/***************************************************************************
 * Copyright 1998-2020 by authors (see AUTHORS.txt)                        *
 *                                                                         *
 *   This file is part of LuxCoreRender.                                   *
 *                                                                         *
 * Licensed under the Apache License, Version 2.0 (the "License");         *
 * you may not use this file except in compliance with the License.        *
 * You may obtain a copy of the License at                                 *
 *                                                                         *
 *     http://www.apache.org/licenses/LICENSE-2.0                          *
 *                                                                         *
 * Unless required by applicable law or agreed to in writing, software     *
 * distributed under the License is distributed on an "AS IS" BASIS,       *
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.*
 * See the License for the specific language governing permissions and     *
 * limitations under the License.                                          *
 ***************************************************************************/

#ifndef _SLG_MIST_PLUGIN_H
#define	_SLG_MIST_PLUGIN_H

#include <vector>
#include <memory>
#include <typeinfo> 

#include "luxrays/luxrays.h"
#include "luxrays/core/color/color.h"
#include "luxrays/utils/serializationutils.h"
#include "slg/film/imagepipeline/imagepipeline.h"

namespace slg {

//------------------------------------------------------------------------------
// Mist plugin
//------------------------------------------------------------------------------

class MistPlugin : public ImagePipelinePlugin {
public:
	MistPlugin(const luxrays::Spectrum &color, float amount, float start, float end, bool excludeBackground);
	virtual ~MistPlugin() {};

	virtual ImagePipelinePlugin *Copy() const;

	virtual void Apply(Film &film, const u_int index);

	friend class boost::serialization::access;

private:
	MistPlugin();

	template<class Archive> void serialize(Archive &ar, const u_int version) {
		ar & BOOST_SERIALIZATION_BASE_OBJECT_NVP(ImagePipelinePlugin);
		// TODO: implement serialization for ImageMap
		throw std::runtime_error("MistPlugin serialization not yet supported");
	}

	luxrays::Spectrum color;
	float amount;
	float start;
	float end;
	bool excludeBackground;
};

}

BOOST_CLASS_VERSION(slg::MistPlugin, 1)

BOOST_CLASS_EXPORT_KEY(slg::MistPlugin)

#endif	/*  _SLG_MIST_PLUGIN_H */
