
#include <string>

namespace luxrays { namespace ocl {
std::string KernelSource_exttrianglemesh_funcs =
"#line 2 \"exttrianglemesh_funcs.cl\"\n"
"\n"
"/***************************************************************************\n"
" * Copyright 1998-2020 by authors (see AUTHORS.txt)                        *\n"
" *                                                                         *\n"
" *   This file is part of LuxCoreRender.                                   *\n"
" *                                                                         *\n"
" * Licensed under the Apache License, Version 2.0 (the \"License\");         *\n"
" * you may not use this file except in compliance with the License.        *\n"
" * You may obtain a copy of the License at                                 *\n"
" *                                                                         *\n"
" *     http://www.apache.org/licenses/LICENSE-2.0                          *\n"
" *                                                                         *\n"
" * Unless required by applicable law or agreed to in writing, software     *\n"
" * distributed under the License is distributed on an \"AS IS\" BASIS,       *\n"
" * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.*\n"
" * See the License for the specific language governing permissions and     *\n"
" * limitations under the License.                                          *\n"
" ***************************************************************************/\n"
"\n"
"OPENCL_FORCE_INLINE float3 ExtMesh_GetGeometryNormal(\n"
"		__global const Transform* restrict localToWorld,\n"
"		const uint meshIndex, const uint triangleIndex\n"
"		EXTMESH_PARAM_DECL) {\n"
"	__global const ExtMesh* restrict meshDesc = &meshDescs[meshIndex];\n"
"	__global const Normal* restrict tn = &triNormals[meshDesc->triNormalsOffset];\n"
"\n"
"	float3 geometryN = VLOAD3F(&tn[triangleIndex].x);\n"
"\n"
"	switch (meshDesc->type) {\n"
"		case TYPE_EXT_TRIANGLE: {\n"
"			// geometryN is already in world coordinates for TYPE_EXT_TRIANGLE\n"
"			// and\n"
"			// pre-computed geometry normals already factor appliedTransSwapsHandedness\n"
"			break;\n"
"		}\n"
"		case TYPE_EXT_TRIANGLE_INSTANCE: {\n"
"			// Transform to global coordinates\n"
"			geometryN = (meshDesc->instance.transSwapsHandedness ? -1.f : 1.f) * normalize(Transform_ApplyNormal(localToWorld, geometryN));\n"
"			break;\n"
"		}\n"
"		case TYPE_EXT_TRIANGLE_MOTION: {\n"
"			const bool swapsHandedness = Transform_SwapsHandedness(localToWorld); \n"
"			// Transform to global coordinates\n"
"			geometryN = (swapsHandedness ? -1.f : 1.f) * normalize(Transform_ApplyNormal(localToWorld, geometryN));\n"
"			break;\n"
"		}\n"
"	}\n"
"\n"
"	return geometryN;\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 ExtMesh_GetInterpolateNormal(\n"
"		__global const Transform* restrict localToWorld,\n"
"		const uint meshIndex, const uint triangleIndex,\n"
"		const float b1, const float b2\n"
"		EXTMESH_PARAM_DECL) {\n"
"	__global const ExtMesh* restrict meshDesc = &meshDescs[meshIndex];\n"
"\n"
"	float3 interpolatedN;\n"
"	if (meshDesc->normalsOffset != NULL_INDEX) {\n"
"		// Shading normal expressed in local coordinates\n"
"		__global const Triangle* restrict tri = &triangles[meshDesc->trisOffset + triangleIndex];\n"
"		__global const Normal* restrict vns = &vertNormals[meshDesc->normalsOffset];\n"
"		const float3 n0 = VLOAD3F(&vns[tri->v[0]].x);\n"
"		const float3 n1 = VLOAD3F(&vns[tri->v[1]].x);\n"
"		const float3 n2 = VLOAD3F(&vns[tri->v[2]].x);\n"
"\n"
"		const float b0 = 1.f - b1 - b2;\n"
"		interpolatedN =  Triangle_InterpolateNormal(n0, n1, n2, b0, b1, b2);\n"
"\n"
"		switch (meshDesc->type) {\n"
"			case TYPE_EXT_TRIANGLE: {\n"
"				// interpolatedN is already in world coordinates for TYPE_EXT_TRIANGLE\n"
"				interpolatedN = (meshDesc->triangle.appliedTransSwapsHandedness ? -1.f : 1.f) * interpolatedN;\n"
"				break;\n"
"			}\n"
"			case TYPE_EXT_TRIANGLE_INSTANCE: {\n"
"				// Transform to global coordinates\n"
"				interpolatedN = (meshDesc->instance.transSwapsHandedness ? -1.f : 1.f) * normalize(Transform_ApplyNormal(localToWorld, interpolatedN));\n"
"				break;\n"
"			}\n"
"			case TYPE_EXT_TRIANGLE_MOTION: {\n"
"				const bool swapsHandedness = Transform_SwapsHandedness(localToWorld); \n"
"				// Transform to global coordinates\n"
"				interpolatedN = (swapsHandedness ? -1.f : 1.f) * normalize(Transform_ApplyNormal(localToWorld, interpolatedN));\n"
"				break;\n"
"			}\n"
"		}\n"
"	} else\n"
"		interpolatedN = ExtMesh_GetGeometryNormal(localToWorld, meshIndex, triangleIndex EXTMESH_PARAM);\n"
"\n"
"	return interpolatedN;\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float2 ExtMesh_GetInterpolateUV(\n"
"		const uint meshIndex, const uint triangleIndex,\n"
"		const float b1, const float b2, const uint dataIndex\n"
"		EXTMESH_PARAM_DECL) {\n"
"	__global const ExtMesh* restrict meshDesc = &meshDescs[meshIndex];\n"
"	\n"
"	float2 uv = MAKE_FLOAT2(0.f, 0.f);\n"
"	if (meshDesc->uvsOffset[dataIndex] != NULL_INDEX) {\n"
"		__global const UV* restrict uvs = &vertUVs[meshDesc->uvsOffset[dataIndex]];\n"
"		__global const Triangle* restrict tri = &triangles[meshDesc->trisOffset + triangleIndex];\n"
"\n"
"		const float2 uv0 = VLOAD2F(&uvs[tri->v[0]].u);\n"
"		const float2 uv1 = VLOAD2F(&uvs[tri->v[1]].u);\n"
"		const float2 uv2 = VLOAD2F(&uvs[tri->v[2]].u);\n"
"\n"
"		const float b0 = 1.f - b1 - b2;\n"
"		uv = Triangle_InterpolateUV(uv0, uv1, uv2, b0, b1, b2);\n"
"	}\n"
"\n"
"	return uv;\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 ExtMesh_GetInterpolateColor(\n"
"		const uint meshIndex, const uint triangleIndex,\n"
"		const float b1, const float b2, const uint dataIndex\n"
"		EXTMESH_PARAM_DECL) {\n"
"	__global const ExtMesh* restrict meshDesc = &meshDescs[meshIndex];\n"
"\n"
"	float3 c = WHITE;\n"
"	if (meshDesc->colsOffset[dataIndex] != NULL_INDEX) {\n"
"		__global const Spectrum* restrict vcs = &vertCols[meshDesc->colsOffset[dataIndex]];\n"
"		__global const Triangle* restrict tri = &triangles[meshDesc->trisOffset + triangleIndex];\n"
"		const float3 rgb0 = VLOAD3F(vcs[tri->v[0]].c);\n"
"		const float3 rgb1 = VLOAD3F(vcs[tri->v[1]].c);\n"
"		const float3 rgb2 = VLOAD3F(vcs[tri->v[2]].c);\n"
"\n"
"		const float b0 = 1.f - b1 - b2;\n"
"		c = Triangle_InterpolateColor(rgb0, rgb1, rgb2, b0, b1, b2);\n"
"	}\n"
"	\n"
"	return c;\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float ExtMesh_GetInterpolateAlpha(\n"
"		const uint meshIndex, const uint triangleIndex,\n"
"		const float b1, const float b2, const uint dataIndex\n"
"		EXTMESH_PARAM_DECL) {\n"
"	__global const ExtMesh* restrict meshDesc = &meshDescs[meshIndex];\n"
"	\n"
"	float a = 1.f;\n"
"	if (meshDesc->alphasOffset[dataIndex] != NULL_INDEX) {\n"
"		__global const float* restrict vas = &vertAlphas[meshDesc->alphasOffset[dataIndex]];\n"
"		__global const Triangle* restrict tri = &triangles[meshDesc->trisOffset + triangleIndex];\n"
"		const float a0 = vas[tri->v[0]];\n"
"		const float a1 = vas[tri->v[1]];\n"
"		const float a2 = vas[tri->v[2]];\n"
"\n"
"		const float b0 = 1.f - b1 - b2;\n"
"		a =  Triangle_InterpolateAlpha(a0, a1, a2, b0, b1, b2);\n"
"	}\n"
"\n"
"	return a;\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float ExtMesh_GetInterpolateVertexAOV(\n"
"		const uint meshIndex, const uint triangleIndex,\n"
"		const float b1, const float b2, const uint dataIndex\n"
"		EXTMESH_PARAM_DECL) {\n"
"	__global const ExtMesh* restrict meshDesc = &meshDescs[meshIndex];\n"
"	\n"
"	float v = 0.f;\n"
"	if (meshDesc->vertexAOVOffset[dataIndex] != NULL_INDEX) {\n"
"		__global const float* restrict vs = &vertexAOVs[meshDesc->vertexAOVOffset[dataIndex]];\n"
"		__global const Triangle* restrict tri = &triangles[meshDesc->trisOffset + triangleIndex];\n"
"		const float v0 = vs[tri->v[0]];\n"
"		const float v1 = vs[tri->v[1]];\n"
"		const float v2 = vs[tri->v[2]];\n"
"\n"
"		const float b0 = 1.f - b1 - b2;\n"
"		v =  Triangle_InterpolateVertexAOV(v0, v1, v2, b0, b1, b2);\n"
"	}\n"
"\n"
"	return v;\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float ExtMesh_GetTriAOV(\n"
"		const uint meshIndex, const uint triangleIndex,\n"
"		const uint dataIndex\n"
"		EXTMESH_PARAM_DECL) {\n"
"	__global const ExtMesh* restrict meshDesc = &meshDescs[meshIndex];\n"
"	\n"
"	float t = 0.f;\n"
"	if (meshDesc->triAOVOffset[dataIndex] != NULL_INDEX) {\n"
"		__global const float* restrict ts = &triAOVs[meshDesc->triAOVOffset[dataIndex]];\n"
"		\n"
"		t = ts[triangleIndex];\n"
"	}\n"
"\n"
"	return t;\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void ExtMesh_GetDifferentials(\n"
"		__global const Transform* restrict localToWorld,\n"
"		const uint meshIndex,\n"
"		const uint triangleIndex,\n"
"		float3 shadeNormal,\n"
"		const uint dataIndex,\n"
"		float3 *dpdu, float3 *dpdv,\n"
"        float3 *dndu, float3 *dndv\n"
"		EXTMESH_PARAM_DECL) {\n"
"	__global const ExtMesh* restrict meshDesc = &meshDescs[meshIndex];\n"
"	__global const Point* restrict iVertices = &vertices[meshDesc->vertsOffset];\n"
"	__global const Triangle* restrict iTriangles = &triangles[meshDesc->trisOffset];\n"
"\n"
"	// Compute triangle partial derivatives\n"
"	__global const Triangle* restrict tri = &iTriangles[triangleIndex];\n"
"	const uint vi0 = tri->v[0];\n"
"	const uint vi1 = tri->v[1];\n"
"	const uint vi2 = tri->v[2];\n"
"\n"
"	float2 uv0, uv1, uv2;\n"
"	if (meshDesc->uvsOffset[dataIndex] != NULL_INDEX) {\n"
"		// Ok, UV coordinates are available, use them to build the reference\n"
"		// system around the shading normal.\n"
"\n"
"		__global const UV* restrict iVertUVs = &vertUVs[meshDesc->uvsOffset[dataIndex]];\n"
"		uv0 = VLOAD2F(&iVertUVs[vi0].u);\n"
"		uv1 = VLOAD2F(&iVertUVs[vi1].u);\n"
"		uv2 = VLOAD2F(&iVertUVs[vi2].u);\n"
"	} else {\n"
"		uv0 = MAKE_FLOAT2(.5f, .5f);\n"
"		uv1 = MAKE_FLOAT2(.5f, .5f);\n"
"		uv2 = MAKE_FLOAT2(.5f, .5f);\n"
"	}\n"
"\n"
"	// Compute deltas for triangle partial derivatives\n"
"	const float du1 = uv0.x - uv2.x;\n"
"	const float du2 = uv1.x - uv2.x;\n"
"	const float dv1 = uv0.y - uv2.y;\n"
"	const float dv2 = uv1.y - uv2.y;\n"
"	const float determinant = du1 * dv2 - dv1 * du2;\n"
"\n"
"	if (determinant == 0.f) {\n"
"		// Handle 0 determinant for triangle partial derivative matrix\n"
"		CoordinateSystem(shadeNormal, dpdu, dpdv);\n"
"		*dndu = ZERO;\n"
"		*dndv = ZERO;\n"
"	} else {\n"
"		const float invdet = 1.f / determinant;\n"
"\n"
"		// Vertices expressed in local coordinates\n"
"		const float3 p0 = VLOAD3F(&iVertices[vi0].x);\n"
"		const float3 p1 = VLOAD3F(&iVertices[vi1].x);\n"
"		const float3 p2 = VLOAD3F(&iVertices[vi2].x);\n"
"		\n"
"		float3 dp1 = p0 - p2;\n"
"		float3 dp2 = p1 - p2;\n"
"\n"
"		// dp1 and dp2 are already in world coordinates for TYPE_EXT_TRIANGLE\n"
"		if (meshDesc->type != TYPE_EXT_TRIANGLE) {\n"
"			// Transform to global coordinates\n"
"			dp1 = Transform_ApplyVector(localToWorld, dp1);\n"
"			dp2 = Transform_ApplyVector(localToWorld, dp2);\n"
"		}\n"
"\n"
"		//------------------------------------------------------------------\n"
"		// Compute dpdu and dpdv\n"
"		//------------------------------------------------------------------\n"
"\n"
"		const float3 geometryDpDu = ( dv2 * dp1 - dv1 * dp2) * invdet;\n"
"		const float3 geometryDpDv = (-du2 * dp1 + du1 * dp2) * invdet;\n"
"\n"
"		*dpdu = cross(shadeNormal, cross(geometryDpDu, shadeNormal));\n"
"		*dpdv = cross(shadeNormal, cross(geometryDpDv, shadeNormal));\n"
"\n"
"		//------------------------------------------------------------------\n"
"		// Compute dndu and dndv\n"
"		//------------------------------------------------------------------\n"
"\n"
"		if (meshDesc->normalsOffset != NULL_INDEX) {\n"
"			__global const Normal* restrict iVertNormals = &vertNormals[meshDesc->normalsOffset];\n"
"			// Shading normals expressed in local coordinates\n"
"			const float3 n0 = VLOAD3F(&iVertNormals[tri->v[0]].x);\n"
"			const float3 n1 = VLOAD3F(&iVertNormals[tri->v[1]].x);\n"
"			const float3 n2 = VLOAD3F(&iVertNormals[tri->v[2]].x);\n"
"			const float3 dn1 = n0 - n2;\n"
"			const float3 dn2 = n1 - n2;\n"
"\n"
"			*dndu = ( dv2 * dn1 - dv1 * dn2) * invdet;\n"
"			*dndv = (-du2 * dn1 + du1 * dn2) * invdet;\n"
"			\n"
"			// dndu and dndv are already in world coordinates for TYPE_EXT_TRIANGLE\n"
"			if (meshDesc->type != TYPE_EXT_TRIANGLE) {\n"
"				// Transform to global coordinates\n"
"				*dndu = Transform_ApplyNormal(localToWorld, *dndu);\n"
"				*dndv = Transform_ApplyNormal(localToWorld, *dndv);\n"
"			}\n"
"		} else {\n"
"			*dndu = ZERO;\n"
"			*dndv = ZERO;\n"
"		}\n"
"	}\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void ExtMesh_Sample(\n"
"		__global const Transform* restrict localToWorld,\n"
"		const uint meshIndex, const uint triangleIndex,\n"
"		const float u0, const float u1,\n"
"		float3 *samplePoint, float *b0, float *b1, float *b2\n"
"		EXTMESH_PARAM_DECL) {\n"
"	__global const ExtMesh* restrict meshDesc = &meshDescs[meshIndex];\n"
"	__global const Point* restrict triVerts = &vertices[meshDesc->vertsOffset];\n"
"	__global const Triangle* restrict tri = &triangles[meshDesc->trisOffset + triangleIndex];\n"
"\n"
"	// Vertices are in local object space\n"
"	float3 p0 = VLOAD3F(&triVerts[tri->v[0]].x);\n"
"	float3 p1 = VLOAD3F(&triVerts[tri->v[1]].x);\n"
"	float3 p2 = VLOAD3F(&triVerts[tri->v[2]].x);\n"
"	*samplePoint = Triangle_Sample(\n"
"			p0, p1, p2,\n"
"			u0, u1,\n"
"			b0, b1, b2);\n"
"\n"
"	// samplePoint is already in world coordinates for TYPE_EXT_TRIANGLE\n"
"	if (meshDesc->type != TYPE_EXT_TRIANGLE) {\n"
"		// Transform to global coordinates\n"
"		*samplePoint = Transform_ApplyPoint(localToWorld, *samplePoint);\n"
"	}\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void ExtMesh_GetLocal2World(const float time,\n"
"		const uint meshIndex, const uint triangleIndex,\n"
"		__global Transform *local2World\n"
"		EXTMESH_PARAM_DECL) {\n"
"	// Initialized world to local object space transformation\n"
"	__global const ExtMesh* restrict meshDesc = &meshDescs[meshIndex];\n"
"\n"
"	switch (meshDesc->type) {\n"
"		case TYPE_EXT_TRIANGLE:\n"
"			*local2World = meshDesc->triangle.appliedTrans;\n"
"			break;\n"
"		case TYPE_EXT_TRIANGLE_INSTANCE:\n"
"			*local2World = meshDesc->instance.trans;\n"
"			break;\n"
"		case TYPE_EXT_TRIANGLE_MOTION: {\n"
"			Matrix4x4 m;\n"
"\n"
"			MotionSystem_Sample(&meshDesc->motion.motionSystem, time, interpolatedTransforms, &m);\n"
"			local2World->mInv = m;\n"
"\n"
"			MotionSystem_SampleInverse(&meshDesc->motion.motionSystem, time, interpolatedTransforms, &m);\n"
"			local2World->m = m;\n"
"\n"
"			break;\n"
"		}\n"
"		default:\n"
"			break;\n"
"	}\n"
"}\n";
} }
