
#include <string>

namespace slg { namespace ocl {
std::string KernelSource_bsdf_funcs =
"#line 2 \"bsdf_funcs.cl\"\n"
"\n"
"/***************************************************************************\n"
" * Copyright 1998-2020 by authors (see AUTHORS.txt)                        *\n"
" *                                                                         *\n"
" *   This file is part of LuxCoreRender.                                   *\n"
" *                                                                         *\n"
" * Licensed under the Apache License, Version 2.0 (the \"License\");         *\n"
" * you may not use this file except in compliance with the License.        *\n"
" * You may obtain a copy of the License at                                 *\n"
" *                                                                         *\n"
" *     http://www.apache.org/licenses/LICENSE-2.0                          *\n"
" *                                                                         *\n"
" * Unless required by applicable law or agreed to in writing, software     *\n"
" * distributed under the License is distributed on an \"AS IS\" BASIS,       *\n"
" * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.*\n"
" * See the License for the specific language governing permissions and     *\n"
" * limitations under the License.                                          *\n"
" ***************************************************************************/\n"
"\n"
"// Used when hitting a surface\n"
"OPENCL_FORCE_INLINE void BSDF_Init(\n"
"		__global BSDF *bsdf,\n"
"		const bool throughShadowTransparency,\n"
"		__global Ray *ray,\n"
"		__global const RayHit *rayHit,\n"
"		const float passThroughEvent,\n"
"		__global PathVolumeInfo *volInfo\n"
"		MATERIALS_PARAM_DECL\n"
"		) {\n"
"	const uint meshIndex = rayHit->meshIndex;\n"
"	const uint triangleIndex = rayHit->triangleIndex;\n"
"\n"
"	// Initialized local to world object space transformation\n"
"	ExtMesh_GetLocal2World(ray->time, meshIndex, triangleIndex, &bsdf->hitPoint.localToWorld EXTMESH_PARAM);\n"
"\n"
"	const float3 rayOrig = VLOAD3F(&ray->o.x);\n"
"	const float3 rayDir = VLOAD3F(&ray->d.x);\n"
"	const float3 hitPointP = rayOrig + rayHit->t * rayDir;\n"
"\n"
"	HitPoint_Init(&bsdf->hitPoint, throughShadowTransparency,\n"
"		meshIndex, triangleIndex,\n"
"		hitPointP, -rayDir,\n"
"		rayHit->b1, rayHit->b2,\n"
"		passThroughEvent\n"
"		MATERIALS_PARAM);\n"
"\n"
"	// Save the scene object index\n"
"	bsdf->sceneObjectIndex = meshIndex;\n"
"\n"
"	// Get the material\n"
"	const uint matIndex = sceneObjs[meshIndex].materialIndex;\n"
"	bsdf->materialIndex = matIndex;\n"
"\n"
"	//--------------------------------------------------------------------------\n"
"	// Set interior and exterior volumes\n"
"	//--------------------------------------------------------------------------\n"
"\n"
"	PathVolumeInfo_SetHitPointVolumes(\n"
"			volInfo,\n"
"			&bsdf->hitPoint,\n"
"			Material_GetInteriorVolume(matIndex, &bsdf->hitPoint,\n"
"				passThroughEvent\n"
"				MATERIALS_PARAM),\n"
"			Material_GetExteriorVolume(matIndex, &bsdf->hitPoint,\n"
"				passThroughEvent\n"
"				MATERIALS_PARAM)\n"
"			MATERIALS_PARAM);\n"
"\n"
"	//--------------------------------------------------------------------------\n"
"\n"
"	// Check if it is a light source\n"
"	const uint offset = lightIndexOffsetByMeshIndex[meshIndex];\n"
"	bsdf->triangleLightSourceIndex = (offset == NULL_INDEX) ? NULL_INDEX : lightIndexByTriIndex[offset + triangleIndex];\n"
"\n"
"	//--------------------------------------------------------------------------\n"
"	// Apply bump or normal mapping\n"
"	//--------------------------------------------------------------------------\n"
"\n"
"	Material_Bump(matIndex, &bsdf->hitPoint\n"
"			MATERIALS_PARAM);\n"
"	// Re-read the shadeN modified by Material_Bump()\n"
"	const float3 shadeN = VLOAD3F(&bsdf->hitPoint.shadeN.x);\n"
"	const float3 dpdu = VLOAD3F(&bsdf->hitPoint.dpdu.x);\n"
"	const float3 dpdv = VLOAD3F(&bsdf->hitPoint.dpdv.x);\n"
"\n"
"	//--------------------------------------------------------------------------\n"
"	// Build the local reference system\n"
"	//--------------------------------------------------------------------------\n"
"	Frame_Set(&bsdf->frame, dpdu, dpdv, shadeN);\n"
"\n"
"	bsdf->isVolume = false;\n"
"}\n"
"\n"
"// Used when hitting a volume scatter point\n"
"OPENCL_FORCE_INLINE void BSDF_InitVolume(\n"
"		__global BSDF *bsdf,\n"
"		const bool throughShadowTransparency,\n"
"		__global const Material* restrict mats,\n"
"		__global Ray *ray,\n"
"		const uint volumeIndex, const float t, const float passThroughEvent) {\n"
"	bsdf->hitPoint.throughShadowTransparency = throughShadowTransparency;\n"
"	bsdf->hitPoint.passThroughEvent = passThroughEvent;\n"
"\n"
"	const float3 rayOrig = VLOAD3F(&ray->o.x);\n"
"	const float3 rayDir = VLOAD3F(&ray->d.x);\n"
"\n"
"	const float3 hitPointP = rayOrig + t * rayDir;\n"
"	VSTORE3F(hitPointP, &bsdf->hitPoint.p.x);\n"
"	const float3 geometryN = -rayDir;\n"
"	VSTORE3F(geometryN, &bsdf->hitPoint.fixedDir.x);\n"
"\n"
"	bsdf->sceneObjectIndex = NULL_INDEX;\n"
"	Matrix4x4_IdentityGlobal(&bsdf->hitPoint.localToWorld.m);\n"
"	bsdf->materialIndex = volumeIndex;\n"
"\n"
"	VSTORE3F(geometryN, &bsdf->hitPoint.geometryN.x);\n"
"	VSTORE3F(geometryN, &bsdf->hitPoint.interpolatedN.x);\n"
"	VSTORE3F(geometryN, &bsdf->hitPoint.shadeN.x);\n"
"\n"
"	bsdf->hitPoint.intoObject = true;\n"
"	bsdf->hitPoint.interiorVolumeIndex = volumeIndex;\n"
"	bsdf->hitPoint.exteriorVolumeIndex = volumeIndex;\n"
"	const uint iorTexIndex = (volumeIndex != NULL_INDEX) ? mats[volumeIndex].volume.iorTexIndex : NULL_INDEX;\n"
"	bsdf->hitPoint.interiorIorTexIndex = iorTexIndex;\n"
"	bsdf->hitPoint.exteriorIorTexIndex = iorTexIndex;\n"
"\n"
"	bsdf->triangleLightSourceIndex = NULL_INDEX;\n"
"\n"
"	VSTORE2F(MAKE_FLOAT2(0.f, 0.f), &bsdf->hitPoint.defaultUV.u);\n"
"\n"
"	float3 dpdu, dpdv;\n"
"	CoordinateSystem(geometryN, &dpdu, &dpdv);\n"
"	VSTORE3F(dpdu, &bsdf->hitPoint.dpdu.x);\n"
"	VSTORE3F(dpdv, &bsdf->hitPoint.dpdv.x);\n"
"	VSTORE3F(MAKE_FLOAT3(0.f, 0.f, 0.f), &bsdf->hitPoint.dndu.x);\n"
"	VSTORE3F(MAKE_FLOAT3(0.f, 0.f, 0.f), &bsdf->hitPoint.dndv.x);\n"
"\n"
"	bsdf->hitPoint.meshIndex = NULL_INDEX;\n"
"	bsdf->hitPoint.triangleIndex = NULL_INDEX;\n"
"	bsdf->hitPoint.triangleBariCoord1 = 0.f;\n"
"	bsdf->hitPoint.triangleBariCoord2 = 0.f;\n"
"\n"
"	bsdf->hitPoint.objectID = NULL_INDEX;\n"
"\n"
"	// Build the local reference system\n"
"	Frame_SetFromZ(&bsdf->frame, geometryN);\n"
"\n"
"	bsdf->isVolume = true;\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 BSDF_Albedo(__global const BSDF *bsdf\n"
"		MATERIALS_PARAM_DECL) {\n"
"	return Material_Albedo(bsdf->materialIndex, &bsdf->hitPoint\n"
"			MATERIALS_PARAM);\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// \"Taming the Shadow Terminator\"\n"
"// by Matt Jen-Yuan Chiang, Yining Karl Li and Brent Burley\n"
"// https://www.yiningkarlli.com/projects/shadowterminator.html\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE float BSDF_ShadowTerminatorAvoidanceFactor(const float3 Ni, const float3 Ns,\n"
"		const float3 lightDir) {\n"
"	const float dotNsLightDir = dot(Ns, lightDir);\n"
"	if (dotNsLightDir <= 0.f)\n"
"		return 0.f;\n"
"\n"
"	const float dotNiNs = dot(Ni, Ns);\n"
"	if (dotNiNs <= 0.f)\n"
"		return 0.f;\n"
"\n"
"	const float G = fmin(1.f, dot(Ni, lightDir) / (dotNsLightDir * dotNiNs));\n"
"	if (G <= 0.f)\n"
"		return 0.f;\n"
"	\n"
"	const float G2 = G * G;\n"
"	const float G3 = G2 * G;\n"
"\n"
"	return -G3 + G2 + G;\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 BSDF_Evaluate(__global const BSDF *bsdf,\n"
"		const float3 generatedDir, BSDFEvent *event, float *directPdfW\n"
"		MATERIALS_PARAM_DECL) {\n"
"	//const Vector &eyeDir = fromLight ? generatedDir : hitPoint.fixedDir;\n"
"	//const Vector &lightDir = fromLight ? hitPoint.fixedDir : generatedDir;\n"
"	const float3 eyeDir = VLOAD3F(&bsdf->hitPoint.fixedDir.x);\n"
"	const float3 lightDir = generatedDir;\n"
"	const float3 geometryN = VLOAD3F(&bsdf->hitPoint.geometryN.x);\n"
"	const float3 interpolatedN =  VLOAD3F(&bsdf->hitPoint.interpolatedN.x);\n"
"	const float3 shadeN =  VLOAD3F(&bsdf->hitPoint.shadeN.x);\n"
"\n"
"	const float dotLightDirNG = dot(lightDir, geometryN);\n"
"	const float absDotLightDirNG = fabs(dotLightDirNG);\n"
"	const float dotEyeDirNG = dot(eyeDir, geometryN);\n"
"	const float absDotEyeDirNG = fabs(dotEyeDirNG);\n"
"\n"
"	if (!bsdf->isVolume) {\n"
"		// These kind of tests make sense only for materials\n"
"\n"
"		// Avoid glancing angles\n"
"		if ((absDotLightDirNG < DEFAULT_COS_EPSILON_STATIC) ||\n"
"				(absDotEyeDirNG < DEFAULT_COS_EPSILON_STATIC))\n"
"			return BLACK;\n"
"\n"
"		// Check geometry normal and light direction side\n"
"		const float sideTest = dotEyeDirNG * dotLightDirNG;\n"
"		const BSDFEvent matEvents = Material_GetEventTypes(bsdf->materialIndex\n"
"				MATERIALS_PARAM);\n"
"		if (((sideTest > 0.f) && !(matEvents & REFLECT)) ||\n"
"				((sideTest < 0.f) && !(matEvents & TRANSMIT)))\n"
"			return BLACK;\n"
"\n"
"		// Check shading normal and light direction side\n"
"		const float sideTestIS = dot(eyeDir, interpolatedN) * dot(lightDir, interpolatedN);\n"
"		if (((sideTestIS > 0.f) && !(matEvents & REFLECT)) ||\n"
"				((sideTestIS < 0.f) && !(matEvents & TRANSMIT)))\n"
"			return BLACK;\n"
"	}\n"
"\n"
"	__global const Frame *frame = &bsdf->frame;\n"
"	const float3 localLightDir = Frame_ToLocal(frame, lightDir);\n"
"	const float3 localEyeDir = Frame_ToLocal(frame, eyeDir);\n"
"	float3 result = Material_Evaluate(bsdf->materialIndex, &bsdf->hitPoint,\n"
"			localLightDir, localEyeDir,	event, directPdfW\n"
"			MATERIALS_PARAM);\n"
"	if (Spectrum_IsBlack(result))\n"
"		return BLACK;\n"
"\n"
"	if (!bsdf->isVolume) {\n"
"		// Shadow terminator artefact avoidance\n"
"		if ((*event & REFLECT) &&\n"
"				(*event & (DIFFUSE | GLOSSY)) &&\n"
"				((shadeN.x != interpolatedN.x) || (shadeN.y != interpolatedN.y) || (shadeN.z != interpolatedN.z)))\n"
"			result *= BSDF_ShadowTerminatorAvoidanceFactor(BSDF_GetLandingInterpolatedN(bsdf),\n"
"					BSDF_GetLandingShadeN(bsdf), lightDir);\n"
"	}\n"
"\n"
"	return result;\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 BSDF_Sample(__global const BSDF *bsdf, const float u0, const float u1,\n"
"		float3 *sampledDir, float *pdfW, float *absCosSampledDir, BSDFEvent *event\n"
"		MATERIALS_PARAM_DECL) {\n"
"	const float3 fixedDir = VLOAD3F(&bsdf->hitPoint.fixedDir.x);\n"
"	const float3 localFixedDir = Frame_ToLocal(&bsdf->frame, fixedDir);\n"
"	float3 localSampledDir;\n"
"\n"
"	float3 result = Material_Sample(bsdf->materialIndex, &bsdf->hitPoint,\n"
"			localFixedDir, &localSampledDir, u0, u1,\n"
"			bsdf->hitPoint.passThroughEvent,\n"
"			pdfW, event\n"
"			MATERIALS_PARAM);\n"
"	if (Spectrum_IsBlack(result))\n"
"		return BLACK;\n"
"\n"
"	*absCosSampledDir = fabs(CosTheta(localSampledDir));\n"
"	*sampledDir = Frame_ToWorld(&bsdf->frame, localSampledDir);\n"
"\n"
"	if (!bsdf->isVolume) {\n"
"		// Shadow terminator artefact avoidance\n"
"		const float3 shadeN = VLOAD3F(&bsdf->hitPoint.shadeN.x);\n"
"		const float3 interpolatedN = VLOAD3F(&bsdf->hitPoint.interpolatedN.x);\n"
"		if ((*event & REFLECT) &&\n"
"				(*event & (DIFFUSE | GLOSSY)) &&\n"
"				((shadeN.x != interpolatedN.x) || (shadeN.y != interpolatedN.y) || (shadeN.z != interpolatedN.z)))\n"
"			result *= BSDF_ShadowTerminatorAvoidanceFactor(BSDF_GetLandingInterpolatedN(bsdf),\n"
"					BSDF_GetLandingShadeN(bsdf), *sampledDir);\n"
"	}\n"
"\n"
"	return result;\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE bool BSDF_IsLightSource(__global const BSDF *bsdf) {\n"
"	return (bsdf->triangleLightSourceIndex != NULL_INDEX);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 BSDF_GetEmittedRadiance(__global const BSDF *bsdf, float *directPdfA\n"
"		LIGHTS_PARAM_DECL) {\n"
"	const uint triangleLightSourceIndex = bsdf->triangleLightSourceIndex;\n"
"	if (triangleLightSourceIndex == NULL_INDEX)\n"
"		return BLACK;\n"
"	else\n"
"		return IntersectableLight_GetRadiance(&lights[triangleLightSourceIndex],\n"
"				&bsdf->hitPoint, directPdfA\n"
"				LIGHTS_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 BSDF_GetPassThroughTransparency(__global const BSDF *bsdf, const bool backTracing\n"
"		MATERIALS_PARAM_DECL) {\n"
"	const float3 localFixedDir = Frame_ToLocal(&bsdf->frame, VLOAD3F(&bsdf->hitPoint.fixedDir.x));\n"
"\n"
"	return Material_GetPassThroughTransparency(bsdf->materialIndex,\n"
"			&bsdf->hitPoint, localFixedDir, bsdf->hitPoint.passThroughEvent,backTracing\n"
"			MATERIALS_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float BSDF_IsPhotonGIEnabled(__global const BSDF *bsdf\n"
"		MATERIALS_PARAM_DECL) {\n"
"	return Material_IsPhotonGIEnabled(bsdf->materialIndex\n"
"			MATERIALS_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE bool BSDF_IsHoldout(__global const BSDF *bsdf\n"
"		MATERIALS_PARAM_DECL) {\n"
"	return Material_IsHoldout(bsdf->materialIndex\n"
"			MATERIALS_PARAM);\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Shadow catcher related functions\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE bool BSDF_IsShadowCatcher(__global const BSDF *bsdf\n"
"		MATERIALS_PARAM_DECL) {\n"
"	const uint matIndex = bsdf->materialIndex;\n"
"\n"
"	return (matIndex == NULL_INDEX) ? false : mats[matIndex].isShadowCatcher;\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE bool BSDF_IsShadowCatcherOnlyInfiniteLights(__global const BSDF *bsdf\n"
"		MATERIALS_PARAM_DECL) {\n"
"	const uint matIndex = bsdf->materialIndex;\n"
"\n"
"	return (matIndex == NULL_INDEX) ? false : (mats[matIndex].isShadowCatcher && mats[matIndex].isShadowCatcherOnlyInfiniteLights);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 BSDF_ShadowCatcherSample(__global const BSDF *bsdf,\n"
"		float3 *sampledDir, float *pdfW, float *absCosSampledDir, BSDFEvent *event\n"
"		MATERIALS_PARAM_DECL) {\n"
"	// Just continue to trace the ray\n"
"	*sampledDir = -VLOAD3F(&bsdf->hitPoint.fixedDir.x);\n"
"	*absCosSampledDir = fabs(dot(*sampledDir, VLOAD3F(&bsdf->hitPoint.geometryN.x)));\n"
"\n"
"	*pdfW = 1.f;\n"
"	*event = SPECULAR | TRANSMIT;\n"
"	const float3 result = WHITE;\n"
"\n"
"	// Adjoint BSDF\n"
"//	if (hitPoint.fromLight) {\n"
"//		const float absDotFixedDirNG = AbsDot(hitPoint.fixedDir, hitPoint.geometryN);\n"
"//		const float absDotSampledDirNG = AbsDot(*sampledDir, hitPoint.geometryN);\n"
"//		return result * (absDotSampledDirNG / absDotFixedDirNG);\n"
"//	} else\n"
"		return result;\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Bake related functions\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE bool BSDF_HasBakeMap(__global const BSDF *bsdf, const BakeMapType type\n"
"		MATERIALS_PARAM_DECL) {\n"
"	return (bsdf->sceneObjectIndex != NULL_INDEX) &&\n"
"			(sceneObjs[bsdf->sceneObjectIndex].bakeMapIndex != NULL_INDEX) &&\n"
"			(sceneObjs[bsdf->sceneObjectIndex].bakeMapType == type);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 BSDF_GetBakeMapValue(__global const BSDF *bsdf\n"
"		MATERIALS_PARAM_DECL) {\n"
"	const uint mapIndex = sceneObjs[bsdf->sceneObjectIndex].bakeMapIndex;\n"
"	__global const ImageMap *imageMap = &imageMapDescs[mapIndex];\n"
"\n"
"	const uint uvIndex = sceneObjs[bsdf->sceneObjectIndex].bakeMapUVIndex;\n"
"	const float2 uv = HitPoint_GetUV(&bsdf->hitPoint, uvIndex EXTMESH_PARAM);\n"
"\n"
"	return ImageMap_GetSpectrum(\n"
"			imageMap,\n"
"			uv.x, uv.y\n"
"			IMAGEMAPS_PARAM);\n"
"}\n";
} }
