
#include <string>

namespace slg { namespace ocl {
std::string KernelSource_camera_funcs =
"#line 2 \"camera_funcs.cl\"\n"
"\n"
"/***************************************************************************\n"
" * Copyright 1998-2016 by authors (see AUTHORS.txt)                        *\n"
" *                                                                         *\n"
" *   This file is part of LuxCoreRender.                                   *\n"
" *                                                                         *\n"
" * Licensed under the Apache License, Version 2.0 (the \"License\");         *\n"
" * you may not use this file except in compliance with the License.        *\n"
" * You may obtain a copy of the License at                                 *\n"
" *                                                                         *\n"
" *     http://www.apache.org/licenses/LICENSE-2.0                          *\n"
" *                                                                         *\n"
" * Unless required by applicable law or agreed to in writing, software     *\n"
" * distributed under the License is distributed on an \"AS IS\" BASIS,       *\n"
" * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.*\n"
" * See the License for the specific language governing permissions and     *\n"
" * limitations under the License.                                          *\n"
" ***************************************************************************/\n"
"\n"
"OPENCL_FORCE_INLINE void Camera_OculusRiftBarrelPostprocess(const float x, const float y, float *barrelX, float *barrelY) {\n"
"	// Express the sample in coordinates relative to the eye center\n"
"	float ex = x * 2.f - 1.f;\n"
"	float ey = y * 2.f - 1.f;\n"
"\n"
"	if ((ex == 0.f) && (ey == 0.f)) {\n"
"		*barrelX = 0.f;\n"
"		*barrelY = 0.f;\n"
"		return;\n"
"	}\n"
"\n"
"	// Distance from the eye center\n"
"	const float distance = sqrt(ex * ex + ey * ey);\n"
"\n"
"	// \"Push\" the sample away base on the distance from the center\n"
"	const float scale = 1.f / 1.4f;\n"
"	const float k0 = 1.f;\n"
"	const float k1 = .22f;\n"
"	const float k2 = .23f;\n"
"	const float k3 = 0.f;\n"
"	const float distance2 = distance * distance;\n"
"	const float distance4 = distance2 * distance2;\n"
"	const float distance6 = distance2 * distance4;\n"
"	const float fr = scale * (k0 + k1 * distance2 + k2 * distance4 + k3 * distance6);\n"
"\n"
"	ex *= fr;\n"
"	ey *= fr;\n"
"\n"
"	// Clamp the coordinates\n"
"	ex = clamp(ex, -1.f, 1.f);\n"
"	ey = clamp(ey, -1.f, 1.f);\n"
"\n"
"	*barrelX = (ex + 1.f) * .5f;\n"
"	*barrelY = (ey + 1.f) * .5f;\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void Camera_ApplyArbitraryClippingPlane(\n"
"		__global Ray *ray,\n"
"		const float3 clippingPlaneCenter, const float3 clippingPlaneNormal) {\n"
"	const float3 rayOrig = MAKE_FLOAT3(ray->o.x, ray->o.y, ray->o.z);\n"
"	const float3 rayDir = MAKE_FLOAT3(ray->d.x, ray->d.y, ray->d.z);\n"
"\n"
"	// Intersect the ray with clipping plane\n"
"	const float denom = dot(clippingPlaneNormal, rayDir);\n"
"	const float3 pr = clippingPlaneCenter - rayOrig;\n"
"	float d = dot(pr, clippingPlaneNormal);\n"
"\n"
"	if (fabs(denom) > DEFAULT_COS_EPSILON_STATIC) {\n"
"		// There is a valid intersection\n"
"		d /= denom; \n"
"\n"
"		if (d > 0.f) {\n"
"			// The plane is in front of the camera\n"
"			if (denom < 0.f) {\n"
"				// The plane points toward the camera\n"
"				ray->maxt = clamp(d, ray->mint, ray->maxt);\n"
"			} else {\n"
"				// The plane points away from the camera\n"
"				ray->mint = clamp(d, ray->mint, ray->maxt);\n"
"			}\n"
"		} else {\n"
"			if ((denom < 0.f) && (d < 0.f)) {\n"
"				// No intersection possible, I use a trick here to avoid any\n"
"				// intersection by setting mint=maxt\n"
"				ray->mint = ray->maxt;\n"
"			} else {\n"
"				// Nothing to do\n"
"			}\n"
"		}\n"
"	} else {\n"
"		// The plane is parallel to the view directions. Check if I'm on the\n"
"		// visible side of the plane or not\n"
"		if (d >= 0.f) {\n"
"			// No intersection possible, I use a trick here to avoid any\n"
"			// intersection by setting mint=maxt\n"
"			ray->mint = ray->maxt;\n"
"		} else {\n"
"			// Nothing to do\n"
"		}\n"
"	}\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Perspective camera\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE void PerspectiveCamera_LocalSampleLens(\n"
"		__global const PerspectiveCamera* restrict cameraPersp,\n"
"		__global const float* restrict cameraBokehDistribution,\n"
"		const float u1, const float u2,\n"
"		float *lensU, float *lensV) {\n"
"	*lensU = 0.f;\n"
"	*lensV = 0.f;\n"
"\n"
"	if (cameraPersp->projCamera.lensRadius > 0.f) {\n"
"		const float bokehBlades = cameraPersp->bokehBlades;\n"
"		const BokehDistributionType bokehDistribution = cameraPersp->bokehDistribution;\n"
"		if ((bokehDistribution == DIST_NONE) || ((bokehDistribution != DIST_CUSTOM) && (bokehBlades < 3)))\n"
"			ConcentricSampleDisk(u1, u2, lensU, lensV);\n"
"		else {\n"
"			// Bokeh support\n"
"			\n"
"			if (bokehDistribution == DIST_CUSTOM) {\n"
"				float2 sampleUV;\n"
"				float distPdf;\n"
"				Distribution2D_SampleContinuous(cameraBokehDistribution, u1, u2, &sampleUV, &distPdf);\n"
"				if (distPdf > 0.f) {\n"
"					*lensU = sampleUV.x * cameraPersp->bokehScaleX;\n"
"					*lensV = sampleUV.y * cameraPersp->bokehScaleY;\n"
"				}	\n"
"			} else {\n"
"				const float halfAngle = M_PI_F / bokehBlades;\n"
"				const float honeyRadius = cos(halfAngle);\n"
"\n"
"				const float theta = 2.f * M_PI_F * u2;\n"
"\n"
"				const uint sector = Floor2UInt(theta / halfAngle);\n"
"				const float rho = (sector % 2 == 0) ? (theta - sector * halfAngle) :\n"
"						((sector + 1) * halfAngle - theta);\n"
"\n"
"				float r = honeyRadius / cos(rho);\n"
"				switch (cameraPersp->bokehDistribution) {\n"
"					case DIST_UNIFORM:\n"
"						r *= sqrt(u1);\n"
"						break;\n"
"					case DIST_EXPONENTIAL:\n"
"						r *= sqrt(ExponentialSampleDisk(u1, cameraPersp->bokehPower));\n"
"						break;\n"
"					case DIST_INVERSEEXPONENTIAL:\n"
"						r *= sqrt(InverseExponentialSampleDisk(u1, cameraPersp->bokehPower));\n"
"						break;\n"
"					case DIST_GAUSSIAN:\n"
"						r *= sqrt(GaussianSampleDisk(u1));\n"
"						break;\n"
"					case DIST_INVERSEGAUSSIAN:\n"
"						r *= sqrt(InverseGaussianSampleDisk(u1));\n"
"						break;\n"
"					case DIST_TRIANGULAR:\n"
"						r *= sqrt(TriangularSampleDisk(u1));\n"
"						break;\n"
"					default:\n"
"						// Something wrong here\n"
"						break;\n"
"				}\n"
"\n"
"				*lensU = r * cos(theta) * cameraPersp->bokehScaleX;\n"
"				*lensV = r * sin(theta) * cameraPersp->bokehScaleY;\n"
"			}\n"
"		}\n"
"\n"
"		const float lensRadius = cameraPersp->projCamera.lensRadius;\n"
"		*lensU *= lensRadius;\n"
"		*lensV *= lensRadius;\n"
"	}\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void PerspectiveCamera_GenerateRayImpl(\n"
"		__global const CameraBase* restrict cameraBase,\n"
"		__global const PerspectiveCamera* restrict cameraPersp,\n"
"		__global const Transform* restrict rasterToCamera,\n"
"		__global const Transform* restrict cameraToWorld,\n"
"		__global const float* restrict cameraBokehDistribution,\n"
"		const uint filmWidth, const uint filmHeight,\n"
"		__global Ray *ray,\n"
"		__global PathVolumeInfo *volInfo,\n"
"		const float filmX, const float filmY, const float timeSample,\n"
"		const float dofSampleX, const float dofSampleY) {\n"
"	PathVolumeInfo_StartVolume(volInfo, cameraBase->volumeIndex);\n"
"\n"
"	float3 Pras;\n"
"	if (cameraPersp->enableOculusRiftBarrel) {\n"
"		float ssx, ssy;\n"
"		Camera_OculusRiftBarrelPostprocess(filmX / filmWidth, (filmHeight - filmY - 1.f) / filmHeight, &ssx, &ssy);\n"
"		Pras = MAKE_FLOAT3(min(ssx * filmWidth, (float) (filmWidth - 1)), min(ssy * filmHeight, (float) (filmHeight - 1)), 0.f);\n"
"	} else\n"
"		Pras = MAKE_FLOAT3(filmX, filmHeight - filmY - 1.f, 0.f);\n"
"\n"
"	float3 rayOrig = Transform_ApplyPoint(rasterToCamera, Pras);\n"
"	float3 rayDir = rayOrig;\n"
"\n"
"	const float hither = cameraBase->hither;\n"
"\n"
"	const float lensRadius = cameraPersp->projCamera.lensRadius;\n"
"	const float focalDistance = cameraPersp->projCamera.focalDistance;\n"
"	if ((lensRadius > 0.f) && (focalDistance > 0.f)) {\n"
"		// Sample point on lens\n"
"		float lensU, lensV;\n"
"		PerspectiveCamera_LocalSampleLens(cameraPersp, cameraBokehDistribution, dofSampleX, dofSampleY, &lensU, &lensV);\n"
"\n"
"		// Compute point on plane of focus\n"
"		const float dist = focalDistance - hither;\n"
"\n"
"		const float ft = dist / rayDir.z;\n"
"		const float3 Pfocus = rayOrig + rayDir * ft;\n"
"\n"
"		// Update ray for effect of lens\n"
"		const float k = dist / focalDistance;\n"
"		rayOrig.x += lensU * k;\n"
"		rayOrig.y += lensV * k;\n"
"\n"
"		rayDir = Pfocus - rayOrig;\n"
"	}\n"
"\n"
"	rayDir = normalize(rayDir);\n"
"\n"
"	const float maxt = (cameraBase->yon - hither) / rayDir.z;\n"
"	const float time = mix(cameraBase->shutterOpen, cameraBase->shutterClose, timeSample);\n"
"\n"
"	// Transform ray in world coordinates\n"
"	rayOrig = Transform_ApplyPoint(cameraToWorld, rayOrig);\n"
"	rayDir = Transform_ApplyVector(cameraToWorld, rayDir);\n"
"\n"
"	const uint interpolatedTransformFirstIndex = cameraBase->motionSystem.interpolatedTransformFirstIndex;\n"
"	if (interpolatedTransformFirstIndex != NULL_INDEX) {\n"
"		Matrix4x4 m;\n"
"		MotionSystem_Sample(&cameraBase->motionSystem, time, cameraBase->interpolatedTransforms, &m);\n"
"\n"
"		rayOrig = Matrix4x4_ApplyPoint_Private(&m, rayOrig);\n"
"		rayDir = Matrix4x4_ApplyVector_Private(&m, rayDir);\n"
"	}\n"
"\n"
"	Ray_Init3(ray, rayOrig, rayDir, maxt, time);\n"
"	\n"
"	if (cameraPersp->projCamera.enableClippingPlane) {\n"
"		Camera_ApplyArbitraryClippingPlane(ray,\n"
"				VLOAD3F(&cameraPersp->projCamera.clippingPlaneCenter.x),\n"
"				VLOAD3F(&cameraPersp->projCamera.clippingPlaneNormal.x));\n"
"	}\n"
"\n"
"	/*printf(\"(%f, %f, %f) (%f, %f, %f) [%f, %f]\\n\",\n"
"		ray->o.x, ray->o.y, ray->o.z, ray->d.x, ray->d.y, ray->d.z,\n"
"		ray->mint, ray->maxt);*/\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void PerspectiveCamera_GenerateRay(\n"
"		__global const Camera* restrict camera,\n"
"		__global const float* restrict cameraBokehDistribution,\n"
"		const uint filmWidth, const uint filmHeight,\n"
"		__global Ray *ray,\n"
"		__global PathVolumeInfo *volInfo,\n"
"		const float filmX, const float filmY, const float timeSample,\n"
"		const float dofSampleX, const float dofSampleY) {\n"
"	PerspectiveCamera_GenerateRayImpl(&camera->base, &camera->persp,\n"
"			&camera->base.rasterToCamera, &camera->base.cameraToWorld,\n"
"			cameraBokehDistribution,\n"
"			filmWidth, filmHeight, ray, volInfo, filmX, filmY,\n"
"			timeSample, dofSampleX, dofSampleY);\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Orthographic camera\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE void OrthographicCamera_GenerateRay(\n"
"		__global const Camera* restrict camera,\n"
"		const uint filmWidth, const uint filmHeight,\n"
"		__global Ray *ray,\n"
"		__global PathVolumeInfo *volInfo,\n"
"		const float filmX, const float filmY, const float timeSample,\n"
"		const float dofSampleX, const float dofSampleY) {\n"
"	PathVolumeInfo_StartVolume(volInfo, camera->base.volumeIndex);\n"
"\n"
"	const float3 Pras = MAKE_FLOAT3(filmX, filmHeight - filmY - 1.f, 0.f);\n"
"	float3 rayOrig = Transform_ApplyPoint(&camera->base.rasterToCamera, Pras);\n"
"	float3 rayDir = MAKE_FLOAT3(0.f, 0.f, 1.f);\n"
"\n"
"	const float hither = camera->base.hither;\n"
"\n"
"	const float lensRadius = camera->ortho.projCamera.lensRadius;\n"
"	const float focalDistance = camera->ortho.projCamera.focalDistance;\n"
"	if ((lensRadius > 0.f) && (focalDistance > 0.f)) {\n"
"		// Sample point on lens\n"
"		float lensU, lensV;\n"
"		ConcentricSampleDisk(dofSampleX, dofSampleY, &lensU, &lensV);\n"
"		lensU *= lensRadius;\n"
"		lensV *= lensRadius;\n"
"\n"
"		// Compute point on plane of focus\n"
"		const float dist = focalDistance - hither;\n"
"\n"
"		const float ft = dist;\n"
"		const float3 Pfocus = rayOrig + rayDir * ft;\n"
"\n"
"		// Update ray for effect of lens\n"
"		const float k = dist / focalDistance;\n"
"		rayOrig.x += lensU * k;\n"
"		rayOrig.y += lensV * k;\n"
"\n"
"		rayDir = Pfocus - rayOrig;\n"
"	}\n"
"\n"
"	rayDir = normalize(rayDir);\n"
"\n"
"	const float maxt = (camera->base.yon - hither);\n"
"	const float time = mix(camera->base.shutterOpen, camera->base.shutterClose, timeSample);\n"
"\n"
"	// Transform ray in world coordinates\n"
"	rayOrig = Transform_ApplyPoint(&camera->base.cameraToWorld, rayOrig);\n"
"	rayDir = Transform_ApplyVector(&camera->base.cameraToWorld, rayDir);\n"
"\n"
"	const uint interpolatedTransformFirstIndex = camera->base.motionSystem.interpolatedTransformFirstIndex;\n"
"	if (interpolatedTransformFirstIndex != NULL_INDEX) {\n"
"		Matrix4x4 m;\n"
"		MotionSystem_Sample(&camera->base.motionSystem, time, camera->base.interpolatedTransforms, &m);\n"
"\n"
"		rayOrig = Matrix4x4_ApplyPoint_Private(&m, rayOrig);\n"
"		rayDir = Matrix4x4_ApplyVector_Private(&m, rayDir);\n"
"	}\n"
"\n"
"	Ray_Init3(ray, rayOrig, rayDir, maxt, time);\n"
"\n"
"	if (camera->ortho.projCamera.enableClippingPlane) {\n"
"		Camera_ApplyArbitraryClippingPlane(ray,\n"
"				VLOAD3F(&camera->ortho.projCamera.clippingPlaneCenter.x),\n"
"				VLOAD3F(&camera->ortho.projCamera.clippingPlaneNormal.x));\n"
"	}\n"
"\n"
"\n"
"	/*printf(\"(%f, %f, %f) (%f, %f, %f) [%f, %f]\\n\",\n"
"		ray->o.x, ray->o.y, ray->o.z, ray->d.x, ray->d.y, ray->d.z,\n"
"		ray->mint, ray->maxt);*/\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Environment camera\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE void EnvironmentCamera_GenerateRayImpl(\n"
"		__global const CameraBase* restrict cameraBase,\n"
"		const float degrees,\n"
"		__global const Transform* restrict cameraToWorld,\n"
"		const uint filmWidth, const uint filmHeight,\n"
"		__global Ray *ray,\n"
"		__global PathVolumeInfo *volInfo,\n"
"		const float filmX, const float filmY, const float timeSample,\n"
"		const float dofSampleX, const float dofSampleY) {\n"
"	PathVolumeInfo_StartVolume(volInfo, cameraBase->volumeIndex);\n"
"\n"
"	const float theta = M_PI_F * (filmHeight - filmY) / filmHeight;\n"
"	const float phi = Radians(360.f - ((360.f - degrees) * .5f + degrees * filmX / filmWidth)) - .5 * M_PI_F;\n"
"\n"
"	float3 rayOrig = MAKE_FLOAT3(0.f, 0.f, 0.f);\n"
"	float3 rayDir = MAKE_FLOAT3(sin(theta)*cos(phi), cos(theta), sin(theta)*sin(phi));\n"
"	\n"
"	const float maxt = (cameraBase->yon - cameraBase->hither);\n"
"	const float time = mix(cameraBase->shutterOpen, cameraBase->shutterClose, timeSample);\n"
"\n"
"	// Transform ray in world coordinates\n"
"	rayOrig = Transform_ApplyPoint(cameraToWorld, rayOrig);\n"
"	rayDir = Transform_ApplyVector(cameraToWorld, rayDir);\n"
"\n"
"	const uint interpolatedTransformFirstIndex = cameraBase->motionSystem.interpolatedTransformFirstIndex;\n"
"	if (interpolatedTransformFirstIndex != NULL_INDEX) {\n"
"		Matrix4x4 m;\n"
"		MotionSystem_Sample(&cameraBase->motionSystem, time, cameraBase->interpolatedTransforms, &m);\n"
"\n"
"		rayOrig = Matrix4x4_ApplyPoint_Private(&m, rayOrig);\n"
"		rayDir = Matrix4x4_ApplyVector_Private(&m, rayDir);\n"
"	}\n"
"\n"
"	Ray_Init3(ray, rayOrig, rayDir, maxt, time);\n"
"\n"
"	/*printf(\"(%f, %f, %f) (%f, %f, %f) [%f, %f]\\n\",\n"
"		ray->o.x, ray->o.y, ray->o.z, ray->d.x, ray->d.y, ray->d.z,\n"
"		ray->mint, ray->maxt);*/\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void EnvironmentCamera_GenerateRay(\n"
"		__global const Camera* restrict camera,\n"
"		const uint filmWidth, const uint filmHeight,\n"
"		__global Ray *ray,\n"
"		__global PathVolumeInfo *volInfo,\n"
"		const float filmX, const float filmY, const float timeSample,\n"
"		const float dofSampleX, const float dofSampleY) {\n"
"	EnvironmentCamera_GenerateRayImpl(&camera->base, camera->env.degrees, &camera->base.cameraToWorld,\n"
"			filmWidth, filmHeight, ray, volInfo,\n"
"			filmX, filmY, timeSample, dofSampleX, dofSampleY);\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Stereo camera\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE void StereoCamera_GenerateRay(\n"
"		__global const Camera* restrict camera,\n"
"		__global const float* restrict cameraBokehDistribution,\n"
"		const uint origFilmWidth, const uint origFilmHeight,\n"
"		__global Ray *ray,\n"
"		__global PathVolumeInfo *volInfo,\n"
"		const float origFilmX, const float origFilmY, const float timeSample,\n"
"		const float dofSampleX, const float dofSampleY) {\n"
"	PathVolumeInfo_StartVolume(volInfo, camera->base.volumeIndex);\n"
"\n"
"	switch(camera->stereo.stereoCameraType) {\n"
"		case STEREO_PERSPECTIVE: {\n"
"			__global const Transform* restrict rasterToCamera;\n"
"			__global const Transform* restrict cameraToWorld;\n"
"			float filmX, filmY;\n"
"			const float filmWidth = origFilmWidth / 2;\n"
"			const float filmHeight = origFilmHeight;\n"
"			if (origFilmX < filmWidth) {\n"
"				rasterToCamera = &camera->stereo.leftEyeRasterToCamera;\n"
"				cameraToWorld = &camera->stereo.leftEyeCameraToWorld;\n"
"				filmX = origFilmX;\n"
"			} else {\n"
"				rasterToCamera = &camera->stereo.rightEyeRasterToCamera;\n"
"				cameraToWorld = &camera->stereo.rightEyeCameraToWorld;\n"
"				filmX = origFilmX - filmWidth;\n"
"			}\n"
"			filmY = origFilmY;\n"
"\n"
"			PerspectiveCamera_GenerateRayImpl(&camera->base, &camera->stereo.perspCamera,\n"
"					rasterToCamera, cameraToWorld, cameraBokehDistribution,\n"
"					filmWidth, filmHeight, ray, volInfo, filmX, filmY, timeSample, dofSampleX, dofSampleY);\n"
"			break;\n"
"		}\n"
"		case STEREO_ENVIRONMENT_180: {\n"
"			__global const Transform* restrict rasterToCamera;\n"
"			__global const Transform* restrict cameraToWorld;\n"
"			float filmX, filmY;\n"
"			const float filmWidth = origFilmWidth / 2;\n"
"			const float filmHeight = origFilmHeight;\n"
"			if (origFilmX < filmWidth) {\n"
"				rasterToCamera = &camera->stereo.leftEyeRasterToCamera;\n"
"				cameraToWorld = &camera->stereo.leftEyeCameraToWorld;\n"
"				filmX = origFilmX;\n"
"			} else {\n"
"				rasterToCamera = &camera->stereo.rightEyeRasterToCamera;\n"
"				cameraToWorld = &camera->stereo.rightEyeCameraToWorld;\n"
"				filmX = origFilmX - filmWidth;\n"
"			}\n"
"			filmY = origFilmY;\n"
"\n"
"			EnvironmentCamera_GenerateRayImpl(&camera->base, 180.f, cameraToWorld,\n"
"					filmWidth, filmHeight, ray, volInfo,\n"
"					filmX, filmY, timeSample, dofSampleX, dofSampleY);\n"
"			break;\n"
"		}\n"
"		case STEREO_ENVIRONMENT_360: {\n"
"			__global const Transform* restrict rasterToCamera;\n"
"			__global const Transform* restrict cameraToWorld;\n"
"			float filmX, filmY;\n"
"			const float filmWidth = origFilmWidth;\n"
"			const float filmHeight = origFilmHeight / 2;\n"
"			filmX = origFilmX;\n"
"			if (origFilmY < filmHeight) {\n"
"				rasterToCamera = &camera->stereo.leftEyeRasterToCamera;\n"
"				cameraToWorld = &camera->stereo.leftEyeCameraToWorld;\n"
"				filmY = origFilmY;\n"
"			} else {\n"
"				rasterToCamera = &camera->stereo.rightEyeRasterToCamera;\n"
"				cameraToWorld = &camera->stereo.rightEyeCameraToWorld;\n"
"				filmY = origFilmY - filmHeight;\n"
"			}\n"
"\n"
"			EnvironmentCamera_GenerateRayImpl(&camera->base, 360.f, cameraToWorld,\n"
"					filmWidth, filmHeight, ray, volInfo,\n"
"					filmX, filmY, timeSample, dofSampleX, dofSampleY);\n"
"			break;\n"
"		}\n"
"		default:\n"
"			// Something very wrong here...\n"
"			break;\n"
"	}\n"
"\n"
"	/*printf(\"(%f, %f, %f) (%f, %f, %f) [%f, %f]\\n\",\n"
"		ray->o.x, ray->o.y, ray->o.z, ray->d.x, ray->d.y, ray->d.z,\n"
"		ray->mint, ray->maxt);*/\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Generic function\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_NOT_INLINE void Camera_GenerateRay(\n"
"		__global const Camera* restrict camera,\n"
"		__global const float* restrict cameraBokehDistribution,\n"
"		const uint filmWidth, const uint filmHeight,\n"
"		__global Ray *ray,\n"
"		__global PathVolumeInfo *volInfo,\n"
"		const float filmX, const float filmY, const float timeSample,\n"
"		const float dofSampleX, const float dofSampleY) {\n"
"	switch (camera->type) {\n"
"		case PERSPECTIVE:\n"
"			PerspectiveCamera_GenerateRay(camera, cameraBokehDistribution, filmWidth, filmHeight, ray, volInfo, filmX, filmY, timeSample, dofSampleX, dofSampleY);\n"
"			break;\n"
"		case ORTHOGRAPHIC:\n"
"			OrthographicCamera_GenerateRay(camera, filmWidth, filmHeight, ray, volInfo, filmX, filmY, timeSample, dofSampleX, dofSampleY);\n"
"			break;\n"
"		case STEREO:\n"
"			StereoCamera_GenerateRay(camera, cameraBokehDistribution, filmWidth, filmHeight, ray, volInfo, filmX, filmY, timeSample, dofSampleX, dofSampleY);\n"
"			break;\n"
"		case ENVIRONMENT:\n"
"			EnvironmentCamera_GenerateRay(camera, filmWidth, filmHeight, ray, volInfo, filmX, filmY, timeSample, dofSampleX, dofSampleY);\n"
"			break;\n"
"		default:\n"
"			// Something really wrong here\n"
"			break;\n"
"	}\n"
"}\n";
} }
