
#include <string>

namespace slg { namespace ocl {
std::string KernelSource_elvc_funcs =
"#line 2 \"elvc_funcs.cl\"\n"
"\n"
"/***************************************************************************\n"
" * Copyright 1998-2020 by authors (see AUTHORS.txt)                        *\n"
" *                                                                         *\n"
" *   This file is part of LuxCoreRender.                                   *\n"
" *                                                                         *\n"
" * Licensed under the Apache License, Version 2.0 (the \"License\");         *\n"
" * you may not use this file except in compliance with the License.        *\n"
" * You may obtain a copy of the License at                                 *\n"
" *                                                                         *\n"
" *     http://www.apache.org/licenses/LICENSE-2.0                          *\n"
" *                                                                         *\n"
" * Unless required by applicable law or agreed to in writing, software     *\n"
" * distributed under the License is distributed on an \"AS IS\" BASIS,       *\n"
" * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.*\n"
" * See the License for the specific language governing permissions and     *\n"
" * limitations under the License.                                          *\n"
" ***************************************************************************/\n"
"\n"
"OPENCL_FORCE_INLINE bool EnvLightVisibilityCache_IsCacheEnabled(__global const BSDF *bsdf\n"
"		MATERIALS_PARAM_DECL) {\n"
"	return !BSDF_IsDelta(bsdf MATERIALS_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE __global const ELVCacheEntry* restrict EnvLightVisibilityCache_GetNearestEntry(\n"
"		__global const ELVCacheEntry* restrict elvcAllEntries,\n"
"		__global const float* restrict elvcDistributions,\n"
"		__global const IndexBVHArrayNode* restrict elvcBVHNodes,\n"
"		const float elvcRadius2, const float elvcNormalCosAngle,\n"
"		const float3 p, const float3 n, const bool isVolume\n"
"		) {\n"
"	__global const ELVCacheEntry* restrict nearestEntry = NULL;\n"
"	float nearestDistance2 = elvcRadius2;\n"
"\n"
"	uint currentNode = 0; // Root Node\n"
"	const uint stopNode = IndexBVHNodeData_GetSkipIndex(elvcBVHNodes[0].nodeData); // Non-existent\n"
"\n"
"	while (currentNode < stopNode) {\n"
"		__global const IndexBVHArrayNode* restrict node = &elvcBVHNodes[currentNode];\n"
"\n"
"		const uint nodeData = node->nodeData;\n"
"		if (IndexBVHNodeData_IsLeaf(nodeData)) {\n"
"			// It is a leaf, check the entry\n"
"			__global const ELVCacheEntry* restrict entry = &elvcAllEntries[node->entryLeaf.entryIndex];\n"
"\n"
"			const float distance2 = DistanceSquared(p, VLOAD3F(&entry->p.x));\n"
"			if ((distance2 <= nearestDistance2) &&\n"
"					(isVolume == entry->isVolume) && \n"
"					(isVolume ||\n"
"					(dot(n, VLOAD3F(&entry->n.x)) >= elvcNormalCosAngle)\n"
"					)\n"
"				) {\n"
"				// I have found a valid entry\n"
"\n"
"				nearestEntry = entry;\n"
"				nearestDistance2 = distance2;\n"
"			}\n"
"\n"
"			++currentNode;\n"
"		} else {\n"
"			// It is a node, check the bounding box\n"
"			if (p.x >= node->bvhNode.bboxMin[0] && p.x <= node->bvhNode.bboxMax[0] &&\n"
"					p.y >= node->bvhNode.bboxMin[1] && p.y <= node->bvhNode.bboxMax[1] &&\n"
"					p.z >= node->bvhNode.bboxMin[2] && p.z <= node->bvhNode.bboxMax[2])\n"
"				++currentNode;\n"
"			else {\n"
"				// I don't need to use IndexBVHNodeData_GetSkipIndex() here because\n"
"				// I already know the leaf flag is 0\n"
"				currentNode = nodeData;\n"
"			}\n"
"		}\n"
"	}\n"
"\n"
"	return nearestEntry;\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE __global const float* restrict EnvLightVisibilityCache_GetVisibilityMap(__global const BSDF *bsdf\n"
"		LIGHTS_PARAM_DECL) {\n"
"	__global const ELVCacheEntry* restrict cacheEntry = EnvLightVisibilityCache_GetNearestEntry(\n"
"			elvcAllEntries, elvcDistributions, elvcBVHNodes, elvcRadius2, elvcNormalCosAngle,\n"
"			VLOAD3F(&bsdf->hitPoint.p.x), BSDF_GetLandingShadeN(bsdf), bsdf->isVolume\n"
"			);\n"
"\n"
"	return (cacheEntry && (cacheEntry->distributionOffset != NULL_INDEX)) ? &elvcDistributions[cacheEntry->distributionOffset] : NULL;\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Sample\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE void EnvLightVisibilityCache_Sample(__global const BSDF *bsdf,\n"
"		const float u0, const float u1, float2 *uv, float *pdf\n"
"		LIGHTS_PARAM_DECL) {\n"
"	*pdf = 0.f;\n"
"\n"
"	__global const float* restrict cacheDist = EnvLightVisibilityCache_GetVisibilityMap(bsdf LIGHTS_PARAM);\n"
"\n"
"	if (cacheDist) {\n"
"		uint cacheDistXY[2];\n"
"		float cacheDistPdf, du0, du1;\n"
"		Distribution2D_SampleDiscrete(cacheDist, u0, u1, cacheDistXY, &cacheDistPdf, &du0, &du1);\n"
"\n"
"		if (cacheDistPdf > 0.f) {\n"
"			if (elvcTileDistributionOffsets) {\n"
"				const uint tileDistributionOffset = elvcTileDistributionOffsets[cacheDistXY[0] + cacheDistXY[1] * elvcTilesXCount];\n"
"				__global const float* restrict tileDistribution = &elvcDistributions[tileDistributionOffset];\n"
"\n"
"				float2 tileXY;\n"
"				float tileDistPdf;\n"
"				Distribution2D_SampleContinuous(tileDistribution, du0, du1, &tileXY, &tileDistPdf);\n"
"\n"
"				if (tileDistPdf > 0.f) {\n"
"					(*uv).x = (cacheDistXY[0] + tileXY.x) / elvcTilesXCount;\n"
"					(*uv).y = (cacheDistXY[1] + tileXY.y) / elvcTilesYCount;\n"
"\n"
"					*pdf = cacheDistPdf * tileDistPdf * (elvcTilesXCount * elvcTilesYCount);\n"
"				}\n"
"			} else {\n"
"				(*uv).x = (cacheDistXY[0] + du0) / elvcTilesXCount;\n"
"				(*uv).y = (cacheDistXY[1] + du1) / elvcTilesYCount;\n"
"\n"
"				*pdf = cacheDistPdf * (elvcTilesXCount * elvcTilesYCount);\n"
"			}\n"
"		}\n"
"	}\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Pdf\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE float EnvLightVisibilityCache_Pdf(__global const BSDF *bsdf,\n"
"		const float u, const float v\n"
"		LIGHTS_PARAM_DECL) {\n"
"	float pdf = 0.f;\n"
"\n"
"	__global const float* restrict cacheDist = EnvLightVisibilityCache_GetVisibilityMap(bsdf LIGHTS_PARAM);\n"
"\n"
"	if (cacheDist) {\n"
"		uint offsetU, offsetV;\n"
"		float du, dv;\n"
"		const float cacheDistPdf = Distribution2D_PdfExt(cacheDist, u, v, &du, &dv, &offsetU, &offsetV);\n"
"\n"
"		if (cacheDistPdf > 0.f) {\n"
"			if (elvcTileDistributionOffsets) {\n"
"				const uint tileDistributionOffset = elvcTileDistributionOffsets[offsetU + offsetV * elvcTilesXCount];\n"
"				__global const float* restrict tileDistribution = &elvcDistributions[tileDistributionOffset];\n"
"\n"
"				const float tileDistPdf = Distribution2D_Pdf(tileDistribution, du, dv);\n"
"\n"
"				pdf = cacheDistPdf * tileDistPdf;\n"
"			} else\n"
"				pdf = cacheDistPdf;\n"
"		}\n"
"	}\n"
"\n"
"	return pdf;\n"
"}\n";
} }
