
#include <string>

namespace slg { namespace ocl {
std::string KernelSource_film_mergesamplebuffer_funcs =
"#line 2 \"film_mergesamplebuffer_funcs.cl\"\n"
"\n"
"/***************************************************************************\n"
" * Copyright 1998-2020 by authors (see AUTHORS.txt)                        *\n"
" *                                                                         *\n"
" *   This file is part of LuxCoreRender.                                   *\n"
" *                                                                         *\n"
" * Licensed under the Apache License, Version 2.0 (the \"License\");         *\n"
" * you may not use this file except in compliance with the License.        *\n"
" * You may obtain a copy of the License at                                 *\n"
" *                                                                         *\n"
" *     http://www.apache.org/licenses/LICENSE-2.0                          *\n"
" *                                                                         *\n"
" * Unless required by applicable law or agreed to in writing, software     *\n"
" * distributed under the License is distributed on an \"AS IS\" BASIS,       *\n"
" * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either expixelRess or implied.*\n"
" * See the License for the specific language governing permissions and     *\n"
" * limitations under the License.                                          *\n"
" ***************************************************************************/\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Film_MergeBufferInitialize \n"
"//------------------------------------------------------------------------------\n"
"\n"
"__kernel void Film_MergeBufferInitialize(\n"
"		const uint filmWidth, const uint filmHeight,\n"
"		__global float *channel_IMAGEPIPELINE) {\n"
"	const size_t gid = get_global_id(0);\n"
"	if (gid > filmWidth * filmHeight)\n"
"		return;\n"
"\n"
"	__global float *channelBufferPixel = &channel_IMAGEPIPELINE[gid * 3];\n"
"	// I use INFINITY to mark pixel with no samples\n"
"	channelBufferPixel[0] = INFINITY;\n"
"	channelBufferPixel[1] = INFINITY;\n"
"	channelBufferPixel[2] = INFINITY;\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Film_MergeRADIANCE_PER_PIXEL_NORMALIZED \n"
"//------------------------------------------------------------------------------\n"
"\n"
"__kernel void Film_MergeRADIANCE_PER_PIXEL_NORMALIZED(\n"
"		const uint filmWidth, const uint filmHeight,\n"
"		__global float *channel_IMAGEPIPELINE,\n"
"		__global float *mergeBuffer,\n"
"		const float scaleR, const float scaleG, const float scaleB) {\n"
"	const size_t gid = get_global_id(0);\n"
"	if (gid > filmWidth * filmHeight)\n"
"		return;\n"
"\n"
"	__global const float *mergeBufferPixel = &mergeBuffer[gid * 4];\n"
"	const float w = mergeBufferPixel[3];\n"
"\n"
"	if (w > 0.f) {\n"
"		float r = mergeBufferPixel[0];\n"
"		float g = mergeBufferPixel[1];\n"
"		float b = mergeBufferPixel[2];\n"
"\n"
"		const float iw = 1.f / w;\n"
"		r *= iw;\n"
"		g *= iw;\n"
"		b *= iw;\n"
"\n"
"		r *= scaleR;\n"
"		g *= scaleG;\n"
"		b *= scaleB;\n"
"\n"
"		__global float *channelBufferPixel = &channel_IMAGEPIPELINE[gid * 3];\n"
"		const float pixelR = channelBufferPixel[0];\n"
"		const float pixelG = channelBufferPixel[1];\n"
"		const float pixelB = channelBufferPixel[2];\n"
"\n"
"		channelBufferPixel[0] = isinf(pixelR) ? r : (pixelR + r);\n"
"		channelBufferPixel[1] = isinf(pixelG) ? g : (pixelG + g);\n"
"		channelBufferPixel[2] = isinf(pixelB) ? b : (pixelB + b);\n"
"	}\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Film_MergeRADIANCE_PER_SCREEN_NORMALIZED \n"
"//------------------------------------------------------------------------------\n"
"\n"
"__kernel void Film_MergeRADIANCE_PER_SCREEN_NORMALIZED(\n"
"		const uint filmWidth, const uint filmHeight,\n"
"		__global float *channel_IMAGEPIPELINE,\n"
"		__global float *mergeBuffer,\n"
"		const float scaleR, const float scaleG, const float scaleB) {\n"
"	const size_t gid = get_global_id(0);\n"
"	if (gid > filmWidth * filmHeight)\n"
"		return;\n"
"\n"
"	__global const float *mergeBufferPixel = &mergeBuffer[gid * 3];\n"
"	float r = mergeBufferPixel[0];\n"
"	float g = mergeBufferPixel[1];\n"
"	float b = mergeBufferPixel[2];\n"
"\n"
"	if ((r != 0.f) || (g != 0.f) || (b != 0.f)) {\n"
"		r *= scaleR;\n"
"		g *= scaleG;\n"
"		b *= scaleB;\n"
"\n"
"		__global float *channelBufferPixel = &channel_IMAGEPIPELINE[gid * 3];\n"
"		const float pixelR = channelBufferPixel[0];\n"
"		const float pixelG = channelBufferPixel[1];\n"
"		const float pixelB = channelBufferPixel[2];\n"
"\n"
"		channelBufferPixel[0] = isinf(pixelR) ? r : (pixelR + r);\n"
"		channelBufferPixel[1] = isinf(pixelG) ? g : (pixelG + g);\n"
"		channelBufferPixel[2] = isinf(pixelB) ? b : (pixelB + b);\n"
"	}\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Film_MergeBufferfinalize \n"
"//------------------------------------------------------------------------------\n"
"\n"
"__kernel void Film_MergeBufferFinalize(\n"
"		const uint filmWidth, const uint filmHeight,\n"
"		__global float *channel_IMAGEPIPELINE) {\n"
"	const size_t gid = get_global_id(0);\n"
"	if (gid > filmWidth * filmHeight)\n"
"		return;\n"
"\n"
"	__global float *channelBufferPixel = &channel_IMAGEPIPELINE[gid * 3];\n"
"	const float pixelR = channelBufferPixel[0];\n"
"	const float pixelG = channelBufferPixel[1];\n"
"	const float pixelB = channelBufferPixel[2];\n"
"\n"
"	// I use INFINITY to mark pixel with no samples and I need now to replace\n"
"	// all INFINITY with 0.0\n"
"	if (isinf(pixelR))\n"
"		channelBufferPixel[0] = 0.f;\n"
"	if (isinf(pixelG))\n"
"		channelBufferPixel[1] = 0.f;\n"
"	if (isinf(pixelB))\n"
"		channelBufferPixel[2] = 0.f;\n"
"}\n";
} }
