
#include <string>

namespace slg { namespace ocl {
std::string KernelSource_hitpoint_funcs =
"#line 2 \"hitpoint_funcs.cl\"\n"
"\n"
"/***************************************************************************\n"
" * Copyright 1998-2020 by authors (see AUTHORS.txt)                        *\n"
" *                                                                         *\n"
" *   This file is part of LuxCoreRender.                                   *\n"
" *                                                                         *\n"
" * Licensed under the Apache License, Version 2.0 (the \"License\");         *\n"
" * you may not use this file except in compliance with the License.        *\n"
" * You may obtain a copy of the License at                                 *\n"
" *                                                                         *\n"
" *     http://www.apache.org/licenses/LICENSE-2.0                          *\n"
" *                                                                         *\n"
" * Unless required by applicable law or agreed to in writing, software     *\n"
" * distributed under the License is distributed on an \"AS IS\" BASIS,       *\n"
" * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.*\n"
" * See the License for the specific language governing permissions and     *\n"
" * limitations under the License.                                          *\n"
" ***************************************************************************/\n"
"\n"
"// Used when hitting a surface\n"
"OPENCL_FORCE_INLINE void HitPoint_Init(__global HitPoint *hitPoint, const bool throughShadowTransp,\n"
"		const uint meshIndex, const uint triIndex,\n"
"		const float3 pnt, const float3 fixedDir,\n"
"		const float b1, const float b2,\n"
"		const float passThroughEvnt\n"
"		MATERIALS_PARAM_DECL) {\n"
"	hitPoint->throughShadowTransparency = throughShadowTransp;\n"
"	hitPoint->passThroughEvent = passThroughEvnt;\n"
"\n"
"	VSTORE3F(pnt, &hitPoint->p.x);\n"
"	VSTORE3F(fixedDir, &hitPoint->fixedDir.x);\n"
"\n"
"	hitPoint->objectID = sceneObjs[meshIndex].objectID;\n"
"\n"
"	// Interpolate face normal\n"
"	const float3 geometryN = ExtMesh_GetGeometryNormal(&hitPoint->localToWorld, meshIndex, triIndex EXTMESH_PARAM);\n"
"	VSTORE3F(geometryN,  &hitPoint->geometryN.x);\n"
"	const float3 interpolatedN = ExtMesh_GetInterpolateNormal(&hitPoint->localToWorld, meshIndex, triIndex, b1, b2 EXTMESH_PARAM);\n"
"	VSTORE3F(interpolatedN,  &hitPoint->interpolatedN.x);\n"
"	const float3 shadeN = interpolatedN;\n"
"	VSTORE3F(shadeN,  &hitPoint->shadeN.x);\n"
"\n"
"	hitPoint->intoObject = (dot(-fixedDir, geometryN) < 0.f);\n"
"\n"
"	// Interpolate UV coordinates\n"
"	const float2 defaultUV = ExtMesh_GetInterpolateUV(meshIndex, triIndex, b1, b2, 0 EXTMESH_PARAM);\n"
"	VSTORE2F(defaultUV, &hitPoint->defaultUV.u);\n"
"\n"
"	hitPoint->meshIndex = meshIndex;\n"
"	hitPoint->triangleIndex = triIndex;\n"
"	hitPoint->triangleBariCoord1 = b1;\n"
"	hitPoint->triangleBariCoord2 = b2;\n"
"\n"
"	// Compute geometry differentials\n"
"	float3 dndu, dndv, dpdu, dpdv;\n"
"	ExtMesh_GetDifferentials(\n"
"			&hitPoint->localToWorld,\n"
"			meshIndex,\n"
"			triIndex,\n"
"			shadeN, 0,\n"
"			&dpdu, &dpdv,\n"
"			&dndu, &dndv\n"
"			EXTMESH_PARAM);\n"
"	VSTORE3F(dpdu, &hitPoint->dpdu.x);\n"
"	VSTORE3F(dpdv, &hitPoint->dpdv.x);\n"
"	VSTORE3F(dndu, &hitPoint->dndu.x);\n"
"	VSTORE3F(dndv, &hitPoint->dndv.x);\n"
"}\n"
"\n"
"// Initialize all fields\n"
"OPENCL_FORCE_INLINE void HitPoint_InitDefault(__global HitPoint *hitPoint) {\n"
"	hitPoint->meshIndex = NULL_INDEX;\n"
"\n"
"	hitPoint->throughShadowTransparency = false;\n"
"	hitPoint->passThroughEvent = 0.f;\n"
"\n"
"	VSTORE3F(MAKE_FLOAT3(0.f, 0.f, 0.f), &hitPoint->p.x);\n"
"	VSTORE3F(MAKE_FLOAT3(0.f, 0.f, 0.f), &hitPoint->fixedDir.x);\n"
"\n"
"	hitPoint->objectID = 0;\n"
"	\n"
"	VSTORE3F(MAKE_FLOAT3(0.f, 0.f, 0.f),  &hitPoint->geometryN.x);\n"
"	VSTORE3F(MAKE_FLOAT3(0.f, 0.f, 0.f),  &hitPoint->interpolatedN.x);\n"
"	VSTORE3F(MAKE_FLOAT3(0.f, 0.f, 0.f),  &hitPoint->shadeN.x);\n"
"\n"
"	hitPoint->intoObject = true;\n"
"\n"
"	VSTORE2F(MAKE_FLOAT2(0.f, 0.f), &hitPoint->defaultUV.u);\n"
"	\n"
"	VSTORE3F(MAKE_FLOAT3(0.f, 0.f, 0.f), &hitPoint->dpdu.x);\n"
"	VSTORE3F(MAKE_FLOAT3(0.f, 0.f, 0.f), &hitPoint->dpdv.x);\n"
"	VSTORE3F(MAKE_FLOAT3(0.f, 0.f, 0.f), &hitPoint->dndu.x);\n"
"	VSTORE3F(MAKE_FLOAT3(0.f, 0.f, 0.f), &hitPoint->dndv.x);\n"
"\n"
"	Transform_Init(&hitPoint->localToWorld);\n"
"\n"
"	hitPoint->interiorVolumeIndex = NULL_INDEX;\n"
"	hitPoint->exteriorVolumeIndex = NULL_INDEX;\n"
"	hitPoint->interiorIorTexIndex = NULL_INDEX;\n"
"	hitPoint->exteriorIorTexIndex = NULL_INDEX;\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void HitPoint_GetFrame(__global const HitPoint *hitPoint, Frame *frame) {\n"
"	Frame_Set_Private(frame, VLOAD3F(&hitPoint->dpdu.x), VLOAD3F(&hitPoint->dpdv.x), VLOAD3F(&hitPoint->shadeN.x));\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 HitPoint_GetGeometryN(__global const HitPoint *hitPoint) {\n"
"	return (hitPoint->intoObject ? 1.f : -1.f) * VLOAD3F(&hitPoint->geometryN.x);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 HitPoint_GetInterpolatedN(__global const HitPoint *hitPoint) {\n"
"	return (hitPoint->intoObject ? 1.f : -1.f) * VLOAD3F(&hitPoint->interpolatedN.x);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 HitPoint_GetShadeN(__global const HitPoint *hitPoint) {\n"
"	return (hitPoint->intoObject ? 1.f : -1.f) * VLOAD3F(&hitPoint->shadeN.x);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float2 HitPoint_GetUV(__global const HitPoint *hitPoint, const uint dataIndex EXTMESH_PARAM_DECL) {\n"
"	const uint meshIndex = hitPoint->meshIndex;\n"
"\n"
"	if (meshIndex != NULL_INDEX) {\n"
"		return (dataIndex == 0) ?\n"
"			VLOAD2F(&hitPoint->defaultUV.u) :\n"
"			ExtMesh_GetInterpolateUV(meshIndex, hitPoint->triangleIndex, hitPoint->triangleBariCoord1, hitPoint->triangleBariCoord2, dataIndex EXTMESH_PARAM);\n"
"	} else\n"
"		return MAKE_FLOAT2(0.f, 0.f);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 HitPoint_GetColor(__global const HitPoint *hitPoint, const uint dataIndex EXTMESH_PARAM_DECL) {\n"
"	const uint meshIndex = hitPoint->meshIndex;\n"
"\n"
"	if (meshIndex != NULL_INDEX)\n"
"		return ExtMesh_GetInterpolateColor(meshIndex, hitPoint->triangleIndex, hitPoint->triangleBariCoord1, hitPoint->triangleBariCoord2, dataIndex EXTMESH_PARAM);\n"
"	else\n"
"		return WHITE;\n"
"}	\n"
"\n"
"OPENCL_FORCE_INLINE float HitPoint_GetAlpha(__global const HitPoint *hitPoint, const uint dataIndex EXTMESH_PARAM_DECL) {\n"
"	const uint meshIndex = hitPoint->meshIndex;\n"
"\n"
"	if (meshIndex != NULL_INDEX)\n"
"		return ExtMesh_GetInterpolateAlpha(meshIndex, hitPoint->triangleIndex, hitPoint->triangleBariCoord1, hitPoint->triangleBariCoord2, dataIndex EXTMESH_PARAM);\n"
"	else\n"
"		return 1.f;\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float HitPoint_GetVertexAOV(__global const HitPoint *hitPoint, const uint dataIndex EXTMESH_PARAM_DECL) {\n"
"	const uint meshIndex = hitPoint->meshIndex;\n"
"\n"
"	if (meshIndex != NULL_INDEX)\n"
"		return ExtMesh_GetInterpolateVertexAOV(meshIndex, hitPoint->triangleIndex, hitPoint->triangleBariCoord1, hitPoint->triangleBariCoord2, dataIndex EXTMESH_PARAM);\n"
"	else\n"
"		return 0.f;\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float HitPoint_GetTriAOV(__global const HitPoint *hitPoint, const uint dataIndex EXTMESH_PARAM_DECL) {\n"
"	const uint meshIndex = hitPoint->meshIndex;\n"
"\n"
"	if (meshIndex != NULL_INDEX)\n"
"		return ExtMesh_GetTriAOV(meshIndex, hitPoint->triangleIndex, dataIndex EXTMESH_PARAM);\n"
"	else\n"
"		return 0.f;\n"
"}\n";
} }
