
#include <string>

namespace slg { namespace ocl {
std::string KernelSource_light_funcs =
"#line 2 \"light_funcs.cl\"\n"
"\n"
"/***************************************************************************\n"
" * Copyright 1998-2020 by authors (see AUTHORS.txt)                        *\n"
" *                                                                         *\n"
" *   This file is part of LuxCoreRender.                                   *\n"
" *                                                                         *\n"
" * Licensed under the Apache License, Version 2.0 (the \"License\");         *\n"
" * you may not use this file except in compliance with the License.        *\n"
" * You may obtain a copy of the License at                                 *\n"
" *                                                                         *\n"
" *     http://www.apache.org/licenses/LICENSE-2.0                          *\n"
" *                                                                         *\n"
" * Unless required by applicable law or agreed to in writing, software     *\n"
" * distributed under the License is distributed on an \"AS IS\" BASIS,       *\n"
" * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.*\n"
" * See the License for the specific language governing permissions and     *\n"
" * limitations under the License.                                          *\n"
" ***************************************************************************/\n"
"\n"
"OPENCL_FORCE_INLINE float EnvLightSource_GetEnvRadius(const float sceneRadius) {\n"
"	// This is used to scale the world radius in sun/sky/infinite lights in order to\n"
"	// avoid problems with objects that are near the borderline of the world bounding sphere\n"
"	return LIGHT_WORLD_RADIUS_SCALE * sceneRadius;\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void EnvLightSource_ToLatLongMapping(const float3 w,\n"
"		float *s, float *t, float *pdf) {\n"
"	const float theta = SphericalTheta(w);\n"
"\n"
"	*s = SphericalPhi(w) * (1.f / (2.f * M_PI_F));\n"
"	*t = theta * M_1_PI_F;\n"
"\n"
"	if (pdf) {\n"
"		const float sinTheta = sin(theta);\n"
"		*pdf = (sinTheta > 0.f) ? ((1.f / (2.f * M_PI_F)) * M_1_PI_F / sinTheta) : 0.f;\n"
"	}\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void EnvLightSource_FromLatLongMapping(const float s, const float t,\n"
"		float3 *w, float *pdf) {\n"
"	const float phi = s * 2.f * M_PI_F;\n"
"	const float theta = t * M_PI_F;\n"
"	const float sinTheta = sin(theta);\n"
"\n"
"	*w = SphericalDirection(sinTheta, cos(theta), phi);\n"
"\n"
"	if (pdf)\n"
"		*pdf = (sinTheta > 0.f) ? ((1.f / (2.f * M_PI_F)) * M_1_PI_F / sinTheta) : 0.f;\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// ConstantInfiniteLight\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE float3 ConstantInfiniteLight_GetRadiance(__global const LightSource *constantInfiniteLight,\n"
"		__global const BSDF *bsdf, const float3 dir, float *directPdfA\n"
"		LIGHTS_PARAM_DECL) {\n"
"	const bool useVisibilityMapCache = constantInfiniteLight->notIntersectable.constantInfinite.useVisibilityMapCache;\n"
"\n"
"	if (useVisibilityMapCache && (!bsdf || EnvLightVisibilityCache_IsCacheEnabled(bsdf MATERIALS_PARAM))) {\n"
"		const float3 localDir = normalize(Transform_InvApplyVector(&constantInfiniteLight->notIntersectable.light2World, -dir));\n"
"\n"
"		float u, v, latLongMappingPdf;\n"
"		EnvLightSource_ToLatLongMapping(localDir, &u, &v, &latLongMappingPdf);\n"
"		if (latLongMappingPdf == 0.f)\n"
"			return BLACK;\n"
"\n"
"		if (bsdf)\n"
"			*directPdfA = EnvLightVisibilityCache_Pdf(bsdf, u, v LIGHTS_PARAM) * latLongMappingPdf;\n"
"		else\n"
"			*directPdfA = 0.f;\n"
"	} else\n"
"		*directPdfA = bsdf ? UniformSpherePdf() : 0.f;\n"
"\n"
"	return VLOAD3F(constantInfiniteLight->notIntersectable.temperatureScale.c) *\n"
"			VLOAD3F(constantInfiniteLight->notIntersectable.gain.c) *\n"
"			VLOAD3F(constantInfiniteLight->notIntersectable.constantInfinite.color.c);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 ConstantInfiniteLight_Illuminate(__global const LightSource *constantInfiniteLight,\n"
"		const float worldCenterX, const float worldCenterY, const float worldCenterZ,\n"
"		const float sceneRadius,\n"
"		__global const BSDF *bsdf, const float time, const float u0, const float u1,\n"
"		__global Ray *shadowRay, float *directPdfW\n"
"		LIGHTS_PARAM_DECL) {\n"
"	const bool useVisibilityMapCache = constantInfiniteLight->notIntersectable.constantInfinite.useVisibilityMapCache;\n"
"\n"
"	float3 shadowRayDir;\n"
"	if (useVisibilityMapCache && EnvLightVisibilityCache_IsCacheEnabled(bsdf MATERIALS_PARAM)) {\n"
"		float2 sampleUV;\n"
"		float distPdf;\n"
"\n"
"		EnvLightVisibilityCache_Sample(bsdf, u0, u1, &sampleUV, &distPdf LIGHTS_PARAM);\n"
"		if (distPdf == 0.f)\n"
"			return BLACK;\n"
"\n"
"		float latLongMappingPdf;\n"
"		EnvLightSource_FromLatLongMapping(sampleUV.x, sampleUV.y, &shadowRayDir, &latLongMappingPdf);\n"
"		if (latLongMappingPdf == 0.f)\n"
"			return BLACK;\n"
"\n"
"		*directPdfW = distPdf * latLongMappingPdf;\n"
"	} else {\n"
"		shadowRayDir = UniformSampleSphere(u0, u1);\n"
"\n"
"		*directPdfW = UniformSpherePdf();\n"
"	}\n"
"\n"
"	const float3 worldCenter = MAKE_FLOAT3(worldCenterX, worldCenterY, worldCenterZ);\n"
"	const float envRadius = EnvLightSource_GetEnvRadius(sceneRadius);\n"
"\n"
"	const float3 pSurface = BSDF_GetRayOrigin(bsdf, shadowRayDir);\n"
"	const float3 toCenter = worldCenter - pSurface;\n"
"	const float centerDistance = dot(toCenter, toCenter);\n"
"	const float approach = dot(toCenter, shadowRayDir);\n"
"	const float shadowRayDistance = approach + sqrt(max(0.f, envRadius * envRadius -\n"
"		centerDistance + approach * approach));\n"
"\n"
"	const float3 emisPoint = pSurface + shadowRayDistance * shadowRayDir;\n"
"	const float3 emisNormal = normalize(worldCenter - emisPoint);\n"
"\n"
"	const float cosAtLight = dot(emisNormal, -shadowRayDir);\n"
"	if (cosAtLight < DEFAULT_COS_EPSILON_STATIC)\n"
"		return BLACK;\n"
"\n"
"	// Setup the shadow ray\n"
"	const float3 shadowRayOrig = BSDF_GetRayOrigin(bsdf, shadowRayDir);\n"
"	Ray_Init4(shadowRay, shadowRayOrig, shadowRayDir, 0.f, shadowRayDistance, time);\n"
"\n"
"	return VLOAD3F(constantInfiniteLight->notIntersectable.temperatureScale.c) *\n"
"			VLOAD3F(constantInfiniteLight->notIntersectable.gain.c) *\n"
"			VLOAD3F(constantInfiniteLight->notIntersectable.constantInfinite.color.c);\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// InfiniteLight\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE float3 InfiniteLight_GetRadiance(__global const LightSource *infiniteLight,\n"
"		__global const BSDF *bsdf, const float3 dir, float *directPdfA\n"
"		LIGHTS_PARAM_DECL) {\n"
"	const float3 localDir = normalize(Transform_InvApplyVector(&infiniteLight->notIntersectable.light2World, -dir));\n"
"\n"
" 	float u, v, latLongMappingPdf;\n"
"	EnvLightSource_ToLatLongMapping(localDir, &u, &v, &latLongMappingPdf);\n"
"	if (latLongMappingPdf == 0.f)\n"
"		return BLACK;\n"
"\n"
"	if (!bsdf)\n"
"		*directPdfA = 0.f;\n"
"	else if (infiniteLight->notIntersectable.infinite.useVisibilityMapCache &&\n"
"			EnvLightVisibilityCache_IsCacheEnabled(bsdf MATERIALS_PARAM)) {\n"
"		*directPdfA = EnvLightVisibilityCache_Pdf(bsdf, u, v LIGHTS_PARAM) * latLongMappingPdf;\n"
"	} else {\n"
"		__global const float *infiniteLightDist = &envLightDistribution[infiniteLight->notIntersectable.infinite.distributionOffset];\n"
"\n"
"		const float distPdf = Distribution2D_Pdf(infiniteLightDist, u, v);\n"
"		*directPdfA = distPdf * latLongMappingPdf;\n"
"	}\n"
"\n"
"	__global const ImageMap *imageMap = &imageMapDescs[infiniteLight->notIntersectable.infinite.imageMapIndex];\n"
"	return VLOAD3F(infiniteLight->notIntersectable.temperatureScale.c) *\n"
"			VLOAD3F(infiniteLight->notIntersectable.gain.c) *\n"
"			ImageMap_GetSpectrum(\n"
"				imageMap,\n"
"				u, v\n"
"				IMAGEMAPS_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 InfiniteLight_Illuminate(__global const LightSource *infiniteLight,\n"
"		const float worldCenterX, const float worldCenterY, const float worldCenterZ,\n"
"		const float sceneRadius,\n"
"		__global const BSDF *bsdf, const float time, const float u0, const float u1,\n"
"		__global Ray *shadowRay, float *directPdfW\n"
"		LIGHTS_PARAM_DECL) {\n"
"	float2 sampleUV;\n"
"	float distPdf;\n"
"	if (infiniteLight->notIntersectable.infinite.useVisibilityMapCache &&\n"
"			EnvLightVisibilityCache_IsCacheEnabled(bsdf MATERIALS_PARAM)) {\n"
"		EnvLightVisibilityCache_Sample(bsdf, u0, u1, &sampleUV, &distPdf LIGHTS_PARAM);\n"
"	} else {\n"
"		__global const float *infiniteLightDistribution = &envLightDistribution[infiniteLight->notIntersectable.infinite.distributionOffset];\n"
"\n"
"		Distribution2D_SampleContinuous(infiniteLightDistribution, u0, u1, &sampleUV, &distPdf);\n"
"	}\n"
"\n"
"	if (distPdf == 0.f)\n"
"		return BLACK;\n"
"\n"
"	float3 localDir;\n"
"	float latLongMappingPdf;\n"
"	EnvLightSource_FromLatLongMapping(sampleUV.x, sampleUV.y, &localDir, &latLongMappingPdf);\n"
"	if (latLongMappingPdf == 0.f)\n"
"		return BLACK;\n"
"\n"
"	const float3 shadowRayDir = normalize(Transform_ApplyVector(&infiniteLight->notIntersectable.light2World, localDir));\n"
"\n"
"	const float3 worldCenter = MAKE_FLOAT3(worldCenterX, worldCenterY, worldCenterZ);\n"
"	const float envRadius = EnvLightSource_GetEnvRadius(sceneRadius);\n"
"\n"
"	const float3 pSurface = BSDF_GetRayOrigin(bsdf, worldCenter - VLOAD3F(&bsdf->hitPoint.p.x));\n"
"	const float3 toCenter = worldCenter - pSurface;\n"
"	const float centerDistance = dot(toCenter, toCenter);\n"
"	const float approach = dot(toCenter, shadowRayDir);\n"
"	const float shadowRayDistance = approach + sqrt(max(0.f, envRadius * envRadius -\n"
"		centerDistance + approach * approach));\n"
"\n"
"	const float3 emisPoint = pSurface + shadowRayDistance * shadowRayDir;\n"
"	const float3 emisNormal = normalize(worldCenter - emisPoint);\n"
"\n"
"	const float cosAtLight = dot(emisNormal, -shadowRayDir);\n"
"	if (cosAtLight < DEFAULT_COS_EPSILON_STATIC)\n"
"		return BLACK;\n"
"\n"
"	*directPdfW = distPdf * latLongMappingPdf;\n"
"\n"
"	// Setup the shadow ray\n"
"	const float3 shadowRayOrig = BSDF_GetRayOrigin(bsdf, shadowRayDir);\n"
"	Ray_Init4(shadowRay, shadowRayOrig, shadowRayDir, 0.f, shadowRayDistance, time);\n"
"\n"
"	// InfiniteLight_GetRadiance is expended here\n"
"	__global const ImageMap *imageMap = &imageMapDescs[infiniteLight->notIntersectable.infinite.imageMapIndex];\n"
"\n"
"	const float2 uv = MAKE_FLOAT2(sampleUV.x, sampleUV.y);\n"
"\n"
"	return VLOAD3F(infiniteLight->notIntersectable.temperatureScale.c) *\n"
"			VLOAD3F(infiniteLight->notIntersectable.gain.c) *\n"
"			ImageMap_GetSpectrum(\n"
"				imageMap,\n"
"				uv.x, uv.y\n"
"				IMAGEMAPS_PARAM);\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Sky2Light\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE float RiCosBetween(const float3 w1, const float3 w2) {\n"
"	return clamp(dot(w1, w2), -1.f, 1.f);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 Sky2Light_ComputeSkyRadiance(__global const LightSource *sky2Light, const float3 w) {\n"
"	const float3 absoluteSunDir = VLOAD3F(&sky2Light->notIntersectable.sky2.absoluteSunDir.x);\n"
"	const float cosG = RiCosBetween(w, absoluteSunDir);\n"
"	const float cosG2 = cosG * cosG;\n"
"	const float gamma = acos(cosG);\n"
"	const float cosT = fmax(0.f, CosTheta(w));\n"
"\n"
"	const float3 aTerm = VLOAD3F(sky2Light->notIntersectable.sky2.aTerm.c);\n"
"	const float3 bTerm = VLOAD3F(sky2Light->notIntersectable.sky2.bTerm.c);\n"
"	const float3 cTerm = VLOAD3F(sky2Light->notIntersectable.sky2.cTerm.c);\n"
"	const float3 dTerm = VLOAD3F(sky2Light->notIntersectable.sky2.dTerm.c);\n"
"	const float3 eTerm = VLOAD3F(sky2Light->notIntersectable.sky2.eTerm.c);\n"
"	const float3 fTerm = VLOAD3F(sky2Light->notIntersectable.sky2.fTerm.c);\n"
"	const float3 gTerm = VLOAD3F(sky2Light->notIntersectable.sky2.gTerm.c);\n"
"	const float3 hTerm = VLOAD3F(sky2Light->notIntersectable.sky2.hTerm.c);\n"
"	const float3 iTerm = VLOAD3F(sky2Light->notIntersectable.sky2.iTerm.c);\n"
"	const float3 radianceTerm = VLOAD3F(sky2Light->notIntersectable.sky2.radianceTerm.c);\n"
"	\n"
"	const float3 expTerm = dTerm * Spectrum_Exp(eTerm * gamma);\n"
"	const float3 rayleighTerm = fTerm * cosG2;\n"
"	const float3 mieTerm = gTerm * (1.f + cosG2) /\n"
"		Spectrum_Pow(1.f + iTerm * (iTerm - (2.f * cosG)), 1.5f);\n"
"	const float3 zenithTerm = hTerm * sqrt(cosT);\n"
"\n"
"	// 683 is a scaling factor to convert W to lm\n"
"	return 683.f * (1.f + aTerm * Spectrum_Exp(bTerm / (cosT + .01f))) *\n"
"		(cTerm + expTerm + rayleighTerm + mieTerm + zenithTerm) * radianceTerm;\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 Sky2Light_ComputeRadiance(__global const LightSource *sky2Light, const float3 w) {\n"
"	if (sky2Light->notIntersectable.sky2.hasGround &&\n"
"			(dot(w, VLOAD3F(&sky2Light->notIntersectable.sky2.absoluteUpDir.x)) < 0.f)) {\n"
"		// Lower hemisphere\n"
"		return VLOAD3F(sky2Light->notIntersectable.sky2.scaledGroundColor.c);\n"
"	} else\n"
"		return VLOAD3F(sky2Light->notIntersectable.temperatureScale.c) *\n"
"				VLOAD3F(sky2Light->notIntersectable.gain.c) *\n"
"				Sky2Light_ComputeSkyRadiance(sky2Light, w);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 Sky2Light_GetRadiance(__global const LightSource *sky2Light,\n"
"		__global const BSDF *bsdf, const float3 dir, float *directPdfA\n"
"		LIGHTS_PARAM_DECL) {\n"
"	const float3 w = -dir;\n"
"	float u, v, latLongMappingPdf;\n"
"	EnvLightSource_ToLatLongMapping(w, &u, &v, &latLongMappingPdf);\n"
"	if (latLongMappingPdf == 0.f)\n"
"		return BLACK;\n"
"\n"
"	if (!bsdf)\n"
"		*directPdfA = 0.f;\n"
"	else if (sky2Light->notIntersectable.sky2.useVisibilityMapCache &&\n"
"			EnvLightVisibilityCache_IsCacheEnabled(bsdf MATERIALS_PARAM)) {\n"
"		*directPdfA = EnvLightVisibilityCache_Pdf(bsdf, u, v LIGHTS_PARAM) * latLongMappingPdf;\n"
"	} else {\n"
"		__global const float *skyLightDist = &envLightDistribution[sky2Light->notIntersectable.sky2.distributionOffset];\n"
"\n"
"		const float distPdf = Distribution2D_Pdf(skyLightDist, u, v);\n"
"		*directPdfA = distPdf * latLongMappingPdf;\n"
"	}\n"
"\n"
"	return Sky2Light_ComputeRadiance(sky2Light, w);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 Sky2Light_Illuminate(__global const LightSource *sky2Light,\n"
"		const float worldCenterX, const float worldCenterY, const float worldCenterZ,\n"
"		const float sceneRadius,\n"
"		__global const BSDF *bsdf, const float time, const float u0, const float u1,\n"
"		__global Ray *shadowRay, float *directPdfW\n"
"		LIGHTS_PARAM_DECL) {\n"
"	float2 sampleUV;\n"
"	float distPdf;\n"
"	if (sky2Light->notIntersectable.sky2.useVisibilityMapCache &&\n"
"			EnvLightVisibilityCache_IsCacheEnabled(bsdf MATERIALS_PARAM)) {\n"
"		EnvLightVisibilityCache_Sample(bsdf, u0, u1, &sampleUV, &distPdf LIGHTS_PARAM);\n"
"	} else {\n"
"		__global const float *skyLightDistribution = &envLightDistribution[sky2Light->notIntersectable.sky2.distributionOffset];\n"
"\n"
"		Distribution2D_SampleContinuous(skyLightDistribution, u0, u1, &sampleUV, &distPdf);\n"
"	}\n"
"\n"
"	if (distPdf == 0.f)\n"
"			return BLACK;\n"
"\n"
"	float3 shadowRayDir;\n"
"	float latLongMappingPdf;\n"
"	EnvLightSource_FromLatLongMapping(sampleUV.x, sampleUV.y, &shadowRayDir, &latLongMappingPdf);\n"
"	if (latLongMappingPdf == 0.f)\n"
"		return BLACK;\n"
"\n"
"	const float3 worldCenter = MAKE_FLOAT3(worldCenterX, worldCenterY, worldCenterZ);\n"
"	const float envRadius = EnvLightSource_GetEnvRadius(sceneRadius);\n"
"\n"
"	const float3 pSurface = BSDF_GetRayOrigin(bsdf, shadowRayDir);\n"
"	const float3 toCenter = worldCenter - pSurface;\n"
"	const float centerDistance = dot(toCenter, toCenter);\n"
"	const float approach = dot(toCenter, shadowRayDir);\n"
"	const float shadowRayDistance = approach + sqrt(max(0.f, envRadius * envRadius -\n"
"		centerDistance + approach * approach));\n"
"\n"
"	const float3 emisPoint = pSurface + shadowRayDistance * shadowRayDir;\n"
"	const float3 emisNormal = normalize(worldCenter - emisPoint);\n"
"\n"
"	const float cosAtLight = dot(emisNormal, -shadowRayDir);\n"
"	if (cosAtLight < DEFAULT_COS_EPSILON_STATIC)\n"
"		return BLACK;\n"
"\n"
"	*directPdfW = distPdf * latLongMappingPdf;\n"
"\n"
"	// Setup the shadow ray\n"
"	const float3 shadowRayOrig = BSDF_GetRayOrigin(bsdf, shadowRayDir);\n"
"	Ray_Init4(shadowRay, shadowRayOrig, shadowRayDir, 0.f, shadowRayDistance, time);\n"
"\n"
"	return Sky2Light_ComputeRadiance(sky2Light, shadowRayDir);\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// SunLight\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE float3 SunLight_GetRadiance(__global const LightSource *sunLight,\n"
"		__global const BSDF *bsdf, const float3 dir, float *directPdfA) {\n"
"	const float cosThetaMax = sunLight->notIntersectable.sun.cosThetaMax;\n"
"	const float sin2ThetaMax = sunLight->notIntersectable.sun.sin2ThetaMax;\n"
"	const float3 x = VLOAD3F(&sunLight->notIntersectable.sun.x.x);\n"
"	const float3 y = VLOAD3F(&sunLight->notIntersectable.sun.y.x);\n"
"	const float3 absoluteSunDir = VLOAD3F(&sunLight->notIntersectable.sun.absoluteDir.x);\n"
"\n"
"	const float xD = dot(-dir, x);\n"
"	const float yD = dot(-dir, y);\n"
"	const float zD = dot(-dir, absoluteSunDir);\n"
"	if ((cosThetaMax == 1.f) || (zD < 0.f) || ((xD * xD + yD * yD) > sin2ThetaMax))\n"
"		return BLACK;\n"
"\n"
"	if (directPdfA)\n"
"		*directPdfA = UniformConePdf(cosThetaMax);\n"
"\n"
"	return VLOAD3F(sunLight->notIntersectable.sun.color.c);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 SunLight_Illuminate(__global const LightSource *sunLight,\n"
"		const float worldCenterX, const float worldCenterY, const float worldCenterZ,\n"
"		const float sceneRadius,\n"
"		__global const BSDF *bsdf, const float time, const float u0, const float u1,\n"
"		__global Ray *shadowRay, float *directPdfW) {\n"
"\n"
"	const float cosThetaMax = sunLight->notIntersectable.sun.cosThetaMax;\n"
"	const float3 sunDir = VLOAD3F(&sunLight->notIntersectable.sun.absoluteDir.x);\n"
"	const float3 shadowRayDir = UniformSampleCone(u0, u1, cosThetaMax, VLOAD3F(&sunLight->notIntersectable.sun.x.x), VLOAD3F(&sunLight->notIntersectable.sun.y.x), sunDir);\n"
"\n"
"	// Check if the point can be inside the sun cone of light\n"
"	const float cosAtLight = dot(sunDir, shadowRayDir);\n"
"	if (cosAtLight <= cosThetaMax)\n"
"		return BLACK;\n"
"\n"
"	const float3 worldCenter = MAKE_FLOAT3(worldCenterX, worldCenterY, worldCenterZ);\n"
"	const float envRadius = EnvLightSource_GetEnvRadius(sceneRadius);\n"
"	\n"
"	const float3 pSurface = BSDF_GetRayOrigin(bsdf, worldCenter - VLOAD3F(&bsdf->hitPoint.p.x));\n"
"	const float3 toCenter = worldCenter - pSurface;\n"
"	const float centerDistance = dot(toCenter, toCenter);\n"
"	const float approach = dot(toCenter, shadowRayDir);\n"
"	const float shadowRayDistance = approach + sqrt(max(0.f, envRadius * envRadius -\n"
"		centerDistance + approach * approach));\n"
"\n"
"	*directPdfW = UniformConePdf(cosThetaMax);\n"
"\n"
"	// Setup the shadow ray\n"
"	const float3 shadowRayOrig = BSDF_GetRayOrigin(bsdf, shadowRayDir);\n"
"	Ray_Init4(shadowRay, shadowRayOrig, shadowRayDir, 0.f, shadowRayDistance, time);\n"
"\n"
"	return VLOAD3F(sunLight->notIntersectable.sun.color.c);\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// TriangleLight\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE float3 TriangleLight_GetRadiance(__global const LightSource *triLight,\n"
"		 __global const HitPoint *hitPoint, float *directPdfA\n"
"		MATERIALS_PARAM_DECL) {\n"
"	const uint materialIndex = sceneObjs[triLight->triangle.meshIndex].materialIndex;\n"
"\n"
"	const float3 dir = VLOAD3F(&hitPoint->fixedDir.x);\n"
"	const float3 hitPointNormal = VLOAD3F(&hitPoint->shadeN.x);\n"
"	const float cosOutLight = dot(hitPointNormal, dir);\n"
"	const float cosThetaMax = Material_GetEmittedCosThetaMax(materialIndex\n"
"			MATERIALS_PARAM);\n"
"	// emissionFunc can emit light even backward, this is for compatibility with classic Lux\n"
"	if (((triLight->triangle.imageMapIndex == NULL_INDEX) && (cosOutLight < cosThetaMax + DEFAULT_COS_EPSILON_STATIC)) ||\n"
"			// A safety check to avoid NaN/Inf\n"
"			(triLight->triangle.invTriangleArea == 0.f) || (triLight->triangle.invMeshArea == 0.f))\n"
"		return BLACK;\n"
"\n"
"	if (directPdfA)\n"
"		*directPdfA = triLight->triangle.invTriangleArea;\n"
"\n"
"	float3 emissionColor = WHITE;\n"
"\n"
"	if (triLight->triangle.imageMapIndex != NULL_INDEX) {\n"
"		// Build the local frame\n"
"		Frame frame;\n"
"		HitPoint_GetFrame(hitPoint, &frame);\n"
"\n"
"		const float3 localFromLight = normalize(Frame_ToLocal_Private(&frame, dir));\n"
"\n"
"		// Retrieve the image map information\n"
"		__global const ImageMap *imageMap = &imageMapDescs[triLight->triangle.imageMapIndex];\n"
"		const float2 uv = MAKE_FLOAT2(SphericalPhi(localFromLight) * (1.f / (2.f * M_PI_F)), SphericalTheta(localFromLight) * M_1_PI_F);\n"
"		emissionColor = ImageMap_GetSpectrum(\n"
"				imageMap,\n"
"				uv.x, uv.y\n"
"				IMAGEMAPS_PARAM) / triLight->triangle.average;\n"
"	}\n"
"\n"
"	return Material_GetEmittedRadiance(materialIndex,\n"
"			hitPoint, triLight->triangle.invMeshArea\n"
"			MATERIALS_PARAM) * emissionColor;\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 TriangleLight_Illuminate(__global const LightSource *triLight,\n"
"		__global HitPoint *tmpHitPoint, __global const BSDF *bsdf,\n"
"		const float time, const float u0, const float u1,\n"
"		const float passThroughEvent,\n"
"		__global Ray *shadowRay, float *directPdfW\n"
"		MATERIALS_PARAM_DECL) {\n"
"	// A safety check to avoid NaN/Inf\n"
"	if ((triLight->triangle.invTriangleArea == 0.f) || (triLight->triangle.invMeshArea == 0.f))\n"
"		return BLACK;\n"
"\n"
"	//--------------------------------------------------------------------------\n"
"	// Compute the sample point and direction\n"
"	//--------------------------------------------------------------------------\n"
"\n"
"	const uint meshIndex = triLight->triangle.meshIndex;\n"
"	const uint triangleIndex = triLight->triangle.triangleIndex;\n"
"	const uint materialIndex = sceneObjs[meshIndex].materialIndex;\n"
"\n"
"	// Initialized local to world object space transformation\n"
"	ExtMesh_GetLocal2World(time, meshIndex, triangleIndex, &tmpHitPoint->localToWorld EXTMESH_PARAM);\n"
"\n"
"	float b0, b1, b2;\n"
"	float3 samplePoint;\n"
"	ExtMesh_Sample(&tmpHitPoint->localToWorld,\n"
"			meshIndex, triangleIndex,\n"
"			u0, u1,\n"
"			&samplePoint, &b0, &b1, &b2\n"
"			EXTMESH_PARAM);\n"
"\n"
"	float3 sampleDir = samplePoint - VLOAD3F(&bsdf->hitPoint.p.x);\n"
"	const float distanceSquared = dot(sampleDir, sampleDir);\n"
"	const float distance = sqrt(distanceSquared);\n"
"	sampleDir /= distance;\n"
"	\n"
"	// Build a temporary HitPoint\n"
"	HitPoint_Init(tmpHitPoint, false,\n"
"		meshIndex, triangleIndex,\n"
"		samplePoint, -sampleDir,\n"
"		b1, b2,\n"
"		passThroughEvent\n"
"		MATERIALS_PARAM);\n"
"	// Add bump?\n"
"		// lightMaterial->Bump(&hitPoint, 1.f);\n"
"\n"
"	const float3 geometryN = VLOAD3F(&tmpHitPoint->geometryN.x);\n"
"	const float cosAtLight = dot(geometryN, -sampleDir);\n"
"	const float cosThetaMax = Material_GetEmittedCosThetaMax(materialIndex\n"
"		MATERIALS_PARAM);\n"
"	// emissionFunc can emit light even backward, this is for compatibility with classic Lux\n"
"	if ((triLight->triangle.imageMapIndex == NULL_INDEX) && (cosAtLight < cosThetaMax + DEFAULT_COS_EPSILON_STATIC))\n"
"		return BLACK;\n"
"\n"
"	//--------------------------------------------------------------------------\n"
"	// Initialize the shadow ray\n"
"	//--------------------------------------------------------------------------\n"
"	\n"
"	// Move shadow ray origin along the geometry normal by an epsilon to avoid self-shadow problems\n"
"	const float3 shadowRayOrig = BSDF_GetRayOrigin(bsdf, sampleDir);\n"
"\n"
"	// Compute shadow ray direction with displaced start and end point to avoid self-shadow problems\n"
"	float3 shadowRayDir = samplePoint + geometryN * MachineEpsilon_E_Float3(samplePoint) *\n"
"			(tmpHitPoint->intoObject ? 1.f : -1.f) - shadowRayOrig;\n"
"	const float shadowRayDistanceSquared = dot(shadowRayDir, shadowRayDir);\n"
"	const float shadowRayDistance = sqrt(shadowRayDistanceSquared);\n"
"	shadowRayDir /= shadowRayDistance;\n"
"\n"
"	//--------------------------------------------------------------------------\n"
"	// Compute the PDF and emission color\n"
"	//--------------------------------------------------------------------------\n"
"	\n"
"	float3 emissionColor = WHITE;\n"
"\n"
"	if (triLight->triangle.imageMapIndex != NULL_INDEX) {\n"
"		// Build the local frame\n"
"		Frame frame;\n"
"		HitPoint_GetFrame(tmpHitPoint, &frame);\n"
"\n"
"		const float3 localFromLight = normalize(Frame_ToLocal_Private(&frame, -shadowRayDir));\n"
"\n"
"		// Retrieve the image map information\n"
"		__global const ImageMap *imageMap = &imageMapDescs[triLight->triangle.imageMapIndex];\n"
"		const float2 uv = MAKE_FLOAT2(SphericalPhi(localFromLight) * (1.f / (2.f * M_PI_F)), SphericalTheta(localFromLight) * M_1_PI_F);\n"
"		emissionColor = ImageMap_GetSpectrum(\n"
"				imageMap,\n"
"				uv.x, uv.y\n"
"				IMAGEMAPS_PARAM) / triLight->triangle.average;\n"
"\n"
"		*directPdfW = triLight->triangle.invTriangleArea * shadowRayDistanceSquared ;\n"
"	} else\n"
"		*directPdfW = triLight->triangle.invTriangleArea * shadowRayDistanceSquared / fabs(cosAtLight);\n"
"\n"
"	// Setup the shadow ray\n"
"	Ray_Init4(shadowRay, shadowRayOrig, shadowRayDir, 0.f, shadowRayDistance, time);\n"
"\n"
"	return Material_GetEmittedRadiance(materialIndex,\n"
"			tmpHitPoint, triLight->triangle.invMeshArea\n"
"			MATERIALS_PARAM) * emissionColor;\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// PointLight\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE float3 PointLight_Illuminate(__global const LightSource *pointLight,\n"
"		__global const BSDF *bsdf, const float time,\n"
"		__global Ray *shadowRay, float *directPdfW) {\n"
"	const float3 pLight = VLOAD3F(&pointLight->notIntersectable.point.absolutePos.x);	\n"
"	const float3 pSurface = BSDF_GetRayOrigin(bsdf, pLight - VLOAD3F(&bsdf->hitPoint.p.x));\n"
"\n"
"	const float3 toLight = pLight - pSurface;\n"
"	const float shadowRayDistanceSquared = dot(toLight, toLight);\n"
"	const float shadowRayDistance = sqrt(shadowRayDistanceSquared);\n"
"	const float3 shadowRayDir = toLight / shadowRayDistance;\n"
"\n"
"	*directPdfW = shadowRayDistanceSquared;\n"
"\n"
"	// Setup the shadow ray\n"
"	const float3 shadowRayOrig = BSDF_GetRayOrigin(bsdf, shadowRayDir);\n"
"	Ray_Init4(shadowRay, shadowRayOrig, shadowRayDir, 0.f, shadowRayDistance, time);\n"
"\n"
"	return VLOAD3F(pointLight->notIntersectable.point.emittedFactor.c) * (1.f / (4.f * M_PI_F));\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// SphereLight\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE bool SphereLight_SphereIntersect(const float3 absolutePos, const float radiusSquared,\n"
"		const float3 rayOrig, const float3 rayDir, float *hitT) {\n"
"	const float3 op = absolutePos - rayOrig;\n"
"	const float b = dot(op, rayDir);\n"
"\n"
"	float det = b * b - dot(op, op) + radiusSquared;\n"
"	if (det < 0.f)\n"
"		return false;\n"
"	else\n"
"		det = sqrt(det);\n"
"\n"
"	const float mint = MachineEpsilon_E_Float3(rayOrig);\n"
"	const float maxt = INFINITY;\n"
"\n"
"	float t = b - det;\n"
"	if ((t > mint) && ((t < maxt)))\n"
"		*hitT = t;\n"
"	else {\n"
"		t = b + det;\n"
"\n"
"		if ((t > mint) && ((t < maxt)))\n"
"			*hitT = t;\n"
"		else\n"
"			return false;\n"
"	}\n"
"\n"
"	return true;\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 SphereLight_Illuminate(__global const LightSource *sphereLight,\n"
"		__global const BSDF *bsdf, const float time, const float u0, const float u1,\n"
"		__global Ray *shadowRay, float *directPdfW) {\n"
"	const float3 absolutePos = VLOAD3F(&sphereLight->notIntersectable.sphere.absolutePos.x);\n"
"	const float3 toLight = absolutePos - VLOAD3F(&bsdf->hitPoint.p.x);\n"
"	const float centerDistanceSquared = dot(toLight, toLight);\n"
"	const float centerDistance = sqrt(centerDistanceSquared);\n"
"\n"
"	const float radius = sphereLight->notIntersectable.sphere.radius;\n"
"	const float radiusSquared = radius * radius;\n"
"\n"
"	// Check if the point is inside the sphere\n"
"	if (centerDistanceSquared - radiusSquared < DEFAULT_EPSILON_STATIC) {\n"
"		// The point is inside the sphere, return black\n"
"		return BLACK;\n"
"	}\n"
"\n"
"	// The point isn't inside the sphere\n"
"\n"
"	const float cosThetaMax = sqrt(max(0.f, 1.f - radiusSquared / centerDistanceSquared));\n"
"\n"
"	// Build a local coordinate system\n"
"	const float3 localZ = toLight * (1.f / centerDistance);\n"
"	Frame localFrame;\n"
"	Frame_SetFromZ_Private(&localFrame, localZ);\n"
"\n"
"	// Sample sphere uniformly inside subtended cone\n"
"	const float3 localShadowRayDir = UniformSampleConeLocal(u0, u1, cosThetaMax);\n"
"	if (CosTheta(localShadowRayDir) < DEFAULT_COS_EPSILON_STATIC)\n"
"		return BLACK;\n"
"\n"
"	const float3 shadowRayDir = Frame_ToWorld_Private(&localFrame, localShadowRayDir);\n"
"\n"
"	// Check the intersection with the sphere\n"
"	const float3 shadowRayOrig = BSDF_GetRayOrigin(bsdf, shadowRayDir);\n"
"	float shadowRayDistance;\n"
"	if (!SphereLight_SphereIntersect(absolutePos, radiusSquared, shadowRayOrig, shadowRayDir, &shadowRayDistance))\n"
"		shadowRayDistance = dot(toLight, shadowRayDir);\n"
"\n"
"	if (cosThetaMax > 1.f - DEFAULT_EPSILON_STATIC) {\n"
"		// If the subtended angle is too small, I replace the computation for\n"
"		// directPdfW  and return factor with that of a point light source in\n"
"		// order to avoiding banding due to (lack of) numerical precision.\n"
"		*directPdfW = shadowRayDistance * shadowRayDistance;\n"
"\n"
"		// Setup the shadow ray\n"
"		Ray_Init4(shadowRay, shadowRayOrig, shadowRayDir, 0.f, shadowRayDistance, time);\n"
"\n"
"		return VLOAD3F(sphereLight->notIntersectable.sphere.emittedFactor.c) * (1.f / (4.f * M_PI_F));\n"
"	} else {\n"
"		*directPdfW = UniformConePdf(cosThetaMax);\n"
"\n"
"		// Setup the shadow ray\n"
"		Ray_Init4(shadowRay, shadowRayOrig, shadowRayDir, 0.f, shadowRayDistance, time);\n"
"\n"
"		const float invArea = 1.f / (4.f * M_PI_F * radiusSquared);\n"
"\n"
"		return VLOAD3F(sphereLight->notIntersectable.sphere.emittedFactor.c) * invArea * M_1_PI_F;\n"
"	}\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// MapPointLight\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE float3 MapPointLight_Illuminate(__global const LightSource *mapPointLight,\n"
"		__global const BSDF *bsdf, const float time,\n"
"		__global Ray *shadowRay, float *directPdfW\n"
"		IMAGEMAPS_PARAM_DECL) {\n"
"	const float3 pLight = VLOAD3F(&mapPointLight->notIntersectable.mapPoint.absolutePos.x);\n"
"	const float3 toLight = pLight - VLOAD3F(&bsdf->hitPoint.p.x);\n"
"	const float shadowRayDistanceSquared = dot(toLight, toLight);\n"
"	const float shadowRayDistance = sqrt(shadowRayDistanceSquared);\n"
"	const float3 shadowRayDir = toLight / shadowRayDistance;\n"
"\n"
"	*directPdfW = shadowRayDistanceSquared;\n"
"\n"
"	// Setup the shadow ray\n"
"	const float3 shadowRayOrig = BSDF_GetRayOrigin(bsdf, shadowRayDir);\n"
"	Ray_Init4(shadowRay, shadowRayOrig, shadowRayDir, 0.f, shadowRayDistance, time);\n"
"\n"
"	// Retrieve the image map information\n"
"	__global const ImageMap *imageMap = &imageMapDescs[mapPointLight->notIntersectable.mapPoint.imageMapIndex];\n"
"\n"
"	const float3 localFromLight = normalize(Transform_InvApplyVector(\n"
"			&mapPointLight->notIntersectable.light2World, -shadowRayDir));\n"
"	const float2 uv = MAKE_FLOAT2(SphericalPhi(localFromLight) * (1.f / (2.f * M_PI_F)), SphericalTheta(localFromLight) * M_1_PI_F);\n"
"	const float3 emissionColor = ImageMap_GetSpectrum(\n"
"			imageMap,\n"
"			uv.x, uv.y\n"
"			IMAGEMAPS_PARAM) / (4.f * M_PI_F * mapPointLight->notIntersectable.mapPoint.average);\n"
"\n"
"	return VLOAD3F(mapPointLight->notIntersectable.mapPoint.emittedFactor.c) * emissionColor;\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// MapSphereLight\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE float3 MapSphereLight_Illuminate(__global const LightSource *mapSphereLight,\n"
"		__global const BSDF *bsdf,	const float time, const float u0, const float u1,\n"
"		__global Ray *shadowRay, float *directPdfW\n"
"		IMAGEMAPS_PARAM_DECL) {\n"
"	const float3 result = SphereLight_Illuminate(mapSphereLight, bsdf, time, u0, u1,\n"
"			shadowRay, directPdfW);\n"
"\n"
"	// Retrieve the image map information\n"
"	__global const ImageMap *imageMap = &imageMapDescs[mapSphereLight->notIntersectable.mapSphere.imageMapIndex];\n"
"\n"
"	const float3 localFromLight = normalize(Transform_InvApplyVector(\n"
"			&mapSphereLight->notIntersectable.light2World, -VLOAD3F(&shadowRay->d.x)));\n"
"	const float2 uv = MAKE_FLOAT2(SphericalPhi(localFromLight) * (1.f / (2.f * M_PI_F)), SphericalTheta(localFromLight) * M_1_PI_F);\n"
"	const float3 emissionColor = ImageMap_GetSpectrum(\n"
"			imageMap,\n"
"			uv.x, uv.y\n"
"			IMAGEMAPS_PARAM) * (1.f / mapSphereLight->notIntersectable.mapSphere.average);\n"
"\n"
"	return result * emissionColor;\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// SpotLight\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE float SpotLight_LocalFalloff(const float3 w, const float cosTotalWidth, const float cosFalloffStart) {\n"
"	if (CosTheta(w) < cosTotalWidth)\n"
"		return 0.f;\n"
" 	if (CosTheta(w) > cosFalloffStart)\n"
"		return 1.f;\n"
"\n"
"	// Compute falloff inside spotlight cone\n"
"	const float delta = (CosTheta(w) - cosTotalWidth) / (cosFalloffStart - cosTotalWidth);\n"
"	return pow(delta, 4.f);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 SpotLight_Illuminate(__global const LightSource *spotLight,\n"
"		__global const BSDF *bsdf, const float time,\n"
"		__global Ray *shadowRay, float *directPdfW) {\n"
"	const float3 pLight = VLOAD3F(&spotLight->notIntersectable.spot.absolutePos.x);\n"
"	const float3 pSurface = BSDF_GetRayOrigin(bsdf, pLight - VLOAD3F(&bsdf->hitPoint.p.x));\n"
"\n"
"	const float3 toLight = pLight - pSurface;\n"
"	const float shadowRayDistanceSquared = dot(toLight, toLight);\n"
"	const float shadowRayDistance = sqrt(shadowRayDistanceSquared);\n"
"	const float3 shadowRayDir = toLight / shadowRayDistance;\n"
"\n"
"	const float3 localFromLight = normalize(Transform_InvApplyVector(\n"
"			&spotLight->notIntersectable.light2World, -shadowRayDir));\n"
"	const float falloff = SpotLight_LocalFalloff(localFromLight,\n"
"			spotLight->notIntersectable.spot.cosTotalWidth,\n"
"			spotLight->notIntersectable.spot.cosFalloffStart);\n"
"	if (falloff == 0.f)\n"
"		return BLACK;\n"
"\n"
"	*directPdfW = shadowRayDistanceSquared;\n"
"\n"
"	// Setup the shadow ray\n"
"	const float3 shadowRayOrig = BSDF_GetRayOrigin(bsdf, shadowRayDir);\n"
"	Ray_Init4(shadowRay, shadowRayOrig, shadowRayDir, 0.f, shadowRayDistance, time);\n"
"\n"
"	return VLOAD3F(spotLight->notIntersectable.spot.emittedFactor.c) *\n"
"			(falloff / fabs(CosTheta(localFromLight)));\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// ProjectionLight\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE float3 ProjectionLight_Illuminate(__global const LightSource *projectionLight,\n"
"		__global const BSDF *bsdf, const float time,\n"
"		__global Ray *shadowRay, float *directPdfW\n"
"		IMAGEMAPS_PARAM_DECL) {\n"
"	const float3 pLight = VLOAD3F(&projectionLight->notIntersectable.projection.absolutePos.x);\n"
"	const float3 pSurface = BSDF_GetRayOrigin(bsdf, pLight - VLOAD3F(&bsdf->hitPoint.p.x));\n"
"\n"
"	const float3 toLight = pLight - pSurface;\n"
"	const float shadowRayDistanceSquared = dot(toLight, toLight);\n"
"	const float shadowRayDistance = sqrt(shadowRayDistanceSquared);\n"
"	const float3 shadowRayDir = toLight / shadowRayDistance;\n"
"\n"
"	// Check the side\n"
"	if (dot(-shadowRayDir, VLOAD3F(&projectionLight->notIntersectable.projection.lightNormal.x)) < 0.f)\n"
"		return BLACK;\n"
"\n"
"	// Check if the point is inside the image plane\n"
"	const float3 localFromLight = normalize(Transform_InvApplyVector(\n"
"			&projectionLight->notIntersectable.light2World, -shadowRayDir));\n"
"	const float3 p0 = Matrix4x4_ApplyPoint(\n"
"			&projectionLight->notIntersectable.projection.lightProjection, localFromLight);\n"
"\n"
"	const float screenX0 = projectionLight->notIntersectable.projection.screenX0;\n"
"	const float screenX1 = projectionLight->notIntersectable.projection.screenX1;\n"
"	const float screenY0 = projectionLight->notIntersectable.projection.screenY0;\n"
"	const float screenY1 = projectionLight->notIntersectable.projection.screenY1;\n"
"	if ((p0.x < screenX0) || (p0.x >= screenX1) || (p0.y < screenY0) || (p0.y >= screenY1))\n"
"		return BLACK;\n"
"\n"
"	*directPdfW = shadowRayDistanceSquared;\n"
"\n"
"	// Setup the shadow ray\n"
"	const float3 shadowRayOrig = BSDF_GetRayOrigin(bsdf, shadowRayDir);\n"
"	Ray_Init4(shadowRay, shadowRayOrig, shadowRayDir, 0.f, shadowRayDistance, time);\n"
"\n"
"	float3 c = VLOAD3F(projectionLight->notIntersectable.projection.emittedFactor.c);\n"
"\n"
"	const uint imageMapIndex = projectionLight->notIntersectable.projection.imageMapIndex;\n"
"	if (imageMapIndex != NULL_INDEX) {\n"
"		const float u = (p0.x - screenX0) / (screenX1 - screenX0);\n"
"		const float v = (p0.y - screenY0) / (screenY1 - screenY0);\n"
"		\n"
"		// Retrieve the image map information\n"
"		__global const ImageMap *imageMap = &imageMapDescs[imageMapIndex];\n"
"\n"
"		c *= ImageMap_GetSpectrum(\n"
"				imageMap,\n"
"				u, v\n"
"				IMAGEMAPS_PARAM);\n"
"	}\n"
"\n"
"	return c;\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// SharpDistantLight\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE float3 SharpDistantLight_Illuminate(__global const LightSource *sharpDistantLight,\n"
"		const float worldCenterX, const float worldCenterY, const float worldCenterZ,\n"
"		const float sceneRadius,\n"
"		__global const BSDF *bsdf, const float time,\n"
"		__global Ray *shadowRay, float *directPdfW) {\n"
"	const float3 shadowRayDir = -VLOAD3F(&sharpDistantLight->notIntersectable.sharpDistant.absoluteLightDir.x);\n"
"\n"
"	const float3 worldCenter = MAKE_FLOAT3(worldCenterX, worldCenterY, worldCenterZ);\n"
"	const float envRadius = EnvLightSource_GetEnvRadius(sceneRadius);\n"
"	\n"
"	const float3 pSurface = BSDF_GetRayOrigin(bsdf, worldCenter - VLOAD3F(&bsdf->hitPoint.p.x));\n"
"	const float3 toCenter = worldCenter - pSurface;\n"
"	const float centerDistance = dot(toCenter, toCenter);\n"
"	const float approach = dot(toCenter, shadowRayDir);\n"
"	const float shadowRayDistance = approach + sqrt(max(0.f, envRadius * envRadius -\n"
"		centerDistance + approach * approach));\n"
"\n"
"	*directPdfW = 1.f;\n"
"\n"
"	// Setup the shadow ray\n"
"	const float3 shadowRayOrig = BSDF_GetRayOrigin(bsdf, shadowRayDir);\n"
"	Ray_Init4(shadowRay, shadowRayOrig, shadowRayDir, 0.f, shadowRayDistance, time);\n"
"\n"
"	return VLOAD3F(sharpDistantLight->notIntersectable.temperatureScale.c) *\n"
"			VLOAD3F(sharpDistantLight->notIntersectable.gain.c) *\n"
"			VLOAD3F(sharpDistantLight->notIntersectable.sharpDistant.color.c);\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// DistantLight\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE float3 DistantLight_Illuminate(__global const LightSource *distantLight,\n"
"		const float worldCenterX, const float worldCenterY, const float worldCenterZ,\n"
"		const float sceneRadius,\n"
"		__global const BSDF *bsdf, const float time, const float u0, const float u1,\n"
"		__global Ray *shadowRay, float *directPdfW) {\n"
"	const float3 absoluteLightDir = VLOAD3F(&distantLight->notIntersectable.distant.absoluteLightDir.x);\n"
"	const float3 x = VLOAD3F(&distantLight->notIntersectable.distant.x.x);\n"
"	const float3 y = VLOAD3F(&distantLight->notIntersectable.distant.y.x);\n"
"	const float cosThetaMax = distantLight->notIntersectable.distant.cosThetaMax;\n"
"	const float3 shadowRayDir = -UniformSampleCone(u0, u1, cosThetaMax, x, y, absoluteLightDir);\n"
"\n"
"	const float3 worldCenter = MAKE_FLOAT3(worldCenterX, worldCenterY, worldCenterZ);\n"
"	const float envRadius = EnvLightSource_GetEnvRadius(sceneRadius);\n"
"	\n"
"	const float3 pSurface = BSDF_GetRayOrigin(bsdf, worldCenter - VLOAD3F(&bsdf->hitPoint.p.x));\n"
"	const float3 toCenter = worldCenter - pSurface;\n"
"	const float centerDistance = dot(toCenter, toCenter);\n"
"	const float approach = dot(toCenter, shadowRayDir);\n"
"	const float shadowRayDistance = approach + sqrt(max(0.f, envRadius * envRadius -\n"
"		centerDistance + approach * approach));\n"
"\n"
"	const float uniformConePdf = UniformConePdf(cosThetaMax);\n"
"	*directPdfW = uniformConePdf;\n"
"\n"
"	// Setup the shadow ray\n"
"	const float3 shadowRayOrig = BSDF_GetRayOrigin(bsdf, shadowRayDir);\n"
"	Ray_Init4(shadowRay, shadowRayOrig, shadowRayDir, 0.f, shadowRayDistance, time);\n"
"\n"
"	return VLOAD3F(distantLight->notIntersectable.temperatureScale.c) *\n"
"			VLOAD3F(distantLight->notIntersectable.gain.c) *\n"
"			VLOAD3F(distantLight->notIntersectable.sharpDistant.color.c);\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// LaserLight\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE float3 LaserLight_Illuminate(__global const LightSource *laserLight,\n"
"		__global const BSDF *bsdf, const float time,\n"
"		__global Ray *shadowRay, float *directPdfW) {\n"
"	const float3 absoluteLightPos = VLOAD3F(&laserLight->notIntersectable.laser.absoluteLightPos.x);\n"
"	const float3 absoluteLightDir = VLOAD3F(&laserLight->notIntersectable.laser.absoluteLightDir.x);\n"
"\n"
"	const float3 shadowRayDir = -absoluteLightDir;\n"
"	const float3 rayOrig = BSDF_GetRayOrigin(bsdf, shadowRayDir);\n"
"	const float3 planeCenter = absoluteLightPos;\n"
"	const float3 planeNormal = absoluteLightDir;\n"
"\n"
"	// Intersect the shadow ray with light plane\n"
"	const float denom = dot(planeNormal, shadowRayDir);\n"
"	const float3 pr = planeCenter - rayOrig;\n"
"	float shadowRayDistance = dot(pr, planeNormal);\n"
"\n"
"	if (fabs(denom) > DEFAULT_COS_EPSILON_STATIC) {\n"
"		// There is a valid intersection\n"
"		shadowRayDistance /= denom; \n"
"\n"
"		if ((shadowRayDistance <= 0.f) || (denom >= 0.f))\n"
"			return BLACK;\n"
"	} else\n"
"		return BLACK;\n"
"\n"
"	const float3 lightPoint = rayOrig + shadowRayDistance * shadowRayDir;\n"
"\n"
"	// Check if the point is inside the emitting circle\n"
"	const float radius = laserLight->notIntersectable.laser.radius;\n"
"	const float3 dist = lightPoint - absoluteLightPos;\n"
"	if (dot(dist, dist) > radius * radius)\n"
"		return BLACK;\n"
"	\n"
"	// Ok, the light is visible\n"
"	\n"
"	*directPdfW = 1.f;\n"
"\n"
"	// Setup the shadow ray\n"
"	const float3 shadowRayOrig = BSDF_GetRayOrigin(bsdf, shadowRayDir);\n"
"	Ray_Init4(shadowRay, shadowRayOrig, shadowRayDir, 0.f, shadowRayDistance, time);\n"
"\n"
"	return VLOAD3F(laserLight->notIntersectable.laser.emittedFactor.c);\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Generic light functions\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float3 EnvLight_GetRadiance(__global const LightSource *light,\n"
"		__global const BSDF *bsdf, const float3 dir, float *directPdfA\n"
"		LIGHTS_PARAM_DECL) {\n"
"	switch (light->type) {\n"
"		case TYPE_IL_CONSTANT:\n"
"			return ConstantInfiniteLight_GetRadiance(light,\n"
"					bsdf,\n"
"					dir, directPdfA\n"
"					LIGHTS_PARAM);\n"
"		case TYPE_IL:\n"
"			return InfiniteLight_GetRadiance(light,\n"
"					bsdf,\n"
"					dir, directPdfA\n"
"					LIGHTS_PARAM);\n"
"		case TYPE_IL_SKY2:\n"
"			return Sky2Light_GetRadiance(light,\n"
"					bsdf,\n"
"					dir, directPdfA\n"
"					LIGHTS_PARAM);\n"
"		case TYPE_SUN:\n"
"			return SunLight_GetRadiance(light,\n"
"					bsdf,\n"
"					dir, directPdfA);\n"
"		case TYPE_SHARPDISTANT:\n"
"			// Just return Black\n"
"		case TYPE_DISTANT:\n"
"			// Just return Black\n"
"		default:\n"
"			return BLACK;\n"
"	}\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 IntersectableLight_GetRadiance(__global const LightSource *light,\n"
"		 __global const HitPoint *hitPoint, float *directPdfA\n"
"		LIGHTS_PARAM_DECL) {\n"
"	return TriangleLight_GetRadiance(light, hitPoint, directPdfA\n"
"			MATERIALS_PARAM);\n"
"}\n"
"\n"
"// Cuda reports large argument size, so overrides noinline attribute anyway\n"
"OPENCL_FORCE_INLINE float3 Light_Illuminate(\n"
"		__global const LightSource *light,\n"
"		__global const BSDF *bsdf,\n"
"		const float time, const float u0, const float u1,\n"
"		const float passThroughEvent,\n"
"		const float worldCenterX,\n"
"		const float worldCenterY,\n"
"		const float worldCenterZ,\n"
"		const float envRadius,\n"
"		__global HitPoint *tmpHitPoint,\n"
"		__global Ray *shadowRay, float *directPdfW\n"
"		LIGHTS_PARAM_DECL) {\n"
"	switch (light->type) {\n"
"		case TYPE_IL_CONSTANT:\n"
"			return ConstantInfiniteLight_Illuminate(\n"
"					light,\n"
"					worldCenterX, worldCenterY, worldCenterZ, envRadius,\n"
"					bsdf, time, u0, u1,\n"
"					shadowRay, directPdfW\n"
"					LIGHTS_PARAM);\n"
"		case TYPE_IL:\n"
"			return InfiniteLight_Illuminate(\n"
"					light,\n"
"					worldCenterX, worldCenterY, worldCenterZ, envRadius,\n"
"					bsdf, time, u0, u1,\n"
"					shadowRay, directPdfW\n"
"					LIGHTS_PARAM);\n"
"		case TYPE_IL_SKY2:\n"
"			return Sky2Light_Illuminate(\n"
"					light,\n"
"					worldCenterX, worldCenterY, worldCenterZ, envRadius,\n"
"					bsdf, time, u0, u1,\n"
"					shadowRay, directPdfW\n"
"					LIGHTS_PARAM);\n"
"		case TYPE_SUN:\n"
"			return SunLight_Illuminate(\n"
"					light,\n"
"					worldCenterX, worldCenterY, worldCenterZ, envRadius,\n"
"					bsdf, time, u0, u1,\n"
"					shadowRay, directPdfW);\n"
"		case TYPE_TRIANGLE:\n"
"			return TriangleLight_Illuminate(\n"
"					light,\n"
"					tmpHitPoint,\n"
"					bsdf, time ,u0, u1,\n"
"					passThroughEvent,\n"
"					shadowRay, directPdfW\n"
"					MATERIALS_PARAM);\n"
"		case TYPE_POINT:\n"
"			return PointLight_Illuminate(\n"
"					light,\n"
"					bsdf, time,\n"
"					shadowRay, directPdfW);\n"
"		case TYPE_MAPPOINT:\n"
"			return MapPointLight_Illuminate(\n"
"					light,\n"
"					bsdf, time,\n"
"					shadowRay, directPdfW\n"
"					IMAGEMAPS_PARAM);\n"
"		case TYPE_SPOT:\n"
"			return SpotLight_Illuminate(\n"
"					light,\n"
"					bsdf, time,\n"
"					shadowRay, directPdfW);\n"
"		case TYPE_PROJECTION:\n"
"			return ProjectionLight_Illuminate(\n"
"					light,\n"
"					bsdf, time,\n"
"					shadowRay, directPdfW\n"
"					IMAGEMAPS_PARAM);\n"
"		case TYPE_SHARPDISTANT:\n"
"			return SharpDistantLight_Illuminate(\n"
"					light,\n"
"					worldCenterX, worldCenterY, worldCenterZ, envRadius,\n"
"					bsdf, time,\n"
"					shadowRay, directPdfW);\n"
"		case TYPE_DISTANT:\n"
"			return DistantLight_Illuminate(\n"
"					light,\n"
"					worldCenterX, worldCenterY, worldCenterZ, envRadius,\n"
"					bsdf, time, u0, u1,\n"
"					shadowRay, directPdfW);\n"
"		case TYPE_LASER:\n"
"			return LaserLight_Illuminate(\n"
"					light,\n"
"					bsdf, time,\n"
"					shadowRay, directPdfW);\n"
"		case TYPE_SPHERE:\n"
"			return SphereLight_Illuminate(\n"
"					light,\n"
"					bsdf, time, u0, u1,\n"
"					shadowRay, directPdfW);\n"
"		case TYPE_MAPSPHERE:\n"
"			return MapSphereLight_Illuminate(\n"
"					light,\n"
"					bsdf, time, u0, u1,\n"
"					shadowRay, directPdfW\n"
"					IMAGEMAPS_PARAM);\n"
"		default:\n"
"			return BLACK;\n"
"	}\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE bool Light_IsEnvOrIntersectable(__global const LightSource *light) {\n"
"	switch (light->type) {\n"
"		case TYPE_IL_CONSTANT:\n"
"		case TYPE_IL:\n"
"		case TYPE_IL_SKY2:\n"
"		case TYPE_SUN:\n"
"		case TYPE_TRIANGLE:\n"
"			return true;\n"
"		case TYPE_SPHERE:\n"
"		case TYPE_MAPSPHERE:\n"
"		case TYPE_POINT:\n"
"		case TYPE_MAPPOINT:\n"
"		case TYPE_SPOT:\n"
"		case TYPE_PROJECTION:\n"
"		case TYPE_SHARPDISTANT:\n"
"		case TYPE_DISTANT:\n"
"		case TYPE_LASER:\n"
"		default:\n"
"			return false;\n"
"	}\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float Light_GetAvgPassThroughTransparency(\n"
"		__global const LightSource *light\n"
"		LIGHTS_PARAM_DECL) {\n"
"	if (light->type == TYPE_TRIANGLE) {\n"
"		const uint meshIndex = light->triangle.meshIndex;\n"
"		const uint materialIndex = sceneObjs[meshIndex].materialIndex;\n"
"		\n"
"		return mats[materialIndex].avgPassThroughTransparency;\n"
"	} else\n"
"		return 1.f;\n"
"}\n";
} }
