
#include <string>

namespace slg { namespace ocl {
std::string KernelSource_light_types =
"#line 2 \"light_types.cl\"\n"
"\n"
"/***************************************************************************\n"
" * Copyright 1998-2020 by authors (see AUTHORS.txt)                        *\n"
" *                                                                         *\n"
" *   This file is part of LuxCoreRender.                                   *\n"
" *                                                                         *\n"
" * Licensed under the Apache License, Version 2.0 (the \"License\");         *\n"
" * you may not use this file except in compliance with the License.        *\n"
" * You may obtain a copy of the License at                                 *\n"
" *                                                                         *\n"
" *     http://www.apache.org/licenses/LICENSE-2.0                          *\n"
" *                                                                         *\n"
" * Unless required by applicable law or agreed to in writing, software     *\n"
" * distributed under the License is distributed on an \"AS IS\" BASIS,       *\n"
" * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.*\n"
" * See the License for the specific language governing permissions and     *\n"
" * limitations under the License.                                          *\n"
" ***************************************************************************/\n"
"\n"
"// This is used to scale the world radius in sun/sky/infinite lights in order to\n"
"// avoid problems with objects that are near the borderline of the world bounding sphere\n"
"#define LIGHT_WORLD_RADIUS_SCALE 1.05f\n"
"\n"
"typedef enum {\n"
"	TYPE_IL, TYPE_SUN, TYPE_TRIANGLE, TYPE_POINT, TYPE_MAPPOINT,\n"
"	TYPE_SPOT, TYPE_PROJECTION, TYPE_IL_CONSTANT, TYPE_SHARPDISTANT, TYPE_DISTANT,\n"
"	TYPE_IL_SKY2, TYPE_LASER, TYPE_SPHERE, TYPE_MAPSPHERE,\n"
"	LIGHT_SOURCE_TYPE_COUNT\n"
"} LightSourceType;\n"
"\n"
"typedef struct {\n"
"	unsigned int imageMapIndex;\n"
"	unsigned int distributionOffset;\n"
"	int useVisibilityMapCache;\n"
"} InfiniteLightParam;\n"
"\n"
"typedef struct {\n"
"	Vector absoluteSunDir, absoluteUpDir;\n"
"	Spectrum aTerm, bTerm, cTerm, dTerm, eTerm, fTerm,\n"
"		gTerm, hTerm, iTerm, radianceTerm;\n"
"	int hasGround, isGroundBlack;\n"
"	Spectrum scaledGroundColor;\n"
"	unsigned int distributionOffset;\n"
"	int useVisibilityMapCache;\n"
"} SkyLight2Param;\n"
"\n"
"typedef struct {\n"
"	Vector absoluteDir;\n"
"	float turbidity, relSize;\n"
"	// XY Vectors for cone sampling\n"
"	Vector x, y;\n"
"	float cosThetaMax, sin2ThetaMax;\n"
"	Spectrum color;\n"
"} SunLightParam;\n"
"\n"
"typedef struct {\n"
"	Vector absolutePos;\n"
"	Spectrum emittedFactor;\n"
"} PointLightParam;\n"
"\n"
"typedef struct {\n"
"	Vector absolutePos, localPos;\n"
"	Spectrum emittedFactor;\n"
"	float average;\n"
"	unsigned int imageMapIndex;\n"
"} MapPointLightParam;\n"
"\n"
"typedef struct {\n"
"	Vector absolutePos;\n"
"	Spectrum emittedFactor;\n"
"	float cosTotalWidth, cosFalloffStart;\n"
"} SpotLightParam;\n"
"\n"
"typedef struct {\n"
"	Vector absolutePos, lightNormal;\n"
"	Spectrum emittedFactor;\n"
"	Matrix4x4 lightProjection;\n"
"	float screenX0, screenX1, screenY0, screenY1;\n"
"	unsigned int imageMapIndex;\n"
"} ProjectionLightParam;\n"
"\n"
"typedef struct {\n"
"	Spectrum color;\n"
"	int useVisibilityMapCache;\n"
"} ConstantInfiniteLightParam;\n"
"\n"
"typedef struct {\n"
"	Spectrum color;\n"
"	Vector absoluteLightDir;\n"
"} SharpDistantLightParam;\n"
"\n"
"typedef struct {\n"
"	Spectrum color;\n"
"	Vector absoluteLightDir, x, y;\n"
"	float cosThetaMax;\n"
"} DistantLightParam;\n"
"\n"
"typedef struct {\n"
"	Vector absoluteLightPos, absoluteLightDir;\n"
"	Spectrum emittedFactor;\n"
"	float radius;\n"
"} LaserLightParam;\n"
"\n"
"typedef struct {\n"
"	Vector absolutePos;\n"
"	Spectrum emittedFactor;\n"
"	float radius;\n"
"} SphereLightParam;\n"
"\n"
"typedef struct {\n"
"	SphereLightParam sphere;\n"
"	float average;\n"
"	unsigned int imageMapIndex;\n"
"} MapSphereLightParam;\n"
"\n"
"typedef struct {\n"
"	Transform light2World;\n"
"	Spectrum gain, temperatureScale;\n"
"\n"
"	union {\n"
"		SunLightParam sun;\n"
"		SkyLight2Param sky2;\n"
"		InfiniteLightParam infinite;\n"
"		PointLightParam point;\n"
"		MapPointLightParam mapPoint;\n"
"		SpotLightParam spot;\n"
"		ProjectionLightParam projection;\n"
"		ConstantInfiniteLightParam constantInfinite;\n"
"		SharpDistantLightParam sharpDistant;\n"
"		DistantLightParam distant;\n"
"		LaserLightParam laser;\n"
"		SphereLightParam sphere;\n"
"		MapSphereLightParam mapSphere;\n"
"	};\n"
"} NotIntersectableLightSource;\n"
"\n"
"typedef struct {\n"
"	float invTriangleArea, invMeshArea;\n"
"\n"
"	unsigned int meshIndex, triangleIndex;\n"
"\n"
"	// Used for image map and/or IES map\n"
"	float average;\n"
"	unsigned int imageMapIndex;\n"
"} TriangleLightParam;\n"
"\n"
"typedef struct {\n"
"	LightSourceType type;\n"
"	unsigned int lightSceneIndex;\n"
"	unsigned int lightID;\n"
"	// Type of indirect paths where a light source is visible with a direct hit. It is\n"
"	// an OR of DIFFUSE, GLOSSY and SPECULAR.\n"
"	BSDFEvent visibility;\n"
"	int isDirectLightSamplingEnabled;\n"
"	\n"
"	union {\n"
"		NotIntersectableLightSource notIntersectable;\n"
"		TriangleLightParam triangle;\n"
"	};\n"
"} LightSource;\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Some macro trick in order to have more readable code\n"
"//------------------------------------------------------------------------------\n"
"\n"
"#if defined(SLG_OPENCL_KERNEL)\n"
"\n"
"#define LIGHTS_PARAM_DECL , __global const LightSource* restrict lights, \\\n"
"	__global const uint* restrict envLightIndices, \\\n"
"	const uint envLightCount, \\\n"
"	__global const float* restrict envLightDistribution, \\\n"
"	__global const float* restrict lightsDistribution, \\\n"
"	__global const float* restrict infiniteLightSourcesDistribution, \\\n"
"	__global const DLSCacheEntry* restrict dlscAllEntries, \\\n"
"	__global const float* restrict dlscDistributions, \\\n"
"	__global const IndexBVHArrayNode* restrict dlscBVHNodes, \\\n"
"	const float dlscRadius2, const float dlscNormalCosAngle, \\\n"
"	__global const ELVCacheEntry* restrict elvcAllEntries, \\\n"
"	__global const float* restrict elvcDistributions, \\\n"
"	__global const uint* restrict elvcTileDistributionOffsets, \\\n"
"	__global const IndexBVHArrayNode* restrict elvcBVHNodes, \\\n"
"	const float elvcRadius2, const float elvcNormalCosAngle, \\\n"
"	const uint elvcTilesXCount, const uint elvcTilesYCount \\\n"
"	MATERIALS_PARAM_DECL\n"
"#define LIGHTS_PARAM , lights, \\\n"
"	envLightIndices, \\\n"
"	envLightCount, \\\n"
"	envLightDistribution, \\\n"
"	lightsDistribution, \\\n"
"	infiniteLightSourcesDistribution, \\\n"
"	dlscAllEntries, \\\n"
"	dlscDistributions, \\\n"
"	dlscBVHNodes, \\\n"
"	dlscRadius2, \\\n"
"	dlscNormalCosAngle, \\\n"
"	elvcAllEntries, \\\n"
"	elvcDistributions, \\\n"
"	elvcTileDistributionOffsets, \\\n"
"	elvcBVHNodes, \\\n"
"	elvcRadius2, \\\n"
"	elvcNormalCosAngle, \\\n"
"	elvcTilesXCount, \\\n"
"	elvcTilesXCount \\\n"
"	MATERIALS_PARAM\n"
"\n"
"#endif\n";
} }
