
#include <string>

namespace slg { namespace ocl {
std::string KernelSource_mapping_funcs =
"#line 2 \"mapping_funcs.cl\"\n"
"\n"
"/***************************************************************************\n"
" * Copyright 1998-2020 by authors (see AUTHORS.txt)                        *\n"
" *                                                                         *\n"
" *   This file is part of LuxCoreRender.                                   *\n"
" *                                                                         *\n"
" * Licensed under the Apache License, Version 2.0 (the \"License\");         *\n"
" * you may not use this file except in compliance with the License.        *\n"
" * You may obtain a copy of the License at                                 *\n"
" *                                                                         *\n"
" *     http://www.apache.org/licenses/LICENSE-2.0                          *\n"
" *                                                                         *\n"
" * Unless required by applicable law or agreed to in writing, software     *\n"
" * distributed under the License is distributed on an \"AS IS\" BASIS,       *\n"
" * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.*\n"
" * See the License for the specific language governing permissions and     *\n"
" * limitations under the License.                                          *\n"
" ***************************************************************************/\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// UVMapping2D\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE float2 UVMapping2D_Map(__global const TextureMapping2D *mapping,\n"
"		__global const HitPoint *hitPoint TEXTURES_PARAM_DECL) {\n"
"	const float2 uv = HitPoint_GetUV(hitPoint, mapping->dataIndex EXTMESH_PARAM);\n"
"\n"
"	// Scale\n"
"	const float uScaled = uv.x * mapping->uvMapping2D.uScale;\n"
"	const float vScaled = uv.y * mapping->uvMapping2D.vScale;\n"
"\n"
"	// Rotate\n"
"	const float sinTheta = mapping->uvMapping2D.sinTheta;\n"
"	const float cosTheta = mapping->uvMapping2D.cosTheta;\n"
"	const float uRotated = uScaled * cosTheta - vScaled * sinTheta;\n"
"	const float vRotated = vScaled * cosTheta + uScaled * sinTheta;\n"
"\n"
"	// Translate\n"
"	const float uTranslated = uRotated + mapping->uvMapping2D.uDelta;\n"
"	const float vTranslated = vRotated + mapping->uvMapping2D.vDelta;\n"
"\n"
"	return MAKE_FLOAT2(uTranslated, vTranslated);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float2 UVMapping2D_MapDuv(__global const TextureMapping2D *mapping,\n"
"		__global const HitPoint *hitPoint, float2 *ds, float2 *dt TEXTURES_PARAM_DECL) {\n"
"	const float signUScale = mapping->uvMapping2D.uScale > 0 ? 1.f : -1.f;\n"
"	const float signVScale = mapping->uvMapping2D.vScale > 0 ? 1.f : -1.f;\n"
"	const float sinTheta = mapping->uvMapping2D.sinTheta;\n"
"	const float cosTheta = mapping->uvMapping2D.cosTheta;\n"
"	\n"
"	*ds = MAKE_FLOAT2(signUScale * cosTheta, signUScale * sinTheta);\n"
"	*dt = MAKE_FLOAT2(-signVScale * sinTheta, signVScale * cosTheta);\n"
"	\n"
"	return UVMapping2D_Map(mapping, hitPoint TEXTURES_PARAM);\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// UVRandomMapping2D\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE float2 UVRandomMapping2D_MapImpl(__global const TextureMapping2D *mapping,\n"
"		__global const HitPoint *hitPoint, float2 *ds, float2 *dt TEXTURES_PARAM_DECL) {\n"
"	// Select random parameters\n"
"	uint seed;\n"
"	switch (mapping->uvRandomMapping2D.seedType) {\n"
"		default:\n"
"		case OBJECT_ID:\n"
"			seed = hitPoint->objectID;\n"
"			break;\n"
"		case TRIANGLE_AOV:\n"
"			seed = (uint)HitPoint_GetTriAOV(hitPoint, mapping->uvRandomMapping2D.triAOVIndex EXTMESH_PARAM);\n"
"			break;\n"
"		case OBJECT_ID_OFFSET:\n"
"			seed = hitPoint->objectID + mapping->uvRandomMapping2D.objectIDOffset;\n"
"			break;\n"
"	}\n"
"\n"
"	Seed rndSeed;\n"
"	Rnd_Init(seed, &rndSeed);\n"
"\n"
"	const float uvRotation = LerpWithStep(Rnd_FloatValue(&rndSeed),\n"
"			mapping->uvRandomMapping2D.uvRotationMin, mapping->uvRandomMapping2D.uvRotationMax,\n"
"			mapping->uvRandomMapping2D.uvRotationStep);\n"
"	const float uScale = Lerp(Rnd_FloatValue(&rndSeed),\n"
"			mapping->uvRandomMapping2D.uScaleMin, mapping->uvRandomMapping2D.uScaleMax);\n"
"	const float vScale = mapping->uvRandomMapping2D.uniformScale ?\n"
"		uScale :\n"
"		Lerp(Rnd_FloatValue(&rndSeed),\n"
"			mapping->uvRandomMapping2D.vScaleMin, mapping->uvRandomMapping2D.vScaleMax);\n"
"	const float uDelta = Lerp(Rnd_FloatValue(&rndSeed),\n"
"			mapping->uvRandomMapping2D.uDeltaMin, mapping->uvRandomMapping2D.uDeltaMax);\n"
"	const float vDelta = Lerp(Rnd_FloatValue(&rndSeed),\n"
"			mapping->uvRandomMapping2D.vDeltaMin, mapping->uvRandomMapping2D.vDeltaMax);\n"
"\n"
"	// Get the hit point UV\n"
"	const float2 uv = HitPoint_GetUV(hitPoint, mapping->dataIndex EXTMESH_PARAM);\n"
"	\n"
"	// Scale\n"
"	const float uScaled = uv.x * uScale;\n"
"	const float vScaled = uv.y * vScale;\n"
"\n"
"	// Rotate\n"
"	const float rad = Radians(-uvRotation);\n"
"	const float sinTheta = sin(rad);\n"
"	const float cosTheta = cos(rad);\n"
"	const float uRotated = uScaled * cosTheta - vScaled * sinTheta;\n"
"	const float vRotated = vScaled * cosTheta + uScaled * sinTheta;\n"
"\n"
"	// Translate\n"
"	const float uTranslated = uRotated + uDelta;\n"
"	const float vTranslated = vRotated + vDelta;\n"
"	\n"
"	if (ds && dt) {\n"
"		const float signUScale = uScale > 0 ? 1.f : -1.f;;\n"
"		const float signVScale = vScale > 0 ? 1.f : -1.f;\n"
"\n"
"		*ds = MAKE_FLOAT2(signUScale * cosTheta, signUScale * sinTheta);\n"
"		*dt = MAKE_FLOAT2(-signVScale * sinTheta, signVScale * cosTheta);\n"
"	}\n"
"	\n"
"	return MAKE_FLOAT2(uTranslated, vTranslated);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float2 UVRandomMapping2D_Map(__global const TextureMapping2D *mapping,\n"
"		__global const HitPoint *hitPoint TEXTURES_PARAM_DECL) {\n"
"	return  UVRandomMapping2D_MapImpl(mapping, hitPoint, NULL, NULL TEXTURES_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float2 UVRandomMapping2D_MapDuv(__global const TextureMapping2D *mapping,\n"
"		__global const HitPoint *hitPoint, float2 *ds, float2 *dt TEXTURES_PARAM_DECL) {\n"
"	return  UVRandomMapping2D_MapImpl(mapping, hitPoint, ds, dt TEXTURES_PARAM);\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// TextureMapping2D\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float2 TextureMapping2D_Map(__global const TextureMapping2D *mapping,\n"
"		__global const HitPoint *hitPoint TEXTURES_PARAM_DECL) {\n"
"	switch (mapping->type) {\n"
"		case UVMAPPING2D:\n"
"			return UVMapping2D_Map(mapping, hitPoint TEXTURES_PARAM);\n"
"		case UVRANDOMMAPPING2D:\n"
"			return UVRandomMapping2D_Map(mapping, hitPoint TEXTURES_PARAM);\n"
"		default:\n"
"			return MAKE_FLOAT2(0.f, 0.f);\n"
"	}\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float2 TextureMapping2D_MapDuv(__global const TextureMapping2D *mapping,\n"
"		__global const HitPoint *hitPoint, float2 *ds, float2 *dt TEXTURES_PARAM_DECL) {\n"
"	switch (mapping->type) {\n"
"		case UVMAPPING2D:\n"
"			return UVMapping2D_MapDuv(mapping, hitPoint, ds, dt TEXTURES_PARAM);\n"
"		case UVRANDOMMAPPING2D:\n"
"			return UVRandomMapping2D_MapDuv(mapping, hitPoint, ds, dt TEXTURES_PARAM);\n"
"		default:\n"
"			return MAKE_FLOAT2(0.f, 0.f);\n"
"	}\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// UVMapping3D\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE float3 UVMapping3D_Map(__global const TextureMapping3D *mapping,\n"
"		__global const HitPoint *hitPoint, float3 *shadeN TEXTURES_PARAM_DECL) {\n"
"	if (shadeN)\n"
"		*shadeN = normalize(Transform_ApplyNormal(&mapping->worldToLocal, VLOAD3F(&hitPoint->shadeN.x)));\n"
"\n"
"	const float2 uv = HitPoint_GetUV(hitPoint, mapping->uvMapping3D.dataIndex EXTMESH_PARAM);\n"
"	return Transform_ApplyPoint(&mapping->worldToLocal, MAKE_FLOAT3(uv.x, uv.y, 0.f));\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// GlobalMapping3D\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE float3 GlobalMapping3D_Map(__global const TextureMapping3D *mapping,\n"
"		__global const HitPoint *hitPoint, float3 *shadeN) {\n"
"	if (shadeN)\n"
"		*shadeN = normalize(Transform_ApplyNormal(&mapping->worldToLocal, VLOAD3F(&hitPoint->shadeN.x)));\n"
"\n"
"	const float3 p = VLOAD3F(&hitPoint->p.x);\n"
"	return Transform_ApplyPoint(&mapping->worldToLocal, p);\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// LocalMapping3D\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE float3 LocalMapping3D_Map(__global const TextureMapping3D *mapping,\n"
"		__global const HitPoint *hitPoint, float3 *shadeN) {\n"
"	if (shadeN) {\n"
"		const Matrix4x4 mInv = Matrix4x4_Mul(&hitPoint->localToWorld.m, &mapping->worldToLocal.mInv);\n"
"		\n"
"		const float3 sn = VLOAD3F(&hitPoint->shadeN.x);\n"
"		*shadeN = normalize(Matrix4x4_ApplyNormal_Private(&mInv, sn));\n"
"	}\n"
"\n"
"	const Matrix4x4 m = Matrix4x4_Mul(&mapping->worldToLocal.m, &hitPoint->localToWorld.mInv);\n"
"\n"
"	const float3 p = VLOAD3F(&hitPoint->p.x);\n"
"	return Matrix4x4_ApplyPoint_Private(&m, p);\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// LocalRandomMapping3D\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE float3 LocalRandomMapping3D_Map(__global const TextureMapping3D *mapping,\n"
"		__global const HitPoint *hitPoint, float3 *shadeN TEXTURES_PARAM_DECL) {\n"
"	if (shadeN) {\n"
"		const Matrix4x4 mInv = Matrix4x4_Mul(&hitPoint->localToWorld.m, &mapping->worldToLocal.mInv);\n"
"		\n"
"		const float3 sn = VLOAD3F(&hitPoint->shadeN.x);\n"
"		*shadeN = normalize(Matrix4x4_ApplyNormal_Private(&mInv, sn));\n"
"	}\n"
"\n"
"	Matrix4x4 m = Matrix4x4_Mul(&mapping->worldToLocal.m, &hitPoint->localToWorld.mInv);\n"
"\n"
"	// Select random parameters\n"
"	uint seed;\n"
"	switch (mapping->localRandomMapping.seedType) {\n"
"		default:\n"
"		case OBJECT_ID:\n"
"			seed = hitPoint->objectID;\n"
"			break;\n"
"		case TRIANGLE_AOV:\n"
"			seed = (uint)HitPoint_GetTriAOV(hitPoint, mapping->localRandomMapping.triAOVIndex EXTMESH_PARAM);\n"
"			break;\n"
"		case OBJECT_ID_OFFSET:\n"
"			seed = hitPoint->objectID + mapping->localRandomMapping.objectIDOffset;\n"
"			break;\n"
"	}\n"
"\n"
"	Seed rndSeed;\n"
"	Rnd_Init(seed, &rndSeed);\n"
"	\n"
"	const float xRotation = LerpWithStep(Rnd_FloatValue(&rndSeed),\n"
"			mapping->localRandomMapping.xRotationMin,\n"
"			mapping->localRandomMapping.xRotationMax,\n"
"			mapping->localRandomMapping.xRotationStep);\n"
"	const float yRotation = LerpWithStep(Rnd_FloatValue(&rndSeed),\n"
"			mapping->localRandomMapping.yRotationMin,\n"
"			mapping->localRandomMapping.yRotationMax,\n"
"			mapping->localRandomMapping.yRotationStep);\n"
"	const float zRotation = LerpWithStep(Rnd_FloatValue(&rndSeed),\n"
"			mapping->localRandomMapping.zRotationMin,\n"
"			mapping->localRandomMapping.zRotationMax,\n"
"			mapping->localRandomMapping.yRotationStep);\n"
"	\n"
"	const float xScale = Lerp(Rnd_FloatValue(&rndSeed), mapping->localRandomMapping.xScaleMin, mapping->localRandomMapping.xScaleMax);\n"
"	const bool uniformScale = mapping->localRandomMapping.uniformScale;\n"
"	const float yScale = uniformScale ? xScale : Lerp(Rnd_FloatValue(&rndSeed), mapping->localRandomMapping.yScaleMin, mapping->localRandomMapping.yScaleMax);\n"
"	const float zScale = uniformScale ? xScale : Lerp(Rnd_FloatValue(&rndSeed), mapping->localRandomMapping.zScaleMin, mapping->localRandomMapping.zScaleMax);\n"
"	\n"
"	const float xTranslate = Lerp(Rnd_FloatValue(&rndSeed), mapping->localRandomMapping.xTranslateMin, mapping->localRandomMapping.xTranslateMax);\n"
"	const float yTranslate = Lerp(Rnd_FloatValue(&rndSeed), mapping->localRandomMapping.yTranslateMin, mapping->localRandomMapping.yTranslateMax);\n"
"	const float zTranslate = Lerp(Rnd_FloatValue(&rndSeed), mapping->localRandomMapping.zTranslateMin, mapping->localRandomMapping.zTranslateMax);\n"
"\n"
"	Matrix4x4 mRandomTrans = Matrix4x4_Scale(xScale, yScale, zScale);\n"
"\n"
"	const Matrix4x4 mRotateX = Matrix4x4_RotateX(xRotation);\n"
"	mRandomTrans = Matrix4x4_Mul_Private(&mRandomTrans, &mRotateX);\n"
"	const Matrix4x4 mRotateY = Matrix4x4_RotateY(yRotation);\n"
"	mRandomTrans = Matrix4x4_Mul_Private(&mRandomTrans, &mRotateY);\n"
"	const Matrix4x4 mRotateZ = Matrix4x4_RotateZ(zRotation);\n"
"	mRandomTrans = Matrix4x4_Mul_Private(&mRandomTrans, &mRotateZ);\n"
"\n"
"	const Matrix4x4 mTranslate = Matrix4x4_Translate(xTranslate, yTranslate, zTranslate);\n"
"	mRandomTrans = Matrix4x4_Mul_Private(&mRandomTrans, &mTranslate);\n"
"	\n"
"	m = Matrix4x4_Mul_Private(&mRandomTrans, &m);\n"
"\n"
"	const float3 p = VLOAD3F(&hitPoint->p.x);\n"
"	return Matrix4x4_ApplyPoint_Private(&m, p);\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float3 TextureMapping3D_Map(__global const TextureMapping3D *mapping,\n"
"		__global const HitPoint *hitPoint, float3 *shadeN TEXTURES_PARAM_DECL) {\n"
"	switch (mapping->type) {\n"
"		case UVMAPPING3D:\n"
"			return UVMapping3D_Map(mapping, hitPoint, shadeN TEXTURES_PARAM);\n"
"		case GLOBALMAPPING3D:\n"
"			return GlobalMapping3D_Map(mapping, hitPoint, shadeN);\n"
"		case LOCALMAPPING3D:\n"
"			return LocalMapping3D_Map(mapping, hitPoint, shadeN);\n"
"		case LOCALRANDOMMAPPING3D:\n"
"			return LocalRandomMapping3D_Map(mapping, hitPoint, shadeN TEXTURES_PARAM);\n"
"		default:\n"
"			return BLACK;\n"
"	}\n"
"}\n";
} }
