
#include <string>

namespace slg { namespace ocl {
std::string KernelSource_material_funcs_evalops =
"#line 2 \"material_eval_funcs.cl\"\n"
"\n"
"/***************************************************************************\n"
" * Copyright 1998-2020 by authors (see AUTHORS.txt)                        *\n"
" *                                                                         *\n"
" *   This file is part of LuxCoreRender.                                   *\n"
" *                                                                         *\n"
" * Licensed under the Apache License, Version 2.0 (the License);         *\n"
" * you may not use this file except in compliance with the License.        *\n"
" * You may obtain a copy of the License at                                 *\n"
" *                                                                         *\n"
" *     http://www.apache.org/licenses/LICENSE-2.0                          *\n"
" *                                                                         *\n"
" * Unless required by applicable law or agreed to in writing, software     *\n"
" * distributed under the License is distributed on an AS IS BASIS,       *\n"
" * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.*\n"
" * See the License for the specific language governing permissions and     *\n"
" * limitations under the License.                                          *\n"
" ***************************************************************************/\n"
"\n"
"//#define DEBUG_PRINTF_KERNEL_NAME 1\n"
"//#define DEBUG_PRINTF_MATERIAL_EVAL 1\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Material evaluation functions\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_NOT_INLINE uint Material_EvalOp(\n"
"		__global const MaterialEvalOp* restrict evalOp,\n"
"		__global float *evalStack,\n"
"		uint *evalStackOffset,\n"
"		__global const HitPoint *hitPoint\n"
"		MATERIALS_PARAM_DECL) {\n"
"	const uint matIndex = evalOp->matIndex;\n"
"	__global const Material* restrict material = &mats[matIndex];\n"
"\n"
"#if defined(DEBUG_PRINTF_MATERIAL_EVAL)\n"
"	printf(\"EvalOp material index=%d type=%d evalType=%d *evalStackOffset=%d\\n\", evalOp->matIndex, material->type, evalOp->evalType, *evalStackOffset);\n"
"#endif\n"
"\n"
"	const MaterialEvalOpType evalType = evalOp->evalType;\n"
"\n"
"	//--------------------------------------------------------------------------\n"
"	// The support for generic ops\n"
"	//--------------------------------------------------------------------------\n"
"\n"
"	switch (evalType) {\n"
"		case EVAL_CONDITIONAL_GOTO: {\n"
"				bool condition;\n"
"				EvalStack_PopInt(condition);\n"
"\n"
"				if (condition)\n"
"					return evalOp->opData.opsCount;\n"
"				break;\n"
"		}\n"
"		case EVAL_UNCONDITIONAL_GOTO:\n"
"			return evalOp->opData.opsCount;\n"
"		default:\n"
"			// It will be handled by the following switch\n"
"			break;\n"
"	}\n"
"\n"
"	//--------------------------------------------------------------------------\n"
"	// The support for material specific ops\n"
"	//--------------------------------------------------------------------------\n"
"\n"
"	switch (material->type) {\n"
"		//----------------------------------------------------------------------\n"
"		// MATTE\n"
"		//----------------------------------------------------------------------\n"
"		case MATTE:\n"
"			MatteMaterial_EvalOp(material, evalType, evalStack, evalStackOffset, hitPoint MATERIALS_PARAM);\n"
"			break;\n"
"		//----------------------------------------------------------------------\n"
"		// MATTETRANSLUCENT\n"
"		//----------------------------------------------------------------------\n"
"		case MATTETRANSLUCENT:\n"
"			MatteTranslucentMaterial_EvalOp(material, evalType, evalStack, evalStackOffset, hitPoint MATERIALS_PARAM);\n"
"			break;\n"
"		//----------------------------------------------------------------------\n"
"		// GLOSSY2\n"
"		//----------------------------------------------------------------------\n"
"		case GLOSSY2:\n"
"			Glossy2Material_EvalOp(material, evalType, evalStack, evalStackOffset, hitPoint MATERIALS_PARAM);\n"
"			break;			\n"
"		//----------------------------------------------------------------------\n"
"		// METAL2\n"
"		//----------------------------------------------------------------------\n"
"		case METAL2:\n"
"			Metal2Material_EvalOp(material, evalType, evalStack, evalStackOffset, hitPoint MATERIALS_PARAM);\n"
"			break;			\n"
"		//----------------------------------------------------------------------\n"
"		// VELVET\n"
"		//----------------------------------------------------------------------\n"
"		case VELVET:\n"
"			VelvetMaterial_EvalOp(material, evalType, evalStack, evalStackOffset, hitPoint MATERIALS_PARAM);\n"
"			break;	\n"
"		//----------------------------------------------------------------------\n"
"		// CLOTH\n"
"		//----------------------------------------------------------------------\n"
"		case CLOTH:\n"
"			ClothMaterial_EvalOp(material, evalType, evalStack, evalStackOffset, hitPoint MATERIALS_PARAM);\n"
"			break;\n"
"		//----------------------------------------------------------------------\n"
"		// CARPAINT\n"
"		//----------------------------------------------------------------------\n"
"		case CARPAINT:\n"
"			CarPaintMaterial_EvalOp(material, evalType, evalStack, evalStackOffset, hitPoint MATERIALS_PARAM);\n"
"			break;\n"
"		//----------------------------------------------------------------------\n"
"		// ROUGHMATTE\n"
"		//----------------------------------------------------------------------\n"
"		case ROUGHMATTE:\n"
"			RoughMatteMaterial_EvalOp(material, evalType, evalStack, evalStackOffset, hitPoint MATERIALS_PARAM);\n"
"			break;\n"
"		//----------------------------------------------------------------------\n"
"		// ROUGHMATTETRANSLUCENT\n"
"		//----------------------------------------------------------------------\n"
"		case ROUGHMATTETRANSLUCENT:\n"
"			RoughMatteTranslucentMaterial_EvalOp(material, evalType, evalStack, evalStackOffset, hitPoint MATERIALS_PARAM);\n"
"			break;\n"
"		//----------------------------------------------------------------------\n"
"		// GLOSSYTRANSLUCENT\n"
"		//----------------------------------------------------------------------\n"
"		case GLOSSYTRANSLUCENT:\n"
"			GlossyTranslucentMaterial_EvalOp(material, evalType, evalStack, evalStackOffset, hitPoint MATERIALS_PARAM);\n"
"			break;\n"
"		//----------------------------------------------------------------------\n"
"		// DISNEY\n"
"		//----------------------------------------------------------------------\n"
"		case DISNEY:\n"
"			DisneyMaterial_EvalOp(material, evalType, evalStack, evalStackOffset, hitPoint MATERIALS_PARAM);\n"
"			break;\n"
"		//----------------------------------------------------------------------\n"
"		// NULLMAT\n"
"		//----------------------------------------------------------------------\n"
"		case NULLMAT:\n"
"			NullMaterial_EvalOp(material, evalType, evalStack, evalStackOffset, hitPoint MATERIALS_PARAM);\n"
"			break;\n"
"		//----------------------------------------------------------------------\n"
"		// MIX\n"
"		//----------------------------------------------------------------------\n"
"		case MIX:\n"
"			MixMaterial_EvalOp(material, evalType, evalStack, evalStackOffset, hitPoint MATERIALS_PARAM);\n"
"			break;\n"
"		//----------------------------------------------------------------------\n"
"		// GLOSSYCOATING\n"
"		//----------------------------------------------------------------------\n"
"		case GLOSSYCOATING:\n"
"			GlossyCoatingMaterial_EvalOp(material, evalType, evalStack, evalStackOffset, hitPoint MATERIALS_PARAM);\n"
"			break;\n"
"		//----------------------------------------------------------------------\n"
"		// MIRROR\n"
"		//----------------------------------------------------------------------\n"
"		case MIRROR:\n"
"			MirrorMaterial_EvalOp(material, evalType, evalStack, evalStackOffset, hitPoint MATERIALS_PARAM);\n"
"			break;\n"
"		//----------------------------------------------------------------------\n"
"		// GLASS\n"
"		//----------------------------------------------------------------------\n"
"		case GLASS:\n"
"			GlassMaterial_EvalOp(material, evalType, evalStack, evalStackOffset, hitPoint MATERIALS_PARAM);\n"
"			break;\n"
"		//----------------------------------------------------------------------\n"
"		// ARCHGLASS\n"
"		//----------------------------------------------------------------------\n"
"		case ARCHGLASS:\n"
"			ArchGlassMaterial_EvalOp(material, evalType, evalStack, evalStackOffset, hitPoint MATERIALS_PARAM);\n"
"			break;\n"
"		//----------------------------------------------------------------------\n"
"		// ROUGHGLASS\n"
"		//----------------------------------------------------------------------\n"
"		case ROUGHGLASS:\n"
"			RoughGlassMaterial_EvalOp(material, evalType, evalStack, evalStackOffset, hitPoint MATERIALS_PARAM);\n"
"			break;\n"
"		//----------------------------------------------------------------------\n"
"		// TWOSIDED\n"
"		//----------------------------------------------------------------------\n"
"		case TWOSIDED:\n"
"			TwoSidedMaterial_EvalOp(material, evalType, evalStack, evalStackOffset, hitPoint MATERIALS_PARAM);\n"
"			break;\n"
"		//----------------------------------------------------------------------\n"
"		// CLEAR_VOL\n"
"		//----------------------------------------------------------------------\n"
"		case CLEAR_VOL:\n"
"			ClearVolMaterial_EvalOp(material, evalType, evalStack, evalStackOffset, hitPoint MATERIALS_PARAM);\n"
"			break;\n"
"		//----------------------------------------------------------------------\n"
"		// HOMOGENEOUS_VOL\n"
"		//----------------------------------------------------------------------\n"
"		case HOMOGENEOUS_VOL:\n"
"			HomogeneousVolMaterial_EvalOp(material, evalType, evalStack, evalStackOffset, hitPoint MATERIALS_PARAM);\n"
"			break;\n"
"		//----------------------------------------------------------------------\n"
"		// HETEROGENEOUS_VOL\n"
"		//----------------------------------------------------------------------\n"
"		case HETEROGENEOUS_VOL:\n"
"			HeterogeneousVolMaterial_EvalOp(material, evalType, evalStack, evalStackOffset, hitPoint MATERIALS_PARAM);\n"
"			break;\n"
"		default:\n"
"			// Something wrong here\n"
"			break;\n"
"	}\n"
"\n"
"	return 0;\n"
"}\n";
} }
