
#include <string>

namespace slg { namespace ocl {
std::string KernelSource_material_funcs =
"#line 2 \"material_main.cl\"\n"
"\n"
"/***************************************************************************\n"
" * Copyright 1998-2020 by authors (see AUTHORS.txt)                        *\n"
" *                                                                         *\n"
" *   This file is part of LuxCoreRender.                                   *\n"
" *                                                                         *\n"
" * Licensed under the Apache License, Version 2.0 (the License);         *\n"
" * you may not use this file except in compliance with the License.        *\n"
" * You may obtain a copy of the License at                                 *\n"
" *                                                                         *\n"
" *     http://www.apache.org/licenses/LICENSE-2.0                          *\n"
" *                                                                         *\n"
" * Unless required by applicable law or agreed to in writing, software     *\n"
" * distributed under the License is distributed on an AS IS BASIS,       *\n"
" * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.*\n"
" * See the License for the specific language governing permissions and     *\n"
" * limitations under the License.                                          *\n"
" ***************************************************************************/\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Main material functions\n"
"//------------------------------------------------------------------------------\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Material_Albedo\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float3 Material_Albedo(const uint matIndex,\n"
"		__global const HitPoint *hitPoint\n"
"		MATERIALS_PARAM_DECL) {\n"
"#if defined(DEBUG_PRINTF_MATERIAL_EVAL)\n"
"	printf(\"===============================================================\\n\");\n"
"	printf(\"Material_Albedo()\\n\");\n"
"	printf(\"===============================================================\\n\");\n"
"#endif\n"
"\n"
"	__global const Material* restrict startMat = &mats[matIndex];\n"
"	const size_t gid = get_global_id(0);\n"
"	__global float *evalStack = &matEvalStacks[gid * maxMaterialEvalStackSize];\n"
"\n"
"	const uint evalOpStartIndex = startMat->evalAlbedoOpStartIndex;\n"
"	const uint evalOpLength = startMat->evalAlbedoOpLength;\n"
"\n"
"#if defined(DEBUG_PRINTF_MATERIAL_EVAL)\n"
"	printf(\"matIndex=%d evalOpStartIndex=%d evalOpLength=%d\\n\", matIndex, evalOpStartIndex, evalOpLength);\n"
"#endif\n"
"\n"
"	uint evalStackOffsetVal = 0;\n"
"	uint *evalStackOffset = &evalStackOffsetVal; // Used by macros\n"
"	for (uint i = 0; i < evalOpLength; ++i) {\n"
"#if defined(DEBUG_PRINTF_MATERIAL_EVAL)\n"
"		printf(\"EvalOp: #%d evalStackOffset=%d\\n\", i, evalStackOffsetVal);\n"
"#endif\n"
"\n"
"		__global const MaterialEvalOp* restrict evalOp = &matEvalOps[evalOpStartIndex + i];\n"
"\n"
"		i += Material_EvalOp(evalOp, evalStack, &evalStackOffsetVal, hitPoint MATERIALS_PARAM);\n"
"	}\n"
"#if defined(DEBUG_PRINTF_MATERIAL_EVAL)\n"
"	printf(\"evalStackOffset=#%d\\n\", evalStackOffsetVal);\n"
"#endif\n"
"	\n"
"	float3 result;\n"
"	EvalStack_PopFloat3(result);\n"
"\n"
"#if defined(DEBUG_PRINTF_MATERIAL_EVAL)\n"
"	printf(\"Result=(%f, %f, %f)\\n\", result.s0, result.s1, result.s2);\n"
"#endif\n"
"\n"
"	return result;\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Material_Evaluate\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float3 Material_Evaluate(const uint matIndex,\n"
"		__global const HitPoint *hitPoint, const float3 lightDir, const float3 eyeDir,\n"
"		BSDFEvent *event, float *directPdfW\n"
"		MATERIALS_PARAM_DECL) {\n"
"#if defined(DEBUG_PRINTF_MATERIAL_EVAL)\n"
"	printf(\"===============================================================\\n\");\n"
"	printf(\"Material_Evaluate()\\n\");\n"
"	printf(\"===============================================================\\n\");\n"
"#endif\n"
"\n"
"	__global const Material* restrict startMat = &mats[matIndex];\n"
"	const size_t gid = get_global_id(0);\n"
"	__global float *evalStack = &matEvalStacks[gid * maxMaterialEvalStackSize];\n"
"\n"
"	const uint evalOpStartIndex = startMat->evalEvaluateOpStartIndex;\n"
"	const uint evalOpLength = startMat->evalEvaluateOpLength;\n"
"\n"
"#if defined(DEBUG_PRINTF_MATERIAL_EVAL)\n"
"	printf(\"matIndex=%d evalOpStartIndex=%d evalOpLength=%d\\n\", matIndex, evalOpStartIndex, evalOpLength);\n"
"#endif\n"
"\n"
"	uint evalStackOffsetVal = 0;\n"
"	uint *evalStackOffset = &evalStackOffsetVal; // Used by macros\n"
"	// Evaluation parameters\n"
"	EvalStack_PushFloat3(lightDir);\n"
"	EvalStack_PushFloat3(eyeDir);\n"
"\n"
"	for (uint i = 0; i < evalOpLength; ++i) {\n"
"#if defined(DEBUG_PRINTF_MATERIAL_EVAL)\n"
"		printf(\"EvalOp: #%d evalStackOffset=%d\\n\", i, evalStackOffsetVal);\n"
"#endif\n"
"\n"
"		__global const MaterialEvalOp* restrict evalOp = &matEvalOps[evalOpStartIndex + i];\n"
"\n"
"		i += Material_EvalOp(evalOp, evalStack, &evalStackOffsetVal, hitPoint MATERIALS_PARAM);\n"
"	}\n"
"#if defined(DEBUG_PRINTF_MATERIAL_EVAL)\n"
"	printf(\"evalStackOffset=#%d\\n\", evalStackOffsetVal);\n"
"#endif\n"
"	\n"
"	float pdfResult;\n"
"	EvalStack_PopFloat(pdfResult);\n"
"	BSDFEvent eventResult;\n"
"	EvalStack_PopBSDFEvent(eventResult);\n"
"	float3 result;\n"
"	EvalStack_PopFloat3(result);\n"
"\n"
"#if defined(DEBUG_PRINTF_MATERIAL_EVAL)\n"
"	printf(\"Result=(%f, %f, %f) %d %f\\n\", result.s0, result.s1, result.s2, eventResult, pdfResult);\n"
"#endif\n"
"\n"
"	*event = eventResult;\n"
"	if (directPdfW)\n"
"		*directPdfW = pdfResult;\n"
"\n"
"	return result;\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Material_Sample\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float3 Material_Sample(const uint matIndex, __global const HitPoint *hitPoint,\n"
"		const float3 fixedDir, float3 *sampledDir,\n"
"		const float u0, const float u1,\n"
"		const float passThroughEvent,\n"
"		float *pdfW, BSDFEvent *event\n"
"		MATERIALS_PARAM_DECL) {\n"
"#if defined(DEBUG_PRINTF_MATERIAL_EVAL)\n"
"	printf(\"===============================================================\\n\");\n"
"	printf(\"Material_Sample()\\n\");\n"
"	printf(\"===============================================================\\n\");\n"
"#endif\n"
"\n"
"	__global const Material* restrict startMat = &mats[matIndex];\n"
"	const size_t gid = get_global_id(0);\n"
"	__global float *evalStack = &matEvalStacks[gid * maxMaterialEvalStackSize];\n"
"\n"
"	const uint evalOpStartIndex = startMat->evalSampleOpStartIndex;\n"
"	const uint evalOpLength = startMat->evalSampleOpLength;\n"
"\n"
"#if defined(DEBUG_PRINTF_MATERIAL_EVAL)\n"
"	printf(\"matIndex=%d evalOpStartIndex=%d evalOpLength=%d\\n\", matIndex, evalOpStartIndex, evalOpLength);\n"
"#endif\n"
"\n"
"	uint evalStackOffsetVal = 0;\n"
"	uint *evalStackOffset = &evalStackOffsetVal; // Used by macros\n"
"	// Evaluation parameters\n"
"	EvalStack_PushFloat3(fixedDir);\n"
"	EvalStack_PushFloat(u0);\n"
"	EvalStack_PushFloat(u1);\n"
"	EvalStack_PushFloat(passThroughEvent);\n"
"\n"
"	for (uint i = 0; i < evalOpLength; ++i) {\n"
"#if defined(DEBUG_PRINTF_MATERIAL_EVAL)\n"
"		printf(\"EvalOp: #%d evalStackOffset=%d\\n\", i, evalStackOffsetVal);\n"
"#endif\n"
"\n"
"		__global const MaterialEvalOp* restrict evalOp = &matEvalOps[evalOpStartIndex + i];\n"
"\n"
"		i += Material_EvalOp(evalOp, evalStack, &evalStackOffsetVal, hitPoint MATERIALS_PARAM);\n"
"	}\n"
"#if defined(DEBUG_PRINTF_MATERIAL_EVAL)\n"
"	printf(\"evalStackOffset=#%d\\n\", evalStackOffsetVal);\n"
"#endif\n"
"	\n"
"	BSDFEvent eventResult;\n"
"	EvalStack_PopBSDFEvent(eventResult);\n"
"	float pdfResult;\n"
"	EvalStack_PopFloat(pdfResult);\n"
"	float3 result, sampledDirResult;\n"
"	EvalStack_PopFloat3(sampledDirResult);\n"
"	EvalStack_PopFloat3(result);\n"
"\n"
"#if defined(DEBUG_PRINTF_MATERIAL_EVAL)\n"
"	printf(\"Result=(%f, %f, %f) (%f, %f, %f) %f %d\\n\", result.s0, result.s1, result.s2,\n"
"			sampledDirResult.s0, sampledDirResult.s1, sampledDirResult.s2, pdfResult, eventResult);\n"
"#endif\n"
"\n"
"	*sampledDir = sampledDirResult;\n"
"	if (pdfW)\n"
"		*pdfW = pdfResult;\n"
"	*event = eventResult;\n"
"\n"
"	return result;\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Material_GetPassThroughTransparency\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float3 Material_GetPassThroughTransparency(const uint matIndex,\n"
"		__global const HitPoint *hitPoint,\n"
"		const float3 localFixedDir, const float passThroughEvent, const bool backTracing\n"
"		MATERIALS_PARAM_DECL) {\n"
"#if defined(DEBUG_PRINTF_MATERIAL_EVAL)\n"
"	printf(\"===============================================================\\n\");\n"
"	printf(\"Material_GetPassThroughTransparency()\\n\");\n"
"	printf(\"===============================================================\\n\");\n"
"#endif\n"
"\n"
"	__global const Material* restrict startMat = &mats[matIndex];\n"
"	const size_t gid = get_global_id(0);\n"
"	__global float *evalStack = &matEvalStacks[gid * maxMaterialEvalStackSize];\n"
"\n"
"	const uint evalOpStartIndex = startMat->evalGetPassThroughTransparencyOpStartIndex;\n"
"	const uint evalOpLength = startMat->evalGetPassThroughTransparencyOpLength;\n"
"\n"
"#if defined(DEBUG_PRINTF_MATERIAL_EVAL)\n"
"	printf(\"matIndex=%d evalOpStartIndex=%d evalOpLength=%d\\n\", matIndex, evalOpStartIndex, evalOpLength);\n"
"#endif\n"
"\n"
"	uint evalStackOffsetVal = 0;\n"
"	uint *evalStackOffset = &evalStackOffsetVal; // Used by macros\n"
"	// Evaluation parameters\n"
"	EvalStack_PushFloat3(localFixedDir);\n"
"	EvalStack_PushFloat(passThroughEvent);\n"
"	EvalStack_PushUInt(backTracing);\n"
"\n"
"	for (uint i = 0; i < evalOpLength; ++i) {\n"
"#if defined(DEBUG_PRINTF_MATERIAL_EVAL)\n"
"		printf(\"EvalOp: #%d evalStackOffset=%d\\n\", i, evalStackOffsetVal);\n"
"#endif\n"
"\n"
"		__global const MaterialEvalOp* restrict evalOp = &matEvalOps[evalOpStartIndex + i];\n"
"\n"
"		i += Material_EvalOp(evalOp, evalStack, &evalStackOffsetVal, hitPoint MATERIALS_PARAM);\n"
"	}\n"
"#if defined(DEBUG_PRINTF_MATERIAL_EVAL)\n"
"	printf(\"evalStackOffset=#%d\\n\", evalStackOffsetVal);\n"
"#endif\n"
"	\n"
"	float3 result;\n"
"	EvalStack_PopFloat3(result);\n"
"\n"
"#if defined(DEBUG_PRINTF_MATERIAL_EVAL)\n"
"	printf(\"Result=(%f, %f, %f)\\n\", result.s0, result.s1, result.s2);\n"
"#endif\n"
"\n"
"	return result;\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Material_GetEmittedRadiance\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float3 Material_GetEmittedRadiance(const uint matIndex,\n"
"		__global const HitPoint *hitPoint, const float oneOverPrimitiveArea\n"
"		MATERIALS_PARAM_DECL) {\n"
"#if defined(DEBUG_PRINTF_MATERIAL_EVAL)\n"
"	printf(\"===============================================================\\n\");\n"
"	printf(\"Material_GetEmittedRadiance()\\n\");\n"
"	printf(\"===============================================================\\n\");\n"
"#endif\n"
"\n"
"	__global const Material* restrict startMat = &mats[matIndex];\n"
"	const size_t gid = get_global_id(0);\n"
"	__global float *evalStack = &matEvalStacks[gid * maxMaterialEvalStackSize];\n"
"\n"
"	const uint evalOpStartIndex = startMat->evalGetEmittedRadianceOpStartIndex;\n"
"	const uint evalOpLength = startMat->evalGetEmittedRadianceOpLength;\n"
"\n"
"#if defined(DEBUG_PRINTF_MATERIAL_EVAL)\n"
"	printf(\"matIndex=%d evalOpStartIndex=%d evalOpLength=%d\\n\", matIndex, evalOpStartIndex, evalOpLength);\n"
"#endif\n"
"\n"
"	uint evalStackOffsetVal = 0;\n"
"	uint *evalStackOffset = &evalStackOffsetVal; // Used by macros\n"
"	// Evaluation parameters\n"
"	EvalStack_PushFloat(oneOverPrimitiveArea);\n"
"\n"
"	for (uint i = 0; i < evalOpLength; ++i) {\n"
"#if defined(DEBUG_PRINTF_MATERIAL_EVAL)\n"
"		printf(\"EvalOp: #%d evalStackOffset=%d\\n\", i, evalStackOffsetVal);\n"
"#endif\n"
"\n"
"		__global const MaterialEvalOp* restrict evalOp = &matEvalOps[evalOpStartIndex + i];\n"
"\n"
"		i += Material_EvalOp(evalOp, evalStack, &evalStackOffsetVal, hitPoint MATERIALS_PARAM);\n"
"	}\n"
"#if defined(DEBUG_PRINTF_MATERIAL_EVAL)\n"
"	printf(\"evalStackOffset=#%d\\n\", evalStackOffsetVal);\n"
"#endif\n"
"\n"
"	float3 result;\n"
"	EvalStack_PopFloat3(result);\n"
"\n"
"#if defined(DEBUG_PRINTF_MATERIAL_EVAL)\n"
"	printf(\"Result=(%f, %f, %f)\\n\", result.s0, result.s1, result.s2);\n"
"#endif\n"
"\n"
"	return result;\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Material_GetInteriorVolume\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_NOT_INLINE uint Material_GetInteriorVolume(const uint matIndex,\n"
"		__global const HitPoint *hitPoint,\n"
"		const float passThroughEvent\n"
"		MATERIALS_PARAM_DECL) {\n"
"#if defined(DEBUG_PRINTF_MATERIAL_EVAL)\n"
"	printf(\"===============================================================\\n\");\n"
"	printf(\"Material_GetInteriorVolume()\\n\");\n"
"	printf(\"===============================================================\\n\");\n"
"#endif\n"
"\n"
"	__global const Material* restrict startMat = &mats[matIndex];\n"
"	const size_t gid = get_global_id(0);\n"
"	__global float *evalStack = &matEvalStacks[gid * maxMaterialEvalStackSize];\n"
"\n"
"	const uint evalOpStartIndex = startMat->evalGetInteriorVolumeOpStartIndex;\n"
"	const uint evalOpLength = startMat->evalGetInteriorVolumeOpLength;\n"
"\n"
"#if defined(DEBUG_PRINTF_MATERIAL_EVAL)\n"
"	printf(\"matIndex=%d evalOpStartIndex=%d evalOpLength=%d\\n\", matIndex, evalOpStartIndex, evalOpLength);\n"
"#endif\n"
"\n"
"	uint evalStackOffsetVal = 0;\n"
"	uint *evalStackOffset = &evalStackOffsetVal; // Used by macros\n"
"	// Evaluation parameters\n"
"	EvalStack_PushFloat(passThroughEvent);\n"
"\n"
"	for (uint i = 0; i < evalOpLength; ++i) {\n"
"#if defined(DEBUG_PRINTF_MATERIAL_EVAL)\n"
"		printf(\"EvalOp: #%d evalStackOffset=%d\\n\", i, evalStackOffsetVal);\n"
"#endif\n"
"\n"
"		__global const MaterialEvalOp* restrict evalOp = &matEvalOps[evalOpStartIndex + i];\n"
"\n"
"		i += Material_EvalOp(evalOp, evalStack, &evalStackOffsetVal, hitPoint MATERIALS_PARAM);\n"
"	}\n"
"#if defined(DEBUG_PRINTF_MATERIAL_EVAL)\n"
"	printf(\"evalStackOffset=#%d\\n\", evalStackOffsetVal);\n"
"#endif\n"
"\n"
"	uint result;\n"
"	EvalStack_PopUInt(result);\n"
"\n"
"#if defined(DEBUG_PRINTF_MATERIAL_EVAL)\n"
"	printf(\"Result=(%d)\\n\", result);\n"
"#endif\n"
"\n"
"	return result;\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Material_GetExteriorVolume\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_NOT_INLINE uint Material_GetExteriorVolume(const uint matIndex,\n"
"		__global const HitPoint *hitPoint,\n"
"		const float passThroughEvent\n"
"		MATERIALS_PARAM_DECL) {\n"
"#if defined(DEBUG_PRINTF_MATERIAL_EVAL)\n"
"	printf(\"===============================================================\\n\");\n"
"	printf(\"Material_GetExteriorVolume()\\n\");\n"
"	printf(\"===============================================================\\n\");\n"
"#endif\n"
"\n"
"	__global const Material* restrict startMat = &mats[matIndex];\n"
"	const size_t gid = get_global_id(0);\n"
"	__global float *evalStack = &matEvalStacks[gid * maxMaterialEvalStackSize];\n"
"\n"
"	const uint evalOpStartIndex = startMat->evalGetExteriorVolumeOpStartIndex;\n"
"	const uint evalOpLength = startMat->evalGetExteriorVolumeOpLength;\n"
"\n"
"#if defined(DEBUG_PRINTF_MATERIAL_EVAL)\n"
"	printf(\"matIndex=%d evalOpStartIndex=%d evalOpLength=%d\\n\", matIndex, evalOpStartIndex, evalOpLength);\n"
"#endif\n"
"\n"
"	uint evalStackOffsetVal = 0;\n"
"	uint *evalStackOffset = &evalStackOffsetVal; // Used by macros\n"
"	// Evaluation parameters\n"
"	EvalStack_PushFloat(passThroughEvent);\n"
"\n"
"	for (uint i = 0; i < evalOpLength; ++i) {\n"
"#if defined(DEBUG_PRINTF_MATERIAL_EVAL)\n"
"		printf(\"EvalOp: #%d evalStackOffset=%d\\n\", i, evalStackOffsetVal);\n"
"#endif\n"
"\n"
"		__global const MaterialEvalOp* restrict evalOp = &matEvalOps[evalOpStartIndex + i];\n"
"\n"
"		i += Material_EvalOp(evalOp, evalStack, &evalStackOffsetVal, hitPoint MATERIALS_PARAM);\n"
"	}\n"
"#if defined(DEBUG_PRINTF_MATERIAL_EVAL)\n"
"	printf(\"evalStackOffset=#%d\\n\", evalStackOffsetVal);\n"
"#endif\n"
"\n"
"	uint result;\n"
"	EvalStack_PopUInt(result);\n"
"\n"
"#if defined(DEBUG_PRINTF_MATERIAL_EVAL)\n"
"	printf(\"Result=(%d)\\n\", result);\n"
"#endif\n"
"\n"
"	return result;\n"
"}\n";
} }
