
#include <string>

namespace slg { namespace ocl {
std::string KernelSource_materialdefs_funcs_carpaint =
"#line 2 \"materialdefs_funcs_carpaint.cl\"\n"
"\n"
"/***************************************************************************\n"
" * Copyright 1998-2020 by authors (see AUTHORS.txt)                        *\n"
" *                                                                         *\n"
" *   This file is part of LuxCoreRender.                                   *\n"
" *                                                                         *\n"
" * Licensed under the Apache License, Version 2.0 (the \"License\");         *\n"
" * you may not use this file except in compliance with the License.        *\n"
" * You may obtain a copy of the License at                                 *\n"
" *                                                                         *\n"
" *     http://www.apache.org/licenses/LICENSE-2.0                          *\n"
" *                                                                         *\n"
" * Unless required by applicable law or agreed to in writing, software     *\n"
" * distributed under the License is distributed on an \"AS IS\" BASIS,       *\n"
" * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.*\n"
" * See the License for the specific language governing permissions and     *\n"
" * limitations under the License.                                          *\n"
" ***************************************************************************/\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// CarPaint material\n"
"//\n"
"// LuxRender carpaint material porting.\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE void CarPaintMaterial_Albedo(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"    const float3 albedo = Spectrum_Clamp(Texture_GetSpectrumValue(material->carpaint.KdTexIndex, hitPoint TEXTURES_PARAM));\n"
"\n"
"	EvalStack_PushFloat3(albedo);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void CarPaintMaterial_GetInteriorVolume(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	DefaultMaterial_GetInteriorVolume(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void CarPaintMaterial_GetExteriorVolume(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	DefaultMaterial_GetExteriorVolume(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void CarPaintMaterial_GetPassThroughTransparency(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	DefaultMaterial_GetPassThroughTransparency(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void CarPaintMaterial_GetEmittedRadiance(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	DefaultMaterial_GetEmittedRadiance(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void CarPaintMaterial_Evaluate(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	float3 lightDir, eyeDir;\n"
"	EvalStack_PopFloat3(eyeDir);\n"
"	EvalStack_PopFloat3(lightDir);\n"
"\n"
"	float3 H = normalize(lightDir + eyeDir);\n"
"	if ((H.x == 0.f) && (H.y == 0.f) && (H.z == 0.f)) {\n"
"		MATERIAL_EVALUATE_RETURN_BLACK;\n"
"	}\n"
"	if (H.z < 0.f)\n"
"		H = -H;\n"
"\n"
"	float pdf = 0.f;\n"
"	int n = 1; // already counts the diffuse layer\n"
"\n"
"	// Absorption\n"
"	const float cosi = fabs(lightDir.z);\n"
"	const float coso = fabs(eyeDir.z);\n"
"	const float3 kaVal = Texture_GetSpectrumValue(material->carpaint.KaTexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float3 alpha = Spectrum_Clamp(kaVal);\n"
"	const float d = Texture_GetFloatValue(material->carpaint.depthTexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float3 absorption = CoatingAbsorption(cosi, coso, alpha, d);\n"
"\n"
"	// Diffuse layer\n"
"	const float3 kdVal = Texture_GetSpectrumValue(material->carpaint.KdTexIndex, hitPoint TEXTURES_PARAM);\n"
"	float3 result = absorption * Spectrum_Clamp(kdVal) * M_1_PI_F * fabs(lightDir.z);\n"
"\n"
"	// 1st glossy layer\n"
"	const float3 ks1Val = Texture_GetSpectrumValue(material->carpaint.Ks1TexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float3 ks1 = Spectrum_Clamp(ks1Val);\n"
"	const float m1 = Texture_GetFloatValue(material->carpaint.M1TexIndex, hitPoint TEXTURES_PARAM);\n"
"	if (Spectrum_Filter(ks1) > 0.f && m1 > 0.f)\n"
"	{\n"
"		const float rough1 = m1 * m1;\n"
"		const float r1 = Texture_GetFloatValue(material->carpaint.R1TexIndex, hitPoint TEXTURES_PARAM);\n"
"		result += (SchlickDistribution_D(rough1, H, 0.f) * SchlickDistribution_G(rough1, lightDir, eyeDir) / (4.f * coso)) * (ks1 * FresnelSchlick_Evaluate(TO_FLOAT3(r1), dot(eyeDir, H)));\n"
"		pdf += SchlickDistribution_Pdf(rough1, H, 0.f);\n"
"		++n;\n"
"	}\n"
"	const float3 ks2Val = Texture_GetSpectrumValue(material->carpaint.Ks2TexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float3 ks2 = Spectrum_Clamp(ks2Val);\n"
"	const float m2 = Texture_GetFloatValue(material->carpaint.M2TexIndex, hitPoint TEXTURES_PARAM);\n"
"	if (Spectrum_Filter(ks2) > 0.f && m2 > 0.f)\n"
"	{\n"
"		const float rough2 = m2 * m2;\n"
"		const float r2 = Texture_GetFloatValue(material->carpaint.R2TexIndex, hitPoint TEXTURES_PARAM);\n"
"		result += (SchlickDistribution_D(rough2, H, 0.f) * SchlickDistribution_G(rough2, lightDir, eyeDir) / (4.f * coso)) * (ks2 * FresnelSchlick_Evaluate(TO_FLOAT3(r2), dot(eyeDir, H)));\n"
"		pdf += SchlickDistribution_Pdf(rough2, H, 0.f);\n"
"		++n;\n"
"	}\n"
"	const float3 ks3Val = Texture_GetSpectrumValue(material->carpaint.Ks3TexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float3 ks3 = Spectrum_Clamp(ks3Val);\n"
"	const float m3 = Texture_GetFloatValue(material->carpaint.M3TexIndex, hitPoint TEXTURES_PARAM);\n"
"	if (Spectrum_Filter(ks3) > 0.f && m3 > 0.f)\n"
"	{\n"
"		const float rough3 = m3 * m3;\n"
"		const float r3 = Texture_GetFloatValue(material->carpaint.R3TexIndex, hitPoint TEXTURES_PARAM);\n"
"		result += (SchlickDistribution_D(rough3, H, 0.f) * SchlickDistribution_G(rough3, lightDir, eyeDir) / (4.f * coso)) * (ks3 * FresnelSchlick_Evaluate(TO_FLOAT3(r3), dot(eyeDir, H)));\n"
"		pdf += SchlickDistribution_Pdf(rough3, H, 0.f);\n"
"		++n;\n"
"	}\n"
"\n"
"	// Front face: coating+base\n"
"	const BSDFEvent event = GLOSSY | REFLECT;\n"
"\n"
"	// Finish pdf computation\n"
"	pdf /= 4.f * fabs(dot(lightDir, H));\n"
"	const float directPdfW = (pdf + fabs(lightDir.z) * M_1_PI_F) / n;\n"
"\n"
"	EvalStack_PushFloat3(result);\n"
"	EvalStack_PushBSDFEvent(event);\n"
"	EvalStack_PushFloat(directPdfW);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void CarPaintMaterial_Sample(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	float u0, u1, passThroughEvent;\n"
"	EvalStack_PopFloat(passThroughEvent);\n"
"	EvalStack_PopFloat(u1);\n"
"	EvalStack_PopFloat(u0);\n"
"	float3 fixedDir;\n"
"	EvalStack_PopFloat3(fixedDir);\n"
"\n"
"	if (fabs(fixedDir.z) < DEFAULT_COS_EPSILON_STATIC) {\n"
"		MATERIAL_SAMPLE_RETURN_BLACK;\n"
"	}\n"
"\n"
"	const float3 kaVal = Texture_GetSpectrumValue(material->carpaint.KaTexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float d = Texture_GetFloatValue(material->carpaint.depthTexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float3 kdVal = Texture_GetSpectrumValue(material->carpaint.KdTexIndex, hitPoint TEXTURES_PARAM);\n"
"\n"
"	const float m1 = Texture_GetFloatValue(material->carpaint.M1TexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float m2 = Texture_GetFloatValue(material->carpaint.M2TexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float m3 = Texture_GetFloatValue(material->carpaint.M3TexIndex, hitPoint TEXTURES_PARAM);\n"
"\n"
"	const float r1 = Texture_GetFloatValue(material->carpaint.R1TexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float r2 = Texture_GetFloatValue(material->carpaint.R2TexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float r3 = Texture_GetFloatValue(material->carpaint.R3TexIndex, hitPoint TEXTURES_PARAM);\n"
"\n"
"	// Test presence of components\n"
"	int n = 1; // already count the diffuse layer\n"
"	int sampled = 0; // sampled layer\n"
"	float3 result = BLACK;\n"
"	float pdf = 0.f;\n"
"	bool l1 = false, l2 = false, l3 = false;\n"
"	// 1st glossy layer\n"
"	const float3 ks1Val = Texture_GetSpectrumValue(material->carpaint.Ks1TexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float3 ks1 = Spectrum_Clamp(ks1Val);\n"
"	if (Spectrum_Filter(ks1) > 0.f && m1 > 0.f)\n"
"	{\n"
"		l1 = true;\n"
"		++n;\n"
"	}\n"
"	// 2nd glossy layer\n"
"	const float3 ks2Val = Texture_GetSpectrumValue(material->carpaint.Ks2TexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float3 ks2 = Spectrum_Clamp(ks2Val);\n"
"	if (Spectrum_Filter(ks2) > 0.f && m2 > 0.f)\n"
"	{\n"
"		l2 = true;\n"
"		++n;\n"
"	}\n"
"	// 3rd glossy layer\n"
"	const float3 ks3Val = Texture_GetSpectrumValue(material->carpaint.Ks3TexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float3 ks3 = Spectrum_Clamp(ks3Val);\n"
"	if (Spectrum_Filter(ks3) > 0.f && m3 > 0.f)\n"
"	{\n"
"		l3 = true;\n"
"		++n;\n"
"	}\n"
"\n"
"	float3 sampledDir, wh;\n"
"	float cosWH;\n"
"	if (passThroughEvent < 1.f / n) {\n"
"		// Sample diffuse layer\n"
"		sampledDir = (signbit(fixedDir.z) ? -1.f : 1.f) * CosineSampleHemisphereWithPdf(u0, u1, &pdf);\n"
"\n"
"		if (fabs(CosTheta(sampledDir)) < DEFAULT_COS_EPSILON_STATIC) {\n"
"			MATERIAL_SAMPLE_RETURN_BLACK;\n"
"		}\n"
"\n"
"		// Absorption\n"
"		const float cosi = fabs(fixedDir.z);\n"
"		const float coso = fabs(sampledDir.z);\n"
"		const float3 alpha = Spectrum_Clamp(kaVal);\n"
"		const float3 absorption = CoatingAbsorption(cosi, coso, alpha, d);\n"
"\n"
"		// Evaluate base BSDF\n"
"		result = absorption * Spectrum_Clamp(kdVal) * pdf;\n"
"\n"
"		wh = normalize(sampledDir + fixedDir);\n"
"		if (wh.z < 0.f)\n"
"			wh = -wh;\n"
"		cosWH = fabs(dot(fixedDir, wh));\n"
"	} else if (passThroughEvent < 2.f / n && l1) {\n"
"		// Sample 1st glossy layer\n"
"		sampled = 1;\n"
"		const float rough1 = m1 * m1;\n"
"		float d;\n"
"		SchlickDistribution_SampleH(rough1, 0.f, u0, u1, &wh, &d, &pdf);\n"
"		cosWH = dot(fixedDir, wh);\n"
"		sampledDir = 2.f * cosWH * wh - fixedDir;\n"
"		cosWH = fabs(cosWH);\n"
"\n"
"		if ((sampledDir.z < DEFAULT_COS_EPSILON_STATIC) ||\n"
"			(fixedDir.z * sampledDir.z < 0.f)) {\n"
"			MATERIAL_SAMPLE_RETURN_BLACK;\n"
"		}\n"
"\n"
"		pdf /= 4.f * cosWH;\n"
"		if (pdf <= 0.f) {\n"
"			MATERIAL_SAMPLE_RETURN_BLACK;\n"
"		}\n"
"\n"
"		result = ks1 * FresnelSchlick_Evaluate(TO_FLOAT3(r1), cosWH);\n"
"\n"
"		const float G = SchlickDistribution_G(rough1, fixedDir, sampledDir);\n"
"		result *= d * G / (4.f * fabs(fixedDir.z));\n"
"	} else if ((passThroughEvent < 2.f / n  ||\n"
"		(!l1 && passThroughEvent < 3.f / n)) && l2) {\n"
"		// Sample 2nd glossy layer\n"
"		sampled = 2;\n"
"		const float rough2 = m2 * m2;\n"
"		float d;\n"
"		SchlickDistribution_SampleH(rough2, 0.f, u0, u1, &wh, &d, &pdf);\n"
"		cosWH = dot(fixedDir, wh);\n"
"		sampledDir = 2.f * cosWH * wh - fixedDir;\n"
"		cosWH = fabs(cosWH);\n"
"\n"
"		if ((sampledDir.z < DEFAULT_COS_EPSILON_STATIC) ||\n"
"			(fixedDir.z * sampledDir.z < 0.f)) {\n"
"			MATERIAL_SAMPLE_RETURN_BLACK;\n"
"		}\n"
"\n"
"		pdf /= 4.f * cosWH;\n"
"		if (pdf <= 0.f) {\n"
"			MATERIAL_SAMPLE_RETURN_BLACK;\n"
"		}\n"
"\n"
"		result = ks2 * FresnelSchlick_Evaluate(TO_FLOAT3(r2), cosWH);\n"
"\n"
"		const float G = SchlickDistribution_G(rough2, fixedDir, sampledDir);\n"
"		result *= d * G / (4.f * fabs(fixedDir.z));\n"
"	} else if (l3) {\n"
"		// Sample 3rd glossy layer\n"
"		sampled = 3;\n"
"		const float rough3 = m3 * m3;\n"
"		float d;\n"
"		SchlickDistribution_SampleH(rough3, 0.f, u0, u1, &wh, &d, &pdf);\n"
"		cosWH = dot(fixedDir, wh);\n"
"		sampledDir = 2.f * cosWH * wh - fixedDir;\n"
"		cosWH = fabs(cosWH);\n"
"\n"
"		if ((sampledDir.z < DEFAULT_COS_EPSILON_STATIC) ||\n"
"			(fixedDir.z * sampledDir.z < 0.f)) {\n"
"			MATERIAL_SAMPLE_RETURN_BLACK;\n"
"		}\n"
"\n"
"		pdf /= 4.f * cosWH;\n"
"		if (pdf <= 0.f) {\n"
"			MATERIAL_SAMPLE_RETURN_BLACK;\n"
"		}\n"
"\n"
"		result = ks3 * FresnelSchlick_Evaluate(TO_FLOAT3(r3), cosWH);\n"
"\n"
"		const float G = SchlickDistribution_G(rough3, fixedDir, sampledDir);\n"
"		result *= d * G / (4.f * fabs(fixedDir.z));\n"
"	} else {\n"
"		// Sampling issue\n"
" 		MATERIAL_SAMPLE_RETURN_BLACK;\n"
"	}\n"
"\n"
"	const BSDFEvent event = GLOSSY | REFLECT;\n"
"\n"
"	// Add other components\n"
"	// Diffuse\n"
"	if (sampled != 0) {\n"
"		// Absorption\n"
"		const float cosi = fabs(fixedDir.z);\n"
"		const float coso = fabs(sampledDir.z);\n"
"		const float3 alpha = Spectrum_Clamp(kaVal);\n"
"		const float3 absorption = CoatingAbsorption(cosi, coso, alpha, d);\n"
"\n"
"		const float pdf0 = fabs(sampledDir.z) * M_1_PI_F;\n"
"		pdf += pdf0;\n"
"		result += absorption * Spectrum_Clamp(kdVal) * pdf0;\n"
"	}\n"
"	// 1st glossy\n"
"	if (l1 && sampled != 1) {\n"
"		const float rough1 = m1 * m1;\n"
"		const float d1 = SchlickDistribution_D(rough1, wh, 0.f);\n"
"		const float pdf1 = SchlickDistribution_Pdf(rough1, wh, 0.f) / (4.f * cosWH);\n"
"		if (pdf1 > 0.f) {\n"
"			result += ks1 * (d1 *\n"
"				SchlickDistribution_G(rough1, fixedDir, sampledDir) /\n"
"				(4.f * fabs(fixedDir.z))) *\n"
"				FresnelSchlick_Evaluate(TO_FLOAT3(r1), cosWH);\n"
"			pdf += pdf1;\n"
"		}\n"
"	}\n"
"	// 2nd glossy\n"
"	if (l2 && sampled != 2) {\n"
"		const float rough2 = m2 * m2;\n"
"		const float d2 = SchlickDistribution_D(rough2, wh, 0.f);\n"
"		const float pdf2 = SchlickDistribution_Pdf(rough2, wh, 0.f) / (4.f * cosWH);\n"
"		if (pdf2 > 0.f) {\n"
"			result += ks2 * (d2 *\n"
"				SchlickDistribution_G(rough2, fixedDir, sampledDir) /\n"
"				(4.f * fabs(fixedDir.z))) *\n"
"				FresnelSchlick_Evaluate(TO_FLOAT3(r2), cosWH);\n"
"			pdf += pdf2;\n"
"		}\n"
"	}\n"
"	// 3rd glossy\n"
"	if (l3 && sampled != 3) {\n"
"		const float rough3 = m3 * m3;\n"
"		const float d3 = SchlickDistribution_D(rough3, wh, 0.f);\n"
"		const float pdf3 = SchlickDistribution_Pdf(rough3, wh, 0.f) / (4.f * cosWH);\n"
"		if (pdf3 > 0.f) {\n"
"			result += ks3 * (d3 *\n"
"				SchlickDistribution_G(rough3, fixedDir, sampledDir) /\n"
"				(4.f * fabs(fixedDir.z))) *\n"
"				FresnelSchlick_Evaluate(TO_FLOAT3(r3), cosWH);\n"
"			pdf += pdf3;\n"
"		}\n"
"	}\n"
"	// Adjust pdf and result\n"
"	const float pdfW = pdf / n;\n"
"	result = result / pdfW;\n"
"\n"
"	EvalStack_PushFloat3(result);\n"
"	EvalStack_PushFloat3(sampledDir);\n"
"	EvalStack_PushFloat(pdfW);\n"
"	EvalStack_PushBSDFEvent(event);\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Material specific EvalOp\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_NOT_INLINE void CarPaintMaterial_EvalOp(\n"
"		__global const Material* restrict material,\n"
"		const MaterialEvalOpType evalType,\n"
"		__global float *evalStack,\n"
"		uint *evalStackOffset,\n"
"		__global const HitPoint *hitPoint\n"
"		MATERIALS_PARAM_DECL) {\n"
"	switch (evalType) {\n"
"		case EVAL_ALBEDO:\n"
"			CarPaintMaterial_Albedo(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_GET_INTERIOR_VOLUME:\n"
"			CarPaintMaterial_GetInteriorVolume(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_GET_EXTERIOR_VOLUME:\n"
"			CarPaintMaterial_GetExteriorVolume(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_GET_EMITTED_RADIANCE:\n"
"			CarPaintMaterial_GetEmittedRadiance(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_GET_PASS_TROUGH_TRANSPARENCY:\n"
"			CarPaintMaterial_GetPassThroughTransparency(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_EVALUATE:\n"
"			CarPaintMaterial_Evaluate(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_SAMPLE:\n"
"			CarPaintMaterial_Sample(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		default:\n"
"			// Something wrong here\n"
"			break;\n"
"	}\n"
"}\n";
} }
