
#include <string>

namespace slg { namespace ocl {
std::string KernelSource_materialdefs_funcs_glossycoating =
"#line 2 \"materialdefs_funcs_glossycoating.cl\"\n"
"\n"
"/***************************************************************************\n"
" * Copyright 1998-2020 by authors (see AUTHORS.txt)                        *\n"
" *                                                                         *\n"
" *   This file is part of LuxCoreRender.                                   *\n"
" *                                                                         *\n"
" * Licensed under the Apache License, Version 2.0 (the \"License\");         *\n"
" * you may not use this file except in compliance with the License.        *\n"
" * You may obtain a copy of the License at                                 *\n"
" *                                                                         *\n"
" *     http://www.apache.org/licenses/LICENSE-2.0                          *\n"
" *                                                                         *\n"
" * Unless required by applicable law or agreed to in writing, software     *\n"
" * distributed under the License is distributed on an \"AS IS\" BASIS,       *\n"
" * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.*\n"
" * See the License for the specific language governing permissions and     *\n"
" * limitations under the License.                                          *\n"
" ***************************************************************************/\n"
"\n"
"OPENCL_FORCE_INLINE void GlossyCoatingMaterial_Albedo(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	// Albedo value is already on the stack so I have nothing to do (Pop/Push the same value would be a waste of time)\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void GlossyCoatingMaterial_GetInteriorVolume(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	const uint volIndex = material->interiorVolumeIndex;\n"
"	if (volIndex != NULL_INDEX) {\n"
"		float passThroughEvent;\n"
"		EvalStack_PopFloat(passThroughEvent);\n"
"\n"
"		EvalStack_PushUInt(volIndex);\n"
"	}\n"
"	// Else nothing to do because there is already the matBase volume\n"
"	// index on the stack\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void GlossyCoatingMaterial_GetExteriorVolume(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	const uint volIndex = material->exteriorVolumeIndex;\n"
"	if (volIndex != NULL_INDEX) {\n"
"		float passThroughEvent;\n"
"		EvalStack_PopFloat(passThroughEvent);\n"
"\n"
"		EvalStack_PushUInt(volIndex);\n"
"	}\n"
"	// Else nothing to do because there is already the matBase volume\n"
"	// index on the stack\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void GlossyCoatingMaterial_GetPassThroughTransparency(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	// Nothing to do there is already the matBase pass trough\n"
"	// transparency on the stack\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void GlossyCoatingMaterial_GetEmittedRadiance(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	if (material->emitTexIndex != NULL_INDEX)\n"
"		DefaultMaterial_GetEmittedRadiance(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"	// Else nothing to do because there is already the matBase emitted\n"
"	// radiance on the stack	\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Material evaluate\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE void GlossyCoatingMaterial_EvaluateSetUp(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	float3 lightDir, eyeDir;\n"
"	EvalStack_PopFloat3(eyeDir);\n"
"	EvalStack_PopFloat3(lightDir);\n"
"\n"
"	// Save current shading normal/dpdu/dpdv and setup the hitPoint with base material bump mapping\n"
"	const float3 originalShadeN = VLOAD3F(&hitPoint->shadeN.x);\n"
"	const float3 originalDpdu = VLOAD3F(&hitPoint->dpdu.x);\n"
"	const float3 originalDpdv = VLOAD3F(&hitPoint->dpdv.x);\n"
"	EvalStack_PushFloat3(originalShadeN);\n"
"	EvalStack_PushFloat3(originalDpdu);\n"
"	EvalStack_PushFloat3(originalDpdv);\n"
"\n"
"	__global HitPoint *hitPointTmp = (__global HitPoint *)hitPoint;\n"
"	Material_Bump(material->glossycoating.matBaseIndex, hitPointTmp\n"
"		MATERIALS_PARAM);\n"
"	// Re-read the shadeN modified by Material_Bump()\n"
"	const float3 matBaseShadeN = VLOAD3F(&hitPointTmp->shadeN.x);\n"
"	const float3 matBaseDpdu = VLOAD3F(&hitPointTmp->dpdu.x);\n"
"	const float3 matBaseDpdv = VLOAD3F(&hitPointTmp->dpdv.x);\n"
"\n"
"	// Save the parameters\n"
"	EvalStack_PushFloat3(lightDir);\n"
"	EvalStack_PushFloat3(eyeDir);\n"
"\n"
"	// Transform lightDir and eyeDir to base material new reference system\n"
"	Frame frameBase;\n"
"	Frame_Set_Private(&frameBase, matBaseDpdu, matBaseDpdv, matBaseShadeN);\n"
"	Frame frameOriginal;\n"
"	Frame_Set_Private(&frameOriginal, originalDpdu, originalDpdv, originalShadeN);\n"
"\n"
"	const float3 matBaseLightDir = Frame_ToLocal_Private(&frameBase, Frame_ToWorld_Private(&frameOriginal, lightDir));\n"
"	const float3 matBaseEyeDir = Frame_ToLocal_Private(&frameBase, Frame_ToWorld_Private(&frameOriginal, eyeDir));\n"
"	\n"
"	// To setup the following EVAL_EVALUATE evaluation of base material\n"
"	EvalStack_PushFloat3(matBaseLightDir);\n"
"	EvalStack_PushFloat3(matBaseEyeDir);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void GlossyCoatingMaterial_Evaluate(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	// Pop the result of the base material evaluation\n"
"	float directPdfWMatBase;\n"
"	BSDFEvent eventMatBase;\n"
"	float3 resultMatBase;\n"
"	EvalStack_PopFloat(directPdfWMatBase);\n"
"	EvalStack_PopBSDFEvent(eventMatBase);\n"
"	EvalStack_PopFloat3(resultMatBase);\n"
"\n"
"	// Pop the parameters\n"
"	float3 lightDir, eyeDir;\n"
"	EvalStack_PopFloat3(eyeDir);\n"
"	EvalStack_PopFloat3(lightDir);\n"
"	\n"
"	// Pop saved original shading normal/dpdu/dpdv\n"
"	float3 originalShadeN, originalDpdu, originalDpdv;\n"
"	EvalStack_PopFloat3(originalDpdv);\n"
"	EvalStack_PopFloat3(originalDpdu);\n"
"	EvalStack_PopFloat3(originalShadeN);\n"
"	// Restore original hitPoint\n"
"	__global HitPoint *hitPointTmp = (__global HitPoint *)hitPoint;\n"
"	VSTORE3F(originalShadeN, &hitPointTmp->shadeN.x);\n"
"	VSTORE3F(originalDpdu, &hitPointTmp->dpdu.x);\n"
"	VSTORE3F(originalDpdv, &hitPointTmp->dpdv.x);\n"
"\n"
"	const float3 fixedDir = eyeDir;\n"
"	const float3 sampledDir = lightDir;\n"
"\n"
"	// Note: this is the same side test used by matte translucent material and\n"
"	// it is different from the CPU test because HitPoint::dpdu and HitPoint::dpdv\n"
"	// are not available here without bump mapping.\n"
"	const float sideTest = CosTheta(lightDir) * CosTheta(eyeDir);\n"
"\n"
"	if (sideTest > DEFAULT_COS_EPSILON_STATIC) {\n"
"		// Reflection\n"
"\n"
"		// Back face: no coating\n"
"		if (eyeDir.z <= 0.f) {\n"
"			EvalStack_PushFloat3(resultMatBase);\n"
"			EvalStack_PushBSDFEvent(eventMatBase);\n"
"			EvalStack_PushFloat(directPdfWMatBase);\n"
"\n"
"			return;\n"
"		}\n"
"\n"
"		const float3 baseF = resultMatBase;\n"
"		BSDFEvent event = eventMatBase;\n"
"		float directPdfW = directPdfWMatBase;\n"
"\n"
"		// I have always to initialized baseF pdf and event because are used below\n"
"		if (Spectrum_IsBlack(baseF)) {\n"
"			directPdfW = 0.f;\n"
"			event = NONE;\n"
"		}\n"
"\n"
"		// Front face: coating+base\n"
"		event |= GLOSSY | REFLECT;\n"
"\n"
"		float3 ks = Texture_GetSpectrumValue(material->glossycoating.ksTexIndex, hitPoint TEXTURES_PARAM);\n"
"		const float i = Texture_GetFloatValue(material->glossycoating.indexTexIndex, hitPoint TEXTURES_PARAM);\n"
"		if (i > 0.f) {\n"
"			const float ti = (i - 1.f) / (i + 1.f);\n"
"			ks *= ti * ti;\n"
"		}\n"
"		ks = Spectrum_Clamp(ks);\n"
"\n"
"		const float nuVal = Texture_GetFloatValue(material->glossycoating.nuTexIndex, hitPoint TEXTURES_PARAM);\n"
"		const float nvVal = Texture_GetFloatValue(material->glossycoating.nvTexIndex, hitPoint TEXTURES_PARAM);\n"
"		const float u = clamp(nuVal, 1e-9f, 1.f);\n"
"		const float v = clamp(nvVal, 1e-9f, 1.f);\n"
"		const float u2 = u * u;\n"
"		const float v2 = v * v;\n"
"		const float anisotropy = (u2 < v2) ? (1.f - u2 / v2) : u2 > 0.f ? (v2 / u2 - 1.f) : 0.f;\n"
"		const float roughness = u * v;\n"
"\n"
"		const float wCoating = SchlickBSDF_CoatingWeight(ks, fixedDir);\n"
"		const float wBase = 1.f - wCoating;\n"
"\n"
"		directPdfW = wBase * directPdfW +\n"
"				wCoating * SchlickBSDF_CoatingPdf(roughness, anisotropy, fixedDir, sampledDir);\n"
"\n"
"		// Absorption\n"
"		const float cosi = fabs(sampledDir.z);\n"
"		const float coso = fabs(fixedDir.z);\n"
"\n"
"		const float3 alpha = Spectrum_Clamp(Texture_GetSpectrumValue(material->glossycoating.kaTexIndex, hitPoint TEXTURES_PARAM));\n"
"		const float d = Texture_GetFloatValue(material->glossycoating.depthTexIndex, hitPoint TEXTURES_PARAM);\n"
"		const float3 absorption = CoatingAbsorption(cosi, coso, alpha, d);\n"
"\n"
"		// Coating fresnel factor\n"
"		const float3 H = normalize(fixedDir + sampledDir);\n"
"		const float3 S = FresnelSchlick_Evaluate(ks, fabs(dot(sampledDir, H)));\n"
"\n"
"		const float3 coatingF = SchlickBSDF_CoatingF(ks, roughness, anisotropy,\n"
"				material->glossycoating.multibounce, fixedDir, sampledDir);\n"
"\n"
"		// Blend in base layer Schlick style\n"
"		// assumes coating bxdf takes fresnel factor S into account\n"
"\n"
"		const float3 result = coatingF + absorption * (WHITE - S) * baseF;\n"
"		\n"
"		EvalStack_PushFloat3(result);\n"
"		EvalStack_PushBSDFEvent(event);\n"
"		EvalStack_PushFloat(directPdfW);\n"
"\n"
"		return;\n"
"	} else if (sideTest < -DEFAULT_COS_EPSILON_STATIC) {\n"
"		// Transmission\n"
"\n"
"		const float3 baseF = resultMatBase;\n"
"		BSDFEvent event = eventMatBase;\n"
"		float directPdfW = directPdfWMatBase;\n"
"\n"
"		// I have always to initialized baseF pdf because it is used below\n"
"		if (Spectrum_IsBlack(baseF)) {\n"
"			directPdfW = 0.f;\n"
"			event = NONE;\n"
"		}\n"
"\n"
"		event |= GLOSSY | TRANSMIT;\n"
"\n"
"		float3 ks = Texture_GetSpectrumValue(material->glossycoating.ksTexIndex, hitPoint TEXTURES_PARAM);\n"
"		const float i = Texture_GetFloatValue(material->glossycoating.indexTexIndex, hitPoint TEXTURES_PARAM);\n"
"		if (i > 0.f) {\n"
"			const float ti = (i - 1.f) / (i + 1.f);\n"
"			ks *= ti * ti;\n"
"		}\n"
"		ks = Spectrum_Clamp(ks);\n"
"\n"
"		const float3 fixedDir = eyeDir;\n"
"		const float wCoating = (fixedDir.z > DEFAULT_COS_EPSILON_STATIC) ? SchlickBSDF_CoatingWeight(ks, fixedDir) : 0.f;\n"
"		const float wBase = 1.f - wCoating;\n"
"\n"
"		directPdfW *= wBase;\n"
"\n"
"		// Absorption\n"
"		const float cosi = fabs(sampledDir.z);\n"
"		const float coso = fabs(fixedDir.z);\n"
"\n"
"		const float3 alpha = Spectrum_Clamp(Texture_GetSpectrumValue(material->glossycoating.kaTexIndex, hitPoint TEXTURES_PARAM));\n"
"		const float d = Texture_GetFloatValue(material->glossycoating.depthTexIndex, hitPoint TEXTURES_PARAM);\n"
"		const float3 absorption = CoatingAbsorption(cosi, coso, alpha, d);\n"
"\n"
"		// Coating fresnel factor\n"
"		const float3 H = normalize(MAKE_FLOAT3 (sampledDir.x + fixedDir.x, sampledDir.y + fixedDir.y,\n"
"				sampledDir.z - fixedDir.z));\n"
"		const float3 S = FresnelSchlick_Evaluate(ks, fabs(dot(fixedDir, H)));\n"
"\n"
"		// Filter base layer, the square root is just a heuristic\n"
"		// so that a sheet coated on both faces gets a filtering factor\n"
"		// of 1-S like a reflection\n"
"		const float3 result = absorption * Spectrum_Sqrt(WHITE - S) * baseF;\n"
"		\n"
"		EvalStack_PushFloat3(result);\n"
"		EvalStack_PushBSDFEvent(event);\n"
"		EvalStack_PushFloat(directPdfW);\n"
"\n"
"		return;\n"
"	} else {\n"
"		MATERIAL_EVALUATE_RETURN_BLACK;\n"
"	}\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Material sample\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE void GlossyCoatingMaterial_SampleSetUp(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	float u0, u1, passThroughEvent;\n"
"	EvalStack_PopFloat(passThroughEvent);\n"
"	EvalStack_PopFloat(u1);\n"
"	EvalStack_PopFloat(u0);\n"
"	float3 fixedDir;\n"
"	EvalStack_PopFloat3(fixedDir);\n"
"\n"
"	// Save the parameters\n"
"	EvalStack_PushFloat3(fixedDir);\n"
"	EvalStack_PushFloat(u0);\n"
"	EvalStack_PushFloat(u1);\n"
"	EvalStack_PushFloat(passThroughEvent);\n"
"\n"
"	float3 ks = Texture_GetSpectrumValue(material->glossycoating.ksTexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float i = Texture_GetFloatValue(material->glossycoating.indexTexIndex, hitPoint TEXTURES_PARAM);\n"
"	if (i > 0.f) {\n"
"		const float ti = (i - 1.f) / (i + 1.f);\n"
"		ks *= ti * ti;\n"
"	}\n"
"	ks = Spectrum_Clamp(ks);\n"
"\n"
"	// Save ks\n"
"	EvalStack_PushFloat3(ks);\n"
"\n"
"	const float nuVal = Texture_GetFloatValue(material->glossycoating.nuTexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float nvVal = Texture_GetFloatValue(material->glossycoating.nvTexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float u = clamp(nuVal, 1e-9f, 1.f);\n"
"	const float v = clamp(nvVal, 1e-9f, 1.f);\n"
"	const float u2 = u * u;\n"
"	const float v2 = v * v;\n"
"	const float anisotropy = (u2 < v2) ? (1.f - u2 / v2) : u2 > 0.f ? (v2 / u2 - 1.f) : 0.f;\n"
"	const float roughness = u * v;\n"
"\n"
"	// Save roughness and anisotropy\n"
"	EvalStack_PushFloat(roughness);\n"
"	EvalStack_PushFloat(anisotropy);\n"
"\n"
"	// Coating is used only on the front face\n"
"	const float wCoating = (fixedDir.z > DEFAULT_COS_EPSILON_STATIC) ? SchlickBSDF_CoatingWeight(ks, fixedDir) : 0.f;\n"
"	const float wBase = 1.f - wCoating;\n"
"\n"
"	// Save wCoating\n"
"	EvalStack_PushFloat(wCoating);\n"
"\n"
"	// Save current shading normal/dpdu/dpdv and setup the hitPoint with base material bump mapping\n"
"	const float3 originalShadeN = VLOAD3F(&hitPoint->shadeN.x);\n"
"	const float3 originalDpdu = VLOAD3F(&hitPoint->dpdu.x);\n"
"	const float3 originalDpdv = VLOAD3F(&hitPoint->dpdv.x);\n"
"	EvalStack_PushFloat3(originalShadeN);\n"
"	EvalStack_PushFloat3(originalDpdu);\n"
"	EvalStack_PushFloat3(originalDpdv);\n"
"\n"
"	__global HitPoint *hitPointTmp = (__global HitPoint *)hitPoint;\n"
"	Material_Bump(material->glossycoating.matBaseIndex, hitPointTmp\n"
"		MATERIALS_PARAM);\n"
"	// Re-read the shadeN modified by Material_Bump()\n"
"	const float3 matBaseShadeN = VLOAD3F(&hitPoint->shadeN.x);\n"
"	const float3 matBaseDpdu = VLOAD3F(&hitPoint->dpdu.x);\n"
"	const float3 matBaseDpdv = VLOAD3F(&hitPoint->dpdv.x);\n"
"\n"
"	if (passThroughEvent < wBase) {\n"
"		// Sample base BSDF\n"
"\n"
"		// Transform fixedDir to base material new reference system\n"
"		Frame frameBase;\n"
"		Frame_Set_Private(&frameBase, matBaseDpdu, matBaseDpdv, matBaseShadeN);\n"
"		Frame frameOriginal;\n"
"		Frame_Set_Private(&frameOriginal, originalDpdu, originalDpdv, originalShadeN);\n"
"\n"
"		const float3 matBaseFixedDir = Frame_ToLocal_Private(&frameBase, Frame_ToWorld_Private(&frameOriginal, fixedDir));\n"
"\n"
"		// To setup the following EVAL_SAMPLE evaluation of base material\n"
"		EvalStack_PushFloat3(matBaseFixedDir);\n"
"		EvalStack_PushFloat(u0);\n"
"		EvalStack_PushFloat(u1);\n"
"		const float passThroughEventMatBase = passThroughEvent / wBase;\n"
"		EvalStack_PushFloat(passThroughEventMatBase);\n"
"	\n"
"		// To setup the following EVAL_CONDITIONAL_GOTO evaluation\n"
"		EvalStack_PushInt(false);\n"
"	} else {\n"
"		// Sample coating BSDF (Schlick BSDF)\n"
"		float3 sampledDir;\n"
"		float coatingPdf;\n"
"		const float3 coatingF = SchlickBSDF_CoatingSampleF(ks, roughness, anisotropy,\n"
"				material->glossycoating.multibounce, fixedDir, &sampledDir, u0, u1, &coatingPdf);\n"
"\n"
"		// Save coatingF, coatingPdf and sampledDir\n"
"		EvalStack_PushFloat3(coatingF);\n"
"		EvalStack_PushFloat(coatingPdf);\n"
"		EvalStack_PushFloat3(sampledDir);\n"
"\n"
"		// Transform lightDir and eyeDir to base material new reference system\n"
"		Frame frameBase;\n"
"		Frame_Set_Private(&frameBase, matBaseDpdu, matBaseDpdv, matBaseShadeN);\n"
"		Frame frameOriginal;\n"
"		Frame_Set_Private(&frameOriginal, originalDpdu, originalDpdv, originalShadeN);\n"
"\n"
"		const float3 matBaseFixedDir = Frame_ToLocal_Private(&frameBase, Frame_ToWorld_Private(&frameOriginal, fixedDir));\n"
"		const float3 matBaseSampleDir = Frame_ToLocal_Private(&frameBase, Frame_ToWorld_Private(&frameOriginal, sampledDir));\n"
"\n"
"		// To setup the following EVAL_EVALUATE evaluation of base material\n"
"		EvalStack_PushFloat3(matBaseSampleDir);\n"
"		EvalStack_PushFloat3(matBaseFixedDir);\n"
"\n"
"		// To setup the following EVAL_CONDITIONAL_GOTO evaluation\n"
"		EvalStack_PushInt(true);		\n"
"	}\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void GlossyCoatingMaterial_Sample(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset,\n"
"		const float wBase, const float wCoating,\n"
"		const float3 baseF, const float basePdf,\n"
"		const float3 coatingF, const float coatingPdf,\n"
"		const float3 fixedDir, const float3 sampledDir,\n"
"		const BSDFEvent event,	const float3 ks\n"
"		MATERIALS_PARAM_DECL) {\n"
"	// Absorption\n"
"	const float cosi = fabs(sampledDir.z);\n"
"	const float coso = fabs(fixedDir.z);\n"
"\n"
"	const float3 alpha = Spectrum_Clamp(Texture_GetSpectrumValue(material->glossycoating.kaTexIndex, hitPoint TEXTURES_PARAM));\n"
"	const float d = Texture_GetFloatValue(material->glossycoating.depthTexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float3 absorption = CoatingAbsorption(cosi, coso, alpha, d);\n"
"\n"
"	// Note: this is the same side test used by matte translucent material and\n"
"	// it is different from the CPU test because HitPoint::dpdu and HitPoint::dpdv\n"
"	// are not available here without bump mapping.\n"
"	const float sideTest = CosTheta(fixedDir) * CosTheta(sampledDir);\n"
"	float3 result;\n"
"	if (sideTest > DEFAULT_COS_EPSILON_STATIC) {\n"
"		// Reflection\n"
"\n"
"		if (!(fixedDir.z > 0.f)) {\n"
"			// Back face reflection: no coating\n"
"			result = baseF;\n"
"		} else {\n"
"			// Front face reflection: coating+base\n"
"\n"
"			// Coating fresnel factor\n"
"			const float3 H = normalize(fixedDir + sampledDir);\n"
"			const float3 S = FresnelSchlick_Evaluate(ks, fabs(dot(sampledDir, H)));\n"
"\n"
"			// blend in base layer Schlick style\n"
"			// coatingF already takes fresnel factor S into account\n"
"			result = (coatingF + absorption * (WHITE - S) * baseF);\n"
"		}\n"
"	} else if (sideTest < -DEFAULT_COS_EPSILON_STATIC) {\n"
"		// Transmission\n"
"		// Coating fresnel factor\n"
"		float3 H = MAKE_FLOAT3((sampledDir).x + fixedDir.x, (sampledDir).y + fixedDir.y,\n"
"			(sampledDir).z - fixedDir.z);\n"
"		const float HLength = dot(H, H);\n"
"		\n"
"		float3 S;\n"
"		// I have to handle the case when HLength is 0.0 (or nearly 0.f) in\n"
"		// order to avoid NaN\n"
"		if (HLength < DEFAULT_EPSILON_STATIC)\n"
"			S = ZERO;\n"
"		else {\n"
"			// Normalize\n"
"			H *= 1.f / HLength;\n"
"			S = FresnelSchlick_Evaluate(ks, fabs(dot(fixedDir, H)));\n"
"		}\n"
"\n"
"		// Filter base layer, the square root is just a heuristic\n"
"		// so that a sheet coated on both faces gets a filtering factor\n"
"		// of 1-S like a reflection\n"
"		result = absorption * Spectrum_Sqrt(WHITE - S) * baseF;\n"
"	} else {\n"
"		MATERIAL_SAMPLE_RETURN_BLACK;\n"
"	}\n"
"	\n"
"	const float pdfW = coatingPdf * wCoating + basePdf * wBase;\n"
"	result /= pdfW;\n"
"\n"
"	EvalStack_PushFloat3(result);\n"
"	EvalStack_PushFloat3(sampledDir);\n"
"	EvalStack_PushFloat(pdfW);\n"
"	EvalStack_PushBSDFEvent(event);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void GlossyCoatingMaterial_SampleMatBaseSample(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	// Pop the result of the base material sample\n"
"	float3 resultMatBase, sampledDirMatBase;\n"
"	float pdfWMatBase;\n"
"	BSDFEvent eventMatBase;\n"
"	EvalStack_PopBSDFEvent(eventMatBase);\n"
"	EvalStack_PopFloat(pdfWMatBase);\n"
"	EvalStack_PopFloat3(sampledDirMatBase);\n"
"	EvalStack_PopFloat3(resultMatBase);\n"
"\n"
"	// Pop saved original shading normal/dpdu/dpdv\n"
"	float3 originalShadeN, originalDpdu, originalDpdv;\n"
"	EvalStack_PopFloat3(originalDpdv);\n"
"	EvalStack_PopFloat3(originalDpdu);\n"
"	EvalStack_PopFloat3(originalShadeN);\n"
"	// Restore original hitPoint\n"
"	__global HitPoint *hitPointTmp = (__global HitPoint *)hitPoint;\n"
"	VSTORE3F(originalShadeN, &hitPointTmp->shadeN.x);\n"
"	VSTORE3F(originalDpdu, &hitPointTmp->dpdu.x);\n"
"	VSTORE3F(originalDpdv, &hitPointTmp->dpdv.x);\n"
"\n"
"	// Pop wCoating\n"
"	float wCoating;\n"
"	EvalStack_PopFloat(wCoating);\n"
"	const float wBase = 1.f - wCoating;\n"
"\n"
"	// Pop roughness and anisotropy\n"
"	float roughness, anisotropy;\n"
"	EvalStack_PopFloat(anisotropy);\n"
"	EvalStack_PopFloat(roughness);\n"
"\n"
"	// Pop ks\n"
"	float3 ks;\n"
"	EvalStack_PopFloat3(ks);\n"
"\n"
"	// Pop parameters\n"
"	float u0, u1, passThroughEvent;\n"
"	EvalStack_PopFloat(passThroughEvent);\n"
"	EvalStack_PopFloat(u1);\n"
"	EvalStack_PopFloat(u0);\n"
"	float3 fixedDir;\n"
"	EvalStack_PopFloat3(fixedDir);\n"
"\n"
"	//--------------------------------------------------------------------------\n"
"	\n"
"	if (Spectrum_IsBlack(resultMatBase)) {\n"
"		MATERIAL_SAMPLE_RETURN_BLACK;\n"
"	}\n"
"\n"
"	const float3 baseF = resultMatBase * pdfWMatBase;\n"
"	const float basePdf = pdfWMatBase;\n"
"	const float3 sampledDir = sampledDirMatBase;\n"
"	const BSDFEvent event = pdfWMatBase;\n"
"\n"
"	// Don't add the coating scattering if the base sampled\n"
"	// component is specular\n"
"	float3 coatingF;\n"
"	float coatingPdf;\n"
"	if (!(event & SPECULAR)) {\n"
"		coatingF = SchlickBSDF_CoatingF(ks, roughness, anisotropy,\n"
"				material->glossycoating.multibounce, fixedDir, sampledDir);\n"
"		coatingPdf = SchlickBSDF_CoatingPdf(roughness, anisotropy, fixedDir, sampledDir);\n"
"	} else {\n"
"		coatingF = BLACK;\n"
"		coatingPdf = 0.f;\n"
"	}\n"
"\n"
"	//--------------------------------------------------------------------------\n"
"\n"
"	GlossyCoatingMaterial_Sample(material, hitPoint, evalStack, evalStackOffset,\n"
"		wBase, wCoating,\n"
"		baseF, basePdf,\n"
"		coatingF, coatingPdf,\n"
"		fixedDir, sampledDir,\n"
"		event, ks\n"
"		MATERIALS_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void GlossyCoatingMaterial_SampleMatBaseEvaluate(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	// Pop the result of the base material evaluate\n"
"	float3 resultMatBase;\n"
"	BSDFEvent eventMatBase;\n"
"	float directPdfWMatBase;\n"
"	EvalStack_PopFloat(directPdfWMatBase);\n"
"	EvalStack_PopBSDFEvent(eventMatBase);\n"
"	EvalStack_PopFloat3(resultMatBase);\n"
"\n"
"	// Save coatingF and coatingPdf\n"
"	float3 coatingF;\n"
"	float coatingPdf;\n"
"	float3 sampledDir;\n"
"	EvalStack_PopFloat3(sampledDir);\n"
"	EvalStack_PopFloat(coatingPdf);\n"
"	EvalStack_PopFloat3(coatingF);\n"
"\n"
"	// Pop saved original shading normal/dpdu/dpdv\n"
"	float3 originalShadeN, originalDpdu, originalDpdv;\n"
"	EvalStack_PopFloat3(originalDpdv);\n"
"	EvalStack_PopFloat3(originalDpdu);\n"
"	EvalStack_PopFloat3(originalShadeN);\n"
"	// Restore original hitPoint\n"
"	__global HitPoint *hitPointTmp = (__global HitPoint *)hitPoint;\n"
"	VSTORE3F(originalShadeN, &hitPointTmp->shadeN.x);\n"
"	VSTORE3F(originalDpdu, &hitPointTmp->dpdu.x);\n"
"	VSTORE3F(originalDpdv, &hitPointTmp->dpdv.x);\n"
"\n"
"	// Pop wCoating\n"
"	float wCoating;\n"
"	EvalStack_PopFloat(wCoating);\n"
"	const float wBase = 1.f - wCoating;\n"
"\n"
"	// Pop roughness and anisotropy\n"
"	float roughness, anisotropy;\n"
"	EvalStack_PopFloat(anisotropy);\n"
"	EvalStack_PopFloat(roughness);\n"
"\n"
"	// Pop ks\n"
"	float3 ks;\n"
"	EvalStack_PopFloat3(ks);\n"
"\n"
"	// Pop parameters\n"
"	float u0, u1, passThroughEvent;\n"
"	EvalStack_PopFloat(passThroughEvent);\n"
"	EvalStack_PopFloat(u1);\n"
"	EvalStack_PopFloat(u0);\n"
"	float3 fixedDir;\n"
"	EvalStack_PopFloat3(fixedDir);\n"
"	\n"
"	//--------------------------------------------------------------------------\n"
"\n"
"	if (Spectrum_IsBlack(coatingF)) {\n"
"		MATERIAL_SAMPLE_RETURN_BLACK;\n"
"	}\n"
"\n"
"	if (fabs(CosTheta(sampledDir)) < DEFAULT_COS_EPSILON_STATIC) {\n"
"		MATERIAL_SAMPLE_RETURN_BLACK;\n"
"	}\n"
"\n"
"	coatingF *= coatingPdf;\n"
"	const float3 baseF = resultMatBase;\n"
"	float basePdf = directPdfWMatBase;\n"
"	// I have always to initialized basePdf because it is used below\n"
"	if (Spectrum_IsBlack(baseF))\n"
"		basePdf = 0.f;\n"
"\n"
"	const BSDFEvent event = GLOSSY | REFLECT;\n"
"\n"
"	//--------------------------------------------------------------------------\n"
"	\n"
"	GlossyCoatingMaterial_Sample(material, hitPoint, evalStack, evalStackOffset,\n"
"		wBase, wCoating,\n"
"		baseF, basePdf,\n"
"		coatingF, coatingPdf,\n"
"		fixedDir, sampledDir,\n"
"		event, ks\n"
"		MATERIALS_PARAM);\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Material specific EvalOp\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_NOT_INLINE void GlossyCoatingMaterial_EvalOp(\n"
"		__global const Material* restrict material,\n"
"		const MaterialEvalOpType evalType,\n"
"		__global float *evalStack,\n"
"		uint *evalStackOffset,\n"
"		__global const HitPoint *hitPoint\n"
"		MATERIALS_PARAM_DECL) {\n"
"	switch (evalType) {\n"
"		case EVAL_ALBEDO:\n"
"			GlossyCoatingMaterial_Albedo(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_GET_INTERIOR_VOLUME:\n"
"			GlossyCoatingMaterial_GetInteriorVolume(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_GET_EXTERIOR_VOLUME:\n"
"			GlossyCoatingMaterial_GetExteriorVolume(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_GET_EMITTED_RADIANCE:\n"
"			GlossyCoatingMaterial_GetEmittedRadiance(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_GET_PASS_TROUGH_TRANSPARENCY:\n"
"			GlossyCoatingMaterial_GetPassThroughTransparency(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_EVALUATE_GLOSSYCOATING_SETUP:\n"
"			GlossyCoatingMaterial_EvaluateSetUp(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_EVALUATE:\n"
"			GlossyCoatingMaterial_Evaluate(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_SAMPLE_GLOSSYCOATING_SETUP:\n"
"			GlossyCoatingMaterial_SampleSetUp(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_SAMPLE_GLOSSYCOATING_CLOSE_SAMPLE_BASE:\n"
"			GlossyCoatingMaterial_SampleMatBaseSample(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_SAMPLE_GLOSSYCOATING_CLOSE_EVALUATE_BASE:\n"
"			GlossyCoatingMaterial_SampleMatBaseEvaluate(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		default:\n"
"			// Something wrong here\n"
"			break;\n"
"	}\n"
"}\n";
} }
