
#include <string>

namespace slg { namespace ocl {
std::string KernelSource_materialdefs_funcs_heterogeneousvol =
"#line 2 \"materialdefs_funcs_heterogenousvol.cl\"\n"
"\n"
"/***************************************************************************\n"
" * Copyright 1998-2020 by authors (see AUTHORS.txt)                        *\n"
" *                                                                         *\n"
" *   This file is part of LuxCoreRender.                                   *\n"
" *                                                                         *\n"
" * Licensed under the Apache License, Version 2.0 (the \"License\");         *\n"
" * you may not use this file except in compliance with the License.        *\n"
" * You may obtain a copy of the License at                                 *\n"
" *                                                                         *\n"
" *     http://www.apache.org/licenses/LICENSE-2.0                          *\n"
" *                                                                         *\n"
" * Unless required by applicable law or agreed to in writing, software     *\n"
" * distributed under the License is distributed on an \"AS IS\" BASIS,       *\n"
" * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.*\n"
" * See the License for the specific language governing permissions and     *\n"
" * limitations under the License.                                          *\n"
" ***************************************************************************/\n"
"\n"
"OPENCL_FORCE_INLINE float3 SchlickScatter_GetColor(const float3 sigmaS, const float3 sigmaA) {\n"
"	float3 r = sigmaS;\n"
"	if (r.x > 0.f)\n"
"		r.x /= r.x + sigmaA.x;\n"
"	else\n"
"		r.x = 1.f;\n"
"	if (r.y > 0.f)\n"
"		r.y /= r.y + sigmaA.y;\n"
"	else\n"
"		r.y = 1.f;\n"
"	if (r.z > 0.f)\n"
"		r.z /= r.z + sigmaA.z;\n"
"	else\n"
"		r.z = 1.f;\n"
"\n"
"	return r;\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 SchlickScatter_Albedo(const float3 sigmaS, const float3 sigmaA) {\n"
"	return Spectrum_Clamp(SchlickScatter_GetColor(sigmaS, sigmaA));\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 SchlickScatter_Evaluate(\n"
"		__global const HitPoint *hitPoint, const float3 localEyeDir, const float3 localLightDir,\n"
"		BSDFEvent *event, float *directPdfW,\n"
"		const float3 sigmaS, const float3 sigmaA, const float3 g) {\n"
"	const float3 gValue = clamp(g, -1.f, 1.f);\n"
"	const float3 k = gValue * (1.55f - .55f * gValue * gValue);\n"
"\n"
"	*event = DIFFUSE | REFLECT;\n"
"\n"
"	const float dotEyeLight = dot(localEyeDir, localLightDir);\n"
"	const float kFilter = Spectrum_Filter(k);\n"
"	// 1+k*cos instead of 1-k*cos because localEyeDir is reversed compared to the\n"
"	// standard phase function definition\n"
"	const float compcostFilter = 1.f + kFilter * dotEyeLight;\n"
"	const float pdf = (1.f - kFilter * kFilter) / (compcostFilter * compcostFilter * (4.f * M_PI_F));\n"
"\n"
"	if (directPdfW)\n"
"		*directPdfW = pdf;\n"
"\n"
"	// 1+k*cos instead of 1-k*cos because localEyeDir is reversed compared to the\n"
"	// standard phase function definition\n"
"	const float3 compcostValue = 1.f + k * dotEyeLight;\n"
"\n"
"	return SchlickScatter_GetColor(sigmaS, sigmaA) * (1.f - k * k) / (compcostValue * compcostValue * (4.f * M_PI_F));\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 SchlickScatter_Sample(\n"
"		__global const HitPoint *hitPoint, const float3 fixedDir, float3 *sampledDir,\n"
"		const float u0, const float u1, \n"
"		const float passThroughEvent,\n"
"		float *pdfW, BSDFEvent *event,\n"
"		const float3 sigmaS, const float3 sigmaA, const float3 g) {\n"
"	const float3 gValue = clamp(g, -1.f, 1.f);\n"
"	const float3 k = gValue * (1.55f - .55f * gValue * gValue);\n"
"	const float kFilter = Spectrum_Filter(k);\n"
"\n"
"	// Add a - because localEyeDir is reversed compared to the standard phase\n"
"	// function definition\n"
"	const float cost = -(2.f * u0 + kFilter - 1.f) / (2.f * kFilter * u0 - kFilter + 1.f);\n"
"\n"
"	float3 x, y;\n"
"	CoordinateSystem(fixedDir, &x, &y);\n"
"	*sampledDir = SphericalDirectionWithFrame(sqrt(fmax(0.f, 1.f - cost * cost)), cost,\n"
"			2.f * M_PI_F * u1, x, y, fixedDir);\n"
"\n"
"	// The - becomes a + because cost has been reversed above\n"
"	const float compcost = 1.f + kFilter * cost;\n"
"	*pdfW = (1.f - kFilter * kFilter) / (compcost * compcost * (4.f * M_PI_F));\n"
"	if (*pdfW <= 0.f)\n"
"		return BLACK;\n"
"\n"
"	*event = DIFFUSE | REFLECT;\n"
"\n"
"	return SchlickScatter_GetColor(sigmaS, sigmaA);\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// HeterogeneousVol material\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE void HeterogeneousVolMaterial_Albedo(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	const float3 sigmaS = Texture_GetSpectrumValue(material->volume.heterogenous.sigmaSTexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float3 sigmaA = Texture_GetSpectrumValue(material->volume.heterogenous.sigmaATexIndex, hitPoint TEXTURES_PARAM);\n"
"\n"
"    const float3 albedo = SchlickScatter_Albedo(\n"
"			clamp(sigmaS, 0.f, INFINITY),\n"
"			clamp(sigmaA, 0.f, INFINITY));\n"
"\n"
"	EvalStack_PushFloat3(albedo);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void HeterogeneousVolMaterial_GetInteriorVolume(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	DefaultMaterial_GetInteriorVolume(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void HeterogeneousVolMaterial_GetExteriorVolume(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	DefaultMaterial_GetExteriorVolume(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void HeterogeneousVolMaterial_GetPassThroughTransparency(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	DefaultMaterial_GetPassThroughTransparency(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void HeterogeneousVolMaterial_GetEmittedRadiance(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	DefaultMaterial_GetEmittedRadiance(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void HeterogeneousVolMaterial_Evaluate(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	float3 lightDir, eyeDir;\n"
"	EvalStack_PopFloat3(eyeDir);\n"
"	EvalStack_PopFloat3(lightDir);\n"
"\n"
"	const float3 sigmaSTexVal = Texture_GetSpectrumValue(material->volume.homogenous.sigmaSTexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float3 sigmaATexVal = Texture_GetSpectrumValue(material->volume.homogenous.sigmaATexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float3 gTexVal = Texture_GetSpectrumValue(material->volume.homogenous.gTexIndex, hitPoint TEXTURES_PARAM);\n"
"\n"
"	BSDFEvent event;\n"
"	float directPdfW;\n"
"	const float3 result = SchlickScatter_Evaluate(\n"
"			hitPoint, eyeDir, lightDir,\n"
"			&event, &directPdfW,\n"
"			clamp(sigmaSTexVal, 0.f, INFINITY), clamp(sigmaATexVal, 0.f, INFINITY), gTexVal);\n"
"\n"
"	EvalStack_PushFloat3(result);\n"
"	EvalStack_PushBSDFEvent(event);\n"
"	EvalStack_PushFloat(directPdfW);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void HeterogeneousVolMaterial_Sample(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	float u0, u1, passThroughEvent;\n"
"	EvalStack_PopFloat(passThroughEvent);\n"
"	EvalStack_PopFloat(u1);\n"
"	EvalStack_PopFloat(u0);\n"
"	float3 fixedDir;\n"
"	EvalStack_PopFloat3(fixedDir);\n"
"\n"
"	const float3 sigmaSTexVal = Texture_GetSpectrumValue(material->volume.homogenous.sigmaSTexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float3 sigmaATexVal = Texture_GetSpectrumValue(material->volume.homogenous.sigmaATexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float3 gTexVal = Texture_GetSpectrumValue(material->volume.homogenous.gTexIndex, hitPoint TEXTURES_PARAM);\n"
"\n"
"	float3 sampledDir;\n"
"	float pdfW;\n"
"	BSDFEvent event;\n"
"	const float3 result = SchlickScatter_Sample(\n"
"			hitPoint, fixedDir, &sampledDir,\n"
"			u0, u1, \n"
"			passThroughEvent,\n"
"			&pdfW, &event,\n"
"			clamp(sigmaSTexVal, 0.f, INFINITY), clamp(sigmaATexVal, 0.f, INFINITY), gTexVal);\n"
"\n"
"	EvalStack_PushFloat3(result);\n"
"	EvalStack_PushFloat3(sampledDir);\n"
"	EvalStack_PushFloat(pdfW);\n"
"	EvalStack_PushBSDFEvent(event);\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Material specific EvalOp\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_NOT_INLINE void HeterogeneousVolMaterial_EvalOp(\n"
"		__global const Material* restrict material,\n"
"		const MaterialEvalOpType evalType,\n"
"		__global float *evalStack,\n"
"		uint *evalStackOffset,\n"
"		__global const HitPoint *hitPoint\n"
"		MATERIALS_PARAM_DECL) {\n"
"	switch (evalType) {\n"
"		case EVAL_ALBEDO:\n"
"			HeterogeneousVolMaterial_Albedo(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_GET_INTERIOR_VOLUME:\n"
"			HeterogeneousVolMaterial_GetInteriorVolume(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_GET_EXTERIOR_VOLUME:\n"
"			HeterogeneousVolMaterial_GetExteriorVolume(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_GET_EMITTED_RADIANCE:\n"
"			HeterogeneousVolMaterial_GetEmittedRadiance(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_GET_PASS_TROUGH_TRANSPARENCY:\n"
"			HeterogeneousVolMaterial_GetPassThroughTransparency(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_EVALUATE:\n"
"			HeterogeneousVolMaterial_Evaluate(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_SAMPLE:\n"
"			HeterogeneousVolMaterial_Sample(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		default:\n"
"			// Something wrong here\n"
"			break;\n"
"	}\n"
"}\n";
} }
