
#include <string>

namespace slg { namespace ocl {
std::string KernelSource_materialdefs_funcs_homogeneousvol =
"#line 2 \"materialdefs_funcs_homogenousvol.cl\"\n"
"\n"
"/***************************************************************************\n"
" * Copyright 1998-2020 by authors (see AUTHORS.txt)                        *\n"
" *                                                                         *\n"
" *   This file is part of LuxCoreRender.                                   *\n"
" *                                                                         *\n"
" * Licensed under the Apache License, Version 2.0 (the \"License\");         *\n"
" * you may not use this file except in compliance with the License.        *\n"
" * You may obtain a copy of the License at                                 *\n"
" *                                                                         *\n"
" *     http://www.apache.org/licenses/LICENSE-2.0                          *\n"
" *                                                                         *\n"
" * Unless required by applicable law or agreed to in writing, software     *\n"
" * distributed under the License is distributed on an \"AS IS\" BASIS,       *\n"
" * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.*\n"
" * See the License for the specific language governing permissions and     *\n"
" * limitations under the License.                                          *\n"
" ***************************************************************************/\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// HomogeneousVol material\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE void HomogeneousVolMaterial_Albedo(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	const float3 sigmaS = Texture_GetSpectrumValue(material->volume.homogenous.sigmaSTexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float3 sigmaA = Texture_GetSpectrumValue(material->volume.homogenous.sigmaATexIndex, hitPoint TEXTURES_PARAM);\n"
"\n"
"    const float3 albedo = SchlickScatter_Albedo(\n"
"			clamp(sigmaS, 0.f, INFINITY),\n"
"			clamp(sigmaA, 0.f, INFINITY));\n"
"\n"
"	EvalStack_PushFloat3(albedo);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void HomogeneousVolMaterial_GetInteriorVolume(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	DefaultMaterial_GetInteriorVolume(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void HomogeneousVolMaterial_GetExteriorVolume(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	DefaultMaterial_GetExteriorVolume(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void HomogeneousVolMaterial_GetPassThroughTransparency(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	DefaultMaterial_GetPassThroughTransparency(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void HomogeneousVolMaterial_GetEmittedRadiance(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	DefaultMaterial_GetEmittedRadiance(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void HomogeneousVolMaterial_Evaluate(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	float3 lightDir, eyeDir;\n"
"	EvalStack_PopFloat3(eyeDir);\n"
"	EvalStack_PopFloat3(lightDir);\n"
"\n"
"	const float3 sigmaSTexVal = Texture_GetSpectrumValue(material->volume.homogenous.sigmaSTexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float3 sigmaATexVal = Texture_GetSpectrumValue(material->volume.homogenous.sigmaATexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float3 gTexVal = Texture_GetSpectrumValue(material->volume.homogenous.gTexIndex, hitPoint TEXTURES_PARAM);\n"
"\n"
"	BSDFEvent event;\n"
"	float directPdfW;\n"
"	const float3 result = SchlickScatter_Evaluate(\n"
"			hitPoint, eyeDir, lightDir,\n"
"			&event, &directPdfW,\n"
"			clamp(sigmaSTexVal, 0.f, INFINITY), clamp(sigmaATexVal, 0.f, INFINITY), gTexVal);\n"
"\n"
"	EvalStack_PushFloat3(result);\n"
"	EvalStack_PushBSDFEvent(event);\n"
"	EvalStack_PushFloat(directPdfW);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void HomogeneousVolMaterial_Sample(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	float u0, u1, passThroughEvent;\n"
"	EvalStack_PopFloat(passThroughEvent);\n"
"	EvalStack_PopFloat(u1);\n"
"	EvalStack_PopFloat(u0);\n"
"	float3 fixedDir;\n"
"	EvalStack_PopFloat3(fixedDir);\n"
"\n"
"	const float3 sigmaSTexVal = Texture_GetSpectrumValue(material->volume.homogenous.sigmaSTexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float3 sigmaATexVal = Texture_GetSpectrumValue(material->volume.homogenous.sigmaATexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float3 gTexVal = Texture_GetSpectrumValue(material->volume.homogenous.gTexIndex, hitPoint TEXTURES_PARAM);\n"
"\n"
"	float3 sampledDir;\n"
"	float pdfW;\n"
"	BSDFEvent event;\n"
"	const float3 result = SchlickScatter_Sample(\n"
"			hitPoint, fixedDir, &sampledDir,\n"
"			u0, u1, \n"
"			passThroughEvent,\n"
"			&pdfW, &event,\n"
"			clamp(sigmaSTexVal, 0.f, INFINITY), clamp(sigmaATexVal, 0.f, INFINITY), gTexVal);\n"
"\n"
"	EvalStack_PushFloat3(result);\n"
"	EvalStack_PushFloat3(sampledDir);\n"
"	EvalStack_PushFloat(pdfW);\n"
"	EvalStack_PushBSDFEvent(event);\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Material specific EvalOp\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_NOT_INLINE void HomogeneousVolMaterial_EvalOp(\n"
"		__global const Material* restrict material,\n"
"		const MaterialEvalOpType evalType,\n"
"		__global float *evalStack,\n"
"		uint *evalStackOffset,\n"
"		__global const HitPoint *hitPoint\n"
"		MATERIALS_PARAM_DECL) {\n"
"	switch (evalType) {\n"
"		case EVAL_ALBEDO:\n"
"			HomogeneousVolMaterial_Albedo(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_GET_INTERIOR_VOLUME:\n"
"			HomogeneousVolMaterial_GetInteriorVolume(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_GET_EXTERIOR_VOLUME:\n"
"			HomogeneousVolMaterial_GetExteriorVolume(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_GET_EMITTED_RADIANCE:\n"
"			HomogeneousVolMaterial_GetEmittedRadiance(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_GET_PASS_TROUGH_TRANSPARENCY:\n"
"			HomogeneousVolMaterial_GetPassThroughTransparency(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_EVALUATE:\n"
"			HomogeneousVolMaterial_Evaluate(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_SAMPLE:\n"
"			HomogeneousVolMaterial_Sample(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		default:\n"
"			// Something wrong here\n"
"			break;\n"
"	}\n"
"}\n";
} }
