
#include <string>

namespace slg { namespace ocl {
std::string KernelSource_materialdefs_funcs_metal2 =
"#line 2 \"materialdefs_funcs_metal2.cl\"\n"
"\n"
"/***************************************************************************\n"
" * Copyright 1998-2020 by authors (see AUTHORS.txt)                        *\n"
" *                                                                         *\n"
" *   This file is part of LuxCoreRender.                                   *\n"
" *                                                                         *\n"
" * Licensed under the Apache License, Version 2.0 (the \"License\");         *\n"
" * you may not use this file except in compliance with the License.        *\n"
" * You may obtain a copy of the License at                                 *\n"
" *                                                                         *\n"
" *     http://www.apache.org/licenses/LICENSE-2.0                          *\n"
" *                                                                         *\n"
" * Unless required by applicable law or agreed to in writing, software     *\n"
" * distributed under the License is distributed on an \"AS IS\" BASIS,       *\n"
" * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.*\n"
" * See the License for the specific language governing permissions and     *\n"
" * limitations under the License.                                          *\n"
" ***************************************************************************/\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Metal2 material\n"
"//\n"
"// LuxRender Metal2 material porting.\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE void Metal2Material_GetNK(__global const Material* restrict material, __global const HitPoint *hitPoint,\n"
"		float3 *n, float3 *k\n"
"		MATERIALS_PARAM_DECL) {\n"
"	const uint fresnelTexIndex = material->metal2.fresnelTexIndex;\n"
"	if (fresnelTexIndex != NULL_INDEX) {\n"
"		__global const Texture* restrict fresnelTex = &texs[fresnelTexIndex];\n"
"\n"
"		if (fresnelTex->type == FRESNELCOLOR_TEX) {\n"
"			const float3 f = Texture_GetSpectrumValue(fresnelTex->fresnelColor.krIndex, hitPoint TEXTURES_PARAM);\n"
"			*n = FresnelApproxN3(f);\n"
"			*k = FresnelApproxK3(f);\n"
"		} else {\n"
"			*n = VLOAD3F(&fresnelTex->fresnelConst.n.c[0]);\n"
"			*k = VLOAD3F(&fresnelTex->fresnelConst.k.c[0]);\n"
"		}\n"
"	} else {\n"
"		*n = clamp(Texture_GetSpectrumValue(material->metal2.nTexIndex, hitPoint TEXTURES_PARAM), .001f, INFINITY);\n"
"		*k = clamp(Texture_GetSpectrumValue(material->metal2.kTexIndex, hitPoint TEXTURES_PARAM), .001f, INFINITY);\n"
"	}\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void Metal2Material_Albedo(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	float3 n, k;\n"
"	Metal2Material_GetNK(material, hitPoint,\n"
"			&n, &k\n"
"			MATERIALS_PARAM);\n"
"\n"
"	const float3 albedo = Spectrum_Clamp(FresnelGeneral_Evaluate(n, k, 1.f));\n"
"\n"
"	EvalStack_PushFloat3(albedo);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void Metal2Material_GetInteriorVolume(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	DefaultMaterial_GetInteriorVolume(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void Metal2Material_GetExteriorVolume(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	DefaultMaterial_GetExteriorVolume(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void Metal2Material_GetPassThroughTransparency(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	DefaultMaterial_GetPassThroughTransparency(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void Metal2Material_GetEmittedRadiance(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	DefaultMaterial_GetEmittedRadiance(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void Metal2Material_Evaluate(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	float3 lightDir, eyeDir;\n"
"	EvalStack_PopFloat3(eyeDir);\n"
"	EvalStack_PopFloat3(lightDir);\n"
"\n"
"	const float uVal = Texture_GetFloatValue(material->metal2.nuTexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float vVal = Texture_GetFloatValue(material->metal2.nvTexIndex, hitPoint TEXTURES_PARAM);\n"
"\n"
"	const float u = clamp(uVal, 1e-9f, 1.f);\n"
"	const float v = clamp(vVal, 1e-9f, 1.f);\n"
"	const float u2 = u * u;\n"
"	const float v2 = v * v;\n"
"	const float anisotropy = (u2 < v2) ? (1.f - u2 / v2) : u2 > 0.f ? (v2 / u2 - 1.f) : 0.f;\n"
"	const float roughness = u * v;\n"
"\n"
"	const float3 wh = normalize(lightDir + eyeDir);\n"
"	const float cosWH = dot(lightDir, wh);\n"
"\n"
"	const float directPdfW = SchlickDistribution_Pdf(roughness, wh, anisotropy) / (4.f * cosWH);\n"
"\n"
"	float3 nVal, kVal;\n"
"	Metal2Material_GetNK(material, hitPoint,\n"
"			&nVal, &kVal\n"
"			MATERIALS_PARAM);\n"
"\n"
"	const float3 F = FresnelGeneral_Evaluate(nVal, kVal, cosWH);\n"
"	Spectrum_Clamp(F);\n"
"\n"
"	const float G = SchlickDistribution_G(roughness, lightDir, eyeDir);\n"
"\n"
"	const BSDFEvent event = GLOSSY | REFLECT;\n"
"	const float3 result = (SchlickDistribution_D(roughness, wh, anisotropy) * G / (4.f * fabs(eyeDir.z))) * F;\n"
"	\n"
"	EvalStack_PushFloat3(result);\n"
"	EvalStack_PushBSDFEvent(event);\n"
"	EvalStack_PushFloat(directPdfW);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void Metal2Material_Sample(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	float u0, u1, passThroughEvent;\n"
"	EvalStack_PopFloat(passThroughEvent);\n"
"	EvalStack_PopFloat(u1);\n"
"	EvalStack_PopFloat(u0);\n"
"	float3 fixedDir;\n"
"	EvalStack_PopFloat3(fixedDir);\n"
"\n"
"	if (fabs(fixedDir.z) < DEFAULT_COS_EPSILON_STATIC) {\n"
"		MATERIAL_SAMPLE_RETURN_BLACK;\n"
"	}\n"
"\n"
"	const float uVal = Texture_GetFloatValue(material->metal2.nuTexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float vVal = Texture_GetFloatValue(material->metal2.nvTexIndex, hitPoint TEXTURES_PARAM);\n"
"\n"
"	const float u = clamp(uVal, 1e-9f, 1.f);\n"
"	const float v = clamp(vVal, 1e-9f, 1.f);\n"
"	const float u2 = u * u;\n"
"	const float v2 = v * v;\n"
"	const float anisotropy = (u2 < v2) ? (1.f - u2 / v2) : u2 > 0.f ? (v2 / u2 - 1.f) : 0.f;\n"
"	const float roughness = u * v;\n"
"\n"
"	float3 wh;\n"
"	float d, specPdf;\n"
"	SchlickDistribution_SampleH(roughness, anisotropy, u0, u1, &wh, &d, &specPdf);\n"
"	const float cosWH = dot(fixedDir, wh);\n"
"	const float3 sampledDir = 2.f * cosWH * wh - fixedDir;\n"
"\n"
"	const float coso = fabs(fixedDir.z);\n"
"	const float cosi = fabs(sampledDir.z);\n"
"	if ((cosi < DEFAULT_COS_EPSILON_STATIC) || (fixedDir.z * sampledDir.z < 0.f)) {\n"
"		MATERIAL_SAMPLE_RETURN_BLACK;\n"
"	}\n"
"\n"
"	const float pdfW = specPdf / (4.f * fabs(cosWH));\n"
"	if (pdfW <= 0.f) {\n"
"		MATERIAL_SAMPLE_RETURN_BLACK;\n"
"	}\n"
"\n"
"	const float G = SchlickDistribution_G(roughness, fixedDir, sampledDir);\n"
"\n"
"	float3 nVal, kVal;\n"
"	Metal2Material_GetNK(material, hitPoint,\n"
"			&nVal, &kVal\n"
"			MATERIALS_PARAM);\n"
"\n"
"	const float3 F = FresnelGeneral_Evaluate(nVal, kVal, cosWH);\n"
"	Spectrum_Clamp(F);\n"
"\n"
"	float factor = (d / specPdf) * G * fabs(cosWH);\n"
"	//if (!fromLight)\n"
"		factor /= coso;\n"
"	//else\n"
"	//	factor /= cosi;\n"
"\n"
"	const BSDFEvent event = GLOSSY | REFLECT;\n"
"\n"
"	const float3 result = factor * F;\n"
"\n"
"	EvalStack_PushFloat3(result);\n"
"	EvalStack_PushFloat3(sampledDir);\n"
"	EvalStack_PushFloat(pdfW);\n"
"	EvalStack_PushBSDFEvent(event);\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Material specific EvalOp\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_NOT_INLINE void Metal2Material_EvalOp(\n"
"		__global const Material* restrict material,\n"
"		const MaterialEvalOpType evalType,\n"
"		__global float *evalStack,\n"
"		uint *evalStackOffset,\n"
"		__global const HitPoint *hitPoint\n"
"		MATERIALS_PARAM_DECL) {\n"
"	switch (evalType) {\n"
"		case EVAL_ALBEDO:\n"
"			Metal2Material_Albedo(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_GET_INTERIOR_VOLUME:\n"
"			Metal2Material_GetInteriorVolume(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_GET_EXTERIOR_VOLUME:\n"
"			Metal2Material_GetExteriorVolume(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_GET_EMITTED_RADIANCE:\n"
"			Metal2Material_GetEmittedRadiance(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_GET_PASS_TROUGH_TRANSPARENCY:\n"
"			Metal2Material_GetPassThroughTransparency(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_EVALUATE:\n"
"			Metal2Material_Evaluate(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_SAMPLE:\n"
"			Metal2Material_Sample(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		default:\n"
"			// Something wrong here\n"
"			break;\n"
"	}\n"
"}\n";
} }
