
#include <string>

namespace slg { namespace ocl {
std::string KernelSource_materialdefs_funcs_roughglass =
"#line 2 \"materialdefs_funcs_roughglass.cl\"\n"
"\n"
"/***************************************************************************\n"
" * Copyright 1998-2020 by authors (see AUTHORS.txt)                        *\n"
" *                                                                         *\n"
" *   This file is part of LuxCoreRender.                                   *\n"
" *                                                                         *\n"
" * Licensed under the Apache License, Version 2.0 (the \"License\");         *\n"
" * you may not use this file except in compliance with the License.        *\n"
" * You may obtain a copy of the License at                                 *\n"
" *                                                                         *\n"
" *     http://www.apache.org/licenses/LICENSE-2.0                          *\n"
" *                                                                         *\n"
" * Unless required by applicable law or agreed to in writing, software     *\n"
" * distributed under the License is distributed on an \"AS IS\" BASIS,       *\n"
" * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.*\n"
" * See the License for the specific language governing permissions and     *\n"
" * limitations under the License.                                          *\n"
" ***************************************************************************/\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// RoughGlass material\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE void RoughGlassMaterial_Albedo(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"    const float3 albedo = WHITE;\n"
"\n"
"	EvalStack_PushFloat3(albedo);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void RoughGlassMaterial_GetInteriorVolume(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	DefaultMaterial_GetInteriorVolume(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void RoughGlassMaterial_GetExteriorVolume(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	DefaultMaterial_GetExteriorVolume(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void RoughGlassMaterial_GetPassThroughTransparency(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	DefaultMaterial_GetPassThroughTransparency(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void RoughGlassMaterial_GetEmittedRadiance(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	DefaultMaterial_GetEmittedRadiance(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void RoughGlassMaterial_Evaluate(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	float3 lightDir, eyeDir;\n"
"	EvalStack_PopFloat3(eyeDir);\n"
"	EvalStack_PopFloat3(lightDir);\n"
"\n"
"	const float3 ktVal = Texture_GetSpectrumValue(material->roughglass.ktTexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float3 krVal = Texture_GetSpectrumValue(material->roughglass.krTexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float3 kt = Spectrum_Clamp(ktVal);\n"
"	const float3 kr = Spectrum_Clamp(krVal);\n"
"\n"
"	const bool isKtBlack = Spectrum_IsBlack(kt);\n"
"	const bool isKrBlack = Spectrum_IsBlack(kr);\n"
"	if (isKtBlack && isKrBlack) {\n"
"		MATERIAL_EVALUATE_RETURN_BLACK;\n"
"	}\n"
"	\n"
"	const float nc = ExtractExteriorIors(hitPoint, material->roughglass.exteriorIorTexIndex TEXTURES_PARAM);\n"
"	const float nt = ExtractInteriorIors(hitPoint, material->roughglass.interiorIorTexIndex TEXTURES_PARAM);\n"
"	const float ntc = nt / nc;\n"
"\n"
"	const float nuVal = Texture_GetFloatValue(material->roughglass.nuTexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float nvVal = Texture_GetFloatValue(material->roughglass.nvTexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float u = clamp(nuVal, 1e-9f, 1.f);\n"
"	const float v = clamp(nvVal, 1e-9f, 1.f);\n"
"	const float u2 = u * u;\n"
"	const float v2 = v * v;\n"
"	const float anisotropy = (u2 < v2) ? (1.f - u2 / v2) : u2 > 0.f ? (v2 / u2 - 1.f) : 0.f;\n"
"	const float roughness = u * v;\n"
"\n"
"	float directPdfW;\n"
"	BSDFEvent event;\n"
"	float3 result;\n"
"	const float threshold = isKrBlack ? 1.f : (isKtBlack ? 0.f : .5f);\n"
"	if (lightDir.z * eyeDir.z < 0.f) {\n"
"		// Transmit\n"
"\n"
"		const bool entering = (CosTheta(lightDir) > 0.f);\n"
"		const float eta = entering ? (nc / nt) : ntc;\n"
"\n"
"		float3 wh = eta * lightDir + eyeDir;\n"
"		if (wh.z < 0.f)\n"
"			wh = -wh;\n"
"\n"
"		const float lengthSquared = dot(wh, wh);\n"
"		if (!(lengthSquared > 0.f)) {\n"
"			MATERIAL_EVALUATE_RETURN_BLACK;\n"
"		}\n"
"		wh /= sqrt(lengthSquared);\n"
"		const float cosThetaI = fabs(CosTheta(eyeDir));\n"
"		const float cosThetaIH = fabs(dot(eyeDir, wh));\n"
"		const float cosThetaOH = dot(lightDir, wh);\n"
"\n"
"		const float D = SchlickDistribution_D(roughness, wh, anisotropy);\n"
"		const float G = SchlickDistribution_G(roughness, lightDir, eyeDir);\n"
"		const float specPdf = SchlickDistribution_Pdf(roughness, wh, anisotropy);\n"
"		const float F = FresnelCauchy_Evaluate(ntc, cosThetaOH);\n"
"\n"
"		directPdfW = threshold * specPdf * (fabs(cosThetaOH) * eta * eta) / lengthSquared;\n"
"\n"
"		//if (reversePdfW)\n"
"		//	*reversePdfW = threshold * specPdf * cosThetaIH / lengthSquared;\n"
"\n"
"		result = (fabs(cosThetaOH) * cosThetaIH * D *\n"
"			G / (cosThetaI * lengthSquared)) *\n"
"			kt * (1.f - F);\n"
"\n"
"        event = GLOSSY | TRANSMIT;\n"
"	} else {\n"
"		// Reflect\n"
"		const float cosThetaO = fabs(CosTheta(lightDir));\n"
"		const float cosThetaI = fabs(CosTheta(eyeDir));\n"
"		if (cosThetaO == 0.f || cosThetaI == 0.f) {\n"
"			MATERIAL_EVALUATE_RETURN_BLACK;\n"
"		}\n"
"		float3 wh = lightDir + eyeDir;\n"
"		if (all(isequal(wh, BLACK))) {\n"
"			MATERIAL_EVALUATE_RETURN_BLACK;\n"
"		}\n"
"		wh = normalize(wh);\n"
"		if (wh.z < 0.f)\n"
"			wh = -wh;\n"
"\n"
"		float cosThetaH = dot(eyeDir, wh);\n"
"		const float D = SchlickDistribution_D(roughness, wh, anisotropy);\n"
"		const float G = SchlickDistribution_G(roughness, lightDir, eyeDir);\n"
"		const float specPdf = SchlickDistribution_Pdf(roughness, wh, anisotropy);\n"
"		const float F = FresnelCauchy_Evaluate(ntc, cosThetaH);\n"
"\n"
"		directPdfW = (1.f - threshold) * specPdf / (4.f * fabs(dot(lightDir, wh)));\n"
"\n"
"		//if (reversePdfW)\n"
"		//	*reversePdfW = (1.f - threshold) * specPdf / (4.f * fabs(dot(lightDir, wh));\n"
"\n"
"		result = (D * G / (4.f * cosThetaI)) * kr * F;\n"
"		\n"
"		const float localFilmThickness = (material->roughglass.filmThicknessTexIndex != NULL_INDEX) \n"
"										 ? Texture_GetFloatValue(material->roughglass.filmThicknessTexIndex, hitPoint TEXTURES_PARAM) : 0.f;\n"
"		if (localFilmThickness > 0.f) {\n"
"			const float localFilmIor = (material->roughglass.filmIorTexIndex != NULL_INDEX) \n"
"									   ? Texture_GetFloatValue(material->roughglass.filmIorTexIndex, hitPoint TEXTURES_PARAM) : 1.f;\n"
"			result *= CalcFilmColor(eyeDir, localFilmThickness, localFilmIor);\n"
"		}\n"
"\n"
"        event = GLOSSY | REFLECT;\n"
"	}\n"
"\n"
"	EvalStack_PushFloat3(result);\n"
"	EvalStack_PushBSDFEvent(event);\n"
"	EvalStack_PushFloat(directPdfW);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void RoughGlassMaterial_Sample(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	float u0, u1, passThroughEvent;\n"
"	EvalStack_PopFloat(passThroughEvent);\n"
"	EvalStack_PopFloat(u1);\n"
"	EvalStack_PopFloat(u0);\n"
"	float3 fixedDir;\n"
"	EvalStack_PopFloat3(fixedDir);\n"
"\n"
"	if (fabs(fixedDir.z) < DEFAULT_COS_EPSILON_STATIC) {\n"
"		MATERIAL_SAMPLE_RETURN_BLACK;\n"
"	}\n"
"\n"
"	const float3 ktVal = Texture_GetSpectrumValue(material->roughglass.ktTexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float3 krVal = Texture_GetSpectrumValue(material->roughglass.krTexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float3 kt = Spectrum_Clamp(ktVal);\n"
"	const float3 kr = Spectrum_Clamp(krVal);\n"
"\n"
"	const bool isKtBlack = Spectrum_IsBlack(kt);\n"
"	const bool isKrBlack = Spectrum_IsBlack(kr);\n"
"	if (isKtBlack && isKrBlack) {\n"
"		MATERIAL_SAMPLE_RETURN_BLACK;\n"
"	}\n"
"\n"
"	const float nuVal = Texture_GetFloatValue(material->roughglass.nuTexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float nvVal = Texture_GetFloatValue(material->roughglass.nvTexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float u = clamp(nuVal, 1e-9f, 1.f);\n"
"	const float v = clamp(nvVal, 1e-9f, 1.f);\n"
"	const float u2 = u * u;\n"
"	const float v2 = v * v;\n"
"	const float anisotropy = (u2 < v2) ? (1.f - u2 / v2) : u2 > 0.f ? (v2 / u2 - 1.f) : 0.f;\n"
"	const float roughness = u * v;\n"
"\n"
"	float3 wh;\n"
"	float d, specPdf;\n"
"	SchlickDistribution_SampleH(roughness, anisotropy, u0, u1, &wh, &d, &specPdf);\n"
"	if (wh.z < 0.f)\n"
"		wh = -wh;\n"
"	const float cosThetaOH = dot(fixedDir, wh);\n"
"\n"
"	const float nc = ExtractExteriorIors(hitPoint, material->roughglass.exteriorIorTexIndex TEXTURES_PARAM);\n"
"	const float nt = ExtractInteriorIors(hitPoint, material->roughglass.interiorIorTexIndex TEXTURES_PARAM);\n"
"	const float ntc = nt / nc;\n"
"\n"
"	const float coso = fabs(fixedDir.z);\n"
"\n"
"	// Decide to transmit or reflect\n"
"	float threshold;\n"
"	if (!isKrBlack) {\n"
"		if (!isKtBlack)\n"
"			threshold = .5f;\n"
"		else\n"
"			threshold = 0.f;\n"
"	} else {\n"
"		if (!isKtBlack)\n"
"			threshold = 1.f;\n"
"		else {\n"
"			MATERIAL_SAMPLE_RETURN_BLACK;\n"
"		}\n"
"	}\n"
"\n"
"	float pdfW;\n"
"	BSDFEvent event;\n"
"	float3 sampledDir;\n"
"	float3 result;\n"
"	if (passThroughEvent < threshold) {\n"
"		// Transmit\n"
"\n"
"		const bool entering = (CosTheta(fixedDir) > 0.f);\n"
"		const float eta = entering ? (nc / nt) : ntc;\n"
"		const float eta2 = eta * eta;\n"
"		const float sinThetaIH2 = eta2 * fmax(0.f, 1.f - cosThetaOH * cosThetaOH);\n"
"		if (sinThetaIH2 >= 1.f) {\n"
"			MATERIAL_SAMPLE_RETURN_BLACK;\n"
"		}\n"
"		float cosThetaIH = sqrt(1.f - sinThetaIH2);\n"
"		if (entering)\n"
"			cosThetaIH = -cosThetaIH;\n"
"		const float length = eta * cosThetaOH + cosThetaIH;\n"
"		sampledDir = length * wh - eta * fixedDir;\n"
"\n"
"		const float lengthSquared = length * length;\n"
"		pdfW = specPdf * fabs(cosThetaIH) / lengthSquared;\n"
"		if (pdfW <= 0.f) {\n"
"			MATERIAL_SAMPLE_RETURN_BLACK;\n"
"		}\n"
"\n"
"		const float cosi = fabs(sampledDir.z);\n"
"\n"
"		const float G = SchlickDistribution_G(roughness, fixedDir, sampledDir);\n"
"		float factor = (d / specPdf) * G * fabs(cosThetaOH) / threshold;\n"
"\n"
"		//if (!hitPoint.fromLight) {\n"
"			const float F = FresnelCauchy_Evaluate(ntc, cosThetaIH);\n"
"			result = (factor / coso) * kt * (1.f - F);\n"
"		//} else {\n"
"		//	const Spectrum F = FresnelCauchy_Evaluate(ntc, cosThetaOH);\n"
"		//	result = (factor / cosi) * kt * (Spectrum(1.f) - F);\n"
"		//}\n"
"\n"
"		pdfW *= threshold;\n"
"		event = GLOSSY | TRANSMIT;\n"
"	} else {\n"
"		// Reflect\n"
"		pdfW = specPdf / (4.f * fabs(cosThetaOH));\n"
"		if (pdfW <= 0.f) {\n"
"			MATERIAL_SAMPLE_RETURN_BLACK;\n"
"		}\n"
"\n"
"		sampledDir = 2.f * cosThetaOH * wh - fixedDir;\n"
"\n"
"		const float cosi = fabs(sampledDir.z);\n"
"		if ((cosi < DEFAULT_COS_EPSILON_STATIC) || (fixedDir.z * sampledDir.z < 0.f)) {\n"
"			MATERIAL_SAMPLE_RETURN_BLACK;\n"
"		}\n"
"\n"
"		const float G = SchlickDistribution_G(roughness, fixedDir, sampledDir);\n"
"		float factor = (d / specPdf) * G * fabs(cosThetaOH) / (1.f - threshold);\n"
"\n"
"		const float F = FresnelCauchy_Evaluate(ntc, cosThetaOH);\n"
"		//factor /= (!hitPoint.fromLight) ? coso : cosi;\n"
"		factor /= coso;\n"
"		result = factor * F * kr;\n"
"		\n"
"		const float localFilmThickness = (material->roughglass.filmThicknessTexIndex != NULL_INDEX) \n"
"										 ? Texture_GetFloatValue(material->roughglass.filmThicknessTexIndex, hitPoint TEXTURES_PARAM) : 0.f;\n"
"		if (localFilmThickness > 0.f) {\n"
"			const float localFilmIor = (material->roughglass.filmIorTexIndex != NULL_INDEX) \n"
"									   ? Texture_GetFloatValue(material->roughglass.filmIorTexIndex, hitPoint TEXTURES_PARAM) : 1.f;\n"
"			result *= CalcFilmColor(fixedDir, localFilmThickness, localFilmIor);\n"
"		}\n"
"\n"
"		pdfW *= (1.f - threshold);\n"
"		event = GLOSSY | REFLECT;\n"
"	}\n"
"\n"
"	EvalStack_PushFloat3(result);\n"
"	EvalStack_PushFloat3(sampledDir);\n"
"	EvalStack_PushFloat(pdfW);\n"
"	EvalStack_PushBSDFEvent(event);\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Material specific EvalOp\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_NOT_INLINE void RoughGlassMaterial_EvalOp(\n"
"		__global const Material* restrict material,\n"
"		const MaterialEvalOpType evalType,\n"
"		__global float *evalStack,\n"
"		uint *evalStackOffset,\n"
"		__global const HitPoint *hitPoint\n"
"		MATERIALS_PARAM_DECL) {\n"
"	switch (evalType) {\n"
"		case EVAL_ALBEDO:\n"
"			RoughGlassMaterial_Albedo(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_GET_INTERIOR_VOLUME:\n"
"			RoughGlassMaterial_GetInteriorVolume(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_GET_EXTERIOR_VOLUME:\n"
"			RoughGlassMaterial_GetExteriorVolume(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_GET_EMITTED_RADIANCE:\n"
"			RoughGlassMaterial_GetEmittedRadiance(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_GET_PASS_TROUGH_TRANSPARENCY:\n"
"			RoughGlassMaterial_GetPassThroughTransparency(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_EVALUATE:\n"
"			RoughGlassMaterial_Evaluate(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_SAMPLE:\n"
"			RoughGlassMaterial_Sample(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		default:\n"
"			// Something wrong here\n"
"			break;\n"
"	}\n"
"}\n";
} }
