
#include <string>

namespace slg { namespace ocl {
std::string KernelSource_materialdefs_funcs_velvet =
"#line 2 \"materialdefs_funcs_velvet.cl\"\n"
"\n"
"/***************************************************************************\n"
" * Copyright 1998-2020 by authors (see AUTHORS.txt)                        *\n"
" *                                                                         *\n"
" *   This file is part of LuxCoreRender.                                   *\n"
" *                                                                         *\n"
" * Licensed under the Apache License, Version 2.0 (the \"License\");         *\n"
" * you may not use this file except in compliance with the License.        *\n"
" * You may obtain a copy of the License at                                 *\n"
" *                                                                         *\n"
" *     http://www.apache.org/licenses/LICENSE-2.0                          *\n"
" *                                                                         *\n"
" * Unless required by applicable law or agreed to in writing, software     *\n"
" * distributed under the License is distributed on an \"AS IS\" BASIS,       *\n"
" * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.*\n"
" * See the License for the specific language governing permissions and     *\n"
" * limitations under the License.                                          *\n"
" ***************************************************************************/\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Velvet material\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE void VelvetMaterial_Albedo(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	const float3 albedo = Spectrum_Clamp(Texture_GetSpectrumValue(material->velvet.kdTexIndex,\n"
"			hitPoint TEXTURES_PARAM));\n"
"\n"
"	EvalStack_PushFloat3(albedo);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void VelvetMaterial_GetInteriorVolume(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	DefaultMaterial_GetInteriorVolume(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void VelvetMaterial_GetExteriorVolume(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	DefaultMaterial_GetExteriorVolume(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void VelvetMaterial_GetPassThroughTransparency(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	DefaultMaterial_GetPassThroughTransparency(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void VelvetMaterial_GetEmittedRadiance(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	DefaultMaterial_GetEmittedRadiance(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void VelvetMaterial_Evaluate(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	float3 lightDir, eyeDir;\n"
"	EvalStack_PopFloat3(eyeDir);\n"
"	EvalStack_PopFloat3(lightDir);\n"
"\n"
"	const float directPdfW = fabs(lightDir.z * M_1_PI_F);\n"
"\n"
"	const BSDFEvent event = DIFFUSE | REFLECT;\n"
"\n"
"	const float3 kdVal = Texture_GetSpectrumValue(material->velvet.kdTexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float3 kd = Spectrum_Clamp(kdVal);\n"
"	const float cosv = -dot(lightDir, eyeDir);\n"
"\n"
"	// Compute phase function\n"
"\n"
"	const float B = 3.0f * cosv;\n"
"\n"
"	const float A1 = Texture_GetFloatValue(material->velvet.p1TexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float A2 = Texture_GetFloatValue(material->velvet.p2TexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float A3 = Texture_GetFloatValue(material->velvet.p3TexIndex, hitPoint TEXTURES_PARAM);\n"
"\n"
"	float p = 1.0f + A1 * cosv + A2 * 0.5f * (B * cosv - 1.0f) + A3 * 0.5 * (5.0f * cosv * cosv * cosv - B);\n"
"	p = p / (4.0f * M_PI_F);\n"
" \n"
"	const float delta = Texture_GetFloatValue(material->velvet.thicknessTexIndex, hitPoint TEXTURES_PARAM);\n"
"	p = (p * delta) / fabs(eyeDir.z);\n"
"\n"
"	// Clamp the BRDF (page 7)\n"
"	if (p > 1.0f)\n"
"		p = 1.0f;\n"
"	else if (p < 0.0f)\n"
"		p = 0.0f;\n"
"\n"
"	const float3 result = p * kd;\n"
"\n"
"	EvalStack_PushFloat3(result);\n"
"	EvalStack_PushBSDFEvent(event);\n"
"	EvalStack_PushFloat(directPdfW);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void VelvetMaterial_Sample(__global const Material* restrict material,\n"
"		__global const HitPoint *hitPoint,\n"
"		__global float *evalStack, uint *evalStackOffset\n"
"		MATERIALS_PARAM_DECL) {\n"
"	float u0, u1, passThroughEvent;\n"
"	EvalStack_PopFloat(passThroughEvent);\n"
"	EvalStack_PopFloat(u1);\n"
"	EvalStack_PopFloat(u0);\n"
"	float3 fixedDir;\n"
"	EvalStack_PopFloat3(fixedDir);\n"
"\n"
"	if (fabs(fixedDir.z) < DEFAULT_COS_EPSILON_STATIC) {\n"
"		MATERIAL_SAMPLE_RETURN_BLACK;\n"
"	}\n"
"\n"
"	float pdfW;\n"
"	const float3 sampledDir = (signbit(fixedDir.z) ? -1.f : 1.f) * CosineSampleHemisphereWithPdf(u0, u1, &pdfW);\n"
"	if (fabs(CosTheta(sampledDir)) < DEFAULT_COS_EPSILON_STATIC) {\n"
"		MATERIAL_SAMPLE_RETURN_BLACK;\n"
"	}\n"
"\n"
"	const BSDFEvent event = DIFFUSE | REFLECT;\n"
"\n"
"	const float3 kdVal = Texture_GetSpectrumValue(material->velvet.kdTexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float3 kd = Spectrum_Clamp(kdVal);\n"
"	const float cosv = dot(-fixedDir, sampledDir);\n"
"\n"
"	// Compute phase function\n"
"\n"
"	const float B = 3.0f * cosv;\n"
"\n"
"	const float A1 = Texture_GetFloatValue(material->velvet.p1TexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float A2 = Texture_GetFloatValue(material->velvet.p2TexIndex, hitPoint TEXTURES_PARAM);\n"
"	const float A3 = Texture_GetFloatValue(material->velvet.p3TexIndex, hitPoint TEXTURES_PARAM);\n"
"\n"
"	float p = 1.0f + A1 * cosv + A2 * 0.5f * (B * cosv - 1.0f) + A3 * 0.5 * (5.0f * cosv * cosv * cosv - B);\n"
"	p = p / (4.0f * M_PI_F);\n"
" \n"
"	const float delta = Texture_GetFloatValue(material->velvet.thicknessTexIndex, hitPoint TEXTURES_PARAM);\n"
"	p = (p * delta) / fabs(fixedDir.z);\n"
"\n"
"	// Clamp the BRDF (page 7)\n"
"	if (p > 1.0f)\n"
"		p = 1.0f;\n"
"	else if (p < 0.0f)\n"
"		p = 0.0f;\n"
"\n"
"	const float3 result = kd * (p / pdfW);\n"
"\n"
"	EvalStack_PushFloat3(result);\n"
"	EvalStack_PushFloat3(sampledDir);\n"
"	EvalStack_PushFloat(pdfW);\n"
"	EvalStack_PushBSDFEvent(event);\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Material specific EvalOp\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_NOT_INLINE void VelvetMaterial_EvalOp(\n"
"		__global const Material* restrict material,\n"
"		const MaterialEvalOpType evalType,\n"
"		__global float *evalStack,\n"
"		uint *evalStackOffset,\n"
"		__global const HitPoint *hitPoint\n"
"		MATERIALS_PARAM_DECL) {\n"
"	switch (evalType) {\n"
"		case EVAL_ALBEDO:\n"
"			VelvetMaterial_Albedo(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_GET_INTERIOR_VOLUME:\n"
"			VelvetMaterial_GetInteriorVolume(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_GET_EXTERIOR_VOLUME:\n"
"			VelvetMaterial_GetExteriorVolume(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_GET_EMITTED_RADIANCE:\n"
"			VelvetMaterial_GetEmittedRadiance(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_GET_PASS_TROUGH_TRANSPARENCY:\n"
"			VelvetMaterial_GetPassThroughTransparency(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_EVALUATE:\n"
"			VelvetMaterial_Evaluate(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		case EVAL_SAMPLE:\n"
"			VelvetMaterial_Sample(material, hitPoint, evalStack, evalStackOffset MATERIALS_PARAM);\n"
"			break;\n"
"		default:\n"
"			// Something wrong here\n"
"			break;\n"
"	}\n"
"}\n";
} }
