
#include <string>

namespace slg { namespace ocl {
std::string KernelSource_pathinfo_funcs =
"#line 2 \"pathinfo_funcs.cl\"\n"
"\n"
"/***************************************************************************\n"
" * Copyright 1998-2020 by authors (see AUTHORS.txt)                        *\n"
" *                                                                         *\n"
" *   This file is part of LuxCoreRender.                                   *\n"
" *                                                                         *\n"
" * Licensed under the Apache License, Version 2.0 (the \"License\");         *\n"
" * you may not use this file except in compliance with the License.        *\n"
" * You may obtain a copy of the License at                                 *\n"
" *                                                                         *\n"
" *     http://www.apache.org/licenses/LICENSE-2.0                          *\n"
" *                                                                         *\n"
" * Unless required by applicable law or agreed to in writing, software     *\n"
" * distributed under the License is distributed on an \"AS IS\" BASIS,       *\n"
" * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.*\n"
" * See the License for the specific language governing permissions and     *\n"
" * limitations under the License.                                          *\n"
" ***************************************************************************/\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// EyePathInfo\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE void EyePathInfo_Init(__global EyePathInfo *pathInfo) {\n"
"	PathDepthInfo_Init(&pathInfo->depth);\n"
"	PathVolumeInfo_Init(&pathInfo->volume);\n"
"	\n"
"	pathInfo->isPassThroughPath = true;\n"
"\n"
"	pathInfo->lastBSDFEvent = SPECULAR; // SPECULAR is required to avoid MIS\n"
"	pathInfo->lastBSDFPdfW = 1.f;\n"
"	pathInfo->lastGlossiness = 0.f;\n"
"	pathInfo->lastFromVolume = false;\n"
"	pathInfo->isTransmittedPath = true;\n"
"\n"
"	pathInfo->isNearlyCaustic = false;\n"
"	pathInfo->isNearlyS = false;\n"
"	pathInfo->isNearlySD = false;\n"
"	pathInfo->isNearlySDS = false;\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE bool EyePathInfo_UseRR(__global EyePathInfo *pathInfo, const uint rrDepth) {\n"
"	return !(pathInfo->lastBSDFEvent & SPECULAR) &&\n"
"			(PathDepthInfo_GetRRDepth(&pathInfo->depth) >= rrDepth);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE bool EyePathInfo_IsNearlySpecular(__global EyePathInfo *pathInfo,\n"
"		const BSDFEvent event, const float glossiness, const float glossinessThreshold) {\n"
"	return (event & SPECULAR) || ((event & GLOSSY) && (glossiness <= glossinessThreshold));\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE bool EyePathInfo_CanBeNearlySpecular(__global EyePathInfo *pathInfo,\n"
"		__global const BSDF *bsdf, const float glossinessThreshold\n"
"		MATERIALS_PARAM_DECL) {\n"
"	const BSDFEvent eventTypes = BSDF_GetEventTypes(bsdf MATERIALS_PARAM);\n"
"\n"
"	return (eventTypes & SPECULAR) || ((eventTypes & GLOSSY) && (BSDF_GetGlossiness(bsdf MATERIALS_PARAM) <= glossinessThreshold));\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void EyePathInfo_AddVertex(__global EyePathInfo *pathInfo,\n"
"		__global const BSDF *bsdf, const BSDFEvent event, const float pdfW,\n"
"		const float glossinessThreshold\n"
"		MATERIALS_PARAM_DECL) {\n"
"	//--------------------------------------------------------------------------\n"
"	// PathInfo::AddVertex() inlined here for performances\n"
"	//--------------------------------------------------------------------------\n"
"\n"
"	// Increment path depth information\n"
"	PathDepthInfo_IncDepths(&pathInfo->depth, event);\n"
"\n"
"	// Update volume information\n"
"	PathVolumeInfo_Update(&pathInfo->volume, event, bsdf MATERIALS_PARAM);\n"
"\n"
"	const float glossiness = BSDF_GetGlossiness(bsdf MATERIALS_PARAM);\n"
"	const bool isNewVertexNearlySpecular = EyePathInfo_IsNearlySpecular(pathInfo,\n"
"			event, glossiness, glossinessThreshold);\n"
"\n"
"	// Update isNearlySDS (must be done before isNearlySD)\n"
"	pathInfo->isNearlySDS = (pathInfo->isNearlySD || pathInfo->isNearlySDS) && isNewVertexNearlySpecular;\n"
"\n"
"	// Update isNearlySD (must be done before isNearlyS)\n"
"	pathInfo->isNearlySD = pathInfo->isNearlyS && !isNewVertexNearlySpecular;\n"
"\n"
"	// Update isNearlySpecular\n"
"	pathInfo->isNearlyS = ((pathInfo->depth.depth == 1) || pathInfo->isNearlyS) && isNewVertexNearlySpecular;\n"
"\n"
"	// Update last path vertex information\n"
"	pathInfo->lastBSDFEvent = event;\n"
"\n"
"	//--------------------------------------------------------------------------\n"
"	// EyePathInfo::AddVertex()\n"
"	//--------------------------------------------------------------------------\n"
"\n"
"	// Update \n"
"	//\n"
"	// Note: depth.depth has been already incremented by 1 with the depth.IncDepths(event);\n"
"	pathInfo->isNearlyCaustic = (pathInfo->depth.depth == 1) ? \n"
"		// First vertex must a nearly diffuse\n"
"		(!isNewVertexNearlySpecular) :\n"
"		// All other vertices must be nearly specular\n"
"		(pathInfo->isNearlyCaustic && isNewVertexNearlySpecular);\n"
"\n"
"	// Update last path vertex information\n"
"	pathInfo->lastBSDFPdfW = pdfW;\n"
"	const float3 shadeN = VLOAD3F(&bsdf->hitPoint.shadeN.x);\n"
"	VSTORE3F(bsdf->hitPoint.intoObject ? shadeN : -shadeN, &pathInfo->lastShadeN.x);\n"
"	pathInfo->lastFromVolume =  bsdf->isVolume;\n"
"	pathInfo->lastGlossiness = glossiness;\n"
"	\n"
"	pathInfo->isTransmittedPath = pathInfo->isTransmittedPath && (event & TRANSMIT) && (event & (SPECULAR | GLOSSY));\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE bool EyePathInfo_IsCausticPath(__global EyePathInfo *pathInfo) {\n"
"	return pathInfo->isNearlyCaustic && (pathInfo->depth.depth > 1);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE bool EyePathInfo_IsCausticPathWithEvent(__global EyePathInfo *pathInfo,\n"
"		const BSDFEvent event, const float glossiness, const float glossinessThreshold) {\n"
"	// Note: the +1 is there for the event passed as method arguments\n"
"	return pathInfo->isNearlyCaustic && (pathInfo->depth.depth + 1 > 1) &&\n"
"			EyePathInfo_IsNearlySpecular(pathInfo, event, glossiness, glossinessThreshold);\n"
"}\n";
} }
