
#include <string>

namespace slg { namespace ocl {
std::string KernelSource_pathoclbase_kernels_micro =
"#line 2 \"pathoclbase_kernels_micro.cl\"\n"
"\n"
"/***************************************************************************\n"
" * Copyright 1998-2020 by authors (see AUTHORS.txt)                        *\n"
" *                                                                         *\n"
" *   This file is part of LuxCoreRender.                                   *\n"
" *                                                                         *\n"
" * Licensed under the Apache License, Version 2.0 (the \"License\");         *\n"
" * you may not use this file except in compliance with the License.        *\n"
" * You may obtain a copy of the License at                                 *\n"
" *                                                                         *\n"
" *     http://www.apache.org/licenses/LICENSE-2.0                          *\n"
" *                                                                         *\n"
" * Unless required by applicable law or agreed to in writing, software     *\n"
" * distributed under the License is distributed on an \"AS IS\" BASIS,       *\n"
" * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.*\n"
" * See the License for the specific language governing permissions and     *\n"
" * limitations under the License.                                          *\n"
" ***************************************************************************/\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// AdvancePaths (Micro-Kernels)\n"
"//------------------------------------------------------------------------------\n"
"\n"
"//#define DEBUG_PRINTF_KERNEL_NAME 1\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Evaluation of the Path finite state machine.\n"
"//\n"
"// From: MK_RT_NEXT_VERTEX\n"
"// To: MK_HIT_NOTHING or MK_HIT_OBJECT or MK_RT_NEXT_VERTEX\n"
"//------------------------------------------------------------------------------\n"
"\n"
"__kernel void AdvancePaths_MK_RT_NEXT_VERTEX(\n"
"		KERNEL_ARGS\n"
"		) {\n"
"	const size_t gid = get_global_id(0);\n"
"	__global SampleResult *sampleResult = &sampleResultsBuff[gid];\n"
"\n"
"	// This has to be done by the first kernel to run after RT kernel\n"
"	sampleResult->rayCount += 1;\n"
"\n"
"	// Read the path state\n"
"	__global GPUTaskState *taskState = &tasksState[gid];\n"
"	PathState pathState = taskState->state;\n"
"#if defined(DEBUG_PRINTF_KERNEL_NAME)\n"
"	if (gid == 0)\n"
"		printf(\"Kernel: AdvancePaths_MK_RT_NEXT_VERTEX(state = %d)\\n\", pathState);\n"
"	else\n"
"		return;\n"
"#endif\n"
"	if (pathState != MK_RT_NEXT_VERTEX)\n"
"		return;\n"
"\n"
"	//--------------------------------------------------------------------------\n"
"	// Start of variables setup\n"
"	//--------------------------------------------------------------------------\n"
"	\n"
"	__global EyePathInfo *pathInfo = &eyePathInfos[gid];\n"
"	__constant const Scene* restrict scene = &taskConfig->scene;\n"
"\n"
"	// Initialize image maps page pointer table\n"
"	INIT_IMAGEMAPS_PAGES\n"
"\n"
"	//--------------------------------------------------------------------------\n"
"	// End of variables setup\n"
"	//--------------------------------------------------------------------------\n"
"\n"
"	float3 connectionThroughput;\n"
"\n"
"	Seed seedPassThroughEvent = taskState->seedPassThroughEvent;\n"
"	const float passThroughEvent = Rnd_FloatValue(&seedPassThroughEvent);\n"
"	taskState->seedPassThroughEvent = seedPassThroughEvent;\n"
"\n"
"	int throughShadowTransparency = taskState->throughShadowTransparency;\n"
"	const bool continueToTrace = Scene_Intersect(taskConfig,\n"
"			EYE_RAY | ((pathInfo->depth.depth == 0) ? CAMERA_RAY : INDIRECT_RAY),\n"
"			&throughShadowTransparency,\n"
"			&pathInfo->volume,\n"
"			&tasks[gid].tmpHitPoint,\n"
"			passThroughEvent,\n"
"			&rays[gid], &rayHits[gid], &taskState->bsdf,\n"
"			&connectionThroughput, VLOAD3F(taskState->throughput.c),\n"
"			sampleResult,\n"
"			false\n"
"			MATERIALS_PARAM\n"
"			);\n"
"	taskState->throughShadowTransparency = throughShadowTransparency;\n"
"	VSTORE3F(connectionThroughput * VLOAD3F(taskState->throughput.c), taskState->throughput.c);\n"
"\n"
"	// If continueToTrace, there is nothing to do, just keep the same state\n"
"	if (!continueToTrace) {\n"
"		if (rayHits[gid].meshIndex == NULL_INDEX)\n"
"			taskState->state = MK_HIT_NOTHING;\n"
"		else {\n"
"			const BSDFEvent eventTypes = BSDF_GetEventTypes(&taskState->bsdf\n"
"					MATERIALS_PARAM);\n"
"\n"
"			sampleResult->lastPathVertex = PathDepthInfo_IsLastPathVertex(&pathInfo->depth, \n"
"					&taskConfig->pathTracer.maxPathDepth, eventTypes);\n"
"\n"
"			taskState->state = MK_HIT_OBJECT;\n"
"		}\n"
"	}\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Evaluation of the Path finite state machine.\n"
"//\n"
"// From: MK_HIT_NOTHING\n"
"// To: MK_SPLAT_SAMPLE\n"
"//------------------------------------------------------------------------------\n"
"\n"
"__kernel void AdvancePaths_MK_HIT_NOTHING(\n"
"		KERNEL_ARGS\n"
"		) {\n"
"	const size_t gid = get_global_id(0);\n"
"\n"
"	// Read the path state\n"
"	__global GPUTaskState *taskState = &tasksState[gid];\n"
"	PathState pathState = taskState->state;\n"
"#if defined(DEBUG_PRINTF_KERNEL_NAME)\n"
"	if (gid == 0)\n"
"		printf(\"Kernel: AdvancePaths_MK_HIT_NOTHING(state = %d)\\n\", pathState);\n"
"	else\n"
"		return;\n"
"#endif\n"
"	if (pathState != MK_HIT_NOTHING)\n"
"		return;\n"
"\n"
"	//--------------------------------------------------------------------------\n"
"	// Start of variables setup\n"
"	//--------------------------------------------------------------------------\n"
"\n"
"	__global EyePathInfo *pathInfo = &eyePathInfos[gid];\n"
"	__constant const Scene* restrict scene = &taskConfig->scene;\n"
"	__global SampleResult *sampleResult = &sampleResultsBuff[gid];\n"
"\n"
"	// Initialize image maps page pointer table\n"
"	INIT_IMAGEMAPS_PAGES\n"
"\n"
"	//--------------------------------------------------------------------------\n"
"	// End of variables setup\n"
"	//--------------------------------------------------------------------------\n"
"\n"
"	// Nothing was hit, add environmental lights radiance\n"
"\n"
"	bool checkDirectLightHit = true;\n"
"	\n"
"	checkDirectLightHit = checkDirectLightHit &&\n"
"			(!(taskConfig->pathTracer.forceBlackBackground && pathInfo->isPassThroughPath) || !pathInfo->isPassThroughPath);\n"
"\n"
"	checkDirectLightHit = checkDirectLightHit &&\n"
"			// Avoid to render caustic path if hybridBackForwardEnable\n"
"			(!taskConfig->pathTracer.hybridBackForward.enabled || !EyePathInfo_IsCausticPath(pathInfo));\n"
"\n"
"	checkDirectLightHit = checkDirectLightHit &&\n"
"			((!taskConfig->pathTracer.pgic.indirectEnabled && !taskConfig->pathTracer.pgic.causticEnabled) ||\n"
"			PhotonGICache_IsDirectLightHitVisible(taskConfig, pathInfo, taskState->photonGICausticCacheUsed));\n"
"\n"
"	if (checkDirectLightHit) {\n"
"		DirectHitInfiniteLight(\n"
"				&taskConfig->film,\n"
"				pathInfo,\n"
"				&taskState->throughput,\n"
"				&rays[gid],\n"
"				sampleResult->firstPathVertex ? NULL : &taskState->bsdf,\n"
"				sampleResult\n"
"				LIGHTS_PARAM);\n"
"	}\n"
"\n"
"	if (pathInfo->depth.depth == 0) {\n"
"		sampleResult->alpha = 0.f;\n"
"		sampleResult->depth = INFINITY;\n"
"		sampleResult->position.x = INFINITY;\n"
"		sampleResult->position.y = INFINITY;\n"
"		sampleResult->position.z = INFINITY;\n"
"		sampleResult->geometryNormal.x = 0.f;\n"
"		sampleResult->geometryNormal.y = 0.f;\n"
"		sampleResult->geometryNormal.z = 0.f;\n"
"		sampleResult->shadingNormal.x = 0.f;\n"
"		sampleResult->shadingNormal.y = 0.f;\n"
"		sampleResult->shadingNormal.z = 0.f;\n"
"		sampleResult->materialID = 0;\n"
"		sampleResult->objectID = 0;\n"
"		sampleResult->uv.u = INFINITY;\n"
"		sampleResult->uv.v = INFINITY;\n"
"		sampleResult->isHoldout = false;\n"
"	} else if (!sampleResult->isHoldout && pathInfo->isTransmittedPath) {\n"
"		// I set to 0.0 also the alpha all purely transmitted paths hitting nothing\n"
"		sampleResult->alpha = 0.f;\n"
"	}\n"
"\n"
"	taskState->state = MK_SPLAT_SAMPLE;\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Evaluation of the Path finite state machine.\n"
"//\n"
"// From: MK_HIT_OBJECT\n"
"// To: MK_DL_ILLUMINATE or MK_SPLAT_SAMPLE\n"
"//------------------------------------------------------------------------------\n"
"\n"
"__kernel void AdvancePaths_MK_HIT_OBJECT(\n"
"		KERNEL_ARGS\n"
"		) {\n"
"	const size_t gid = get_global_id(0);\n"
"\n"
"	// Read the path state\n"
"	__global GPUTaskState *taskState = &tasksState[gid];\n"
"	PathState pathState = taskState->state;\n"
"#if defined(DEBUG_PRINTF_KERNEL_NAME)\n"
"	if (gid == 0)\n"
"		printf(\"Kernel: AdvancePaths_MK_HIT_OBJECT(state = %d)\\n\", pathState);\n"
"	else\n"
"		return;\n"
"#endif\n"
"	if (pathState != MK_HIT_OBJECT)\n"
"		return;\n"
"\n"
"	//--------------------------------------------------------------------------\n"
"	// Start of variables setup\n"
"	//--------------------------------------------------------------------------\n"
"\n"
"	__global BSDF *bsdf = &taskState->bsdf;\n"
"	__global EyePathInfo *pathInfo = &eyePathInfos[gid];\n"
"	__constant const Scene* restrict scene = &taskConfig->scene;\n"
"	__global SampleResult *sampleResult = &sampleResultsBuff[gid];\n"
"	\n"
"\n"
"	// Initialize image maps page pointer table\n"
"	INIT_IMAGEMAPS_PAGES\n"
"\n"
"	//--------------------------------------------------------------------------\n"
"	// End of variables setup\n"
"	//--------------------------------------------------------------------------\n"
"\n"
"	// Something was hit\n"
"\n"
"	if (taskState->albedoToDo && BSDF_IsAlbedoEndPoint(bsdf, taskConfig->pathTracer.albedo.specularSetting,\n"
"			taskConfig->pathTracer.albedo.specularGlossinessThreshold MATERIALS_PARAM)) {\n"
"		const float3 albedo = VLOAD3F(taskState->throughput.c) * BSDF_Albedo(bsdf\n"
"				MATERIALS_PARAM);\n"
"		VSTORE3F(albedo, sampleResult->albedo.c);\n"
"		sampleResult->shadingNormal = bsdf->hitPoint.shadeN;\n"
"\n"
"		taskState->albedoToDo = false;\n"
"	}\n"
"\n"
"	if (pathInfo->depth.depth == 0) {\n"
"		const bool isHoldout = BSDF_IsHoldout(bsdf\n"
"				MATERIALS_PARAM);\n"
"		sampleResult->alpha = isHoldout ? 0.f : 1.f;\n"
"		sampleResult->depth = rayHits[gid].t;\n"
"		sampleResult->position = bsdf->hitPoint.p;\n"
"		sampleResult->geometryNormal = bsdf->hitPoint.geometryN;\n"
"		sampleResult->materialID = BSDF_GetMaterialID(bsdf\n"
"				MATERIALS_PARAM);\n"
"		sampleResult->objectID = BSDF_GetObjectID(bsdf, sceneObjs);\n"
"		sampleResult->uv = bsdf->hitPoint.defaultUV;\n"
"		sampleResult->isHoldout = isHoldout;\n"
"	}\n"
"\n"
"	//----------------------------------------------------------------------\n"
"	// Check if it is a baked material\n"
"	//----------------------------------------------------------------------\n"
"\n"
"	if (BSDF_HasBakeMap(bsdf, COMBINED MATERIALS_PARAM)) {\n"
"		const float3 radiance = VLOAD3F(&taskState->throughput.c[0]) * BSDF_GetBakeMapValue(bsdf MATERIALS_PARAM);\n"
"		VADD3F(sampleResult->radiancePerPixelNormalized[0].c, radiance);\n"
"\n"
"		taskState->state = MK_SPLAT_SAMPLE;\n"
"		return;\n"
"	} else if (BSDF_HasBakeMap(bsdf, LIGHTMAP MATERIALS_PARAM)) {\n"
"		const float3 radiance = VLOAD3F(&taskState->throughput.c[0]) *\n"
"				BSDF_Albedo(bsdf MATERIALS_PARAM) *\n"
"				BSDF_GetBakeMapValue(bsdf MATERIALS_PARAM);\n"
"		VADD3F(sampleResult->radiancePerPixelNormalized[0].c, radiance);\n"
"\n"
"		taskState->state = MK_SPLAT_SAMPLE;\n"
"		return;\n"
"	}\n"
"\n"
"	//--------------------------------------------------------------------------\n"
"	// Check if it is a light source and I have to add light emission\n"
"	//--------------------------------------------------------------------------\n"
"\n"
"	bool checkDirectLightHit = true;\n"
"\n"
"	checkDirectLightHit = checkDirectLightHit &&\n"
"			// Avoid to render caustic path if hybridBackForwardEnable\n"
"			(!taskConfig->pathTracer.hybridBackForward.enabled || !EyePathInfo_IsCausticPath(pathInfo));\n"
"\n"
"	checkDirectLightHit = checkDirectLightHit &&\n"
"			((!taskConfig->pathTracer.pgic.indirectEnabled && !taskConfig->pathTracer.pgic.causticEnabled) ||\n"
"			PhotonGICache_IsDirectLightHitVisible(taskConfig, pathInfo, taskState->photonGICausticCacheUsed));\n"
"\n"
"	// Check if it is a light source (note: I can hit only triangle area light sources)\n"
"	if (BSDF_IsLightSource(bsdf) && checkDirectLightHit) {\n"
"		DirectHitFiniteLight(\n"
"				&taskConfig->film,\n"
"				pathInfo,\n"
"				&taskState->throughput,\n"
"				&rays[gid],\n"
"				rayHits[gid].t,\n"
"				bsdf,\n"
"				sampleResult\n"
"				LIGHTS_PARAM);\n"
"	}\n"
"\n"
"	//----------------------------------------------------------------------\n"
"	// Check if I can use the photon cache\n"
"	//----------------------------------------------------------------------\n"
"\n"
"	if (taskConfig->pathTracer.pgic.indirectEnabled || taskConfig->pathTracer.pgic.causticEnabled) {\n"
"		const bool isPhotonGIEnabled = PhotonGICache_IsPhotonGIEnabled(bsdf,\n"
"				taskConfig->pathTracer.pgic.glossinessUsageThreshold\n"
"				MATERIALS_PARAM);\n"
"\n"
"		switch (taskConfig->pathTracer.pgic.debugType) {\n"
"			case PGIC_DEBUG_SHOWINDIRECT: {\n"
"				if (isPhotonGIEnabled) {\n"
"					__global const Spectrum* restrict radiance = PhotonGICache_GetIndirectRadiance(bsdf,\n"
"							pgicRadiancePhotons, pgicLightGroupCounts, pgicRadiancePhotonsValues, pgicRadiancePhotonsBVHNodes,\n"
"							taskConfig->pathTracer.pgic.indirectLookUpRadius * taskConfig->pathTracer.pgic.indirectLookUpRadius,\n"
"							taskConfig->pathTracer.pgic.indirectLookUpNormalCosAngle);\n"
"					if (radiance) {\n"
"						for (uint i = 0; i < pgicLightGroupCounts; ++i)\n"
"							VADD3F(sampleResult->radiancePerPixelNormalized[i].c, VLOAD3F(radiance[i].c));\n"
"					}\n"
"				}\n"
"				taskState->state = MK_SPLAT_SAMPLE;\n"
"				return;\n"
"			}\n"
"			case PGIC_DEBUG_SHOWCAUSTIC: {\n"
"				if (isPhotonGIEnabled) {\n"
"					PhotonGICache_ConnectWithCausticPaths(bsdf,\n"
"							pgicCausticPhotons, pgicCausticPhotonsBVHNodes,\n"
"							taskConfig->pathTracer.pgic.causticPhotonTracedCount,\n"
"							taskConfig->pathTracer.pgic.causticLookUpRadius,\n"
"							taskConfig->pathTracer.pgic.causticLookUpNormalCosAngle,\n"
"							WHITE,\n"
"							&sampleResult->radiancePerPixelNormalized[0]\n"
"							MATERIALS_PARAM);\n"
"				}\n"
"				taskState->state = MK_SPLAT_SAMPLE;\n"
"				return;\n"
"			}\n"
"			case PGIC_DEBUG_SHOWINDIRECTPATHMIX: {\n"
"				if (isPhotonGIEnabled) {\n"
"					Seed seedPassThroughEvent = taskState->seedPassThroughEvent;\n"
"					const float passThroughEvent = Rnd_FloatValue(&seedPassThroughEvent);\n"
"\n"
"					if (taskState->photonGICacheEnabledOnLastHit &&\n"
"							(rayHits[gid].t > PhotonGICache_GetIndirectUsageThreshold(\n"
"								pathInfo->lastBSDFEvent,\n"
"								pathInfo->lastGlossiness,\n"
"								// I hope to not introduce strange sample correlations\n"
"								// by using passThrough here\n"
"								passThroughEvent,\n"
"								taskConfig->pathTracer.pgic.glossinessUsageThreshold,\n"
"								taskConfig->pathTracer.pgic.indirectUsageThresholdScale,\n"
"								taskConfig->pathTracer.pgic.indirectLookUpRadius))) {\n"
"						VSTORE3F(MAKE_FLOAT3(0.f, 0.f, 1.f), sampleResult->radiancePerPixelNormalized[0].c);\n"
"						taskState->photonGIShowIndirectPathMixUsed = true;\n"
"\n"
"						taskState->state = MK_SPLAT_SAMPLE;\n"
"						return;\n"
"					}\n"
"\n"
"					taskState->photonGICacheEnabledOnLastHit = true;\n"
"				} else\n"
"					taskState->photonGICacheEnabledOnLastHit = false;\n"
"\n"
"				break;\n"
"			}\n"
"			case PGIC_DEBUG_NONE:\n"
"			default: {\n"
"				if (isPhotonGIEnabled) {\n"
"					if (taskConfig->pathTracer.pgic.causticEnabled &&\n"
"							(!taskConfig->pathTracer.hybridBackForward.enabled || (pathInfo->depth.depth != 0))) {\n"
"						const bool isEmpty = PhotonGICache_ConnectWithCausticPaths(bsdf,\n"
"								pgicCausticPhotons, pgicCausticPhotonsBVHNodes,\n"
"								taskConfig->pathTracer.pgic.causticPhotonTracedCount,\n"
"								taskConfig->pathTracer.pgic.causticLookUpRadius,\n"
"								taskConfig->pathTracer.pgic.causticLookUpNormalCosAngle,\n"
"								VLOAD3F(taskState->throughput.c),\n"
"								&sampleResult->radiancePerPixelNormalized[0]\n"
"								MATERIALS_PARAM);\n"
"\n"
"						if (!isEmpty)\n"
"							taskState->photonGICausticCacheUsed = true;\n"
"					}\n"
"\n"
"					if (taskConfig->pathTracer.pgic.indirectEnabled) {\n"
"						Seed seedPassThroughEvent = taskState->seedPassThroughEvent;\n"
"						const float passThroughEvent = Rnd_FloatValue(&seedPassThroughEvent);\n"
"\n"
"						if (taskState->photonGICacheEnabledOnLastHit &&\n"
"								(rayHits[gid].t > PhotonGICache_GetIndirectUsageThreshold(\n"
"									pathInfo->lastBSDFEvent,\n"
"									pathInfo->lastGlossiness,\n"
"									// I hope to not introduce strange sample correlations\n"
"									// by using passThrough here\n"
"									passThroughEvent,\n"
"									taskConfig->pathTracer.pgic.glossinessUsageThreshold,\n"
"									taskConfig->pathTracer.pgic.indirectUsageThresholdScale,\n"
"									taskConfig->pathTracer.pgic.indirectLookUpRadius))) {\n"
"							__global const Spectrum* restrict radiance = PhotonGICache_GetIndirectRadiance(bsdf,\n"
"								pgicRadiancePhotons, pgicLightGroupCounts, pgicRadiancePhotonsValues, pgicRadiancePhotonsBVHNodes,\n"
"								taskConfig->pathTracer.pgic.indirectLookUpRadius * taskConfig->pathTracer.pgic.indirectLookUpRadius,\n"
"								taskConfig->pathTracer.pgic.indirectLookUpNormalCosAngle);\n"
"\n"
"							if (radiance) {\n"
"								for (uint i = 0; i < pgicLightGroupCounts; ++i)\n"
"									VADD3F(sampleResult->radiancePerPixelNormalized[i].c, VLOAD3F(taskState->throughput.c) * VLOAD3F(radiance[i].c));\n"
"							}\n"
"\n"
"							// I can terminate the path, all done\n"
"							taskState->state = MK_SPLAT_SAMPLE;\n"
"							return;\n"
"						}\n"
"					}\n"
"\n"
"					taskState->photonGICacheEnabledOnLastHit = true;\n"
"				} else\n"
"					taskState->photonGICacheEnabledOnLastHit = false;\n"
"\n"
"				break;\n"
"			}\n"
"		}\n"
"	}\n"
"\n"
"	//----------------------------------------------------------------------\n"
"	// Check if this is the last path vertex (but not also the first)\n"
"	//\n"
"	// I handle as a special case when the path vertex is both the first\n"
"	// and the last: I do direct light sampling without MIS.\n"
"	taskState->state = (sampleResult->lastPathVertex && !sampleResult->firstPathVertex) ?\n"
"		MK_SPLAT_SAMPLE : MK_DL_ILLUMINATE;\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Evaluation of the Path finite state machine.\n"
"//\n"
"// From: MK_RT_DL\n"
"// To: MK_SPLAT_SAMPLE or MK_GENERATE_NEXT_VERTEX_RAY\n"
"//------------------------------------------------------------------------------\n"
"\n"
"__kernel void AdvancePaths_MK_RT_DL(\n"
"		KERNEL_ARGS\n"
"		) {\n"
"	const size_t gid = get_global_id(0);\n"
"\n"
"	// Read the path state\n"
"	__global GPUTask *task = &tasks[gid];\n"
"	__global GPUTaskState *taskState = &tasksState[gid];\n"
"	PathState pathState = taskState->state;\n"
"#if defined(DEBUG_PRINTF_KERNEL_NAME)\n"
"	if (gid == 0)\n"
"		printf(\"Kernel: AdvancePaths_MK_RT_DL(state = %d)\\n\", pathState);\n"
"	else\n"
"		return;\n"
"#endif\n"
"	if (pathState != MK_RT_DL)\n"
"		return;\n"
"\n"
" 	//--------------------------------------------------------------------------\n"
"	// Start of variables setup\n"
"	//--------------------------------------------------------------------------\n"
"\n"
"	__global GPUTaskDirectLight *taskDirectLight = &tasksDirectLight[gid];\n"
"	__constant const Scene* restrict scene = &taskConfig->scene;\n"
"	__global SampleResult *sampleResult = &sampleResultsBuff[gid];\n"
"\n"
"	// Initialize image maps page pointer table\n"
"	INIT_IMAGEMAPS_PAGES\n"
"	\n"
"	//--------------------------------------------------------------------------\n"
"	// End of variables setup\n"
"	//--------------------------------------------------------------------------\n"
"\n"
"	float3 connectionThroughput = WHITE;\n"
"\n"
"	Seed seedPassThroughEvent = taskDirectLight->seedPassThroughEvent;\n"
"	const float passThroughEvent = Rnd_FloatValue(&seedPassThroughEvent);\n"
"	taskDirectLight->seedPassThroughEvent = seedPassThroughEvent;\n"
"\n"
"	int throughShadowTransparency = taskDirectLight->throughShadowTransparency;\n"
"	const bool continueToTrace =\n"
"		Scene_Intersect(taskConfig,\n"
"			EYE_RAY | SHADOW_RAY,\n"
"			&throughShadowTransparency,\n"
"			&directLightVolInfos[gid],\n"
"			&task->tmpHitPoint,\n"
"			passThroughEvent,\n"
"			&rays[gid], &rayHits[gid], &task->tmpBsdf,\n"
"			&connectionThroughput, WHITE,\n"
"			NULL,\n"
"			true\n"
"			MATERIALS_PARAM\n"
"			);\n"
"	taskDirectLight->throughShadowTransparency = throughShadowTransparency;\n"
"	VSTORE3F(connectionThroughput * VLOAD3F(taskDirectLight->illumInfo.lightRadiance.c), taskDirectLight->illumInfo.lightRadiance.c);\n"
"	VSTORE3F(connectionThroughput * VLOAD3F(taskDirectLight->illumInfo.lightIrradiance.c), taskDirectLight->illumInfo.lightIrradiance.c);\n"
"\n"
"	const bool rayMiss = (rayHits[gid].meshIndex == NULL_INDEX);\n"
"\n"
"	// If continueToTrace, there is nothing to do, just keep the same state\n"
"	if (!continueToTrace) {\n"
"		if (rayMiss) {\n"
"			// Nothing was hit, the light source is visible\n"
"\n"
"			__global BSDF *bsdf = &taskState->bsdf;\n"
"\n"
"			if (!BSDF_IsShadowCatcher(bsdf MATERIALS_PARAM)) {\n"
"				const float3 lightRadiance = VLOAD3F(taskDirectLight->illumInfo.lightRadiance.c);\n"
"				SampleResult_AddDirectLight(&taskConfig->film,\n"
"						sampleResult, taskDirectLight->illumInfo.lightID,\n"
"						BSDF_GetEventTypes(bsdf\n"
"							MATERIALS_PARAM),\n"
"						VLOAD3F(taskState->throughput.c), lightRadiance,\n"
"						1.f);\n"
"\n"
"				// The first path vertex is not handled by AddDirectLight(). This is valid\n"
"				// for irradiance AOV only if it is not a SPECULAR material.\n"
"				//\n"
"				// Note: irradiance samples the light sources only here (i.e. no\n"
"				// direct hit, no MIS, it would be useless)\n"
"				if ((sampleResult->firstPathVertex) && !(BSDF_GetEventTypes(bsdf\n"
"							MATERIALS_PARAM) & SPECULAR)) {\n"
"					const float3 irradiance = (M_1_PI_F * fabs(dot(\n"
"								VLOAD3F(&bsdf->hitPoint.shadeN.x),\n"
"								VLOAD3F(&rays[gid].d.x)))) *\n"
"							VLOAD3F(taskDirectLight->illumInfo.lightIrradiance.c);\n"
"					VSTORE3F(irradiance, sampleResult->irradiance.c);\n"
"				}\n"
"			}\n"
"\n"
"			taskDirectLight->directLightResult = ILLUMINATED;\n"
"		} else\n"
"			taskDirectLight->directLightResult = SHADOWED;\n"
"\n"
"		// Check if this is the last path vertex\n"
"		if (sampleResult->lastPathVertex)\n"
"			pathState = MK_SPLAT_SAMPLE;\n"
"		else\n"
"			pathState = MK_GENERATE_NEXT_VERTEX_RAY;\n"
"\n"
"		// Save the state\n"
"		taskState->state = pathState;\n"
"	}\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Evaluation of the Path finite state machine.\n"
"//\n"
"// From: MK_DL_ILLUMINATE\n"
"// To: MK_DL_SAMPLE_BSDF or MK_GENERATE_NEXT_VERTEX_RAY\n"
"//------------------------------------------------------------------------------\n"
"\n"
"__kernel void AdvancePaths_MK_DL_ILLUMINATE(\n"
"		KERNEL_ARGS\n"
"		) {\n"
"	const size_t gid = get_global_id(0);\n"
"\n"
"	// Read the path state\n"
"	__global GPUTask *task = &tasks[gid];\n"
"	__global GPUTaskState *taskState = &tasksState[gid];\n"
"	PathState pathState = taskState->state;\n"
"#if defined(DEBUG_PRINTF_KERNEL_NAME)\n"
"	if (gid == 0)\n"
"		printf(\"Kernel: AdvancePaths_MK_DL_ILLUMINATE(state = %d)\\n\", pathState);\n"
"	else\n"
"		return;\n"
"#endif\n"
"	if (pathState != MK_DL_ILLUMINATE)\n"
"		return;\n"
"\n"
" 	//--------------------------------------------------------------------------\n"
"	// Start of variables setup\n"
"	//--------------------------------------------------------------------------\n"
"\n"
"	__global EyePathInfo *pathInfo = &eyePathInfos[gid];\n"
"\n"
"	__global BSDF *bsdf = &taskState->bsdf;\n"
"\n"
"	// Read the seed\n"
"	Seed seedValue = task->seed;\n"
"	// This trick is required by SAMPLER_PARAM macro\n"
"	Seed *seed = &seedValue;\n"
"\n"
"	__global GPUTaskDirectLight *taskDirectLight = &tasksDirectLight[gid];\n"
"	__constant const Scene* restrict scene = &taskConfig->scene;\n"
"	__global SampleResult *sampleResult = &sampleResultsBuff[gid];\n"
"	const uint sampleOffset = taskConfig->pathTracer.eyeSampleBootSize + pathInfo->depth.depth * taskConfig->pathTracer.eyeSampleStepSize;\n"
"\n"
"	// Initialize image maps page pointer table\n"
"	INIT_IMAGEMAPS_PAGES\n"
"	\n"
"	//--------------------------------------------------------------------------\n"
"	// End of variables setup\n"
"	//--------------------------------------------------------------------------\n"
"\n"
"	// It will set eventually to true if the light is visible\n"
"	taskDirectLight->directLightResult = NOT_VISIBLE;\n"
"\n"
"	if (!BSDF_IsDelta(bsdf\n"
"			MATERIALS_PARAM) &&\n"
"			DirectLight_Illuminate(\n"
"				bsdf,\n"
"				&rays[gid],\n"
"				worldCenterX, worldCenterY, worldCenterZ, worldRadius,\n"
"				&task->tmpHitPoint,\n"
"				rays[gid].time,\n"
"				Sampler_GetSample(taskConfig, sampleOffset + IDX_DIRECTLIGHT_X SAMPLER_PARAM),\n"
"				Sampler_GetSample(taskConfig, sampleOffset + IDX_DIRECTLIGHT_Y SAMPLER_PARAM),\n"
"				Sampler_GetSample(taskConfig, sampleOffset + IDX_DIRECTLIGHT_Z SAMPLER_PARAM),\n"
"				Sampler_GetSample(taskConfig, sampleOffset + IDX_DIRECTLIGHT_W SAMPLER_PARAM),\n"
"				&taskDirectLight->illumInfo\n"
"				LIGHTS_PARAM)) {\n"
"		// I have now to evaluate the BSDF\n"
"		taskState->state = MK_DL_SAMPLE_BSDF;\n"
"	} else {\n"
"		// No shadow ray to trace, move to the next vertex ray\n"
"		// however, I have to Check if this is the last path vertex\n"
"		taskState->state = (sampleResult->lastPathVertex) ? MK_SPLAT_SAMPLE : MK_GENERATE_NEXT_VERTEX_RAY;\n"
"	}\n"
"\n"
"	//--------------------------------------------------------------------------\n"
"\n"
"	// Save the seed\n"
"	task->seed = seedValue;\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Evaluation of the Path finite state machine.\n"
"//\n"
"// From: MK_DL_SAMPLE_BSDF\n"
"// To: MK_GENERATE_NEXT_VERTEX_RAY or MK_RT_DL or MK_SPLAT_SAMPLE\n"
"//------------------------------------------------------------------------------\n"
"\n"
"__kernel void AdvancePaths_MK_DL_SAMPLE_BSDF(\n"
"		KERNEL_ARGS\n"
"		) {\n"
"	const size_t gid = get_global_id(0);\n"
"\n"
"	// Read the path state\n"
"	__global GPUTaskState *taskState = &tasksState[gid];\n"
"	PathState pathState = taskState->state;\n"
"#if defined(DEBUG_PRINTF_KERNEL_NAME)\n"
"	if (gid == 0)\n"
"		printf(\"Kernel: AdvancePaths_MK_DL_SAMPLE_BSDF(state = %d)\\n\", pathState);\n"
"	else\n"
"		return;\n"
"#endif\n"
"	if (pathState != MK_DL_SAMPLE_BSDF)\n"
"		return;\n"
"\n"
" 	//--------------------------------------------------------------------------\n"
"	// Start of variables setup\n"
"	//--------------------------------------------------------------------------\n"
"\n"
"	__global GPUTask *task = &tasks[gid];\n"
"	__global EyePathInfo *pathInfo = &eyePathInfos[gid];\n"
"	__constant const Scene* restrict scene = &taskConfig->scene;\n"
"	__global SampleResult *sampleResult = &sampleResultsBuff[gid];\n"
"	const uint sampleOffset = taskConfig->pathTracer.eyeSampleBootSize + pathInfo->depth.depth * taskConfig->pathTracer.eyeSampleStepSize;\n"
"\n"
"	// Initialize image maps page pointer table\n"
"	INIT_IMAGEMAPS_PAGES\n"
"	\n"
"	//--------------------------------------------------------------------------\n"
"	// End of variables setup\n"
"	//--------------------------------------------------------------------------\n"
"\n"
"	if (DirectLight_BSDFSampling(\n"
"			taskConfig,\n"
"			&tasksDirectLight[gid].illumInfo,\n"
"			rays[gid].time, sampleResult->lastPathVertex,\n"
"			pathInfo,\n"
"			&task->tmpPathDepthInfo,\n"
"			&taskState->bsdf,\n"
"			VLOAD3F(&rays[gid].d.x)\n"
"			LIGHTS_PARAM)) {\n"
"		__global GPUTask *task = &tasks[gid];\n"
"		Seed seedValue = task->seed;\n"
"		// This trick is required by SAMPLER_PARAM macro\n"
"		Seed *seed = &seedValue;\n"
"\n"
"		// Initialize the pass-through event for the shadow ray\n"
"		const float passThroughEvent = Sampler_GetSample(taskConfig, sampleOffset + IDX_DIRECTLIGHT_A SAMPLER_PARAM);\n"
"		Seed seedPassThroughEvent;\n"
"		Rnd_InitFloat(passThroughEvent, &seedPassThroughEvent);\n"
"		tasksDirectLight[gid].seedPassThroughEvent = seedPassThroughEvent;\n"
"\n"
"		// Save the seed\n"
"		task->seed = seedValue;\n"
"\n"
"		// Initialize the trough a shadow transparency flag used by Scene_Intersect()\n"
"		tasksDirectLight[gid].throughShadowTransparency = false;\n"
"\n"
"		// Make a copy of current PathVolumeInfo for tracing the\n"
"		// shadow ray\n"
"		directLightVolInfos[gid] = pathInfo->volume;\n"
"\n"
"		// I have to trace the shadow ray\n"
"		taskState->state = MK_RT_DL;\n"
"	} else {\n"
"		// No shadow ray to trace, move to the next vertex ray\n"
"		// however, I have to check if this is the last path vertex\n"
"		taskState->state = (sampleResult->lastPathVertex) ? MK_SPLAT_SAMPLE : MK_GENERATE_NEXT_VERTEX_RAY;\n"
"	}\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Evaluation of the Path finite state machine.\n"
"//\n"
"// From: MK_GENERATE_NEXT_VERTEX_RAY\n"
"// To: MK_SPLAT_SAMPLE or MK_RT_NEXT_VERTEX\n"
"//------------------------------------------------------------------------------\n"
"\n"
"__kernel void AdvancePaths_MK_GENERATE_NEXT_VERTEX_RAY(\n"
"		KERNEL_ARGS\n"
"		) {\n"
"	const size_t gid = get_global_id(0);\n"
"\n"
"	// Read the path state\n"
"	__global GPUTask *task = &tasks[gid];\n"
"	__global GPUTaskState *taskState = &tasksState[gid];\n"
"	PathState pathState = taskState->state;\n"
"#if defined(DEBUG_PRINTF_KERNEL_NAME)\n"
"	if (gid == 0)\n"
"		printf(\"Kernel: AdvancePaths_MK_GENERATE_NEXT_VERTEX_RAY(state = %d)\\n\", pathState);\n"
"	else\n"
"		return;\n"
"#endif\n"
"	if (pathState != MK_GENERATE_NEXT_VERTEX_RAY)\n"
"		return;\n"
"\n"
" 	//--------------------------------------------------------------------------\n"
"	// Start of variables setup\n"
"	//--------------------------------------------------------------------------\n"
"\n"
"	__global EyePathInfo *pathInfo = &eyePathInfos[gid];\n"
"	__global BSDF *bsdf = &taskState->bsdf;\n"
"\n"
"	// Read the seed\n"
"	Seed seedValue = task->seed;\n"
"	// This trick is required by SAMPLER_PARAM macro\n"
"	Seed *seed = &seedValue;\n"
"\n"
"	__constant const Scene* restrict scene = &taskConfig->scene;\n"
"	__global SampleResult *sampleResult = &sampleResultsBuff[gid];\n"
"	const uint sampleOffset = taskConfig->pathTracer.eyeSampleBootSize + pathInfo->depth.depth * taskConfig->pathTracer.eyeSampleStepSize;\n"
"\n"
"	// Initialize image maps page pointer table\n"
"	INIT_IMAGEMAPS_PAGES\n"
"\n"
"	__global Ray *ray = &rays[gid];\n"
"	\n"
"	//--------------------------------------------------------------------------\n"
"	// End of variables setup\n"
"	//--------------------------------------------------------------------------\n"
"\n"
"	// Sample the BSDF\n"
"	float3 sampledDir;\n"
"	float3 bsdfSample;\n"
"	float cosSampledDir;\n"
"	float bsdfPdfW;\n"
"	BSDFEvent bsdfEvent;\n"
"\n"
"	if (BSDF_IsShadowCatcher(bsdf MATERIALS_PARAM) && (tasksDirectLight[gid].directLightResult != SHADOWED)) {\n"
"		bsdfSample = BSDF_ShadowCatcherSample(bsdf,\n"
"				&sampledDir, &bsdfPdfW, &cosSampledDir, &bsdfEvent\n"
"				MATERIALS_PARAM);\n"
"\n"
"		if (sampleResult->firstPathVertex) {\n"
"			// In this case I have also to set the value of the alpha channel to 0.0\n"
"			sampleResult->alpha = 0.f;\n"
"		}\n"
"	} else {\n"
"		const float3 shadowTransparency = BSDF_GetPassThroughShadowTransparency(bsdf\n"
"				MATERIALS_PARAM);\n"
"		if (!sampleResult->firstPathVertex && !Spectrum_IsBlack(shadowTransparency) && !pathInfo->isNearlyS) {\n"
"			sampledDir = -VLOAD3F(&bsdf->hitPoint.fixedDir.x);\n"
"			bsdfSample = shadowTransparency;\n"
"			bsdfPdfW = pathInfo->lastBSDFPdfW;\n"
"			cosSampledDir = -1.f;\n"
"			bsdfEvent = pathInfo->lastBSDFEvent;\n"
"		} else {\n"
"			bsdfSample = BSDF_Sample(bsdf,\n"
"					Sampler_GetSample(taskConfig, sampleOffset + IDX_BSDF_X SAMPLER_PARAM),\n"
"					Sampler_GetSample(taskConfig, sampleOffset + IDX_BSDF_Y SAMPLER_PARAM),\n"
"					&sampledDir, &bsdfPdfW, &cosSampledDir, &bsdfEvent\n"
"					MATERIALS_PARAM);\n"
"\n"
"			pathInfo->isPassThroughPath = false;\n"
"		}\n"
"	}\n"
"\n"
"	if (sampleResult->firstPathVertex)\n"
"		sampleResult->firstPathVertexEvent = bsdfEvent;\n"
"\n"
"	EyePathInfo_AddVertex(pathInfo, bsdf, bsdfEvent, bsdfPdfW,\n"
"			taskConfig->pathTracer.hybridBackForward.glossinessThreshold\n"
"			MATERIALS_PARAM);\n"
"\n"
"	// Russian Roulette\n"
"	const bool rrEnabled = EyePathInfo_UseRR(pathInfo, taskConfig->pathTracer.rrDepth);\n"
"	const float rrProb = rrEnabled ?\n"
"		RussianRouletteProb(taskConfig->pathTracer.rrImportanceCap, bsdfSample) :\n"
"		1.f;\n"
"	const bool rrContinuePath = !rrEnabled ||\n"
"		!(rrProb < Sampler_GetSample(taskConfig, sampleOffset + IDX_RR SAMPLER_PARAM));\n"
"\n"
"	// Max. path depth\n"
"	const bool maxPathDepth = (pathInfo->depth.depth >= taskConfig->pathTracer.maxPathDepth.depth);\n"
"\n"
"	const bool continuePath = !Spectrum_IsBlack(bsdfSample) && rrContinuePath && !maxPathDepth;\n"
"	if (continuePath) {\n"
"		float3 throughputFactor = WHITE;\n"
"\n"
"		// RR increases path contribution\n"
"		throughputFactor /= rrProb;\n"
"		throughputFactor *= bsdfSample;\n"
"\n"
"		VSTORE3F(throughputFactor * VLOAD3F(taskState->throughput.c), taskState->throughput.c);\n"
"\n"
"		// This is valid for irradiance AOV only if it is not a SPECULAR material and\n"
"		// first path vertex. Set or update sampleResult.irradiancePathThroughput\n"
"		if (sampleResult->firstPathVertex) {\n"
"			if (!(BSDF_GetEventTypes(&taskState->bsdf\n"
"						MATERIALS_PARAM) & SPECULAR))\n"
"				VSTORE3F(TO_FLOAT3(M_1_PI_F * fabs(dot(\n"
"						VLOAD3F(&bsdf->hitPoint.shadeN.x),\n"
"						sampledDir)) / rrProb),\n"
"						sampleResult->irradiancePathThroughput.c);\n"
"			else\n"
"				VSTORE3F(BLACK, sampleResult->irradiancePathThroughput.c);\n"
"		} else\n"
"			VSTORE3F(throughputFactor * VLOAD3F(sampleResult->irradiancePathThroughput.c), sampleResult->irradiancePathThroughput.c);\n"
"\n"
"		Ray_Init2(ray, BSDF_GetRayOrigin(bsdf, sampledDir), sampledDir, ray->time);\n"
"\n"
"		sampleResult->firstPathVertex = false;\n"
"\n"
"		// Initialize the pass-through event seed\n"
"		//\n"
"		// Note: I use the IDX_PASSTHROUGH of the next path depth\n"
"		const uint nextSampleOffset = taskConfig->pathTracer.eyeSampleBootSize + pathInfo->depth.depth * taskConfig->pathTracer.eyeSampleStepSize;\n"
"		const float passThroughEvent = Sampler_GetSample(taskConfig, nextSampleOffset + IDX_PASSTHROUGH SAMPLER_PARAM);\n"
"		Seed seedPassThroughEvent;\n"
"		Rnd_InitFloat(passThroughEvent, &seedPassThroughEvent);\n"
"		taskState->seedPassThroughEvent = seedPassThroughEvent;\n"
"\n"
"		// Initialize the trough a shadow transparency flag used by Scene_Intersect()\n"
"		taskState->throughShadowTransparency = false;\n"
"\n"
"\n"
"		pathState = MK_RT_NEXT_VERTEX;\n"
"	} else\n"
"		pathState = MK_SPLAT_SAMPLE;\n"
"\n"
"	// Save the state\n"
"	taskState->state = pathState;\n"
"\n"
"	//--------------------------------------------------------------------------\n"
"\n"
"	// Save the seed\n"
"	task->seed = seedValue;\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Evaluation of the Path finite state machine.\n"
"//\n"
"// From: MK_SPLAT_SAMPLE\n"
"// To: MK_NEXT_SAMPLE\n"
"//------------------------------------------------------------------------------\n"
"\n"
"__kernel void AdvancePaths_MK_SPLAT_SAMPLE(\n"
"		KERNEL_ARGS\n"
"		) {\n"
"	const size_t gid = get_global_id(0);\n"
"\n"
"	// Read the path state\n"
"	__global GPUTask *task = &tasks[gid];\n"
"	__global GPUTaskState *taskState = &tasksState[gid];\n"
"	PathState pathState = taskState->state;\n"
"#if defined(DEBUG_PRINTF_KERNEL_NAME)\n"
"	if (gid == 0)\n"
"		printf(\"Kernel: AdvancePaths_MK_SPLAT_SAMPLE(state = %d)\\n\", pathState);\n"
"	else\n"
"		return;\n"
"#endif\n"
"	if (pathState != MK_SPLAT_SAMPLE)\n"
"		return;\n"
"\n"
"	//--------------------------------------------------------------------------\n"
"	// Start of variables setup\n"
"	//--------------------------------------------------------------------------\n"
"\n"
"	// Read the seed\n"
"	Seed seedValue = task->seed;\n"
"	// This trick is required by SAMPLER_PARAM macro\n"
"	Seed *seed = &seedValue;\n"
"\n"
"	__constant const Film* restrict film = &taskConfig->film;\n"
"	__global SampleResult *sampleResult = &sampleResultsBuff[gid];\n"
"\n"
"	//--------------------------------------------------------------------------\n"
"	// End of variables setup\n"
"	//--------------------------------------------------------------------------\n"
"\n"
"	// Initialize Film radiance group pointer table\n"
"	__global float *filmRadianceGroup[FILM_MAX_RADIANCE_GROUP_COUNT];\n"
"	filmRadianceGroup[0] = filmRadianceGroup0;\n"
"	filmRadianceGroup[1] = filmRadianceGroup1;\n"
"	filmRadianceGroup[2] = filmRadianceGroup2;\n"
"	filmRadianceGroup[3] = filmRadianceGroup3;\n"
"	filmRadianceGroup[4] = filmRadianceGroup4;\n"
"	filmRadianceGroup[5] = filmRadianceGroup5;\n"
"	filmRadianceGroup[6] = filmRadianceGroup6;\n"
"	filmRadianceGroup[7] = filmRadianceGroup7;\n"
"\n"
"	// Initialize Film radiance group scale table\n"
"	float3 filmRadianceGroupScale[FILM_MAX_RADIANCE_GROUP_COUNT];\n"
"	filmRadianceGroupScale[0] = MAKE_FLOAT3(filmRadianceGroupScale0_R, filmRadianceGroupScale0_G, filmRadianceGroupScale0_B);\n"
"	filmRadianceGroupScale[1] = MAKE_FLOAT3(filmRadianceGroupScale1_R, filmRadianceGroupScale1_G, filmRadianceGroupScale1_B);\n"
"	filmRadianceGroupScale[2] = MAKE_FLOAT3(filmRadianceGroupScale2_R, filmRadianceGroupScale2_G, filmRadianceGroupScale2_B);\n"
"	filmRadianceGroupScale[3] = MAKE_FLOAT3(filmRadianceGroupScale3_R, filmRadianceGroupScale3_G, filmRadianceGroupScale3_B);\n"
"	filmRadianceGroupScale[4] = MAKE_FLOAT3(filmRadianceGroupScale4_R, filmRadianceGroupScale4_G, filmRadianceGroupScale4_B);\n"
"	filmRadianceGroupScale[5] = MAKE_FLOAT3(filmRadianceGroupScale5_R, filmRadianceGroupScale5_G, filmRadianceGroupScale5_B);\n"
"	filmRadianceGroupScale[6] = MAKE_FLOAT3(filmRadianceGroupScale6_R, filmRadianceGroupScale6_G, filmRadianceGroupScale6_B);\n"
"	filmRadianceGroupScale[7] = MAKE_FLOAT3(filmRadianceGroupScale7_R, filmRadianceGroupScale7_G, filmRadianceGroupScale7_B);\n"
"\n"
"	if (sampleResult->isHoldout) {\n"
"		SampleResult_ClearRadiance(sampleResult);\n"
"		VSTORE3F(BLACK, sampleResult->albedo.c);\n"
"	}\n"
"\n"
"	if (taskConfig->pathTracer.pgic.indirectEnabled &&\n"
"			(taskConfig->pathTracer.pgic.debugType == PGIC_DEBUG_SHOWINDIRECTPATHMIX) &&\n"
"			!taskState->photonGIShowIndirectPathMixUsed)\n"
"		VSTORE3F(MAKE_FLOAT3(1.f, 0.f, 0.f), sampleResult->radiancePerPixelNormalized[0].c);\n"
"\n"
"	//--------------------------------------------------------------------------\n"
"	// Variance clamping\n"
"	//--------------------------------------------------------------------------\n"
"\n"
"	const float sqrtVarianceClampMaxValue = taskConfig->pathTracer.sqrtVarianceClampMaxValue;\n"
"	if (sqrtVarianceClampMaxValue > 0.f) {\n"
"		// Radiance clamping\n"
"		VarianceClamping_Clamp(sampleResult, sqrtVarianceClampMaxValue\n"
"				FILM_PARAM);\n"
"	}\n"
"\n"
"	//--------------------------------------------------------------------------\n"
"	// Sampler splat sample\n"
"	//--------------------------------------------------------------------------\n"
"\n"
"	Sampler_SplatSample(taskConfig\n"
"			SAMPLER_PARAM\n"
"			FILM_PARAM);\n"
"	taskStats[gid].sampleCount += 1;\n"
"\n"
"	// Save the state\n"
"	taskState->state = MK_NEXT_SAMPLE;\n"
"\n"
"	//--------------------------------------------------------------------------\n"
"\n"
"	// Save the seed\n"
"	task->seed = seedValue;\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Evaluation of the Path finite state machine.\n"
"//\n"
"// From: MK_NEXT_SAMPLE\n"
"// To: MK_GENERATE_CAMERA_RAY\n"
"//------------------------------------------------------------------------------\n"
"\n"
"__kernel void AdvancePaths_MK_NEXT_SAMPLE(\n"
"		KERNEL_ARGS\n"
"		) {\n"
"	const size_t gid = get_global_id(0);\n"
"\n"
"	// Read the path state\n"
"	__global GPUTask *task = &tasks[gid];\n"
"	__global GPUTaskState *taskState = &tasksState[gid];\n"
"	PathState pathState = taskState->state;\n"
"#if defined(DEBUG_PRINTF_KERNEL_NAME)\n"
"	if (gid == 0)\n"
"		printf(\"Kernel: AdvancePaths_MK_NEXT_SAMPLE(state = %d)\\n\", pathState);\n"
"	else\n"
"		return;\n"
"#endif\n"
"	if (pathState != MK_NEXT_SAMPLE)\n"
"		return;\n"
"\n"
"	//--------------------------------------------------------------------------\n"
"	// Start of variables setup\n"
"	//--------------------------------------------------------------------------\n"
"\n"
"	// Read the seed\n"
"	Seed seedValue = task->seed;\n"
"	// This trick is required by SAMPLER_PARAM macro\n"
"	Seed *seed = &seedValue;\n"
"\n"
"	//--------------------------------------------------------------------------\n"
"	// End of variables setup\n"
"	//--------------------------------------------------------------------------\n"
"\n"
"	Sampler_NextSample(taskConfig,\n"
"			filmNoise,\n"
"			filmUserImportance,\n"
"			filmWidth, filmHeight,\n"
"			filmSubRegion0, filmSubRegion1, filmSubRegion2, filmSubRegion3\n"
"			SAMPLER_PARAM);\n"
"\n"
"	// Save the state\n"
"\n"
"	// Generate a new path and camera ray only it is not TILEPATHOCL\n"
"#if !defined(RENDER_ENGINE_TILEPATHOCL) && !defined(RENDER_ENGINE_RTPATHOCL)\n"
"	taskState->state = MK_GENERATE_CAMERA_RAY;\n"
"#else\n"
"	taskState->state = MK_DONE;\n"
"	// Mark the ray like like one to NOT trace\n"
"	rays[gid].flags = RAY_FLAGS_MASKED;\n"
"#endif\n"
"\n"
"	//--------------------------------------------------------------------------\n"
"\n"
"	// Save the seed\n"
"	task->seed = seedValue;\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Evaluation of the Path finite state machine.\n"
"//\n"
"// From: MK_GENERATE_CAMERA_RAY\n"
"// To: MK_RT_NEXT_VERTEX\n"
"//------------------------------------------------------------------------------\n"
"\n"
"__kernel void AdvancePaths_MK_GENERATE_CAMERA_RAY(\n"
"		KERNEL_ARGS\n"
"		) {\n"
"	// Generate a new path and camera ray only it is not TILEPATHOCL: path regeneration\n"
"	// is not used in this case\n"
"#if !defined(RENDER_ENGINE_TILEPATHOCL) && !defined(RENDER_ENGINE_RTPATHOCL)\n"
"	const size_t gid = get_global_id(0);\n"
"\n"
"	// Read the path state\n"
"	__global GPUTask *task = &tasks[gid];\n"
"	__global GPUTaskState *taskState = &tasksState[gid];\n"
"	PathState pathState = taskState->state;\n"
"#if defined(DEBUG_PRINTF_KERNEL_NAME)\n"
"	if (gid == 0)\n"
"		printf(\"Kernel: AdvancePaths_MK_GENERATE_CAMERA_RAY(state = %d)\\n\", pathState);\n"
"	else\n"
"		return;\n"
"#endif\n"
"	if (pathState != MK_GENERATE_CAMERA_RAY)\n"
"		return;\n"
"\n"
"	//--------------------------------------------------------------------------\n"
"	// Start of variables setup\n"
"	//--------------------------------------------------------------------------\n"
"\n"
"	// Read the seed\n"
"	Seed seedValue = task->seed;\n"
"	// This trick is required by SAMPLER_PARAM macro\n"
"	Seed *seed = &seedValue;\n"
"\n"
"	__global Ray *ray = &rays[gid];\n"
"	__global EyePathInfo *pathInfo = &eyePathInfos[gid];\n"
"	\n"
"	//--------------------------------------------------------------------------\n"
"	// End of variables setup\n"
"	//--------------------------------------------------------------------------\n"
"\n"
"	// Re-initialize the volume information\n"
"	PathVolumeInfo_Init(&pathInfo->volume);\n"
"\n"
"	GenerateEyePath(taskConfig,\n"
"			&tasksDirectLight[gid], taskState,\n"
"			camera,\n"
"			cameraBokehDistribution,\n"
"			filmWidth, filmHeight,\n"
"			filmSubRegion0, filmSubRegion1, filmSubRegion2, filmSubRegion3,\n"
"			pixelFilterDistribution,\n"
"			ray,\n"
"			pathInfo\n"
"			SAMPLER_PARAM);\n"
"	// taskState->state is set to RT_NEXT_VERTEX inside GenerateEyePath()\n"
"\n"
"	//--------------------------------------------------------------------------\n"
"\n"
"	// Save the seed\n"
"	task->seed = seedValue;\n"
"\n"
"#endif\n"
"}\n"
"\n";
} }
