
#include <string>

namespace slg { namespace ocl {
std::string KernelSource_sampler_funcs =
"#line 2 \"sampler_funcs.cl\"\n"
"\n"
"/***************************************************************************\n"
" * Copyright 1998-2020 by authors (see AUTHORS.txt)                        *\n"
" *                                                                         *\n"
" *   This file is part of LuxCoreRender.                                   *\n"
" *                                                                         *\n"
" * Licensed under the Apache License, Version 2.0 (the \"License\");         *\n"
" * you may not use this file except in compliance with the License.        *\n"
" * You may obtain a copy of the License at                                 *\n"
" *                                                                         *\n"
" *     http://www.apache.org/licenses/LICENSE-2.0                          *\n"
" *                                                                         *\n"
" * Unless required by applicable law or agreed to in writing, software     *\n"
" * distributed under the License is distributed on an \"AS IS\" BASIS,       *\n"
" * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.*\n"
" * See the License for the specific language governing permissions and     *\n"
" * limitations under the License.                                          *\n"
" ***************************************************************************/\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Sampler functions\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE float Sampler_GetSample(\n"
"		__constant const GPUTaskConfiguration* restrict taskConfig,\n"
"		const uint index\n"
"		SAMPLER_PARAM_DECL) {\n"
"	switch (taskConfig->sampler.type) {\n"
"		case RANDOM:\n"
"			return RandomSampler_GetSample(taskConfig, index SAMPLER_PARAM);\n"
"		case SOBOL:\n"
"			return SobolSampler_GetSample(taskConfig, index SAMPLER_PARAM);\n"
"		case METROPOLIS:\n"
"			return MetropolisSampler_GetSample(taskConfig, index SAMPLER_PARAM);\n"
"		case TILEPATHSAMPLER:\n"
"			return TilePathSampler_GetSample(taskConfig, index SAMPLER_PARAM);\n"
"		default:\n"
"			// Something has gone very wrong here\n"
"			return 0.f;\n"
"	}\n"
"}\n"
"\n"
"// Cuda reports large argument size, so overrides noinline attribute anyway\n"
"OPENCL_FORCE_INLINE void Sampler_SplatSample(\n"
"		__constant const GPUTaskConfiguration* restrict taskConfig\n"
"		SAMPLER_PARAM_DECL\n"
"		FILM_PARAM_DECL\n"
"		) {\n"
"	switch (taskConfig->sampler.type) {\n"
"		case RANDOM:\n"
"			return RandomSampler_SplatSample(taskConfig\n"
"					SAMPLER_PARAM\n"
"					FILM_PARAM);\n"
"		case SOBOL:\n"
"			return SobolSampler_SplatSample(taskConfig\n"
"					SAMPLER_PARAM\n"
"					FILM_PARAM);\n"
"		case METROPOLIS:\n"
"			return MetropolisSampler_SplatSample(taskConfig\n"
"					SAMPLER_PARAM\n"
"					FILM_PARAM);\n"
"		case TILEPATHSAMPLER:\n"
"			return TilePathSampler_SplatSample(taskConfig\n"
"					SAMPLER_PARAM\n"
"					FILM_PARAM);\n"
"		default:\n"
"			// Something has gone very wrong here\n"
"			return;\n"
"	}\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE void Sampler_NextSample(\n"
"		__constant const GPUTaskConfiguration* restrict taskConfig,\n"
"		__global float *filmNoise,\n"
"		__global float *filmUserImportance,\n"
"		const uint filmWidth, const uint filmHeight,\n"
"		const uint filmSubRegion0, const uint filmSubRegion1,\n"
"		const uint filmSubRegion2, const uint filmSubRegion3\n"
"		SAMPLER_PARAM_DECL) {\n"
"	switch (taskConfig->sampler.type) {\n"
"		case RANDOM:\n"
"			return RandomSampler_NextSample(taskConfig,\n"
"					filmNoise,\n"
"					filmUserImportance,\n"
"					filmWidth, filmHeight,\n"
"					filmSubRegion0, filmSubRegion1,\n"
"					filmSubRegion2, filmSubRegion3\n"
"					SAMPLER_PARAM);\n"
"		case SOBOL:\n"
"			return SobolSampler_NextSample(taskConfig,\n"
"					filmNoise,\n"
"					filmUserImportance,\n"
"					filmWidth, filmHeight,\n"
"					filmSubRegion0, filmSubRegion1,\n"
"					filmSubRegion2, filmSubRegion3\n"
"					SAMPLER_PARAM);\n"
"		case METROPOLIS:\n"
"			return MetropolisSampler_NextSample(taskConfig,\n"
"					filmNoise,\n"
"					filmUserImportance,\n"
"					filmWidth, filmHeight,\n"
"					filmSubRegion0, filmSubRegion1,\n"
"					filmSubRegion2, filmSubRegion3\n"
"					SAMPLER_PARAM);\n"
"		case TILEPATHSAMPLER:\n"
"			return TilePathSampler_NextSample(taskConfig,\n"
"					filmNoise,\n"
"					filmUserImportance,\n"
"					filmWidth, filmHeight,\n"
"					filmSubRegion0, filmSubRegion1,\n"
"					filmSubRegion2, filmSubRegion3\n"
"					SAMPLER_PARAM);\n"
"		default:\n"
"			// Something has gone very wrong here\n"
"			return;\n"
"	}\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE bool Sampler_Init(\n"
"		__constant const GPUTaskConfiguration* restrict taskConfig,\n"
"		__global float *filmNoise,\n"
"		__global float *filmUserImportance,\n"
"		const uint filmWidth, const uint filmHeight,\n"
"		const uint filmSubRegion0, const uint filmSubRegion1,\n"
"		const uint filmSubRegion2, const uint filmSubRegion3\n"
"		SAMPLER_PARAM_DECL) {\n"
"	switch (taskConfig->sampler.type) {\n"
"		case RANDOM:\n"
"			return RandomSampler_Init(taskConfig,\n"
"					filmNoise,\n"
"					filmUserImportance,\n"
"					filmWidth, filmHeight,\n"
"					filmSubRegion0, filmSubRegion1,\n"
"					filmSubRegion2, filmSubRegion3\n"
"					SAMPLER_PARAM);\n"
"		case SOBOL:\n"
"			return SobolSampler_Init(taskConfig,\n"
"					filmNoise,\n"
"					filmUserImportance,\n"
"					filmWidth, filmHeight,\n"
"					filmSubRegion0, filmSubRegion1,\n"
"					filmSubRegion2, filmSubRegion3\n"
"					SAMPLER_PARAM);\n"
"		case METROPOLIS:\n"
"			return MetropolisSampler_Init(taskConfig,\n"
"					filmNoise,\n"
"					filmUserImportance,\n"
"					filmWidth, filmHeight,\n"
"					filmSubRegion0, filmSubRegion1,\n"
"					filmSubRegion2, filmSubRegion3\n"
"					SAMPLER_PARAM);\n"
"		case TILEPATHSAMPLER:\n"
"			return TilePathSampler_Init(taskConfig,\n"
"					filmNoise,\n"
"					filmUserImportance,\n"
"					filmWidth, filmHeight,\n"
"					filmSubRegion0, filmSubRegion1,\n"
"					filmSubRegion2, filmSubRegion3\n"
"					SAMPLER_PARAM);\n"
"		default:\n"
"			// Something has gone very wrong here\n"
"			return true;\n"
"	}\n"
"}\n";
} }
