
#include <string>

namespace slg { namespace ocl {
std::string KernelSource_sampler_sobol_funcs =
"#line 2 \"sampler_sobol_funcs.cl\"\n"
"\n"
"/***************************************************************************\n"
" * Copyright 1998-2020 by authors (see AUTHORS.txt)                        *\n"
" *                                                                         *\n"
" *   This file is part of LuxCoreRender.                                   *\n"
" *                                                                         *\n"
" * Licensed under the Apache License, Version 2.0 (the \"License\");         *\n"
" * you may not use this file except in compliance with the License.        *\n"
" * You may obtain a copy of the License at                                 *\n"
" *                                                                         *\n"
" *     http://www.apache.org/licenses/LICENSE-2.0                          *\n"
" *                                                                         *\n"
" * Unless required by applicable law or agreed to in writing, software     *\n"
" * distributed under the License is distributed on an \"AS IS\" BASIS,       *\n"
" * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.*\n"
" * See the License for the specific language governing permissions and     *\n"
" * limitations under the License.                                          *\n"
" ***************************************************************************/\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Sobol Sequence\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE uint SobolSequence_SobolDimension(\n"
"		__global const uint* restrict sobolDirections,\n"
"		const uint index, const uint dimension) {\n"
"	const uint offset = dimension * SOBOL_BITS;\n"
"	uint result = 0;\n"
"	uint i = index;\n"
"\n"
"	for (uint j = 0; i; i >>= 1, j++) {\n"
"		if (i & 1)\n"
"			result ^= sobolDirections[offset + j];\n"
"	}\n"
"\n"
"	return result;\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float SobolSequence_GetSample(\n"
"		__global const uint* restrict sobolDirections,\n"
"		const uint pass, const uint rngPass, const float rng0, const float rng1,\n"
"		const uint index) {\n"
"	// I scramble pass too in order avoid correlations visible with LIGHTCPU and PATHCPU\n"
"	const uint iResult = SobolSequence_SobolDimension(sobolDirections, pass + rngPass, index);\n"
"	const float fResult = iResult * (1.f / 0xffffffffu);\n"
"\n"
"	// Cranley-Patterson rotation to reduce visible regular patterns\n"
"	const float shift = (index & 1) ? rng0 : rng1;\n"
"	const float val = fResult + shift;\n"
"\n"
"	return val - floor(val);\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Sobol Sampler Kernel\n"
"//------------------------------------------------------------------------------\n"
"\n"
"#define SOBOLSAMPLER_TOTAL_U_SIZE 2\n"
"\n"
"OPENCL_FORCE_INLINE __global uint *SobolSampler_GetPassesPtr(__global SobolSamplerSharedData *samplerSharedData) {\n"
"	// Sobol pass array is appended at the end of slg::ocl::SobolSamplerSharedData\n"
"	return (__global uint *)(\n"
"			(__global char *)samplerSharedData +\n"
"			sizeof(SobolSamplerSharedData));\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE __global const uint* restrict SobolSampler_GetSobolDirectionsPtr(__global SobolSamplerSharedData *samplerSharedData) {\n"
"	// Sobol directions array is appended at the end of slg::ocl::SobolSamplerSharedData + all pass values\n"
"	return (__global uint *)(\n"
"			(__global char *)samplerSharedData +\n"
"			sizeof(SobolSamplerSharedData) +\n"
"			sizeof(uint) * samplerSharedData->filmRegionPixelCount);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float SobolSampler_GetSample(\n"
"		__constant const GPUTaskConfiguration* restrict taskConfig,\n"
"		const uint index\n"
"		SAMPLER_PARAM_DECL) {\n"
"	const size_t gid = get_global_id(0);\n"
"\n"
"	switch (index) {\n"
"		case IDX_SCREEN_X: {\n"
"			__global float *samplesData = &samplesDataBuff[gid * SOBOLSAMPLER_TOTAL_U_SIZE];\n"
"			return samplesData[IDX_SCREEN_X];\n"
"		}\n"
"		case IDX_SCREEN_Y: {\n"
"			__global float *samplesData = &samplesDataBuff[gid * SOBOLSAMPLER_TOTAL_U_SIZE];\n"
"			return samplesData[IDX_SCREEN_Y];\n"
"		}\n"
"		default: {\n"
"			__global SobolSamplerSharedData *samplerSharedData = (__global SobolSamplerSharedData *)samplerSharedDataBuff;\n"
"			__global const uint* restrict sobolDirections = SobolSampler_GetSobolDirectionsPtr(samplerSharedData);\n"
"\n"
"			__global SobolSample *samples = (__global SobolSample *)samplesBuff;\n"
"			__global SobolSample *sample = &samples[gid];\n"
"\n"
"			return SobolSequence_GetSample(sobolDirections, sample->pass, sample->rngPass, sample->rng0, sample->rng1, index);	\n"
"		}\n"
"	}\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void SobolSampler_SplatSample(\n"
"		__constant const GPUTaskConfiguration* restrict taskConfig\n"
"		SAMPLER_PARAM_DECL\n"
"		FILM_PARAM_DECL\n"
"		) {\n"
"	const size_t gid = get_global_id(0);\n"
"	__global SampleResult *sampleResult = &sampleResultsBuff[gid];\n"
"\n"
"	Film_AddSample(sampleResult->pixelX, sampleResult->pixelY,\n"
"			sampleResult, 1.f\n"
"			FILM_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void SobolSamplerSharedData_GetNewBucket(__global SobolSamplerSharedData *samplerSharedData,\n"
"		const uint bucketCount, uint *newBucketIndex, uint *seed) {\n"
"	*newBucketIndex = atomic_inc(&samplerSharedData->bucketIndex) % bucketCount;\n"
"\n"
"    *seed = (samplerSharedData->seedBase + *newBucketIndex) % (0xFFFFFFFFu - 1u) + 1u;\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void SobolSampler_InitNewSample(\n"
"		__constant const GPUTaskConfiguration* restrict taskConfig,\n"
"		__global float *filmNoise,\n"
"		__global float * filmUserImportance,\n"
"		const uint filmWidth, const uint filmHeight,\n"
"		const uint filmSubRegion0, const uint filmSubRegion1,\n"
"		const uint filmSubRegion2, const uint filmSubRegion3\n"
"		SAMPLER_PARAM_DECL) {\n"
"	const size_t gid = get_global_id(0);\n"
"	__constant const Sampler *sampler = &taskConfig->sampler;\n"
"	__global SobolSamplerSharedData *samplerSharedData = (__global SobolSamplerSharedData *)samplerSharedDataBuff;\n"
"	__global SobolSample *samples = (__global SobolSample *)samplesBuff;\n"
"	__global SobolSample *sample = &samples[gid];\n"
"	__global float *samplesData = &samplesDataBuff[gid * SOBOLSAMPLER_TOTAL_U_SIZE];\n"
"\n"
"	const uint bucketSize = sampler->sobol.bucketSize;\n"
"	const uint tileSize = sampler->sobol.tileSize;\n"
"	const uint superSampling = sampler->sobol.superSampling;\n"
"	const uint overlapping = sampler->sobol.overlapping;\n"
"	\n"
"	const uint subRegionWidth = filmSubRegion1 - filmSubRegion0 + 1;\n"
"	const uint subRegionHeight = filmSubRegion3 - filmSubRegion2 + 1;\n"
"\n"
"	const uint tiletWidthCount = (subRegionWidth + tileSize - 1) / tileSize;\n"
"	const uint tileHeightCount = (subRegionHeight + tileSize - 1) / tileSize;\n"
"\n"
"	const uint bucketCount = overlapping * (tiletWidthCount * tileSize * tileHeightCount * tileSize + bucketSize - 1) / bucketSize;\n"
"\n"
"	// Pick the pixel to render\n"
"\n"
"	uint bucketIndex = sample->bucketIndex;\n"
"	uint pixelOffset = sample->pixelOffset;\n"
"	uint passOffset = sample->passOffset;\n"
"\n"
"	Seed rngGeneratorSeed = sample->rngGeneratorSeed;\n"
"	for (;;) {\n"
"		passOffset++;\n"
"		if (passOffset >= superSampling) {\n"
"			pixelOffset++;\n"
"			passOffset = 0;\n"
"\n"
"			if (pixelOffset >= bucketSize) {\n"
"				// Ask for a new bucket\n"
"				uint bucketSeed;\n"
"				SobolSamplerSharedData_GetNewBucket(samplerSharedData, bucketCount,\n"
"						&bucketIndex, &bucketSeed);\n"
"\n"
"				sample->bucketIndex = bucketIndex;\n"
"				pixelOffset = 0;\n"
"				passOffset = 0;\n"
"\n"
"				Rnd_Init(bucketSeed, &rngGeneratorSeed);\n"
"			}\n"
"		}\n"
"\n"
"		// Transform the bucket index in a pixel coordinate\n"
"\n"
"		const uint pixelBucketIndex = (bucketIndex / overlapping) * bucketSize + pixelOffset;\n"
"		const uint mortonCurveOffset = pixelBucketIndex % (tileSize * tileSize);\n"
"		const uint pixelTileIndex = pixelBucketIndex / (tileSize * tileSize);\n"
"\n"
"		const uint subRegionPixelX = (pixelTileIndex % tiletWidthCount) * tileSize + DecodeMorton2X(mortonCurveOffset);\n"
"		const uint subRegionPixelY = (pixelTileIndex / tiletWidthCount) * tileSize + DecodeMorton2Y(mortonCurveOffset);\n"
"		if ((subRegionPixelX >= subRegionWidth) || (subRegionPixelY >= subRegionHeight)) {\n"
"			// Skip the pixels out of the film sub region\n"
"			continue;\n"
"		}\n"
"\n"
"		const uint pixelX = filmSubRegion0 + subRegionPixelX;\n"
"		const uint pixelY = filmSubRegion2 + subRegionPixelY;\n"
"\n"
"		if (filmNoise) {\n"
"			const float adaptiveStrength = sampler->sobol.adaptiveStrength;\n"
"\n"
"			if (adaptiveStrength > 0.f) {\n"
"				// Pixels are sampled in accordance with how far from convergence they are\n"
"				const float noise = filmNoise[pixelX + pixelY * filmWidth];\n"
"\n"
"				// Factor user driven importance sampling too\n"
"				float threshold;\n"
"				if (filmUserImportance) {\n"
"					const float userImportance = filmUserImportance[pixelX + pixelY * filmWidth];\n"
"\n"
"					// Noise is initialized to INFINITY at start\n"
"					if (isinf(noise))\n"
"						threshold = userImportance;\n"
"					else\n"
"						threshold = (userImportance > 0.f) ? Lerp(sampler->sobol.adaptiveUserImportanceWeight, noise, userImportance) : 0.f;\n"
"				} else\n"
"					threshold = noise;\n"
"\n"
"				// The floor for the pixel importance is given by the adaptiveness strength\n"
"				threshold = fmax(threshold, 1.f - adaptiveStrength);\n"
"\n"
"				if (Rnd_FloatValue(seed) > threshold) {\n"
"					// Skip this pixel and try the next one\n"
"\n"
"					// Workaround for preserving random number distribution behavior\n"
"					Rnd_UintValue(&rngGeneratorSeed);\n"
"					Rnd_FloatValue(&rngGeneratorSeed);\n"
"					Rnd_FloatValue(&rngGeneratorSeed);\n"
"\n"
"					continue;\n"
"				}\n"
"			}\n"
"		}\n"
"\n"
"		//----------------------------------------------------------------------\n"
"		// This code crashes the AMD OpenCL compiler:\n"
"		//\n"
"		// The array of fields is attached to the SamplerSharedData structure\n"
"#if !defined(LUXCORE_AMD_OPENCL)\n"
"		__global uint *pixelPasses = SobolSampler_GetPassesPtr(samplerSharedData);\n"
"		// Get the pass to do\n"
"		sample->pass = atomic_inc(&pixelPasses[subRegionPixelX + subRegionPixelY * subRegionWidth]);\n"
"#else   //----------------------------------------------------------------------\n"
"		// This one works:\n"
"		//\n"
"		// The array of fields is attached to the SamplerSharedData structure\n"
"		__global uint *pixelPass = SobolSampler_GetPassesPtr(samplerSharedData) + (subRegionPixelX + subRegionPixelY * subRegionWidth);\n"
"		// Get the pass to do\n"
"		uint oldVal, newVal;\n"
"		do {\n"
"				oldVal = *pixelPass;\n"
"				newVal = oldVal + 1;\n"
"		} while (atomic_cmpxchg(pixelPass, oldVal, newVal) != oldVal);\n"
"		sample->pass = oldVal;\n"
"#endif\n"
"		//----------------------------------------------------------------------\n"
"\n"
"		// Initialize rng0 and rng1\n"
"\n"
"		// Limit the number of pass skipped\n"
"		sample->rngPass = Rnd_UintValue(&rngGeneratorSeed);\n"
"		sample->rng0 = Rnd_FloatValue(&rngGeneratorSeed);\n"
"		sample->rng1 = Rnd_FloatValue(&rngGeneratorSeed);\n"
"\n"
"		// Initialize IDX_SCREEN_X and IDX_SCREEN_Y sample\n"
"\n"
"		__global const uint* restrict sobolDirections = SobolSampler_GetSobolDirectionsPtr(samplerSharedData);\n"
"		samplesData[IDX_SCREEN_X] = pixelX + SobolSequence_GetSample(sobolDirections, sample->pass, sample->rngPass, sample->rng0, sample->rng1, IDX_SCREEN_X);\n"
"		samplesData[IDX_SCREEN_Y] = pixelY + SobolSequence_GetSample(sobolDirections, sample->pass, sample->rngPass, sample->rng0, sample->rng1, IDX_SCREEN_Y);\n"
"		break;\n"
"	}\n"
"	\n"
"	sample->rngGeneratorSeed = rngGeneratorSeed;\n"
"\n"
"	// Save the new values\n"
"	sample->pixelOffset = pixelOffset;\n"
"	sample->passOffset = passOffset;\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void SobolSampler_NextSample(\n"
"		__constant const GPUTaskConfiguration* restrict taskConfig,\n"
"		__global float *filmNoise,\n"
"		__global float *filmUserImportance,\n"
"		const uint filmWidth, const uint filmHeight,\n"
"		const uint filmSubRegion0, const uint filmSubRegion1,\n"
"		const uint filmSubRegion2, const uint filmSubRegion3\n"
"		SAMPLER_PARAM_DECL) {\n"
"	SobolSampler_InitNewSample(taskConfig,\n"
"			filmNoise,\n"
"			filmUserImportance,\n"
"			filmWidth, filmHeight,\n"
"			filmSubRegion0, filmSubRegion1, filmSubRegion2, filmSubRegion3\n"
"			SAMPLER_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE bool SobolSampler_Init(__constant const GPUTaskConfiguration* restrict taskConfig,\n"
"		__global float *filmNoise,\n"
"		__global float *filmUserImportance,\n"
"		const uint filmWidth, const uint filmHeight,\n"
"		const uint filmSubRegion0, const uint filmSubRegion1,\n"
"		const uint filmSubRegion2, const uint filmSubRegion3\n"
"		SAMPLER_PARAM_DECL) {\n"
"	const size_t gid = get_global_id(0);\n"
"	__constant const Sampler *sampler = &taskConfig->sampler;\n"
"	__global SobolSample *samples = (__global SobolSample *)samplesBuff;\n"
"	__global SobolSample *sample = &samples[gid];\n"
"\n"
"	const uint bucketSize = sampler->sobol.bucketSize;\n"
"	sample->pixelOffset = bucketSize * bucketSize;\n"
"	sample->passOffset = sampler->sobol.superSampling;\n"
"\n"
"	SobolSampler_NextSample(taskConfig,\n"
"			filmNoise,\n"
"			filmUserImportance,\n"
"			filmWidth, filmHeight,\n"
"			filmSubRegion0, filmSubRegion1, filmSubRegion2, filmSubRegion3\n"
"			SAMPLER_PARAM);\n"
"\n"
"	return true;\n"
"}\n";
} }
