
#include <string>

namespace slg { namespace ocl {
std::string KernelSource_scene_funcs =
"#line 2 \"scene_funcs.cl\"\n"
"\n"
"/***************************************************************************\n"
" * Copyright 1998-2020 by authors (see AUTHORS.txt)                        *\n"
" *                                                                         *\n"
" *   This file is part of LuxCoreRender.                                   *\n"
" *                                                                         *\n"
" * Licensed under the Apache License, Version 2.0 (the \"License\");         *\n"
" * you may not use this file except in compliance with the License.        *\n"
" * You may obtain a copy of the License at                                 *\n"
" *                                                                         *\n"
" *     http://www.apache.org/licenses/LICENSE-2.0                          *\n"
" *                                                                         *\n"
" * Unless required by applicable law or agreed to in writing, software     *\n"
" * distributed under the License is distributed on an \"AS IS\" BASIS,       *\n"
" * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.*\n"
" * See the License for the specific language governing permissions and     *\n"
" * limitations under the License.                                          *\n"
" ***************************************************************************/\n"
"\n"
"OPENCL_FORCE_NOT_INLINE bool Scene_Intersect(\n"
"		__constant const GPUTaskConfiguration* restrict taskConfig,\n"
"		const SceneRayType rayType,\n"
"		int *throughShadowTransparency,\n"
"		__global PathVolumeInfo *volInfo,\n"
"		__global HitPoint *tmpHitPoint,\n"
"		const float passThrough,\n"
"		__global Ray *ray,\n"
"		__global RayHit *rayHit,\n"
"		__global BSDF *bsdf,\n"
"		float3 *connectionThroughput,  const float3 pathThroughput,\n"
"		__global SampleResult *sampleResult,\n"
"		const bool backTracing\n"
"		MATERIALS_PARAM_DECL\n"
"		) {\n"
"	*connectionThroughput = WHITE;\n"
"	\n"
"	const bool cameraRay = rayType & CAMERA_RAY;\n"
"	const bool shadowRay = rayType & SHADOW_RAY;\n"
"\n"
"	const bool hit = (rayHit->meshIndex != NULL_INDEX);\n"
"\n"
"	uint rayVolumeIndex = volInfo->currentVolumeIndex;\n"
"\n"
"	if (hit) {\n"
"		// Initialize the BSDF of the hit point\n"
"		BSDF_Init(bsdf,\n"
"				*throughShadowTransparency,\n"
"				ray, rayHit,\n"
"				passThrough,\n"
"				volInfo\n"
"				MATERIALS_PARAM\n"
"				);\n"
"\n"
"		rayVolumeIndex = bsdf->hitPoint.intoObject ? bsdf->hitPoint.exteriorVolumeIndex : bsdf->hitPoint.interiorVolumeIndex;\n"
"	} else if (rayVolumeIndex == NULL_INDEX) {\n"
"		// No volume information, I use the default volume\n"
"		rayVolumeIndex = scene->defaultVolumeIndex;\n"
"	}\n"
"\n"
"	// Check if there is volume scatter event\n"
"	if (rayVolumeIndex != NULL_INDEX) {\n"
"		// This applies volume transmittance too\n"
"		// Note: by using passThrough here, I introduce subtle correlation\n"
"		// between scattering events and pass-through events\n"
"		float3 connectionEmission = BLACK;\n"
"\n"
"		const float t = Volume_Scatter(&mats[rayVolumeIndex], ray,\n"
"				hit ? rayHit->t : ray->maxt,\n"
"				passThrough, volInfo->scatteredStart,\n"
"				connectionThroughput, &connectionEmission,\n"
"				tmpHitPoint\n"
"				TEXTURES_PARAM);\n"
"\n"
"		// Add the volume emitted light to the appropriate light group\n"
"		if (!Spectrum_IsBlack(connectionEmission) && sampleResult)\n"
"			SampleResult_AddEmission(&taskConfig->film, sampleResult, BSDF_GetLightID(bsdf\n"
"				MATERIALS_PARAM),\n"
"					pathThroughput, connectionEmission);\n"
"\n"
"		if (t > 0.f) {\n"
"			// There was a volume scatter event\n"
"\n"
"			// I have to set RayHit fields even if there wasn't a real\n"
"			// ray hit\n"
"			rayHit->t = t;\n"
"			// This is a trick in order to have RayHit::Miss() return\n"
"			// false. I assume 0xfffffffeu will trigger a memory fault if\n"
"			// used (and the bug will be noticed)\n"
"			rayHit->meshIndex = 0xfffffffeu;\n"
"\n"
"			BSDF_InitVolume(bsdf, *throughShadowTransparency, mats, ray, rayVolumeIndex, t, passThrough);\n"
"			volInfo->scatteredStart = true;\n"
"\n"
"			return false;\n"
"		}\n"
"	}\n"
"\n"
"	if (hit) {\n"
"		bool continueToTrace =\n"
"			// Check if the volume priority system tells me to continue to trace the ray\n"
"			PathVolumeInfo_ContinueToTrace(volInfo, bsdf\n"
"				MATERIALS_PARAM) ||\n"
"			// Check if it is a camera invisible object and we are a tracing a camera ray\n"
"			(cameraRay && sceneObjs[rayHit->meshIndex].cameraInvisible);\n"
"\n"
"		// Check if it is a pass through point\n"
"		if (!continueToTrace) {\n"
"			const float3 passThroughTrans = BSDF_GetPassThroughTransparency(bsdf, backTracing\n"
"				MATERIALS_PARAM);\n"
"			if (!Spectrum_IsBlack(passThroughTrans)) {\n"
"				*connectionThroughput *= passThroughTrans;\n"
"\n"
"				// It is a pass through point, continue to trace the ray\n"
"				continueToTrace = true;\n"
"			}\n"
"		}\n"
"\n"
"		if (!continueToTrace && shadowRay) {\n"
"			const float3 shadowTransparency = BSDF_GetPassThroughShadowTransparency(bsdf\n"
"				MATERIALS_PARAM);\n"
"\n"
"			if (!Spectrum_IsBlack(shadowTransparency)) {\n"
"				*connectionThroughput *= shadowTransparency;\n"
"				*throughShadowTransparency = true;\n"
"				continueToTrace = true;\n"
"			}\n"
"		}\n"
"\n"
"		if (continueToTrace) {\n"
"			// Update volume information\n"
"			const BSDFEvent eventTypes = BSDF_GetEventTypes(bsdf\n"
"						MATERIALS_PARAM);\n"
"			PathVolumeInfo_Update(volInfo, eventTypes, bsdf\n"
"					MATERIALS_PARAM);\n"
"\n"
"			// It is a transparent material, continue to trace the ray\n"
"			ray->mint = rayHit->t + MachineEpsilon_E(rayHit->t);\n"
"\n"
"			// A safety check in case of not enough numerical precision\n"
"			if ((ray->mint == rayHit->t) || (ray->mint >= ray->maxt))\n"
"				return false;\n"
"			else\n"
"				return true;\n"
"		} else\n"
"			return false;\n"
"	} else {\n"
"		// Nothing was hit, stop tracing the ray\n"
"		return false;\n"
"	}\n"
"}\n";
} }
