
#include <string>

namespace slg { namespace ocl {
std::string KernelSource_texture_blender_funcs =
"#line 2 \"texture_blender_funcs.cl\"\n"
"\n"
"/***************************************************************************\n"
" * Copyright 1998-2020 by authors (see AUTHORS.txt)                        *\n"
" *                                                                         *\n"
" *   This file is part of LuxCoreRender.                                   *\n"
" *                                                                         *\n"
" * Licensed under the Apache License, Version 2.0 (the \"License\");         *\n"
" * you may not use this file except in compliance with the License.        *\n"
" * You may obtain a copy of the License at                                 *\n"
" *                                                                         *\n"
" *     http://www.apache.org/licenses/LICENSE-2.0                          *\n"
" *                                                                         *\n"
" * Unless required by applicable law or agreed to in writing, software     *\n"
" * distributed under the License is distributed on an \"AS IS\" BASIS,       *\n"
" * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.*\n"
" * See the License for the specific language governing permissions and     *\n"
" * limitations under the License.                                          *\n"
" ***************************************************************************/\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Blender blend texture\n"
"//------------------------------------------------------------------------------\n"
" \n"
"OPENCL_FORCE_NOT_INLINE float BlenderBlendTexture_Evaluate(__global const HitPoint *hitPoint,\n"
"		const ProgressionType type, const bool direction,\n"
" 		const float contrast, const float bright,\n"
"		__global const TextureMapping3D *mapping TEXTURES_PARAM_DECL) {\n"
"	const float3 P = TextureMapping3D_Map(mapping, hitPoint, NULL TEXTURES_PARAM);\n"
" \n"
"	float result = 0.f;\n"
"	float x, y, t;\n"
"     \n"
"	if(!direction) {\n"
"		//horizontal\n"
"		x = P.x;\n"
"		y = P.y;\n"
"	} else {\n"
"		//vertical\n"
"		x = P.y;\n"
"		y = P.x;\n"
"	};\n"
" \n"
"    if (type == TEX_LIN) { /* lin */\n"
"		result = (1.f + x) / 2.f;\n"
"	} else if (type == TEX_QUAD) { /* quad */\n"
"		result = (1.f + x) / 2.f;\n"
"		if (result < 0.f) result = 0.f;\n"
"		else result *= result;\n"
"	} else if (type == TEX_EASE) { /* ease */\n"
"		result = (1.f + x) / 2.f;\n"
"        if (result <= 0.f) result = 0.f;\n"
"        else if (result >= 1.f) result = 1.f;\n"
"        else {\n"
"			t = result * result;\n"
"            result = (3.f * t - 2.f * t * result);\n"
"        }\n"
"    } else if (type == TEX_DIAG) { /* diag */\n"
"		result = (2.f + x + y) / 4.f;\n"
"    } else if (type == TEX_RAD) { /* radial */\n"
"        result = (atan2(y, x) / (2.f * M_PI_F) + 0.5f);\n"
"    } else { /* sphere TEX_SPHERE */\n"
"        result = 1.f - sqrt(x * x + y * y + P.z * P.z);\n"
"        if (result < 0.f) result = 0.f;\n"
"	    if (type == TEX_HALO) result *= result; /* halo */\n"
"    }\n"
"\n"
"	result = (result - 0.5f) * contrast + bright - 0.5f;\n"
"    if(result < 0.f) result = 0.f; \n"
"	else if(result > 1.f) result = 1.f;\n"
"\n"
"	return result;\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float BlenderBlendTexture_ConstEvaluateFloat(__global const HitPoint *hitPoint,\n"
"		const ProgressionType type, const bool direction,\n"
"		const float contrast, const float bright, __global const TextureMapping3D *mapping\n"
"		TEXTURES_PARAM_DECL) {\n"
"	return BlenderBlendTexture_Evaluate(hitPoint, type, direction, contrast, bright, mapping TEXTURES_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float3 BlenderBlendTexture_ConstEvaluateSpectrum(__global const HitPoint *hitPoint,\n"
"		const ProgressionType type, const bool direction,\n"
"		const float contrast, const float bright, __global const TextureMapping3D *mapping\n"
"		TEXTURES_PARAM_DECL) {\n"
"	return TO_FLOAT3(BlenderBlendTexture_Evaluate(hitPoint, type, direction, contrast, bright, mapping TEXTURES_PARAM));\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE void BlenderBlendTexture_EvalOp(\n"
"		__global const Texture* restrict texture,\n"
"		const TextureEvalOpType evalType,\n"
"		__global float *evalStack,\n"
"		uint *evalStackOffset,\n"
"		__global const HitPoint *hitPoint,\n"
"		const float sampleDistance\n"
"		TEXTURES_PARAM_DECL) {\n"
"	switch (evalType) {\n"
"		case EVAL_FLOAT: {\n"
"			const float eval = BlenderBlendTexture_ConstEvaluateFloat(hitPoint,\n"
"					texture->blenderBlend.type, texture->blenderBlend.direction,\n"
"					texture->blenderBlend.contrast, texture->blenderBlend.bright,\n"
"					&texture->blenderBlend.mapping TEXTURES_PARAM);\n"
"			EvalStack_PushFloat(eval);\n"
"			break;\n"
"		}\n"
"		case EVAL_SPECTRUM: {\n"
"			const float3 eval = BlenderBlendTexture_ConstEvaluateSpectrum(hitPoint,\n"
"					texture->blenderBlend.type, texture->blenderBlend.direction,\n"
"					texture->blenderBlend.contrast, texture->blenderBlend.bright,\n"
"					&texture->blenderBlend.mapping TEXTURES_PARAM);\n"
"			EvalStack_PushFloat3(eval);\n"
"			break;\n"
"		}\n"
"		case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"			Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"			Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		case EVAL_BUMP:\n"
"			Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		default:\n"
"			// Something wrong here\n"
"			break;\n"
"	}\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Blender clouds texture\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float BlenderCloudsTexture_Evaluate(__global const HitPoint *hitPoint,\n"
"		const BlenderNoiseBasis noisebasis, const float noisesize, const int noisedepth,\n"
"		const float contrast, const float bright, const bool hard, __global const TextureMapping3D *mapping\n"
"		TEXTURES_PARAM_DECL) {\n"
"	const float3 P = TextureMapping3D_Map(mapping, hitPoint, NULL TEXTURES_PARAM);\n"
"\n"
"	float clouds = BLI_gTurbulence(noisesize, P.x, P.y, P.z, noisedepth, hard, noisebasis);\n"
"\n"
"	clouds = (clouds - 0.5f) * contrast + bright - 0.5f;\n"
"	clouds = clamp(clouds, 0.f, 1.f);\n"
"\n"
"	return clouds;\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float BlenderCloudsTexture_ConstEvaluateFloat(__global const HitPoint *hitPoint,\n"
"		const BlenderNoiseBasis noisebasis, const float noisesize, const int noisedepth,\n"
"		const float contrast, const float bright, const bool hard,\n"
"		__global const TextureMapping3D *mapping\n"
"		TEXTURES_PARAM_DECL) {\n"
"	return BlenderCloudsTexture_Evaluate(hitPoint, noisebasis, noisesize, noisedepth,\n"
"			contrast, bright, hard, mapping TEXTURES_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float3 BlenderCloudsTexture_ConstEvaluateSpectrum(__global const HitPoint *hitPoint,\n"
"		const BlenderNoiseBasis noisebasis, const float noisesize, const int noisedepth,\n"
"		const float contrast, const float bright, const bool hard,\n"
"		__global const TextureMapping3D *mapping\n"
"		TEXTURES_PARAM_DECL) {\n"
"	return TO_FLOAT3(BlenderCloudsTexture_Evaluate(hitPoint, noisebasis, noisesize, noisedepth,\n"
"			contrast, bright, hard, mapping TEXTURES_PARAM));\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE void BlenderCloudsTexture_EvalOp(\n"
"		__global const Texture* restrict texture,\n"
"		const TextureEvalOpType evalType,\n"
"		__global float *evalStack,\n"
"		uint *evalStackOffset,\n"
"		__global const HitPoint *hitPoint,\n"
"		const float sampleDistance\n"
"		TEXTURES_PARAM_DECL) {\n"
"	switch (evalType) {\n"
"		case EVAL_FLOAT: {\n"
"			const float eval = BlenderCloudsTexture_ConstEvaluateFloat(hitPoint,\n"
"					texture->blenderClouds.noisebasis, texture->blenderClouds.noisesize,\n"
"					texture->blenderClouds.noisedepth, texture->blenderClouds.contrast,\n"
"					texture->blenderClouds.bright, texture->blenderClouds.hard,\n"
"					&texture->blenderClouds.mapping TEXTURES_PARAM);\n"
"			EvalStack_PushFloat(eval);\n"
"			break;\n"
"		}\n"
"		case EVAL_SPECTRUM: {\n"
"			const float3 eval = BlenderCloudsTexture_ConstEvaluateSpectrum(hitPoint,\n"
"					texture->blenderClouds.noisebasis, texture->blenderClouds.noisesize,\n"
"					texture->blenderClouds.noisedepth, texture->blenderClouds.contrast,\n"
"					texture->blenderClouds.bright, texture->blenderClouds.hard,\n"
"					&texture->blenderClouds.mapping TEXTURES_PARAM);\n"
"			EvalStack_PushFloat3(eval);\n"
"			break;\n"
"		}\n"
"		case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"			Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"			Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		case EVAL_BUMP:\n"
"			Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		default:\n"
"			// Something wrong here\n"
"			break;\n"
"	}\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Blender distorted noise texture\n"
"//------------------------------------------------------------------------------\n"
"            \n"
"OPENCL_FORCE_NOT_INLINE float BlenderDistortedNoiseTexture_Evaluate(__global const HitPoint *hitPoint,\n"
"		const BlenderNoiseBasis noisedistortion, const BlenderNoiseBasis noisebasis,\n"
"		const float distortion, const float noisesize,\n"
"		const float contrast, const float bright,\n"
"		__global const TextureMapping3D *mapping\n"
"		TEXTURES_PARAM_DECL) {\n"
"	float3 P = TextureMapping3D_Map(mapping, hitPoint, NULL TEXTURES_PARAM);\n"
"\n"
"	float result = 0.f;\n"
"	float scale = 1.f;\n"
"\n"
"	if(fabs(noisesize) > 0.00001f) scale = (1.f/noisesize);\n"
"	P = scale*P;\n"
"	result = mg_VLNoise(P.x, P.y, P.z, distortion, noisebasis, noisedistortion);\n"
"\n"
"	result = (result - 0.5f) * contrast + bright - 0.5f;\n"
"    if(result < 0.f) result = 0.f; \n"
"	else if(result > 1.f) result = 1.f;\n"
"\n"
"	return result;\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float BlenderDistortedNoiseTexture_ConstEvaluateFloat(__global const HitPoint *hitPoint,\n"
"		const BlenderNoiseBasis noisedistortion, const BlenderNoiseBasis noisebasis,\n"
"		const float distortion, const float noisesize,\n"
"		const float contrast, const float bright,\n"
"		__global const TextureMapping3D *mapping\n"
"		TEXTURES_PARAM_DECL) {\n"
"	return BlenderDistortedNoiseTexture_Evaluate(hitPoint, noisedistortion, noisebasis, distortion, noisesize,\n"
"			contrast, bright, mapping TEXTURES_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float3 BlenderDistortedNoiseTexture_ConstEvaluateSpectrum(__global const HitPoint *hitPoint,\n"
"		const BlenderNoiseBasis noisedistortion, const BlenderNoiseBasis noisebasis,\n"
"		const float distortion, const float noisesize,\n"
"		const float contrast, const float bright,\n"
"		__global const TextureMapping3D *mapping\n"
"		TEXTURES_PARAM_DECL) {\n"
"	return TO_FLOAT3(BlenderDistortedNoiseTexture_Evaluate(hitPoint, noisedistortion, noisebasis, distortion, noisesize,\n"
"			contrast, bright, mapping TEXTURES_PARAM));\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE void BlenderDistortedNoiseTexture_EvalOp(\n"
"		__global const Texture* restrict texture,\n"
"		const TextureEvalOpType evalType,\n"
"		__global float *evalStack,\n"
"		uint *evalStackOffset,\n"
"		__global const HitPoint *hitPoint,\n"
"		const float sampleDistance\n"
"		TEXTURES_PARAM_DECL) {\n"
"	switch (evalType) {\n"
"		case EVAL_FLOAT: {\n"
"			const float eval = BlenderDistortedNoiseTexture_ConstEvaluateFloat(hitPoint,\n"
"					texture->blenderDistortedNoise.noisedistortion, texture->blenderDistortedNoise.noisebasis,\n"
"					texture->blenderDistortedNoise.distortion, texture->blenderDistortedNoise.noisesize,\n"
"					texture->blenderDistortedNoise.contrast, texture->blenderDistortedNoise.bright,\n"
"					&texture->blenderDistortedNoise.mapping TEXTURES_PARAM);\n"
"			EvalStack_PushFloat(eval);\n"
"			break;\n"
"		}\n"
"		case EVAL_SPECTRUM: {\n"
"			const float3 eval = BlenderDistortedNoiseTexture_ConstEvaluateSpectrum(hitPoint,\n"
"					texture->blenderDistortedNoise.noisedistortion, texture->blenderDistortedNoise.noisebasis,\n"
"					texture->blenderDistortedNoise.distortion, texture->blenderDistortedNoise.noisesize,\n"
"					texture->blenderDistortedNoise.contrast, texture->blenderDistortedNoise.bright,\n"
"					&texture->blenderDistortedNoise.mapping TEXTURES_PARAM);\n"
"			EvalStack_PushFloat3(eval);\n"
"			break;\n"
"		}\n"
"		case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"			Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"			Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		case EVAL_BUMP:\n"
"			Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		default:\n"
"			// Something wrong here\n"
"			break;\n"
"	}\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Blender magic texture\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float3 BlenderMagicTexture_Evaluate(__global const HitPoint *hitPoint,\n"
"		const int noisedepth, const float turbulence, const float contrast, const float bright,\n"
"		__global const TextureMapping3D *mapping\n"
"		TEXTURES_PARAM_DECL) {\n"
"	const float3 P = TextureMapping3D_Map(mapping, hitPoint, NULL TEXTURES_PARAM);\n"
"	float3 s;\n"
"\n"
"    float x, y, z, turb = 1.f;\n"
"    float r, g, b;\n"
"    int n;\n"
"\n"
"    n = noisedepth;\n"
"    turb = turbulence / 5.f;\n"
"\n"
"    x = sin((P.x + P.y + P.z)*5.f);\n"
"    y = cos((-P.x + P.y - P.z)*5.f);\n"
"    z = -cos((-P.x - P.y + P.z)*5.f);\n"
"    if (n > 0) {\n"
"        x *= turb;\n"
"        y *= turb;\n"
"        z *= turb;\n"
"        y = -cos(x - y + z);\n"
"        y *= turb;\n"
"        if (n > 1) {\n"
"            x = cos(x - y - z);\n"
"            x *= turb;\n"
"            if (n > 2) {\n"
"                z = sin(-x - y - z);\n"
"                z *= turb;\n"
"                if (n > 3) {\n"
"                    x = -cos(-x + y - z);\n"
"                    x *= turb;\n"
"                    if (n > 4) {\n"
"                        y = -sin(-x + y + z);\n"
"                        y *= turb;\n"
"                        if (n > 5) {\n"
"                            y = -cos(-x + y + z);\n"
"                            y *= turb;\n"
"                            if (n > 6) {\n"
"                                x = cos(x + y + z);\n"
"                                x *= turb;\n"
"                                if (n > 7) {\n"
"                                    z = sin(x + y - z);\n"
"                                    z *= turb;\n"
"                                    if (n > 8) {\n"
"                                        x = -cos(-x - y + z);\n"
"                                        x *= turb;\n"
"                                        if (n > 9) {\n"
"                                            y = -sin(x - y + z);\n"
"                                            y *= turb;\n"
"                                        }\n"
"                                    }\n"
"                                }\n"
"                            }\n"
"                        }\n"
"                    }\n"
"                }\n"
"            }\n"
"        }\n"
"    }\n"
"\n"
"    if (turb != 0.f) {\n"
"        turb *= 2.f;\n"
"        x /= turb;\n"
"        y /= turb;\n"
"        z /= turb;\n"
"    }\n"
"    r = 0.5f - x;\n"
"    g = 0.5f - y;\n"
"    b = 0.5f - z;\n"
"\n"
"	r = (r - 0.5f) * contrast + bright - 0.5f;\n"
"    if(r < 0.f) r = 0.f; \n"
"	else if(r > 1.f) r = 1.f;\n"
"\n"
"	g = (g - 0.5f) * contrast + bright - 0.5f;\n"
"    if(g < 0.f) g = 0.f; \n"
"	else if(g > 1.f) g = 1.f;\n"
"\n"
"	b = (b - 0.5f) * contrast + bright - 0.5f;\n"
"    if(b < 0.f) b = 0.f; \n"
"	else if(b > 1.f) b = 1.f;\n"
"\n"
"	s.x = r;\n"
"	s.y = g;\n"
"	s.z = b;\n"
"\n"
"	return s;\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float BlenderMagicTexture_ConstEvaluateFloat(__global const HitPoint *hitPoint,\n"
"		const int noisedepth, const float turbulence,\n"
"		const float contrast, const float bright,\n"
"		__global const TextureMapping3D *mapping\n"
"		TEXTURES_PARAM_DECL) {\n"
"	float3 result = BlenderMagicTexture_Evaluate(hitPoint, noisedepth, turbulence,\n"
"			contrast, bright, mapping TEXTURES_PARAM);\n"
"\n"
"	return (0.212671f * result.x + 0.715160f * result.y + 0.072169f * result.z);\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float3 BlenderMagicTexture_ConstEvaluateSpectrum(__global const HitPoint *hitPoint,\n"
"		const int noisedepth, const float turbulence,\n"
"		const float contrast, const float bright,\n"
"		__global const TextureMapping3D *mapping\n"
"		TEXTURES_PARAM_DECL) {\n"
"	return BlenderMagicTexture_Evaluate(hitPoint, noisedepth, turbulence, contrast,\n"
"			bright, mapping TEXTURES_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE void BlenderMagicTexture_EvalOp(\n"
"		__global const Texture* restrict texture,\n"
"		const TextureEvalOpType evalType,\n"
"		__global float *evalStack,\n"
"		uint *evalStackOffset,\n"
"		__global const HitPoint *hitPoint,\n"
"		const float sampleDistance\n"
"		TEXTURES_PARAM_DECL) {\n"
"	switch (evalType) {\n"
"		case EVAL_FLOAT: {\n"
"			const float eval = BlenderMagicTexture_ConstEvaluateFloat(hitPoint,\n"
"					texture->blenderMagic.noisedepth, texture->blenderMagic.turbulence,\n"
"					texture->blenderMagic.contrast, texture->blenderMagic.bright,\n"
"					&texture->blenderMagic.mapping TEXTURES_PARAM);\n"
"			EvalStack_PushFloat(eval);\n"
"			break;\n"
"		}\n"
"		case EVAL_SPECTRUM: {\n"
"			const float3 eval = BlenderMagicTexture_ConstEvaluateSpectrum(hitPoint,\n"
"					texture->blenderMagic.noisedepth, texture->blenderMagic.turbulence,\n"
"					texture->blenderMagic.contrast, texture->blenderMagic.bright,\n"
"					&texture->blenderMagic.mapping TEXTURES_PARAM);\n"
"			EvalStack_PushFloat3(eval);\n"
"			break;\n"
"		}\n"
"		case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"			Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"			Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		case EVAL_BUMP:\n"
"			Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		default:\n"
"			// Something wrong here\n"
"			break;\n"
"	}\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Blender marble texture\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float BlenderMarbleTexture_Evaluate(__global const HitPoint *hitPoint,\n"
"		const BlenderMarbleType type, const BlenderNoiseBasis noisebasis,\n"
"		const BlenderNoiseBase noisebasis2, const float noisesize,\n"
"		const float turbulence, const int noisedepth,\n"
"		const float contrast, const float bright,\n"
"		const bool hard, __global const TextureMapping3D *mapping\n"
"		TEXTURES_PARAM_DECL) {\n"
"	const float3 P = TextureMapping3D_Map(mapping, hitPoint, NULL TEXTURES_PARAM);\n"
"\n"
"	float result = 0.f;\n"
"    float n = 5.f * (P.x + P.y + P.z);\n"
"    result = n + turbulence * BLI_gTurbulence(noisesize, P.x, P.y, P.z, noisedepth, hard, noisebasis);\n"
"	\n"
"\n"
"	if(noisebasis2 == TEX_SIN) {\n"
"		result = tex_sin(result);\n"
"	} else if(noisebasis2 == TEX_SAW) {\n"
"		result = tex_saw(result);\n"
"	} else {\n"
"		result = tex_tri(result);\n"
"	}\n"
"	\n"
"	if (type == TEX_SHARP) {\n"
"		result = sqrt(result);\n"
"	} else if (type == TEX_SHARPER) {\n"
"		result = sqrt(sqrt(result));\n"
"	}\n"
"\n"
"	result = (result - 0.5f) * contrast + bright - 0.5f;\n"
"    if(result < 0.f) result = 0.f; \n"
"	else if(result > 1.f) result = 1.f;\n"
"	\n"
"    return result;\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float BlenderMarbleTexture_ConstEvaluateFloat(__global const HitPoint *hitPoint,\n"
"		const BlenderMarbleType type, const BlenderNoiseBasis noisebasis,\n"
"		const BlenderNoiseBase noisebasis2, const float noisesize, \n"
"		const float turbulence, const int noisedepth,\n"
"		const float contrast, const float bright,\n"
"		const bool hard, __global const TextureMapping3D *mapping\n"
"		TEXTURES_PARAM_DECL) {\n"
"	return BlenderMarbleTexture_Evaluate(hitPoint, type, noisebasis, noisebasis2,\n"
"			noisesize, turbulence, noisedepth, contrast, bright, hard, mapping\n"
"			TEXTURES_PARAM);\n"
"}\n"
"\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float3 BlenderMarbleTexture_ConstEvaluateSpectrum(__global const HitPoint *hitPoint,\n"
"		const BlenderMarbleType type, const BlenderNoiseBasis noisebasis,\n"
"		const BlenderNoiseBase noisebasis2, const float noisesize, \n"
"		const float turbulence, const int noisedepth,\n"
"		const float contrast, const float bright, \n"
"		const bool hard, __global const TextureMapping3D *mapping\n"
"		TEXTURES_PARAM_DECL) {\n"
"	return TO_FLOAT3(BlenderMarbleTexture_Evaluate(hitPoint, type, noisebasis, noisebasis2,\n"
"			noisesize, turbulence, noisedepth, contrast, bright, hard, mapping\n"
"			TEXTURES_PARAM));\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE void BlenderMarbleTexture_EvalOp(\n"
"		__global const Texture* restrict texture,\n"
"		const TextureEvalOpType evalType,\n"
"		__global float *evalStack,\n"
"		uint *evalStackOffset,\n"
"		__global const HitPoint *hitPoint,\n"
"		const float sampleDistance\n"
"		TEXTURES_PARAM_DECL) {\n"
"	switch (evalType) {\n"
"		case EVAL_FLOAT: {\n"
"			const float eval = BlenderMarbleTexture_ConstEvaluateFloat(hitPoint,\n"
"					texture->blenderMarble.type, texture->blenderMarble.noisebasis,\n"
"					texture->blenderMarble.noisebasis2, texture->blenderMarble.noisesize,\n"
"					texture->blenderMarble.turbulence, texture->blenderMarble.noisedepth,\n"
"					texture->blenderMarble.contrast, texture->blenderMarble.bright,\n"
"					texture->blenderMarble.hard,\n"
"					&texture->blenderMagic.mapping TEXTURES_PARAM);\n"
"			EvalStack_PushFloat(eval);\n"
"			break;\n"
"		}\n"
"		case EVAL_SPECTRUM: {\n"
"			const float3 eval = BlenderMarbleTexture_ConstEvaluateSpectrum(hitPoint,\n"
"					texture->blenderMarble.type, texture->blenderMarble.noisebasis,\n"
"					texture->blenderMarble.noisebasis2, texture->blenderMarble.noisesize,\n"
"					texture->blenderMarble.turbulence, texture->blenderMarble.noisedepth,\n"
"					texture->blenderMarble.contrast, texture->blenderMarble.bright,\n"
"					texture->blenderMarble.hard,\n"
"					&texture->blenderMagic.mapping TEXTURES_PARAM);\n"
"			EvalStack_PushFloat3(eval);\n"
"			break;\n"
"		}\n"
"		case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"			Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"			Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		case EVAL_BUMP:\n"
"			Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		default:\n"
"			// Something wrong here\n"
"			break;\n"
"	}\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Blender musgrave texture\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float BlenderMusgraveTexture_Evaluate(__global const HitPoint *hitPoint,\n"
"		const BlenderMusgraveType type, const BlenderNoiseBasis noisebasis,\n"
"		const float dimension, const float intensity, const float lacunarity,\n"
"		const float offset, const float gain, const float octaves, const float noisesize,\n"
"		const float contrast, const float bright, __global const TextureMapping3D *mapping\n"
"		TEXTURES_PARAM_DECL) {\n"
"		\n"
"	float scale = 1.f;\n"
"	if(fabs(noisesize) > 0.00001f) scale = (1.f/noisesize);\n"
"	const float3 P = scale * TextureMapping3D_Map(mapping, hitPoint, NULL TEXTURES_PARAM);\n"
"\n"
"	float result = 0.f;\n"
"\n"
"	switch (type) {\n"
"		case TEX_MULTIFRACTAL:\n"
"			result = mg_MultiFractal(P.x, P.y, P.z, dimension, lacunarity, octaves, noisebasis);\n"
"			break;\n"
"		case TEX_FBM:\n"
"			result = mg_fBm(P.x, P.y, P.z, dimension, lacunarity, octaves, noisebasis);\n"
"            break;\n"
"		case TEX_RIDGED_MULTIFRACTAL:\n"
"			result = mg_RidgedMultiFractal(P.x, P.y, P.z, dimension, lacunarity, octaves, offset, gain, noisebasis);\n"
"			break;\n"
"        case TEX_HYBRID_MULTIFRACTAL:\n"
"			result = mg_HybridMultiFractal(P.x, P.y, P.z, dimension, lacunarity, octaves, offset, gain, noisebasis);\n"
"			break;\n"
"        case TEX_HETERO_TERRAIN:\n"
"			result = mg_HeteroTerrain(P.x, P.y, P.z, dimension, lacunarity, octaves, offset, noisebasis);\n"
"			break;\n"
"    };\n"
"\n"
"	result *= intensity;\n"
"           \n"
"	result = (result - 0.5f) * contrast + bright - 0.5f;\n"
"    if(result < 0.f) result = 0.f; \n"
"	else if(result > 1.f) result = 1.f;\n"
"	\n"
"	return result;\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float BlenderMusgraveTexture_ConstEvaluateFloat(__global const HitPoint *hitPoint,\n"
"		const BlenderMusgraveType type, const BlenderNoiseBasis noisebasis,\n"
"		const float dimension, const float intensity, const float lacunarity,\n"
"		const float offset, const float gain, const float octaves, const float noisesize,\n"
"		const float contrast, const float bright, __global const TextureMapping3D *mapping\n"
"		TEXTURES_PARAM_DECL) {\n"
"	return BlenderMusgraveTexture_Evaluate(hitPoint, type, noisebasis, dimension, intensity, lacunarity,\n"
"			offset, gain, octaves, noisesize, contrast, bright, mapping\n"
"			TEXTURES_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float3 BlenderMusgraveTexture_ConstEvaluateSpectrum(__global const HitPoint *hitPoint,\n"
"		const BlenderMusgraveType type,	const BlenderNoiseBasis noisebasis,\n"
"		const float dimension, const float intensity, const float lacunarity,\n"
"		const float offset, const float gain, const float octaves, const float noisesize,\n"
"		const float contrast, const float bright, __global const TextureMapping3D *mapping\n"
"		TEXTURES_PARAM_DECL) {\n"
"	return TO_FLOAT3(BlenderMusgraveTexture_Evaluate(hitPoint, type, noisebasis, dimension, intensity, lacunarity,\n"
"			offset, gain, octaves, noisesize, contrast, bright, mapping\n"
"			TEXTURES_PARAM));\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE void BlenderMusgraveTexture_EvalOp(\n"
"		__global const Texture* restrict texture,\n"
"		const TextureEvalOpType evalType,\n"
"		__global float *evalStack,\n"
"		uint *evalStackOffset,\n"
"		__global const HitPoint *hitPoint,\n"
"		const float sampleDistance\n"
"		TEXTURES_PARAM_DECL) {\n"
"	switch (evalType) {\n"
"		case EVAL_FLOAT: {\n"
"			const float eval = BlenderMusgraveTexture_ConstEvaluateFloat(hitPoint,\n"
"					texture->blenderMusgrave.type, texture->blenderMusgrave.noisebasis,\n"
"					texture->blenderMusgrave.dimension, texture->blenderMusgrave.intensity,\n"
"					texture->blenderMusgrave.lacunarity, texture->blenderMusgrave.offset,\n"
"					texture->blenderMusgrave.gain, texture->blenderMusgrave.octaves,\n"
"					texture->blenderMusgrave.noisesize, texture->blenderMusgrave.contrast,\n"
"					texture->blenderMusgrave.bright,\n"
"					&texture->blenderMusgrave.mapping TEXTURES_PARAM);\n"
"			EvalStack_PushFloat(eval);\n"
"			break;\n"
"		}\n"
"		case EVAL_SPECTRUM: {\n"
"			const float3 eval = BlenderMusgraveTexture_ConstEvaluateSpectrum(hitPoint,\n"
"					texture->blenderMusgrave.type, texture->blenderMusgrave.noisebasis,\n"
"					texture->blenderMusgrave.dimension, texture->blenderMusgrave.intensity,\n"
"					texture->blenderMusgrave.lacunarity, texture->blenderMusgrave.offset,\n"
"					texture->blenderMusgrave.gain, texture->blenderMusgrave.octaves,\n"
"					texture->blenderMusgrave.noisesize, texture->blenderMusgrave.contrast,\n"
"					texture->blenderMusgrave.bright,\n"
"					&texture->blenderMusgrave.mapping TEXTURES_PARAM);\n"
"			EvalStack_PushFloat3(eval);\n"
"			break;\n"
"		}\n"
"		case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"			Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"			Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		case EVAL_BUMP:\n"
"			Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		default:\n"
"			// Something wrong here\n"
"			break;\n"
"	}\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Blender noise texture\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float BlenderNoiseTexture_Evaluate(__global const HitPoint *hitPoint,\n"
"		const int noisedepth, const float bright, const float contrast) {\n"
"	// The original Blender code was not thread-safe\n"
"\n"
"	float result = 0.f;\n"
"\n"
"	float div = 3.f;\n"
"\n"
"	float ran = hitPoint->passThroughEvent;\n"
"	float val = Floor2UInt(ran * 4.f);\n"
"	\n"
"	int loop = noisedepth;\n"
"    while (loop--) {\n"
"		// I generate a new random variable starting from the previous one. I'm\n"
"		// not really sure about the kind of correlation introduced by this\n"
"		// trick.\n"
"		ran = fabs(ran - .5f) * 2.f;\n"
"\n"
"        val *= Floor2UInt(ran * 4.f);\n"
"        div *= 3.f;		\n"
"    }\n"
"\n"
"	result = ((float) val) / div;\n"
"	\n"
"	result = (result - 0.5f) * contrast + bright - 0.5f;\n"
"    if(result < 0.f) result = 0.f;\n"
"	else if(result > 1.f) result = 1.f;\n"
"\n"
"	return result;\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float BlenderNoiseTexture_ConstEvaluateFloat(__global const HitPoint *hitPoint,\n"
"		const int noisedepth, const float bright, const float contrast) {\n"
"	return BlenderNoiseTexture_Evaluate(hitPoint, noisedepth, bright, contrast);\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float3 BlenderNoiseTexture_ConstEvaluateSpectrum(__global const HitPoint *hitPoint,\n"
"		const int noisedepth, const float bright, const float contrast) {\n"
"	return TO_FLOAT3(BlenderNoiseTexture_Evaluate(hitPoint, noisedepth, bright, contrast));\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE void BlenderNoiseTexture_EvalOp(\n"
"		__global const Texture* restrict texture,\n"
"		const TextureEvalOpType evalType,\n"
"		__global float *evalStack,\n"
"		uint *evalStackOffset,\n"
"		__global const HitPoint *hitPoint,\n"
"		const float sampleDistance\n"
"		TEXTURES_PARAM_DECL) {\n"
"	switch (evalType) {\n"
"		case EVAL_FLOAT: {\n"
"			const float eval = BlenderNoiseTexture_ConstEvaluateFloat(hitPoint,\n"
"					texture->blenderNoise.noisedepth, texture->blenderNoise.bright,\n"
"					texture->blenderNoise.contrast);\n"
"			EvalStack_PushFloat(eval);\n"
"			break;\n"
"		}\n"
"		case EVAL_SPECTRUM: {\n"
"			const float3 eval = BlenderNoiseTexture_ConstEvaluateSpectrum(hitPoint,\n"
"					texture->blenderNoise.noisedepth, texture->blenderNoise.bright,\n"
"					texture->blenderNoise.contrast);\n"
"			EvalStack_PushFloat3(eval);\n"
"			break;\n"
"		}\n"
"		case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"			Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"			Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		case EVAL_BUMP:\n"
"			Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		default:\n"
"			// Something wrong here\n"
"			break;\n"
"	}\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Blender stucci texture\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float BlenderStucciTexture_Evaluate(__global const HitPoint *hitPoint,\n"
"		const BlenderStucciType type, const BlenderNoiseBasis noisebasis,\n"
"		const float noisesize, const float turbulence, const float contrast,\n"
"		const float bright, const bool hard, __global const TextureMapping3D *mapping\n"
"		TEXTURES_PARAM_DECL) {\n"
"	const float3 P = TextureMapping3D_Map(mapping, hitPoint, NULL TEXTURES_PARAM);\n"
"	\n"
"	float result = 0.f;\n"
"//	neo2068: only nor[2] is used, so normal float variable is sufficient\n"
"//	float nor[3], b2, ofs;\n"
"	float b2, ofs;\n"
"	\n"
"	b2 = BLI_gNoise(noisesize, P.x, P.y, P.z, hard, noisebasis);\n"
"	ofs = turbulence / 200.f;\n"
"\n"
"	if (type != TEX_PLASTIC) ofs *= (b2 * b2);\n"
"//	neo2068: only nor[2] is used\n"
"//    nor[0] = BLI_gNoise(noisesize, P.x + ofs, P.y, P.z, hard, noisebasis);\n"
"//    nor[1] = BLI_gNoise(noisesize, P.x, P.y + ofs, P.z, hard, noisebasis);\n"
"//    nor[2] = BLI_gNoise(noisesize, P.x, P.y, P.z + ofs, hard, noisebasis);\n"
"    result = BLI_gNoise(noisesize, P.x, P.y, P.z + ofs, hard, noisebasis);\n"
"\n"
"//    result = nor[2];\n"
"\n"
"    if (type == TEX_WALL_OUT)\n"
"        result = 1.f - result;\n"
"\n"
"	result = (result - 0.5f) * contrast + bright - 0.5f;\n"
"    if(result < 0.f) result = 0.f; \n"
"	else if(result > 1.f) result = 1.f;\n"
"\n"
"	return result;\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float BlenderStucciTexture_ConstEvaluateFloat(__global const HitPoint *hitPoint,\n"
"		const BlenderStucciType type, const BlenderNoiseBasis noisebasis,\n"
"		const float noisesize, const float turbulence, const float contrast,\n"
"		const float bright, const bool hard, __global const TextureMapping3D *mapping\n"
"		TEXTURES_PARAM_DECL) {\n"
"	return BlenderStucciTexture_Evaluate(hitPoint, type, noisebasis, noisesize, turbulence,\n"
"			contrast, bright, hard, mapping\n"
"			TEXTURES_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float3 BlenderStucciTexture_ConstEvaluateSpectrum(__global const HitPoint *hitPoint,\n"
"		const BlenderStucciType type, const BlenderNoiseBasis noisebasis,\n"
"		const float noisesize, const float turbulence, const float contrast,\n"
"		const float bright, const bool hard, __global const TextureMapping3D *mapping\n"
"		TEXTURES_PARAM_DECL) {\n"
"	return TO_FLOAT3(BlenderStucciTexture_Evaluate(hitPoint, type, noisebasis, noisesize, turbulence,\n"
"			contrast, bright, hard, mapping TEXTURES_PARAM));\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE void BlenderStucciTexture_EvalOp(\n"
"		__global const Texture* restrict texture,\n"
"		const TextureEvalOpType evalType,\n"
"		__global float *evalStack,\n"
"		uint *evalStackOffset,\n"
"		__global const HitPoint *hitPoint,\n"
"		const float sampleDistance\n"
"		TEXTURES_PARAM_DECL) {\n"
"	switch (evalType) {\n"
"		case EVAL_FLOAT: {\n"
"			const float eval = BlenderStucciTexture_ConstEvaluateFloat(hitPoint,\n"
"					texture->blenderStucci.type, texture->blenderStucci.noisebasis,\n"
"					texture->blenderStucci.noisesize, texture->blenderStucci.turbulence,\n"
"					texture->blenderStucci.contrast, texture->blenderStucci.bright,\n"
"					texture->blenderStucci.hard,\n"
"					&texture->blenderStucci.mapping TEXTURES_PARAM);\n"
"			EvalStack_PushFloat(eval);\n"
"			break;\n"
"		}\n"
"		case EVAL_SPECTRUM: {\n"
"			const float3 eval = BlenderStucciTexture_ConstEvaluateSpectrum(hitPoint,\n"
"					texture->blenderStucci.type, texture->blenderStucci.noisebasis,\n"
"					texture->blenderStucci.noisesize, texture->blenderStucci.turbulence,\n"
"					texture->blenderStucci.contrast, texture->blenderStucci.bright,\n"
"					texture->blenderStucci.hard,\n"
"					&texture->blenderStucci.mapping TEXTURES_PARAM);\n"
"			EvalStack_PushFloat3(eval);\n"
"			break;\n"
"		}\n"
"		case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"			Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"			Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		case EVAL_BUMP:\n"
"			Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		default:\n"
"			// Something wrong here\n"
"			break;\n"
"	}\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Blender wood texture\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float BlenderWoodTexture_Evaluate(__global const HitPoint *hitPoint,\n"
"		const BlenderWoodType type, const BlenderNoiseBase noisebasis2,\n"
"		const BlenderNoiseBasis noisebasis, const float noisesize, const float turbulence,\n"
"		const float contrast, const float bright, const bool hard,\n"
"		__global const TextureMapping3D *mapping TEXTURES_PARAM_DECL) {\n"
"	const float3 P = TextureMapping3D_Map(mapping, hitPoint, NULL TEXTURES_PARAM);\n"
"\n"
"	float wood = 0.f;\n"
"	switch(type) {\n"
"		default:\n"
"		case BANDS:\n"
"			if(noisebasis2 == TEX_SIN) {\n"
"				wood = tex_sin((P.x + P.y + P.z) * 10.f);\n"
"			} else if(noisebasis2 == TEX_SAW) {\n"
"				wood = tex_saw((P.x + P.y + P.z) * 10.f);\n"
"			} else {\n"
"				wood = tex_tri((P.x + P.y + P.z) * 10.f);\n"
"			}\n"
"			break;\n"
"		case RINGS:\n"
"			if(noisebasis2 == TEX_SIN) {\n"
"				wood = tex_sin(sqrt(P.x*P.x + P.y*P.y + P.z*P.z) * 20.f);\n"
"			} else if(noisebasis2 == TEX_SAW) {\n"
"				wood = tex_saw(sqrt(P.x*P.x + P.y*P.y + P.z*P.z) * 20.f);\n"
"			} else {\n"
"				wood = tex_tri(sqrt(P.x*P.x + P.y*P.y + P.z*P.z) * 20.f);\n"
"			}\n"
"			break;\n"
"		case BANDNOISE:			\n"
"			if(hard)	\n"
"				wood = turbulence * fabs(2.f * BLI_gNoise(noisesize, P.x, P.y, P.z, hard, noisebasis) - 1.f);\n"
"			else\n"
"				wood = turbulence * BLI_gNoise(noisesize, P.x, P.y, P.z, hard, noisebasis);\n"
"\n"
"			if(noisebasis2 == TEX_SIN) {\n"
"				wood = tex_sin((P.x + P.y + P.z) * 10.f + wood);\n"
"			} else if(noisebasis2 == TEX_SAW) {\n"
"				wood = tex_saw((P.x + P.y + P.z) * 10.f + wood);\n"
"			} else {\n"
"				wood = tex_tri((P.x + P.y + P.z) * 10.f + wood);\n"
"			}\n"
"			break;\n"
"		case RINGNOISE:\n"
"			if(hard)	\n"
"				wood = turbulence * fabs(2.f * BLI_gNoise(noisesize, P.x, P.y, P.z, hard, noisebasis) - 1.f);\n"
"			else\n"
"				wood = turbulence * BLI_gNoise(noisesize, P.x, P.y, P.z, hard, noisebasis);\n"
"\n"
"			if(noisebasis2 == TEX_SIN) {\n"
"				wood = tex_sin(sqrt(P.x*P.x + P.y*P.y + P.z*P.z) * 20.f + wood);\n"
"			} else if(noisebasis2 == TEX_SAW) {\n"
"				wood = tex_saw(sqrt(P.x*P.x + P.y*P.y + P.z*P.z) * 20.f + wood);\n"
"			} else {\n"
"				wood = tex_tri(sqrt(P.x*P.x + P.y*P.y + P.z*P.z) * 20.f + wood);\n"
"			}\n"
"			break;\n"
"	}\n"
"	wood = (wood - 0.5f) * contrast + bright - 0.5f;\n"
"	wood = clamp(wood, 0.f, 1.f);\n"
"\n"
"	return wood;\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float BlenderWoodTexture_ConstEvaluateFloat(__global const HitPoint *hitPoint,\n"
"		const BlenderWoodType type, const BlenderNoiseBase noisebasis2,\n"
"		const BlenderNoiseBasis noisebasis, const float noisesize, const float turbulence,\n"
"		const float contrast, const float bright, const bool hard,\n"
"		__global const TextureMapping3D *mapping TEXTURES_PARAM_DECL) {\n"
"	return BlenderWoodTexture_Evaluate(hitPoint, type, noisebasis2, noisebasis,\n"
"		noisesize, turbulence, contrast, bright, hard, mapping TEXTURES_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float3 BlenderWoodTexture_ConstEvaluateSpectrum(__global const HitPoint *hitPoint,\n"
"		const BlenderWoodType type, const BlenderNoiseBase noisebasis2, \n"
"		const BlenderNoiseBasis noisebasis, const float noisesize, const float turbulence,\n"
"		const float contrast, const float bright, const bool hard,\n"
"		__global const TextureMapping3D *mapping TEXTURES_PARAM_DECL) {\n"
"    return TO_FLOAT3(BlenderWoodTexture_Evaluate(hitPoint, type, noisebasis2, noisebasis,\n"
"		noisesize, turbulence, contrast, bright, hard, mapping TEXTURES_PARAM));\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE void BlenderWoodTexture_EvalOp(\n"
"		__global const Texture* restrict texture,\n"
"		const TextureEvalOpType evalType,\n"
"		__global float *evalStack,\n"
"		uint *evalStackOffset,\n"
"		__global const HitPoint *hitPoint,\n"
"		const float sampleDistance\n"
"		TEXTURES_PARAM_DECL) {\n"
"	switch (evalType) {\n"
"		case EVAL_FLOAT: {\n"
"			const float eval = BlenderWoodTexture_ConstEvaluateFloat(hitPoint,\n"
"					texture->blenderWood.type, texture->blenderWood.noisebasis2,\n"
"					texture->blenderWood.noisebasis, texture->blenderWood.noisesize,\n"
"					texture->blenderWood.turbulence, texture->blenderWood.contrast,\n"
"					texture->blenderWood.bright, texture->blenderWood.hard,\n"
"					&texture->blenderWood.mapping TEXTURES_PARAM);\n"
"			EvalStack_PushFloat(eval);\n"
"			break;\n"
"		}\n"
"		case EVAL_SPECTRUM: {\n"
"			const float3 eval = BlenderWoodTexture_ConstEvaluateSpectrum(hitPoint,\n"
"					texture->blenderWood.type, texture->blenderWood.noisebasis2,\n"
"					texture->blenderWood.noisebasis, texture->blenderWood.noisesize,\n"
"					texture->blenderWood.turbulence, texture->blenderWood.contrast,\n"
"					texture->blenderWood.bright, texture->blenderWood.hard,\n"
"					&texture->blenderWood.mapping TEXTURES_PARAM);\n"
"			EvalStack_PushFloat3(eval);\n"
"			break;\n"
"		}\n"
"		case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"			Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"			Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		case EVAL_BUMP:\n"
"			Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		default:\n"
"			// Something wrong here\n"
"			break;\n"
"	}\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Blender voronoi texture\n"
"//------------------------------------------------------------------------------\n"
" \n"
"OPENCL_FORCE_NOT_INLINE float BlenderVoronoiTexture_Evaluate(__global const HitPoint *hitPoint,\n"
"		const DistanceMetric distancemetric, const float feature_weight1, const float feature_weight2, \n"
"		const float feature_weight3, const float feature_weight4, const float noisesize, const float intensity,\n"
"		const float exponent, const float contrast, const float bright, __global const TextureMapping3D *mapping\n"
"		TEXTURES_PARAM_DECL) {\n"
"    float da[4], pa[12]; /* distance and point coordinate arrays of 4 nearest neighbours */\n"
"	float scale = 1.f;\n"
"	if(fabs(noisesize) > 0.00001f) scale = (1.f/noisesize);\n"
"	const float3 P = scale * TextureMapping3D_Map(mapping, hitPoint, NULL TEXTURES_PARAM);\n"
" \n"
"	const float aw1 = fabs(feature_weight1);\n"
"    const float aw2 = fabs(feature_weight2);\n"
"    const float aw3 = fabs(feature_weight3);\n"
"    const float aw4 = fabs(feature_weight4);\n"
"\n"
"    float sc = (aw1 + aw2 + aw3 + aw4);\n"
"\n"
"    if (sc > 0.00001f) sc = intensity / sc;\n"
"\n"
"    float result = 1.f;\n"
"\n"
"	voronoi(P.x, P.y, P.z, da, pa, exponent, distancemetric);\n"
"    result = sc * fabs(feature_weight1 * da[0] + feature_weight2 * da[1] + feature_weight3 * da[2] + feature_weight4 * da[3]);\n"
"\n"
"	result = (result - 0.5f) * contrast + bright - 0.5f;\n"
"    if(result < 0.f) result = 0.f; \n"
"	else if(result > 1.f) result = 1.f;\n"
"\n"
"    return result;\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float BlenderVoronoiTexture_ConstEvaluateFloat(__global const HitPoint *hitPoint,	const DistanceMetric distancemetric, const float feature_weight1, \n"
"		const float feature_weight2, const float feature_weight3, const float feature_weight4, const float noisesize, const float intensity,\n"
"		const float exponent, const float contrast, const float bright, __global const TextureMapping3D *mapping\n"
"		TEXTURES_PARAM_DECL) {	\n"
"	return BlenderVoronoiTexture_Evaluate(hitPoint, distancemetric, feature_weight1, feature_weight2, feature_weight3, feature_weight4,\n"
"		noisesize, intensity, exponent, contrast, bright, mapping\n"
"		TEXTURES_PARAM);\n"
"}\n"
" \n"
"OPENCL_FORCE_NOT_INLINE float3 BlenderVoronoiTexture_ConstEvaluateSpectrum(__global const HitPoint *hitPoint, const DistanceMetric distancemetric, const float feature_weight1,\n"
"		const float feature_weight2, const float feature_weight3, const float feature_weight4, const float noisesize, const float intensity,\n"
"		const float exponent, const float contrast, const float bright, __global const TextureMapping3D *mapping\n"
"		TEXTURES_PARAM_DECL) {\n"
"	return TO_FLOAT3(BlenderVoronoiTexture_Evaluate(hitPoint, distancemetric, feature_weight1, feature_weight2, feature_weight3, feature_weight4,\n"
"		noisesize, intensity, exponent, contrast, bright, mapping\n"
"		TEXTURES_PARAM));\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE void BlenderVoronoiTexture_EvalOp(\n"
"		__global const Texture* restrict texture,\n"
"		const TextureEvalOpType evalType,\n"
"		__global float *evalStack,\n"
"		uint *evalStackOffset,\n"
"		__global const HitPoint *hitPoint,\n"
"		const float sampleDistance\n"
"		TEXTURES_PARAM_DECL) {\n"
"	switch (evalType) {\n"
"		case EVAL_FLOAT: {\n"
"			const float eval = BlenderVoronoiTexture_ConstEvaluateFloat(hitPoint,\n"
"					texture->blenderVoronoi.distancemetric, texture->blenderVoronoi.feature_weight1,\n"
"					texture->blenderVoronoi.feature_weight2, texture->blenderVoronoi.feature_weight3,\n"
"					texture->blenderVoronoi.feature_weight4, texture->blenderVoronoi.noisesize,\n"
"					texture->blenderVoronoi.intensity, texture->blenderVoronoi.exponent,\n"
"					texture->blenderVoronoi.contrast, texture->blenderVoronoi.bright,\n"
"					&texture->blenderVoronoi.mapping TEXTURES_PARAM);\n"
"			EvalStack_PushFloat(eval);\n"
"			break;\n"
"		}\n"
"		case EVAL_SPECTRUM: {\n"
"			const float3 eval = BlenderVoronoiTexture_ConstEvaluateSpectrum(hitPoint,\n"
"					texture->blenderVoronoi.distancemetric, texture->blenderVoronoi.feature_weight1,\n"
"					texture->blenderVoronoi.feature_weight2, texture->blenderVoronoi.feature_weight3,\n"
"					texture->blenderVoronoi.feature_weight4, texture->blenderVoronoi.noisesize,\n"
"					texture->blenderVoronoi.intensity, texture->blenderVoronoi.exponent,\n"
"					texture->blenderVoronoi.contrast, texture->blenderVoronoi.bright,\n"
"					&texture->blenderVoronoi.mapping TEXTURES_PARAM);\n"
"			EvalStack_PushFloat3(eval);\n"
"			break;\n"
"		}\n"
"		case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"			Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"			Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		case EVAL_BUMP:\n"
"			Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		default:\n"
"			// Something wrong here\n"
"			break;\n"
"	}\n"
"}\n";
} }
