
#include <string>

namespace slg { namespace ocl {
std::string KernelSource_texture_bombing_funcs =
"#line 2 \"texture_bombing_funcs.cl\"\n"
"\n"
"/***************************************************************************\n"
" * Copyright 1998-2020 by authors (see AUTHORS.txt)                        *\n"
" *                                                                         *\n"
" *   This file is part of LuxCoreRender.                                   *\n"
" *                                                                         *\n"
" * Licensed under the Apache License, Version 2.0 (the \"License\");         *\n"
" * you may not use this file except in compliance with the License.        *\n"
" * You may obtain a copy of the License at                                 *\n"
" *                                                                         *\n"
" *     http://www.apache.org/licenses/LICENSE-2.0                          *\n"
" *                                                                         *\n"
" * Unless required by applicable law or agreed to in writing, software     *\n"
" * distributed under the License is distributed on an \"AS IS\" BASIS,       *\n"
" * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.*\n"
" * See the License for the specific language governing permissions and     *\n"
" * limitations under the License.                                          *\n"
" ***************************************************************************/\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Bombing texture\n"
"//------------------------------------------------------------------------------\n"
"\n"
"//----------------------------------------------------------------------\n"
"// For the very special case of Bombing texture evaluation\n"
"//\n"
"// EVAL_TRIPLANAR_STEP_1, EVAL_TRIPLANAR_STEP_2, EVAL_TRIPLANAR_STEP_3\n"
"// and EVAL_TRIPLANAR\n"
"//----------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE bool BombingTexture_WriteCellUV(__global const Texture* restrict texture,\n"
"		int i, int j, const float2 uv, const float resultPriority,\n"
"		float *currentTexturePriority, __global HitPoint *hitPointTmp\n"
"		TEXTURES_PARAM_DECL) {\n"
"	const float2 cell = MAKE_FLOAT2(floor(uv.x), floor(uv.y));\n"
"	const float2 cellOffset = MAKE_FLOAT2(uv.x - cell.x, uv.y - cell.y);\n"
"\n"
"	const float2 currentCell = cell + MAKE_FLOAT2(i, j);\n"
"	const float2 currentCellOffset = cellOffset - MAKE_FLOAT2(i, j);\n"
"\n"
"	// Pick cell random values\n"
"	__global const ImageMap *randomImageMap = &imageMapDescs[texture->bombingTex.randomImageMapIndex];\n"
"	const uint randomImageMapWidth = randomImageMap->width;\n"
"	const uint randomImageMapHeight = randomImageMap->height;\n"
"\n"
"	const uint randomImageMapStorageIndex = (((int)currentCell.x) % (randomImageMapWidth / 2)) * 2 +\n"
"			(((int)currentCell.y) % randomImageMapHeight) * randomImageMapWidth;\n"
"\n"
"	const float3 noise0 = ImageMapStorage_GetSpectrum(randomImageMap,\n"
"			randomImageMapStorageIndex % randomImageMapWidth, randomImageMapStorageIndex / randomImageMapHeight\n"
"			IMAGEMAPS_PARAM);\n"
"\n"
"	const float currentCellRandomOffsetX = noise0.x;\n"
"	const float currentCellRandomOffsetY = noise0.y;\n"
"	*currentTexturePriority = noise0.z;\n"
"\n"
"	// Check the priority of this cell\n"
"	if (*currentTexturePriority <= resultPriority)\n"
"		return false;\n"
"\n"
"	// Find The cell UV coordinates\n"
"	float2 bulletUV = MAKE_FLOAT2(currentCellOffset.x - currentCellRandomOffsetX, currentCellOffset.y - currentCellRandomOffsetY);\n"
"\n"
"	// Pick some more cell random values\n"
"	const float3 noise1 = ImageMapStorage_GetSpectrum(randomImageMap,\n"
"			(randomImageMapStorageIndex + 1) % randomImageMapWidth, (randomImageMapStorageIndex + 1) / randomImageMapHeight\n"
"			IMAGEMAPS_PARAM);\n"
"\n"
"	const float currentCellRandomRotate = noise1.x;\n"
"	const float currentCellRandomScale = noise1.y;\n"
"	const float currentCellRandomBullet = noise1.z;\n"
"\n"
"	// Translate to the cell center\n"
"	const float2 translatedUV = MAKE_FLOAT2(bulletUV.x - .5f, bulletUV.y - .5f);\n"
"\n"
"	// Apply random scale\n"
"	const float scaleFactor = Lerp(currentCellRandomScale, 1.f, 1.f + texture->bombingTex.randomScaleFactor);\n"
"	const float2 scaledUV = MAKE_FLOAT2(translatedUV.x * scaleFactor, translatedUV.y * scaleFactor);\n"
"\n"
"	// Apply random rotation\n"
"	const float angle = texture->bombingTex.useRandomRotation ? currentCellRandomRotate * (2.f * M_PI_F) : 0.f;\n"
"	const float sinAngle = sin(angle);\n"
"	const float cosAngle = cos(angle);\n"
"	const float2 rotatedUV = MAKE_FLOAT2(\n"
"			scaledUV.x * cosAngle - scaledUV.y * sinAngle,\n"
"			scaledUV.x * sinAngle + scaledUV.y * cosAngle);\n"
"\n"
"	// Translate back the cell center\n"
"	bulletUV.x = rotatedUV.x + .5f;\n"
"	bulletUV.y = rotatedUV.y + .5f;\n"
"\n"
"	// Check if I'm inside the cell\n"
"	//\n"
"	// The check is done outside this function on GPUs\n"
"	if ((bulletUV.x < 0.f) || (bulletUV.x >= 1.f) ||\n"
"			(bulletUV.y < 0.f) || (bulletUV.y >= 1.f))\n"
"		return false;\n"
"\n"
"	// Pick a bullet out if multiple available shapes (if multiBulletCount > 1)\n"
"	const uint multiBulletCount = texture->bombingTex.multiBulletCount;\n"
"	const uint currentCellRandomBulletIndex = Floor2UInt(multiBulletCount * currentCellRandomBullet);\n"
"	bulletUV.x = (currentCellRandomBulletIndex + bulletUV.x) / multiBulletCount;\n"
"\n"
"	VSTORE2F(bulletUV, &hitPointTmp->defaultUV.u);\n"
"\n"
"	return true;\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE void BombingTexture_EvalOp(\n"
"		__global const Texture* restrict texture,\n"
"		const TextureEvalOpType evalType,\n"
"		__global float *evalStack,\n"
"		uint *evalStackOffset,\n"
"		__global const HitPoint *hitPoint,\n"
"		const float sampleDistance\n"
"		TEXTURES_PARAM_DECL) {\n"
"	switch (evalType) {\n"
"		case EVAL_BOMBING_SETUP_11: {\n"
"			// Read background texture evaluation\n"
"			float3 backgroundTexValue;\n"
"			EvalStack_PopFloat3(backgroundTexValue);\n"
"\n"
"			// Save original UV\n"
"			const float2 uv = VLOAD2F(&hitPoint->defaultUV.u);\n"
"			EvalStack_PushFloat2(uv);\n"
"\n"
"			// Save mapped UV\n"
"			const float2 mapUV = TextureMapping2D_Map(&texture->bombingTex.mapping, hitPoint TEXTURES_PARAM);\n"
"			EvalStack_PushFloat2(mapUV);\n"
"\n"
"			// Save background texture value\n"
"			EvalStack_PushFloat3(backgroundTexValue);\n"
"\n"
"			// Save current result\n"
"			EvalStack_PushFloat3(backgroundTexValue);\n"
"\n"
"			// Save result priority\n"
"			float resultPriority = -1.f;\n"
"			EvalStack_PushFloat(resultPriority);\n"
"\n"
"			// Update HitPoint\n"
"			float currentTexturePriority;\n"
"			__global HitPoint *hitPointTmp = (__global HitPoint *)hitPoint;\n"
"			const bool usableResult = BombingTexture_WriteCellUV(texture,\n"
"					-1, -1, mapUV, resultPriority,\n"
"					&currentTexturePriority, hitPointTmp\n"
"					TEXTURES_PARAM);\n"
"\n"
"			// Save current result priority\n"
"			EvalStack_PushFloat(currentTexturePriority);\n"
"\n"
"			// Save usable result\n"
"			EvalStack_PushInt(usableResult);\n"
"			break;\n"
"		}\n"
"		case EVAL_BOMBING_SETUP_10: {\n"
"			// Read bullet mask texture evaluation\n"
"			float bulletMaskTexValue;\n"
"			EvalStack_PopFloat(bulletMaskTexValue);\n"
"			// Read bullet texture evaluation\n"
"			float3 bulletTexValue;\n"
"			EvalStack_PopFloat3(bulletTexValue);\n"
"\n"
"			// Read usable result\n"
"			bool usableResult;\n"
"			EvalStack_PopInt(usableResult);\n"
"\n"
"			// Read current result priority\n"
"			float currentTexturePriority;\n"
"			EvalStack_PopFloat(currentTexturePriority);\n"
"\n"
"			// Read result priority\n"
"			float resultPriority;\n"
"			EvalStack_PopFloat(resultPriority);\n"
"\n"
"			// Read current result\n"
"			float3 currentResult;\n"
"			EvalStack_PopFloat3(currentResult);\n"
"\n"
"			// Read background texture value\n"
"			float3 backgroundTexValue;\n"
"			EvalStack_PopFloat3(backgroundTexValue);\n"
"\n"
"			// Read mapped UV\n"
"			float2 mapUV;\n"
"			EvalStack_PopFloat2(mapUV);\n"
"\n"
"			// Save mapped UV\n"
"			EvalStack_PushFloat2(mapUV);\n"
"\n"
"			// Save background texture value\n"
"			EvalStack_PushFloat3(backgroundTexValue);\n"
"			\n"
"			// Save current result\n"
"			if (usableResult && (bulletMaskTexValue > 0.f)) {\n"
"				resultPriority = currentTexturePriority;\n"
"				currentResult = Lerp3(bulletMaskTexValue, backgroundTexValue, bulletTexValue);\n"
"			}\n"
"			EvalStack_PushFloat3(currentResult);\n"
"\n"
"			// Save current result priority\n"
"			EvalStack_PushFloat(resultPriority);\n"
"\n"
"			// Update HitPoint\n"
"			__global HitPoint *hitPointTmp = (__global HitPoint *)hitPoint;\n"
"			usableResult = BombingTexture_WriteCellUV(texture,\n"
"					-1, 0, mapUV, resultPriority,\n"
"					&currentTexturePriority, hitPointTmp\n"
"					TEXTURES_PARAM);\n"
"\n"
"			// Save current result priority\n"
"			EvalStack_PushFloat(currentTexturePriority);\n"
"\n"
"			// Save usable result\n"
"			EvalStack_PushInt(usableResult);\n"
"			break;\n"
"		}\n"
"		case EVAL_BOMBING_SETUP_01: {\n"
"			// Read bullet mask texture evaluation\n"
"			float bulletMaskTexValue;\n"
"			EvalStack_PopFloat(bulletMaskTexValue);\n"
"			// Read bullet texture evaluation\n"
"			float3 bulletTexValue;\n"
"			EvalStack_PopFloat3(bulletTexValue);\n"
"\n"
"			// Read usable result\n"
"			bool usableResult;\n"
"			EvalStack_PopInt(usableResult);\n"
"\n"
"			// Read current result priority\n"
"			float currentTexturePriority;\n"
"			EvalStack_PopFloat(currentTexturePriority);\n"
"\n"
"			// Read result priority\n"
"			float resultPriority;\n"
"			EvalStack_PopFloat(resultPriority);\n"
"\n"
"			// Read current result\n"
"			float3 currentResult;\n"
"			EvalStack_PopFloat3(currentResult);\n"
"\n"
"			// Read background texture value\n"
"			float3 backgroundTexValue;\n"
"			EvalStack_PopFloat3(backgroundTexValue);\n"
"\n"
"			// Read mapped UV\n"
"			float2 mapUV;\n"
"			EvalStack_PopFloat2(mapUV);\n"
"\n"
"			// Save mapped UV\n"
"			EvalStack_PushFloat2(mapUV);\n"
"\n"
"			// Save background texture value\n"
"			EvalStack_PushFloat3(backgroundTexValue);\n"
"			\n"
"			// Save current result\n"
"			if (usableResult && (bulletMaskTexValue > 0.f)) {\n"
"				resultPriority = currentTexturePriority;\n"
"				currentResult = Lerp3(bulletMaskTexValue, backgroundTexValue, bulletTexValue);\n"
"			}\n"
"			EvalStack_PushFloat3(currentResult);\n"
"\n"
"			// Save current result priority\n"
"			EvalStack_PushFloat(resultPriority);\n"
"\n"
"			// Update HitPoint\n"
"			__global HitPoint *hitPointTmp = (__global HitPoint *)hitPoint;\n"
"			usableResult = BombingTexture_WriteCellUV(texture,\n"
"					0, -1, mapUV, resultPriority,\n"
"					&currentTexturePriority, hitPointTmp\n"
"					TEXTURES_PARAM);\n"
"\n"
"			// Save current result priority\n"
"			EvalStack_PushFloat(currentTexturePriority);\n"
"\n"
"			// Save usable result\n"
"			EvalStack_PushInt(usableResult);\n"
"			break;\n"
"		}\n"
"		case EVAL_BOMBING_SETUP_00: {\n"
"			// Read bullet mask texture evaluation\n"
"			float bulletMaskTexValue;\n"
"			EvalStack_PopFloat(bulletMaskTexValue);\n"
"			// Read bullet texture evaluation\n"
"			float3 bulletTexValue;\n"
"			EvalStack_PopFloat3(bulletTexValue);\n"
"\n"
"			// Read usable result\n"
"			bool usableResult;\n"
"			EvalStack_PopInt(usableResult);\n"
"\n"
"			// Read current result priority\n"
"			float currentTexturePriority;\n"
"			EvalStack_PopFloat(currentTexturePriority);\n"
"\n"
"			// Read result priority\n"
"			float resultPriority;\n"
"			EvalStack_PopFloat(resultPriority);\n"
"\n"
"			// Read current result\n"
"			float3 currentResult;\n"
"			EvalStack_PopFloat3(currentResult);\n"
"\n"
"			// Read background texture value\n"
"			float3 backgroundTexValue;\n"
"			EvalStack_PopFloat3(backgroundTexValue);\n"
"\n"
"			// Read mapped UV\n"
"			float2 mapUV;\n"
"			EvalStack_PopFloat2(mapUV);\n"
"\n"
"			// Save mapped UV\n"
"			EvalStack_PushFloat2(mapUV);\n"
"\n"
"			// Save background texture value\n"
"			EvalStack_PushFloat3(backgroundTexValue);\n"
"			\n"
"			// Save current result\n"
"			if (usableResult && (bulletMaskTexValue > 0.f)) {\n"
"				resultPriority = currentTexturePriority;\n"
"				currentResult = Lerp3(bulletMaskTexValue, backgroundTexValue, bulletTexValue);\n"
"			}\n"
"			EvalStack_PushFloat3(currentResult);\n"
"\n"
"			// Save current result priority\n"
"			EvalStack_PushFloat(resultPriority);\n"
"\n"
"			// Update HitPoint\n"
"			__global HitPoint *hitPointTmp = (__global HitPoint *)hitPoint;\n"
"			usableResult = BombingTexture_WriteCellUV(texture,\n"
"					0, 0, mapUV, resultPriority,\n"
"					&currentTexturePriority, hitPointTmp\n"
"					TEXTURES_PARAM);\n"
"\n"
"			// Save current result priority\n"
"			EvalStack_PushFloat(currentTexturePriority);\n"
"\n"
"			// Save usable result\n"
"			EvalStack_PushInt(usableResult);\n"
"			break;\n"
"		}\n"
"		case EVAL_FLOAT:\n"
"		case EVAL_SPECTRUM: {\n"
"			// Read bullet mask texture evaluation\n"
"			float bulletMaskTexValue;\n"
"			EvalStack_PopFloat(bulletMaskTexValue);\n"
"			// Read bullet texture evaluation\n"
"			float3 bulletTexValue;\n"
"			EvalStack_PopFloat3(bulletTexValue);\n"
"\n"
"			// Read usable result\n"
"			bool usableResult;\n"
"			EvalStack_PopInt(usableResult);\n"
"\n"
"			// Read current result priority\n"
"			float currentTexturePriority;\n"
"			EvalStack_PopFloat(currentTexturePriority);\n"
"\n"
"			// Read result priority\n"
"			float resultPriority;\n"
"			EvalStack_PopFloat(resultPriority);\n"
"\n"
"			// Read current result\n"
"			float3 currentResult;\n"
"			EvalStack_PopFloat3(currentResult);\n"
"\n"
"			// Read background texture value\n"
"			float3 backgroundTexValue;\n"
"			EvalStack_PopFloat3(backgroundTexValue);\n"
"\n"
"			// Read mapped UV\n"
"			float2 mapUV;\n"
"			EvalStack_PopFloat2(mapUV);\n"
"\n"
"			// Read original UV\n"
"			float2 uv;\n"
"			EvalStack_PopFloat2(uv);\n"
"			\n"
"			// Save current result\n"
"			if (usableResult && (bulletMaskTexValue > 0.f)) {\n"
"				resultPriority = currentTexturePriority;\n"
"				currentResult = Lerp3(bulletMaskTexValue, backgroundTexValue, bulletTexValue);\n"
"			}\n"
"\n"
"			// Restore original HitPoint UV\n"
"			__global HitPoint *hitPointTmp = (__global HitPoint *)hitPoint;\n"
"			VSTORE2F(uv, &hitPointTmp->defaultUV.u);\n"
"\n"
"			// Save the texture evaluation result\n"
"			if (evalType == EVAL_FLOAT) {\n"
"				EvalStack_PushFloat(Spectrum_Y(currentResult));\n"
"			} else {\n"
"				EvalStack_PushFloat3(currentResult);\n"
"			}\n"
"			break;\n"
"		}\n"
"		case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"			Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"			Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		case EVAL_BUMP:\n"
"			Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		default:\n"
"			// Something wrong here\n"
"			break;\n"
"	}\n"
"}\n";
} }
