
#include <string>

namespace slg { namespace ocl {
std::string KernelSource_texture_brick_funcs =
"#line 2 \"texture_brick_funcs.cl\"\n"
"\n"
"/***************************************************************************\n"
" * Copyright 1998-2020 by authors (see AUTHORS.txt)                        *\n"
" *                                                                         *\n"
" *   This file is part of LuxCoreRender.                                   *\n"
" *                                                                         *\n"
" * Licensed under the Apache License, Version 2.0 (the \"License\");         *\n"
" * you may not use this file except in compliance with the License.        *\n"
" * You may obtain a copy of the License at                                 *\n"
" *                                                                         *\n"
" *     http://www.apache.org/licenses/LICENSE-2.0                          *\n"
" *                                                                         *\n"
" * Unless required by applicable law or agreed to in writing, software     *\n"
" * distributed under the License is distributed on an \"AS IS\" BASIS,       *\n"
" * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.*\n"
" * See the License for the specific language governing permissions and     *\n"
" * limitations under the License.                                          *\n"
" ***************************************************************************/\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Brick texture\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE float BrickTexture_BrickNoise(uint n) {\n"
"	n = (n + 1013) & 0x7fffffff;\n"
"	n = (n >> 13) ^ n;\n"
"	const uint nn = (n * (n * n * 60493 + 19990303) + 1376312589) & 0x7fffffff;\n"
"	return 0.5f * ((float)nn / 1073741824.0f);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE bool BrickTexture_RunningAlternate(const float3 p, float3 *i, float3 *b,\n"
"		const float run, const float mortarwidth,\n"
"		const float mortarheight, const float mortardepth,\n"
"		int nWhole) {\n"
"	const float sub = nWhole + 0.5f;\n"
"	const float rsub = ceil(sub);\n"
"	(*i).z = floor(p.z);\n"
"	(*b).x = (p.x + (*i).z * run) / sub;\n"
"	(*b).y = (p.y + (*i).z * run) / sub;\n"
"	(*i).x = floor((*b).x);\n"
"	(*i).y = floor((*b).y);\n"
"	(*b).x = ((*b).x - (*i).x) * sub;\n"
"	(*b).y = ((*b).y - (*i).y) * sub;\n"
"	(*b).z = (p.z - (*i).z) * sub;\n"
"	(*i).x += floor((*b).x) / rsub;\n"
"	(*i).y += floor((*b).y) / rsub;\n"
"	(*b).x -= floor((*b).x);\n"
"	(*b).y -= floor((*b).y);\n"
"	return (*b).z > mortarheight && (*b).y > mortardepth &&\n"
"		(*b).x > mortarwidth;\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE bool BrickTexture_Basket(const float3 p, float3 *i,\n"
"		const float mortarwidth, const float mortardepth,\n"
"		const float proportion, const float invproportion) {\n"
"	(*i).x = floor(p.x);\n"
"	(*i).y = floor(p.y);\n"
"	float bx = p.x - (*i).x;\n"
"	float by = p.y - (*i).y;\n"
"	(*i).x += (*i).y - 2.f * floor(0.5f * (*i).y);\n"
"	const bool split = ((*i).x - 2.f * floor(0.5f * (*i).x)) < 1.f;\n"
"	if (split) {\n"
"		bx = fmod(bx, invproportion);\n"
"		(*i).x = floor(proportion * p.x) * invproportion;\n"
"	} else {\n"
"		by = fmod(by, invproportion);\n"
"		(*i).y = floor(proportion * p.y) * invproportion;\n"
"	}\n"
"	return by > mortardepth && bx > mortarwidth;\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE bool BrickTexture_Herringbone(const float3 p, float3 *i,\n"
"		const float mortarwidth, const float mortarheight,\n"
"		const float proportion, const float invproportion) {\n"
"	(*i).y = floor(proportion * p.y);\n"
"	const float px = p.x + (*i).y * invproportion;\n"
"	(*i).x = floor(px);\n"
"	float bx = 0.5f * px - floor(px * 0.5f);\n"
"	bx *= 2.f;\n"
"	float by = proportion * p.y - floor(proportion * p.y);\n"
"	by *= invproportion;\n"
"	if (bx > 1.f + invproportion) {\n"
"		bx = proportion * (bx - 1.f);\n"
"		(*i).y -= floor(bx - 1.f);\n"
"		bx -= floor(bx);\n"
"		bx *= invproportion;\n"
"		by = 1.f;\n"
"	} else if (bx > 1.f) {\n"
"		bx = proportion * (bx - 1.f);\n"
"		(*i).y -= floor(bx - 1.f);\n"
"		bx -= floor(bx);\n"
"		bx *= invproportion;\n"
"	}\n"
"	return by > mortarheight && bx > mortarwidth;\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE bool BrickTexture_Running(const float3 p, float3 *i, float3 *b,\n"
"		const float run, const float mortarwidth,\n"
"		const float mortarheight, const float mortardepth) {\n"
"	(*i).z = floor(p.z);\n"
"	(*b).x = p.x + (*i).z * run;\n"
"	(*b).y = p.y - (*i).z * run;\n"
"	(*i).x = floor((*b).x);\n"
"	(*i).y = floor((*b).y);\n"
"	(*b).z = p.z - (*i).z;\n"
"	(*b).x -= (*i).x;\n"
"	(*b).y -= (*i).y;\n"
"	return (*b).z > mortarheight && (*b).y > mortardepth &&\n"
"		(*b).x > mortarwidth;\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE bool BrickTexture_English(const float3 p, float3 *i, float3 *b,\n"
"		const float run, const float mortarwidth,\n"
"		const float mortarheight, const float mortardepth) {\n"
"	(*i).z = floor(p.z);\n"
"	(*b).x = p.x + (*i).z * run;\n"
"	(*b).y = p.y - (*i).z * run;\n"
"	(*i).x = floor((*b).x);\n"
"	(*i).y = floor((*b).y);\n"
"	(*b).z = p.z - (*i).z;\n"
"	const float divider = floor(fmod(fabs((*i).z), 2.f)) + 1.f;\n"
"	(*b).x = (divider * (*b).x - floor(divider * (*b).x)) / divider;\n"
"	(*b).y = (divider * (*b).y - floor(divider * (*b).y)) / divider;\n"
"	return (*b).z > mortarheight && (*b).y > mortardepth &&\n"
"		(*b).x > mortarwidth;\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE bool BrickTexture_Evaluate(__global const HitPoint *hitPoint,\n"
"		const MasonryBond bond,\n"
"		const float brickwidth, const float brickheight,\n"
"		const float brickdepth, const float mortarsize,\n"
"		const float3 offset,\n"
"		const float run, const float mortarwidth,\n"
"		const float mortarheight, const float mortardepth,\n"
"		const float proportion, const float invproportion,\n"
"		float3 *brickIndex,\n"
"		__global const TextureMapping3D *mapping\n"
"		TEXTURES_PARAM_DECL) {\n"
"#define BRICK_EPSILON 1e-3f\n"
"	const float3 P = TextureMapping3D_Map(mapping, hitPoint, NULL TEXTURES_PARAM);\n"
"\n"
"	const float offs = BRICK_EPSILON + mortarsize;\n"
"	float3 bP = P + MAKE_FLOAT3(offs, offs, offs);\n"
"\n"
"	// Normalize coordinates according brick dimensions\n"
"	bP.x /= brickwidth;\n"
"	bP.y /= brickdepth;\n"
"	bP.z /= brickheight;\n"
"\n"
"	bP += offset;\n"
"\n"
"	float3 bevel;\n"
"	bool b;\n"
"	switch (bond) {\n"
"		case FLEMISH:\n"
"			b = BrickTexture_RunningAlternate(bP, brickIndex, &bevel,\n"
"					run , mortarwidth, mortarheight, mortardepth, 1);\n"
"			break;\n"
"		case RUNNING:\n"
"			b = BrickTexture_Running(bP, brickIndex, &bevel,\n"
"					run, mortarwidth, mortarheight, mortardepth);\n"
"			break;\n"
"		case ENGLISH:\n"
"			b = BrickTexture_English(bP, brickIndex, &bevel,\n"
"					run, mortarwidth, mortarheight, mortardepth);\n"
"			break;\n"
"		case HERRINGBONE:\n"
"			b = BrickTexture_Herringbone(bP, brickIndex,\n"
"					mortarwidth, mortarheight, proportion, invproportion);\n"
"			break;\n"
"		case BASKET:\n"
"			b = BrickTexture_Basket(bP, brickIndex,\n"
"					mortarwidth, mortardepth, proportion, invproportion);\n"
"			break;\n"
"		case KETTING:\n"
"			b = BrickTexture_RunningAlternate(bP, brickIndex, &bevel,\n"
"					run, mortarwidth, mortarheight, mortardepth, 2);\n"
"			break;\n"
"		default:\n"
"			b = true;\n"
"			break;\n"
"	}\n"
"\n"
"	return b;\n"
"#undef BRICK_EPSILON\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float3 BrickTexture_ConstEvaluateSpectrum(__global const HitPoint *hitPoint,\n"
"		const float3 value1, const float3 value2, const float3 value3,\n"
"		const MasonryBond bond,\n"
"		const float brickwidth, const float brickheight,\n"
"		const float brickdepth, const float mortarsize,\n"
"		const float3 offset,\n"
"		const float run, const float mortarwidth,\n"
"		const float mortarheight, const float mortardepth,\n"
"		const float proportion, const float invproportion,\n"
"		const float modulationBias,\n"
"		__global const TextureMapping3D *mapping\n"
"		TEXTURES_PARAM_DECL) {\n"
"	float3 brickIndex;\n"
"	const bool b = BrickTexture_Evaluate(hitPoint,\n"
"			bond,\n"
"			brickwidth, brickheight,\n"
"			brickdepth, mortarsize,\n"
"			offset,\n"
"			run, mortarwidth,\n"
"			mortarheight, mortardepth,\n"
"			proportion, invproportion,\n"
"			&brickIndex,\n"
"			mapping\n"
"			TEXTURES_PARAM);\n"
"	\n"
"	if (b) {\n"
"		// Return brick color\n"
"		if (modulationBias == -1.f) {\n"
"			return value1;\n"
"		} else if (modulationBias == 1.f) {\n"
"			return value3;\n"
"		}\n"
"		\n"
"		const int rownum = brickIndex.y;\n"
"		const int bricknum = brickIndex.x;\n"
"		const float noise = BrickTexture_BrickNoise((rownum << 16) + (bricknum & 0xffff));\n"
"		const float modulation = clamp(noise + modulationBias, 0.f, 1.f);\n"
"		\n"
"		return Lerp3(modulation, value1, value3);\n"
"	} else {\n"
"		// Return mortar color\n"
"		return value2;\n"
"	}\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float BrickTexture_ConstEvaluateFloat(__global const HitPoint *hitPoint,\n"
"		const float3 value1, const float3 value2, const float3 value3,\n"
"		const MasonryBond bond,\n"
"		const float brickwidth, const float brickheight,\n"
"		const float brickdepth, const float mortarsize,\n"
"		const float3 offset,\n"
"		const float run, const float mortarwidth,\n"
"		const float mortarheight, const float mortardepth,\n"
"		const float proportion, const float invproportion,\n"
"		const float modulationBias,\n"
"		__global const TextureMapping3D *mapping TEXTURES_PARAM_DECL) {\n"
"	return Spectrum_Y(BrickTexture_ConstEvaluateSpectrum(\n"
"			hitPoint, value1, value2, value3, bond, brickwidth, brickheight,\n"
"			brickdepth, mortarsize, offset, run, mortarwidth, mortarheight, mortardepth,\n"
"			proportion, invproportion, modulationBias, mapping\n"
"			TEXTURES_PARAM));\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE void BrickTexture_EvalOp(\n"
"		__global const Texture* restrict texture,\n"
"		const TextureEvalOpType evalType,\n"
"		__global float *evalStack,\n"
"		uint *evalStackOffset,\n"
"		__global const HitPoint *hitPoint,\n"
"		const float sampleDistance\n"
"		TEXTURES_PARAM_DECL) {\n"
"	switch (evalType) {\n"
"		case EVAL_FLOAT: {\n"
"			float3 tex1, tex2, tex3;\n"
"			EvalStack_PopFloat3(tex3);\n"
"			EvalStack_PopFloat3(tex2);\n"
"			EvalStack_PopFloat3(tex1);\n"
"\n"
"			const float eval = BrickTexture_ConstEvaluateFloat(hitPoint,\n"
"					tex1, tex2, tex3,\n"
"					texture->brick.bond,\n"
"					texture->brick.brickwidth, texture->brick.brickheight,\n"
"					texture->brick.brickdepth, texture->brick.mortarsize,\n"
"					MAKE_FLOAT3(texture->brick.offsetx, texture->brick.offsety, texture->brick.offsetz),\n"
"					texture->brick.run, texture->brick.mortarwidth,\n"
"					texture->brick.mortarheight, texture->brick.mortardepth,\n"
"					texture->brick.proportion, texture->brick.invproportion,\n"
"					texture->brick.modulationBias,\n"
"					&texture->brick.mapping\n"
"					TEXTURES_PARAM);\n"
"			EvalStack_PushFloat(eval);\n"
"			break;\n"
"		}\n"
"		case EVAL_SPECTRUM: {\n"
"			float3 tex1, tex2, tex3;\n"
"			EvalStack_PopFloat3(tex3);\n"
"			EvalStack_PopFloat3(tex2);\n"
"			EvalStack_PopFloat3(tex1);\n"
"\n"
"			const float3 eval = BrickTexture_ConstEvaluateSpectrum(hitPoint,\n"
"					tex1, tex2, tex3,\n"
"					texture->brick.bond,\n"
"					texture->brick.brickwidth, texture->brick.brickheight,\n"
"					texture->brick.brickdepth, texture->brick.mortarsize,\n"
"					MAKE_FLOAT3(texture->brick.offsetx, texture->brick.offsety, texture->brick.offsetz),\n"
"					texture->brick.run, texture->brick.mortarwidth,\n"
"					texture->brick.mortarheight, texture->brick.mortardepth,\n"
"					texture->brick.proportion, texture->brick.invproportion,\n"
"					texture->brick.modulationBias,\n"
"					&texture->brick.mapping\n"
"					TEXTURES_PARAM);\n"
"			EvalStack_PushFloat3(eval);\n"
"			break;\n"
"		}\n"
"		case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"			Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"			Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		case EVAL_BUMP:\n"
"			Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		default:\n"
"			// Something wrong here\n"
"			break;\n"
"	}\n"
"}\n";
} }
