
#include <string>

namespace slg { namespace ocl {
std::string KernelSource_texture_bump_funcs =
"#line 2 \"texture_bump_funcs.cl\"\n"
"\n"
"/***************************************************************************\n"
" * Copyright 1998-2020 by authors (see AUTHORS.txt)                        *\n"
" *                                                                         *\n"
" *   This file is part of LuxCoreRender.                                   *\n"
" *                                                                         *\n"
" * Licensed under the Apache License, Version 2.0 (the \"License\");         *\n"
" * you may not use this file except in compliance with the License.        *\n"
" * You may obtain a copy of the License at                                 *\n"
" *                                                                         *\n"
" *     http://www.apache.org/licenses/LICENSE-2.0                          *\n"
" *                                                                         *\n"
" * Unless required by applicable law or agreed to in writing, software     *\n"
" * distributed under the License is distributed on an \"AS IS\" BASIS,       *\n"
" * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.*\n"
" * See the License for the specific language governing permissions and     *\n"
" * limitations under the License.                                          *\n"
" ***************************************************************************/\n"
"\n"
"// Texture bump/normal mapping\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Generic texture bump mapping\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE float3 GenericTexture_Bump(\n"
"		__global const HitPoint *hitPoint,\n"
"		const float sampleDistance,\n"
"		const float evalFloatTexBase,\n"
"		const float evalFloatTexOffsetU,\n"
"		const float evalFloatTexOffsetV) {\n"
"	const float3 dpdu = VLOAD3F(&hitPoint->dpdu.x);\n"
"	const float3 dpdv = VLOAD3F(&hitPoint->dpdv.x);\n"
"\n"
"	float2 duv;\n"
"\n"
"	const float base = evalFloatTexBase;\n"
"\n"
"	const float uu = sampleDistance / length(dpdu);\n"
"	const float duValue = evalFloatTexOffsetU;\n"
"	duv.x = (duValue - base) / uu;\n"
"\n"
"	const float vv = sampleDistance / length(dpdv);\n"
"	const float dvValue = evalFloatTexOffsetV;\n"
"	duv.y = (dvValue - base) / vv;\n"
"\n"
"	// Compute the new dpdu and dpdv\n"
"	const float3 shadeN = VLOAD3F(&hitPoint->shadeN.x);\n"
"	const float3 bumpDpdu = dpdu + duv.x * shadeN;\n"
"	const float3 bumpDpdv = dpdv + duv.y * shadeN;\n"
"	float3 newShadeN = normalize(cross(bumpDpdu, bumpDpdv));\n"
"\n"
"	// The above transform keeps the normal in the original normal\n"
"	// hemisphere. If they are opposed, it means UVN was indirect and\n"
"	// the normal needs to be reversed\n"
"	newShadeN *= (dot(shadeN, newShadeN) < 0.f) ? -1.f : 1.f;\n"
"\n"
"	return newShadeN;\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// ConstTexture_Bump() for constant textures\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE float3 ConstTexture_Bump(__global const HitPoint *hitPoint) {\n"
"	return VLOAD3F(&hitPoint->shadeN.x);\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// ImageMapTexture\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE float3 ImageMapTexture_Bump(__global const Texture* restrict tex,\n"
"		__global const HitPoint *hitPoint\n"
"		TEXTURES_PARAM_DECL) {\n"
"	float2 du, dv;\n"
"	const float2 uv = TextureMapping2D_MapDuv(&tex->imageMapTex.mapping, hitPoint, &du, &dv TEXTURES_PARAM);\n"
"	__global const ImageMap *imageMap = &imageMapDescs[tex->imageMapTex.imageMapIndex];\n"
"	const float2 dst = ImageMap_GetDuv(imageMap, uv.x, uv.y IMAGEMAPS_PARAM);\n"
"\n"
"	const float2 duv = tex->imageMapTex.gain * MAKE_FLOAT2(dot(dst, du), dot(dst, dv));\n"
"\n"
"	const float3 shadeN = VLOAD3F(&hitPoint->shadeN.x);\n"
"	const float3 dpdu = VLOAD3F(&hitPoint->dpdu.x) + duv.x * shadeN;\n"
"	const float3 dpdv = VLOAD3F(&hitPoint->dpdv.x) + duv.y * shadeN;\n"
"\n"
"	const float3 n = normalize(cross(dpdu, dpdv));\n"
"\n"
"	return ((dot(n, shadeN) < 0.f) ? -1.f : 1.f) * n;\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// ScaleTexture\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE float3 ScaleTexture_Bump(__global const HitPoint *hitPoint,\n"
"		const float3 bumbNTex1, const float3 bumbNTex2,\n"
"		const float evalFloatTex1, const float evalFloatTex2) {\n"
"	const float3 shadeN = VLOAD3F(&hitPoint->shadeN.x);\n"
"	const float3 dpdu = VLOAD3F(&hitPoint->dpdu.x);\n"
"\n"
"	const float3 u = normalize(dpdu);\n"
"	const float3 v = normalize(cross(shadeN, dpdu));\n"
"\n"
"	const float3 n1 = bumbNTex1;\n"
"	const float nn1 = dot(n1, shadeN);\n"
"	float du1, dv1;\n"
"	if (nn1 != 0.f) {\n"
"		du1 = dot(n1, u) / nn1;\n"
"		dv1 = dot(n1, v) / nn1;\n"
"	} else {\n"
"		du1 = 0.f;\n"
"		dv1 = 0.f;\n"
"	}\n"
"\n"
"	const float3 n2 = bumbNTex2;\n"
"	const float nn2 = dot(n2, shadeN);\n"
"	float du2, dv2;\n"
"	if (nn2 != 0.f) {\n"
"		du2 = dot(n2, u) / nn2;\n"
"		dv2 = dot(n2, v) / nn2;\n"
"	} else {\n"
"		du1 = 0.f;\n"
"		dv1 = 0.f;		\n"
"	}\n"
"\n"
"	const float t1 = evalFloatTex1;\n"
"	const float t2 = evalFloatTex2;\n"
"\n"
"	const float du = du1 * t2 + t1 * du2;\n"
"	const float dv = dv1 * t2 + t1 * dv2;\n"
"\n"
"	return normalize(shadeN + du * u + dv * v);\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// MixTexture\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE float3 MixTexture_Bump(__global const HitPoint *hitPoint,\n"
"		const float3 bumbNTex1, const float3 bumbNTex2, const float3 bumbNAmount,\n"
"		const float evalFloatTex1, const float evalFloatTex2, const float evalFloatAmount) {\n"
"	const float3 shadeN = VLOAD3F(&hitPoint->shadeN.x);\n"
"	const float3 dpdu = VLOAD3F(&hitPoint->dpdu.x);\n"
"	const float3 u = normalize(dpdu);\n"
"	const float3 v = normalize(cross(shadeN, dpdu));\n"
"\n"
"	float3 n = bumbNTex1;\n"
"	float nn = dot(n, shadeN);\n"
"	const float du1 = dot(n, u) / nn;\n"
"	const float dv1 = dot(n, v) / nn;\n"
"\n"
"	n = bumbNTex2;\n"
"	nn = dot(n, shadeN);\n"
"	const float du2 = dot(n, u) / nn;\n"
"	const float dv2 = dot(n, v) / nn;\n"
"\n"
"	n = bumbNTex2;\n"
"	nn = dot(n, shadeN);\n"
"	const float dua = dot(n, u) / nn;\n"
"	const float dva = dot(n, v) / nn;\n"
"\n"
"	const float t1 = evalFloatTex1;\n"
"	const float t2 = evalFloatTex2;\n"
"	const float amt = clamp(evalFloatAmount, 0.f, 1.f);\n"
"\n"
"	const float du = Lerp(amt, du1, du2) + dua * (t2 - t1);\n"
"	const float dv = Lerp(amt, dv1, dv2) + dva * (t2 - t1);\n"
"\n"
"	return normalize(shadeN + du * u + dv * v);\n"
"};\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// AddTexture\n"
"//------------------------------------------------------------------------------\n"
"\n"
" OPENCL_FORCE_INLINE float3 AddTexture_Bump(__global const HitPoint *hitPoint,\n"
"		 const float3 bumbNTex1, const float3 bumbNTex2) {\n"
"	return normalize(bumbNTex1 + bumbNTex2 - VLOAD3F(&hitPoint->shadeN.x));\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// SubtractTexture\n"
"//------------------------------------------------------------------------------\n"
"\n"
" OPENCL_FORCE_INLINE float3 SubtractTexture_Bump(__global const HitPoint *hitPoint,\n"
"		 const float3 bumbNTex1, const float3 bumbNTex2) {\n"
"	return normalize(bumbNTex1 - bumbNTex2 + VLOAD3F(&hitPoint->shadeN.x));\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// NormalMapTexture\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE float3 NormalMapTexture_Bump(\n"
"		__global const Texture* restrict tex,\n"
"		__global const HitPoint *hitPoint,\n"
"		const float3 evalSpectrumTex) {\n"
"	// Normal from normal map\n"
"	const float3 rgb = clamp(evalSpectrumTex, 0.f, 1.f);\n"
"\n"
"	// Normal from normal map\n"
"	float3 n =  MAKE_FLOAT3(2.f, 2.f, 2.f) * rgb - MAKE_FLOAT3(1.f, 1.f, 1.f);\n"
"	const float scale = tex->normalMap.scale;\n"
"	n.x *= scale;\n"
"	n.y *= scale;\n"
"\n"
"	const float3 oldShadeN = VLOAD3F(&hitPoint->shadeN.x);\n"
"	const float3 dpdu = VLOAD3F(&hitPoint->dpdu.x);\n"
"	const float3 dpdv = VLOAD3F(&hitPoint->dpdv.x);\n"
"	\n"
"	Frame frame;\n"
"	Frame_Set_Private(&frame, dpdu, dpdv, oldShadeN);\n"
"\n"
"	// Transform n from tangent to object space\n"
"	float3 shadeN = normalize(Frame_ToWorld_Private(&frame, n));\n"
"	shadeN *= (dot(oldShadeN, shadeN) < 0.f) ? -1.f : 1.f;\n"
"\n"
"	return shadeN;\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// TriplanarTexture\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float3 TriplanarTexture_BumpUVLess(\n"
"		__global const HitPoint *hitPoint,\n"
"		const float sampleDistance,\n"
"		const float evalFloatTexBase,\n"
"		const float evalFloatTexOffsetX,\n"
"		const float evalFloatTexOffsetY,\n"
"		const float evalFloatTexOffsetZ) {\n"
"	// Calculate bump map value at intersection point\n"
"	const float base = evalFloatTexBase;\n"
"\n"
"	float3 dhdx;\n"
"\n"
"	const float offsetX = evalFloatTexOffsetX;\n"
"	dhdx.x = (offsetX - base) / sampleDistance;\n"
"\n"
"	const float offsetY = evalFloatTexOffsetY;\n"
"	dhdx.y = (offsetY - base) / sampleDistance;\n"
"\n"
"	const float offsetZ = evalFloatTexOffsetZ;\n"
"	dhdx.z = (offsetZ - base) / sampleDistance;\n"
"\n"
"	const float3 shadeN = VLOAD3F(&hitPoint->shadeN.x);\n"
"	float3 newShadeN = normalize(shadeN - dhdx);\n"
"	newShadeN *= (dot(shadeN, newShadeN) < 0.f) ? -1.f : 1.f;\n"
"\n"
"	return newShadeN;\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Texture evaluation functions\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_NOT_INLINE void Texture_EvalOpGenericBumpOffsetU(\n"
"		__global float *evalStack,\n"
"		uint *evalStackOffset,\n"
"		__global const HitPoint *hitPoint,\n"
"		const float sampleDistance) {\n"
"	const float3 origP = VLOAD3F(&hitPoint->p.x);\n"
"	const float3 origShadeN = VLOAD3F(&hitPoint->shadeN.x);\n"
"	const float2 origUV = VLOAD2F(&hitPoint->defaultUV.u);\n"
"\n"
"	// Save original P\n"
"	EvalStack_PushFloat3(origP);\n"
"	// Save original shadeN\n"
"	EvalStack_PushFloat3(origShadeN);\n"
"	// Save original UV\n"
"	EvalStack_PushFloat2(origUV);\n"
"\n"
"	// Update HitPoint\n"
"	__global HitPoint *hitPointTmp = (__global HitPoint *)hitPoint;\n"
"	const float3 dpdu = VLOAD3F(&hitPointTmp->dpdu.x);\n"
"	const float3 dndu = VLOAD3F(&hitPoint->dndu.x);\n"
"	// Shift hitPointTmp.du in the u direction and calculate value\n"
"	const float uu = sampleDistance / length(dpdu);\n"
"	VSTORE3F(origP + uu * dpdu, &hitPointTmp->p.x);\n"
"	hitPointTmp->defaultUV.u = origUV.x + uu;\n"
"	hitPointTmp->defaultUV.v = origUV.y;\n"
"	VSTORE3F(normalize(origShadeN + uu * dndu), &hitPointTmp->shadeN.x);\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE void Texture_EvalOpGenericBumpOffsetV(\n"
"		__global float *evalStack,\n"
"		uint *evalStackOffset,\n"
"		__global const HitPoint *hitPoint,\n"
"		const float sampleDistance) {\n"
"	// -1 is for result of EVAL_BUMP_GENERIC_OFFSET_U\n"
"	const float3 origP = EvalStack_ReadFloat3(-8 - 1);\n"
"	const float3 origShadeN = EvalStack_ReadFloat3(-5 - 1);\n"
"	const float2 origUV = EvalStack_ReadFloat2(-2 - 1);\n"
"\n"
"	// Update HitPoint\n"
"	__global HitPoint *hitPointTmp = (__global HitPoint *)hitPoint;\n"
"	const float3 dpdv = VLOAD3F(&hitPointTmp->dpdv.x);\n"
"	const float3 dndv = VLOAD3F(&hitPoint->dndv.x);\n"
"	// Shift hitPointTmp.dv in the v direction and calculate value\n"
"	const float vv = sampleDistance / length(dpdv);\n"
"	VSTORE3F(origP + vv * dpdv, &hitPointTmp->p.x);\n"
"	hitPointTmp->defaultUV.u = origUV.x;\n"
"	hitPointTmp->defaultUV.v = origUV.y + vv;\n"
"	VSTORE3F(normalize(origShadeN + vv * dndv), &hitPointTmp->shadeN.x);\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE void Texture_EvalOpGenericBump(\n"
"		__global float *evalStack,\n"
"		uint *evalStackOffset,\n"
"		__global const HitPoint *hitPoint,\n"
"		const float sampleDistance) {\n"
"	float evalFloatTexBase, evalFloatTexOffsetU, evalFloatTexOffsetV;\n"
"	EvalStack_PopFloat(evalFloatTexOffsetV);\n"
"	EvalStack_PopFloat(evalFloatTexOffsetU);\n"
"\n"
"	float3 origP, origShadeN;\n"
"	float2 origUV;\n"
"	EvalStack_PopFloat2(origUV);\n"
"	EvalStack_PopFloat3(origShadeN);\n"
"	EvalStack_PopFloat3(origP);\n"
"\n"
"	// evalFloatTexBase is the very first evaluation done so\n"
"	// it is the last for a stack pop\n"
"	EvalStack_PopFloat(evalFloatTexBase);\n"
"\n"
"	// Restore original P, shadeN and UV\n"
"	__global HitPoint *hitPointTmp = (__global HitPoint *)hitPoint;\n"
"	VSTORE3F(origP, &hitPointTmp->p.x);\n"
"	VSTORE3F(origShadeN, &hitPointTmp->shadeN.x);\n"
"	hitPointTmp->defaultUV.u = origUV.x;\n"
"	hitPointTmp->defaultUV.v = origUV.y;\n"
"\n"
"	const float3 shadeN = GenericTexture_Bump(hitPoint, sampleDistance,\n"
"			evalFloatTexBase, evalFloatTexOffsetU, evalFloatTexOffsetV);\n"
"	EvalStack_PushFloat3(shadeN);\n"
"}\n";
} }
