
#include <string>

namespace slg { namespace ocl {
std::string KernelSource_texture_densitygrid_funcs =
"#line 2 \"texture_densitygrid_funcs.cl\"\n"
"\n"
"/***************************************************************************\n"
" * Copyright 1998-2020 by authors (see AUTHORS.txt)                        *\n"
" *                                                                         *\n"
" *   This file is part of LuxCoreRender.                                   *\n"
" *                                                                         *\n"
" * Licensed under the Apache License, Version 2.0 (the \"License\");         *\n"
" * you may not use this file except in compliance with the License.        *\n"
" * You may obtain a copy of the License at                                 *\n"
" *                                                                         *\n"
" *     http://www.apache.org/licenses/LICENSE-2.0                          *\n"
" *                                                                         *\n"
" * Unless required by applicable law or agreed to in writing, software     *\n"
" * distributed under the License is distributed on an \"AS IS\" BASIS,       *\n"
" * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.*\n"
" * See the License for the specific language governing permissions and     *\n"
" * limitations under the License.                                          *\n"
" ***************************************************************************/\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// DensityGrid texture\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE float3 DensityGridTexture_D(\n"
"		__global const ImageMap *imageMap,\n"
"		int x, int y, int z,\n"
"		int nx, int ny, int nz\n"
"		IMAGEMAPS_PARAM_DECL) {\n"
"	__global const void *pixels = ImageMap_GetPixelsAddress(\n"
"		imageMapBuff, imageMap->pageIndex, imageMap->pixelsIndex);\n"
"	const ImageMapStorageType storageType = imageMap->storageType;\n"
"	const uint channelCount = imageMap->channelCount;\n"
"\n"
"	const uint index = ((clamp(z, 0, nz - 1) * ny) + clamp(y, 0, ny - 1)) * nx + clamp(x, 0, nx - 1);\n"
"	\n"
"	return ImageMap_GetTexel_SpectrumValue(storageType, pixels, channelCount, index);\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float3 DensityGridTexture_ConstEvaluateSpectrum(__global const HitPoint *hitPoint,\n"
"		const int nx, const int ny, const int nz,\n"
"		const uint imageMapIndex, __global const TextureMapping3D *mapping\n"
"		TEXTURES_PARAM_DECL) {\n"
"	__global const ImageMap *imageMap = &imageMapDescs[imageMapIndex];\n"
"\n"
"	const float3 P = TextureMapping3D_Map(mapping, hitPoint, NULL TEXTURES_PARAM);\n"
"\n"
"	float x, y, z;\n"
"	int vx, vy, vz;\n"
"\n"
"	switch (imageMap->wrapType) {\n"
"		case WRAP_REPEAT:\n"
"			x = P.x * nx;\n"
"			vx = Floor2Int(x);\n"
"			x -= vx;\n"
"			vx = Mod(vx, nx);\n"
"			y = P.y * ny;\n"
"			vy = Floor2Int(y);\n"
"			y -= vy;\n"
"			vy = Mod(vy, ny);\n"
"			z = P.z * nz;\n"
"			vz = Floor2Int(z);\n"
"			z -= vz;\n"
"			vz = Mod(vz, nz);\n"
"			break;\n"
"		case WRAP_BLACK:\n"
"			if (P.x < 0.f || P.x >= 1.f ||\n"
"				P.y < 0.f || P.y >= 1.f ||\n"
"				P.z < 0.f || P.z >= 1.f)\n"
"				return BLACK;\n"
"			x = P.x * nx;\n"
"			vx = Floor2Int(x);\n"
"			x -= vx;\n"
"			y = P.y * ny;\n"
"			vy = Floor2Int(y);\n"
"			y -= vy;\n"
"			z = P.z * nz;\n"
"			vz = Floor2Int(z);\n"
"			z -= vz;\n"
"			break;\n"
"		case WRAP_WHITE:\n"
"			if (P.x < 0.f || P.x >= 1.f ||\n"
"				P.y < 0.f || P.y >= 1.f ||\n"
"				P.z < 0.f || P.z >= 1.f)\n"
"				return WHITE;\n"
"			x = P.x * nx;\n"
"			vx = Floor2Int(x);\n"
"			x -= vx;\n"
"			y = P.y * ny;\n"
"			vy = Floor2Int(y);\n"
"			y -= vy;\n"
"			z = P.z * nz;\n"
"			vz = Floor2Int(z);\n"
"			z -= vz;\n"
"			break;\n"
"		case WRAP_CLAMP:\n"
"			x = clamp(P.x, 0.f, 1.f) * nx;\n"
"			vx = min(Floor2Int(x), nx - 1);\n"
"			x -= vx;\n"
"			y = clamp(P.y, 0.f, 1.f) * ny;\n"
"			vy = min(Floor2Int(P.y * ny), ny - 1);\n"
"			y -= vy;\n"
"			z = clamp(P.z, 0.f, 1.f) * nz;\n"
"			vz = min(Floor2Int(P.z * nz), nz - 1);\n"
"			z -= vz;\n"
"			break;\n"
"		default:\n"
"			return BLACK;\n"
"	}\n"
"\n"
"	// Trilinear interpolation of the grid element\n"
"	return\n"
"		mix(\n"
"			mix(\n"
"				mix(\n"
"					DensityGridTexture_D(imageMap, vx, vy, vz, nx, ny, nz IMAGEMAPS_PARAM),\n"
"					DensityGridTexture_D(imageMap, vx + 1, vy, vz, nx, ny, nz IMAGEMAPS_PARAM),\n"
"				x),\n"
"				mix(\n"
"					DensityGridTexture_D(imageMap, vx, vy + 1, vz, nx, ny, nz IMAGEMAPS_PARAM),\n"
"					DensityGridTexture_D(imageMap, vx + 1, vy + 1, vz, nx, ny, nz IMAGEMAPS_PARAM),\n"
"				x),\n"
"			y),\n"
"			mix(\n"
"				mix(\n"
"					DensityGridTexture_D(imageMap, vx, vy, vz + 1, nx, ny, nz IMAGEMAPS_PARAM),\n"
"					DensityGridTexture_D(imageMap, vx + 1, vy, vz + 1, nx, ny, nz IMAGEMAPS_PARAM),\n"
"				x),\n"
"				mix(\n"
"					DensityGridTexture_D(imageMap, vx, vy + 1, vz + 1, nx, ny, nz IMAGEMAPS_PARAM),\n"
"					DensityGridTexture_D(imageMap, vx + 1, vy + 1, vz + 1, nx, ny, nz IMAGEMAPS_PARAM),\n"
"				x),\n"
"			y),\n"
"		z);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float DensityGridTexture_ConstEvaluateFloat(__global const HitPoint *hitPoint,\n"
"		const int nx, const int ny, const int nz,\n"
"		const uint imageMapIndex, __global const TextureMapping3D *mapping\n"
"		TEXTURES_PARAM_DECL) {\n"
"	return Spectrum_Y(DensityGridTexture_ConstEvaluateSpectrum(hitPoint,\n"
"			nx, ny, nz,\n"
"			imageMapIndex, mapping\n"
"			TEXTURES_PARAM));\n"
"}\n";
} }
