
#include <string>

namespace slg { namespace ocl {
std::string KernelSource_texture_distort_funcs =
"#line 2 \"texture_distort_funcs.cl\"\n"
"\n"
"/***************************************************************************\n"
" * Copyright 1998-2020 by authors (see AUTHORS.txt)                        *\n"
" *                                                                         *\n"
" *   This file is part of LuxCoreRender.                                   *\n"
" *                                                                         *\n"
" * Licensed under the Apache License, Version 2.0 (the \"License\");         *\n"
" * you may not use this file except in compliance with the License.        *\n"
" * You may obtain a copy of the License at                                 *\n"
" *                                                                         *\n"
" *     http://www.apache.org/licenses/LICENSE-2.0                          *\n"
" *                                                                         *\n"
" * Unless required by applicable law or agreed to in writing, software     *\n"
" * distributed under the License is distributed on an \"AS IS\" BASIS,       *\n"
" * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.*\n"
" * See the License for the specific language governing permissions and     *\n"
" * limitations under the License.                                          *\n"
" ***************************************************************************/\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Distort texture\n"
"//------------------------------------------------------------------------------\n"
"\n"
"//----------------------------------------------------------------------\n"
"// For the very special case of Distort texture evaluation\n"
"//\n"
"// EVAL_DISTORT_SETUP\n"
"//----------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_NOT_INLINE void DistortTexture_EvalOp(\n"
"		__global const Texture* restrict texture,\n"
"		const TextureEvalOpType evalType,\n"
"		__global float *evalStack,\n"
"		uint *evalStackOffset,\n"
"		__global const HitPoint *hitPoint,\n"
"		const float sampleDistance\n"
"		TEXTURES_PARAM_DECL) {\n"
"	switch (evalType) {\n"
"		case EVAL_DISTORT_SETUP: {\n"
"			// Read texture evaluation\n"
"			float3 offsetColor;\n"
"			EvalStack_PopFloat3(offsetColor);\n"
"\n"
"			// Save original P\n"
"			float3 p = VLOAD3F(&hitPoint->p.x);\n"
"			EvalStack_PushFloat3(p);\n"
"\n"
"			// Save original UV\n"
"			float2 uv = VLOAD2F(&hitPoint->defaultUV.u);\n"
"			EvalStack_PushFloat(uv.x);\n"
"			EvalStack_PushFloat(uv.y);\n"
"\n"
"			// Distort HitPoint P and UV\n"
"			__global HitPoint *tmpHitPoint = (__global HitPoint *)hitPoint;\n"
"\n"
"			const float3 offset = offsetColor * texture->distortTex.strength;\n"
"			p += offset;\n"
"			VSTORE3F(p, &tmpHitPoint->p.x);\n"
"\n"
"			uv.x += offset.x;\n"
"			uv.y += offset.y;\n"
"			VSTORE2F(uv, &tmpHitPoint->defaultUV.u);\n"
"			break;\n"
"		}\n"
"		case EVAL_FLOAT:\n"
"		case EVAL_SPECTRUM: {\n"
"			// Read textures evaluation\n"
"			float3 result;\n"
"			EvalStack_PopFloat3(result);\n"
"\n"
"			// Restore original P and UV\n"
"			__global HitPoint *tmpHitPoint = (__global HitPoint *)hitPoint;\n"
"			float2 uv;\n"
"			EvalStack_PopFloat2(uv);\n"
"			VSTORE2F(uv, &tmpHitPoint->defaultUV.u);\n"
"\n"
"			float3 p;\n"
"			EvalStack_PopFloat3(p);\n"
"			VSTORE3F(p, &tmpHitPoint->p.x);\n"
"\n"
"			// Save the result\n"
"			if (evalType == EVAL_FLOAT) {\n"
"				EvalStack_PushFloat(Spectrum_Y(result));\n"
"			} else {\n"
"				EvalStack_PushFloat3(result);\n"
"			}\n"
"			break;\n"
"		}\n"
"		case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"			Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"			Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		case EVAL_BUMP:\n"
"			Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		default:\n"
"			// Something wrong here\n"
"			break;\n"
"	}\n"
"}\n";
} }
