
#include <string>

namespace slg { namespace ocl {
std::string KernelSource_texture_funcs_evalops =
"#line 2 \"texture_funcs_evalops.cl\"\n"
"\n"
"/***************************************************************************\n"
" * Copyright 1998-2018 by authors (see AUTHORS.txt)                        *\n"
" *                                                                         *\n"
" *   This file is part of LuxCoreRender.                                   *\n"
" *                                                                         *\n"
" * Licensed under the Apache License, Version 2.0 (the \"License\");         *\n"
" * you may not use this file except in compliance with the License.        *\n"
" * You may obtain a copy of the License at                                 *\n"
" *                                                                         *\n"
" *     http://www.apache.org/licenses/LICENSE-2.0                          *\n"
" *                                                                         *\n"
" * Unless required by applicable law or agreed to in writing, software     *\n"
" * distributed under the License is distributed on an \"AS IS\" BASIS,       *\n"
" * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.*\n"
" * See the License for the specific language governing permissions and     *\n"
" * limitations under the License.                                          *\n"
" ***************************************************************************/\n"
"\n"
"// NOTE: Keep in mind this file must be less than 64 Kbytes because\n"
"// VisulStudio C++ limit of 64 kbytes per file sources\n"
"\n"
"//#define DEBUG_PRINTF_KERNEL_NAME 1\n"
"//#define DEBUG_PRINTF_TEXTURE_EVAL 1\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Texture evaluation functions\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_NOT_INLINE void Texture_EvalOp(\n"
"		__global const TextureEvalOp* restrict evalOp,\n"
"		__global float *evalStack,\n"
"		uint *evalStackOffset,\n"
"		__global const HitPoint *hitPoint,\n"
"		const float sampleDistance\n"
"		TEXTURES_PARAM_DECL) {\n"
"	__global const Texture* restrict texture = &texs[evalOp->texIndex];\n"
"\n"
"#if defined(DEBUG_PRINTF_TEXTURE_EVAL)\n"
"	printf(\"EvalOp texture index=%d type=%d evalType=%d *evalStackOffset=%d\\n\", evalOp->texIndex, texture->type, evalOp->evalType, *evalStackOffset);\n"
"#endif\n"
"\n"
"	const TextureEvalOpType evalType = evalOp->evalType;\n"
"	switch (texture->type) {\n"
"		//----------------------------------------------------------------------\n"
"		// CONST_FLOAT\n"
"		//----------------------------------------------------------------------\n"
"		case CONST_FLOAT: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					const float eval = ConstFloatTexture_ConstEvaluateFloat(texture);\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					const float3 eval = ConstFloatTexture_ConstEvaluateSpectrum(texture);\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_BUMP: {\n"
"					const float3 shadeN = ConstTexture_Bump(hitPoint);\n"
"					EvalStack_PushFloat3(shadeN);\n"
"					break;\n"
"				}\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// CONST_FLOAT3\n"
"		//----------------------------------------------------------------------\n"
"		case CONST_FLOAT3: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					const float eval = ConstFloat3Texture_ConstEvaluateFloat(texture);\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					const float3 eval = ConstFloat3Texture_ConstEvaluateSpectrum(texture);\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}	\n"
"				case EVAL_BUMP: {\n"
"					const float3 shadeN = ConstTexture_Bump(hitPoint);\n"
"					EvalStack_PushFloat3(shadeN);\n"
"					break;\n"
"				}\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// IMAGEMAP\n"
"		//----------------------------------------------------------------------\n"
"		case IMAGEMAP: {\n"
"			ImageMapTexture_EvalOp(texture, evalType, evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance TEXTURES_PARAM);\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// SCALE_TEX\n"
"		//----------------------------------------------------------------------\n"
"		case SCALE_TEX: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					float tex1, tex2;\n"
"					EvalStack_PopFloat(tex2);\n"
"					EvalStack_PopFloat(tex1);\n"
"\n"
"					const float eval = ScaleTexture_ConstEvaluateFloat(tex1, tex2);\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					float3 tex1, tex2;\n"
"					EvalStack_PopFloat3(tex2);\n"
"					EvalStack_PopFloat3(tex1);\n"
"\n"
"					const float3 eval = ScaleTexture_ConstEvaluateSpectrum(tex1, tex2);\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_BUMP: {\n"
"					float evalFloatTex1, evalFloatTex2;\n"
"\n"
"					EvalStack_PopFloat(evalFloatTex2);\n"
"					EvalStack_PopFloat(evalFloatTex1);\n"
"\n"
"					float3 bumbNTex1, bumbNTex2;\n"
"\n"
"					EvalStack_PopFloat3(bumbNTex2);\n"
"					EvalStack_PopFloat3(bumbNTex1);\n"
"\n"
"					const float3 shadeN = ScaleTexture_Bump(hitPoint, bumbNTex1, bumbNTex2, evalFloatTex1, evalFloatTex2);\n"
"					EvalStack_PushFloat3(shadeN);\n"
"					break;\n"
"				}\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// FRESNEL_APPROX_N\n"
"		//----------------------------------------------------------------------\n"
"		case FRESNEL_APPROX_N: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					float tex1;\n"
"					EvalStack_PopFloat(tex1);\n"
"\n"
"					const float eval = FresnelApproxNTexture_ConstEvaluateFloat(tex1);\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					float3 tex1;\n"
"					EvalStack_PopFloat3(tex1);\n"
"\n"
"					const float3 eval = FresnelApproxNTexture_ConstEvaluateSpectrum(tex1);\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"					Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"					Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP:\n"
"					Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// FRESNEL_APPROX_K\n"
"		//----------------------------------------------------------------------\n"
"		case FRESNEL_APPROX_K: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					float tex1;\n"
"					EvalStack_PopFloat(tex1);\n"
"\n"
"					const float eval = FresnelApproxKTexture_ConstEvaluateFloat(tex1);\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					float3 tex1;\n"
"					EvalStack_PopFloat3(tex1);\n"
"\n"
"					const float3 eval = FresnelApproxKTexture_ConstEvaluateSpectrum(tex1);\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"					Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"					Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP:\n"
"					Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// MIX_TEX\n"
"		//----------------------------------------------------------------------\n"
"		case MIX_TEX: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					float amt, tex1, tex2;\n"
"					EvalStack_PopFloat(amt);\n"
"					EvalStack_PopFloat(tex2);\n"
"					EvalStack_PopFloat(tex1);\n"
"\n"
"					const float eval = MixTexture_ConstEvaluateFloat(tex1, tex2, amt);\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					float amt;\n"
"					float3 tex1, tex2;\n"
"					EvalStack_PopFloat(amt);\n"
"					EvalStack_PopFloat3(tex2);\n"
"					EvalStack_PopFloat3(tex1);\n"
"\n"
"					const float3 eval = MixTexture_ConstEvaluateSpectrum(tex1, tex2, MAKE_FLOAT3(amt, amt, amt));\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_BUMP: {\n"
"					float evalFloatTex1, evalFloatTex2, evalFloatAmount;\n"
"\n"
"					EvalStack_PopFloat(evalFloatAmount);\n"
"					EvalStack_PopFloat(evalFloatTex2);\n"
"					EvalStack_PopFloat(evalFloatTex1);\n"
"\n"
"					float3 bumbNTex1, bumbNTex2, bumbNAmount;\n"
"\n"
"					EvalStack_PopFloat3(bumbNAmount);\n"
"					EvalStack_PopFloat3(bumbNTex2);\n"
"					EvalStack_PopFloat3(bumbNTex1);\n"
"\n"
"					const float3 shadeN = MixTexture_Bump(hitPoint,\n"
"							bumbNTex1, bumbNTex2, bumbNAmount,\n"
"							evalFloatTex1, evalFloatTex2, evalFloatAmount);\n"
"					EvalStack_PushFloat3(shadeN);\n"
"					break;\n"
"				}\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// ADD_TEX\n"
"		//----------------------------------------------------------------------\n"
"		case ADD_TEX: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					float tex1, tex2;\n"
"					EvalStack_PopFloat(tex2);\n"
"					EvalStack_PopFloat(tex1);\n"
"\n"
"					const float eval = AddTexture_ConstEvaluateFloat(tex1, tex2);\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					float3 tex1, tex2;\n"
"					EvalStack_PopFloat3(tex2);\n"
"					EvalStack_PopFloat3(tex1);\n"
"\n"
"					const float3 eval = AddTexture_ConstEvaluateSpectrum(tex1, tex2);\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_BUMP: {\n"
"					float3 bumbNTex1, bumbNTex2;\n"
"\n"
"					EvalStack_PopFloat3(bumbNTex2);\n"
"					EvalStack_PopFloat3(bumbNTex1);\n"
"\n"
"					const float3 shadeN = AddTexture_Bump(hitPoint, bumbNTex1, bumbNTex2);\n"
"					EvalStack_PushFloat3(shadeN);\n"
"					break;\n"
"				}\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// SUBTRACT_TEX\n"
"		//----------------------------------------------------------------------\n"
"		case SUBTRACT_TEX: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					float tex1, tex2;\n"
"					EvalStack_PopFloat(tex2);\n"
"					EvalStack_PopFloat(tex1);\n"
"\n"
"					const float eval = SubtractTexture_ConstEvaluateFloat(tex1, tex2);\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					float3 tex1, tex2;\n"
"					EvalStack_PopFloat3(tex2);\n"
"					EvalStack_PopFloat3(tex1);\n"
"\n"
"					const float3 eval = SubtractTexture_ConstEvaluateSpectrum(tex1, tex2);\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_BUMP: {\n"
"					float3 bumbNTex1, bumbNTex2;\n"
"\n"
"					EvalStack_PopFloat3(bumbNTex2);\n"
"					EvalStack_PopFloat3(bumbNTex1);\n"
"\n"
"					const float3 shadeN = SubtractTexture_Bump(hitPoint, bumbNTex1, bumbNTex2);\n"
"					EvalStack_PushFloat3(shadeN);\n"
"					break;\n"
"				}\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// HITPOINTCOLOR\n"
"		//----------------------------------------------------------------------\n"
"		case HITPOINTCOLOR:\n"
"			HitPointColorTexture_EvalOp(texture, evalType, evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance TEXTURES_PARAM);\n"
"			break;\n"
"		//----------------------------------------------------------------------\n"
"		// HITPOINTALPHA\n"
"		//----------------------------------------------------------------------\n"
"		case HITPOINTALPHA:\n"
"			HitPointAlphaTexture_EvalOp(texture, evalType, evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance TEXTURES_PARAM);\n"
"			break;\n"
"		//----------------------------------------------------------------------\n"
"		// HITPOINTGREY\n"
"		//----------------------------------------------------------------------\n"
"		case HITPOINTGREY:\n"
"			HitPointGreyTexture_EvalOp(texture, evalType, evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance TEXTURES_PARAM);\n"
"			break;\n"
"		//----------------------------------------------------------------------\n"
"		// HITPOINTVERTEXAOV\n"
"		//----------------------------------------------------------------------\n"
"		case HITPOINTVERTEXAOV:\n"
"			HitPointVertexAOVTexture_EvalOp(texture, evalType, evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance TEXTURES_PARAM);\n"
"			break;\n"
"		//----------------------------------------------------------------------\n"
"		// HITPOINTTRIANGLEAOV\n"
"		//----------------------------------------------------------------------\n"
"		case HITPOINTTRIANGLEAOV:\n"
"			HitPointTriangleAOVTexture_EvalOp(texture, evalType, evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance TEXTURES_PARAM);\n"
"			break;\n"
"		//----------------------------------------------------------------------\n"
"		// NORMALMAP_TEX\n"
"		//----------------------------------------------------------------------\n"
"		case NORMALMAP_TEX: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					const float eval = NormalMapTexture_ConstEvaluateFloat();\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					const float3 eval = NormalMapTexture_ConstEvaluateSpectrum();\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}	\n"
"				case EVAL_BUMP: {\n"
"					float3 evalSpectrumTex;\n"
"					EvalStack_PopFloat3(evalSpectrumTex);\n"
"\n"
"					const float3 shadeN = NormalMapTexture_Bump(texture, hitPoint, evalSpectrumTex);\n"
"					EvalStack_PushFloat3(shadeN);\n"
"					break;\n"
"				}\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// BLACKBODY_TEX\n"
"		//----------------------------------------------------------------------\n"
"		case BLACKBODY_TEX: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					const float eval = BlackBodyTexture_ConstEvaluateFloat(VLOAD3F(texture->blackBody.rgb.c));\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					const float3 eval = BlackBodyTexture_ConstEvaluateSpectrum(VLOAD3F(texture->blackBody.rgb.c));\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_BUMP: {\n"
"					const float3 shadeN = ConstTexture_Bump(hitPoint);\n"
"					EvalStack_PushFloat3(shadeN);\n"
"					break;\n"
"				}\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// IRREGULARDATA_TEX\n"
"		//----------------------------------------------------------------------\n"
"		case IRREGULARDATA_TEX: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					const float eval = IrregularDataTexture_ConstEvaluateFloat(VLOAD3F(texture->irregularData.rgb.c));\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					const float3 eval = IrregularDataTexture_ConstEvaluateSpectrum(VLOAD3F(texture->irregularData.rgb.c));\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_BUMP: {\n"
"					const float3 shadeN = ConstTexture_Bump(hitPoint);\n"
"					EvalStack_PushFloat3(shadeN);\n"
"					break;\n"
"				}\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// DENSITYGRID_TEX\n"
"		//----------------------------------------------------------------------\n"
"		case DENSITYGRID_TEX: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					const float eval = DensityGridTexture_ConstEvaluateFloat(hitPoint,\n"
"							texture->densityGrid.nx, texture->densityGrid.ny, texture->densityGrid.nz,\n"
"							texture->densityGrid.imageMapIndex, &texture->densityGrid.mapping\n"
"							TEXTURES_PARAM);\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					const float3 eval = DensityGridTexture_ConstEvaluateSpectrum(hitPoint,\n"
"							texture->densityGrid.nx, texture->densityGrid.ny, texture->densityGrid.nz,\n"
"							texture->densityGrid.imageMapIndex, &texture->densityGrid.mapping\n"
"							TEXTURES_PARAM);\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"					Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"					Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP:\n"
"					Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// ABS_TEX\n"
"		//----------------------------------------------------------------------\n"
"		case ABS_TEX:\n"
"			AbsTexture_EvalOp(texture, evalType, evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance TEXTURES_PARAM);\n"
"			break;\n"
"		//----------------------------------------------------------------------\n"
"		// CLAMP_TEX\n"
"		//----------------------------------------------------------------------\n"
"		case CLAMP_TEX:\n"
"			ClampTexture_EvalOp(texture, evalType, evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance TEXTURES_PARAM);\n"
"			break;\n"
"		//----------------------------------------------------------------------\n"
"		// BILERP_TEX\n"
"		//----------------------------------------------------------------------\n"
"		case BILERP_TEX: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					float tex00, tex01, tex10, tex11;\n"
"					EvalStack_PopFloat(tex11);\n"
"					EvalStack_PopFloat(tex10);\n"
"					EvalStack_PopFloat(tex01);\n"
"					EvalStack_PopFloat(tex00);\n"
"					\n"
"\n"
"					const float eval = BilerpTexture_ConstEvaluateFloat(hitPoint,\n"
"							tex00, tex01, tex10, tex11);\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					float3 tex00, tex01, tex10, tex11;\n"
"					EvalStack_PopFloat3(tex11);\n"
"					EvalStack_PopFloat3(tex10);\n"
"					EvalStack_PopFloat3(tex01);\n"
"					EvalStack_PopFloat3(tex00);\n"
"\n"
"					const float3 eval = BilerpTexture_ConstEvaluateSpectrum(hitPoint,\n"
"							tex00, tex01, tex10, tex11);\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"					Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"					Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP:\n"
"					Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// COLORDEPTH_TEX\n"
"		//----------------------------------------------------------------------\n"
"		case COLORDEPTH_TEX: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					float tex1;\n"
"					EvalStack_PopFloat(tex1);\n"
"\n"
"					const float eval = ColorDepthTexture_ConstEvaluateFloat(texture->colorDepthTex.dVal, tex1);\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					float3 tex1;\n"
"					EvalStack_PopFloat3(tex1);\n"
"\n"
"					const float3 eval = ColorDepthTexture_ConstEvaluateSpectrum(texture->colorDepthTex.dVal, tex1);\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"					Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"					Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP:\n"
"					Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// HSV_TEX\n"
"		//----------------------------------------------------------------------\n"
"		case HSV_TEX: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					float3 tex1;\n"
"					float hueTex, satTex, valTex;\n"
"					EvalStack_PopFloat(valTex);\n"
"					EvalStack_PopFloat(satTex);\n"
"					EvalStack_PopFloat(hueTex);\n"
"					EvalStack_PopFloat3(tex1);\n"
"					\n"
"\n"
"					const float eval = HsvTexture_ConstEvaluateFloat(tex1, hueTex, satTex, valTex);\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					float3 tex1;\n"
"					float hueTex, satTex, valTex;\n"
"					EvalStack_PopFloat(valTex);\n"
"					EvalStack_PopFloat(satTex);\n"
"					EvalStack_PopFloat(hueTex);\n"
"					EvalStack_PopFloat3(tex1);\n"
"\n"
"					const float3 eval = HsvTexture_ConstEvaluateSpectrum(tex1, hueTex, satTex, valTex);\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"					Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"					Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP:\n"
"					Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// DIVIDE_TEX\n"
"		//----------------------------------------------------------------------\n"
"		case DIVIDE_TEX: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					float tex1, tex2;\n"
"					EvalStack_PopFloat(tex2);\n"
"					EvalStack_PopFloat(tex1);\n"
"\n"
"					const float eval = DivideTexture_ConstEvaluateFloat(tex1, tex2);\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					float3 tex1, tex2;\n"
"					EvalStack_PopFloat3(tex2);\n"
"					EvalStack_PopFloat3(tex1);\n"
"\n"
"					const float3 eval = DivideTexture_ConstEvaluateSpectrum(tex1, tex2);\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"					Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"					Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP:\n"
"					Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// REMAP_TEX\n"
"		//----------------------------------------------------------------------\n"
"		case REMAP_TEX: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					float valueTex, sourceMinTex, sourceMaxTex, targetMinTex, targetMaxTex;\n"
"					EvalStack_PopFloat(targetMaxTex);\n"
"					EvalStack_PopFloat(targetMinTex);\n"
"					EvalStack_PopFloat(sourceMaxTex);\n"
"					EvalStack_PopFloat(sourceMinTex);\n"
"					EvalStack_PopFloat(valueTex);\n"
"\n"
"					const float eval = RemapTexture_ConstEvaluateFloat(valueTex,\n"
"							sourceMinTex, sourceMaxTex, targetMinTex, targetMaxTex);\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					float3 valueTex;\n"
"					float sourceMinTex, sourceMaxTex, targetMinTex, targetMaxTex;\n"
"					EvalStack_PopFloat(targetMaxTex);\n"
"					EvalStack_PopFloat(targetMinTex);\n"
"					EvalStack_PopFloat(sourceMaxTex);\n"
"					EvalStack_PopFloat(sourceMinTex);\n"
"					EvalStack_PopFloat3(valueTex);\n"
"\n"
"					const float3 eval = RemapTexture_ConstEvaluateSpectrum(valueTex,\n"
"							sourceMinTex, sourceMaxTex, targetMinTex, targetMaxTex);\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"					Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"					Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP:\n"
"					Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// OBJECTID_TEX\n"
"		//----------------------------------------------------------------------\n"
"		case OBJECTID_TEX: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					const float eval = ObjectIDTexture_ConstEvaluateFloat(hitPoint);\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					const float3 eval = ObjectIDTexture_ConstEvaluateSpectrum(hitPoint);\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_BUMP: {\n"
"					const float3 shadeN = ConstTexture_Bump(hitPoint);\n"
"					EvalStack_PushFloat3(shadeN);\n"
"					break;\n"
"				}\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// OBJECTID_COLOR_TEX\n"
"		//----------------------------------------------------------------------\n"
"		case OBJECTID_COLOR_TEX: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					const float eval = ObjectIDColorTexture_ConstEvaluateFloat(hitPoint);\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					const float3 eval = ObjectIDColorTexture_ConstEvaluateSpectrum(hitPoint);\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_BUMP: {\n"
"					const float3 shadeN = ConstTexture_Bump(hitPoint);\n"
"					EvalStack_PushFloat3(shadeN);\n"
"					break;\n"
"				}\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// OBJECTID_NORMALIZED_TEX\n"
"		//----------------------------------------------------------------------\n"
"		case OBJECTID_NORMALIZED_TEX: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					const float eval = ObjectIDNormalizedTexture_ConstEvaluateFloat(hitPoint);\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					const float3 eval = ObjectIDNormalizedTexture_ConstEvaluateSpectrum(hitPoint);\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_BUMP: {\n"
"					const float3 shadeN = ConstTexture_Bump(hitPoint);\n"
"					EvalStack_PushFloat3(shadeN);\n"
"					break;\n"
"				}\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// DOT_PRODUCT_TEX\n"
"		//----------------------------------------------------------------------\n"
"		case DOT_PRODUCT_TEX: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					float3 tex1, tex2;\n"
"					EvalStack_PopFloat3(tex2);\n"
"					EvalStack_PopFloat3(tex1);\n"
"\n"
"					const float eval = DotProductTexture_ConstEvaluateFloat(tex1, tex2);\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					float3 tex1, tex2;\n"
"					EvalStack_PopFloat3(tex2);\n"
"					EvalStack_PopFloat3(tex1);\n"
"\n"
"					const float3 eval = DotProductTexture_ConstEvaluateSpectrum(tex1, tex2);\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"					Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"					Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP:\n"
"					Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// POWER_TEX\n"
"		//----------------------------------------------------------------------\n"
"		case POWER_TEX: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					float tex1, tex2;\n"
"					EvalStack_PopFloat(tex2);\n"
"					EvalStack_PopFloat(tex1);\n"
"\n"
"					const float eval = PowerTexture_ConstEvaluateFloat(tex1, tex2);\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					float tex1, tex2;\n"
"					EvalStack_PopFloat(tex2);\n"
"					EvalStack_PopFloat(tex1);\n"
"\n"
"					const float3 eval = PowerTexture_ConstEvaluateSpectrum(tex1, tex2);\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"					Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"					Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP:\n"
"					Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// LESS_THAN_TEX\n"
"		//----------------------------------------------------------------------\n"
"		case LESS_THAN_TEX: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					float tex1, tex2;\n"
"					EvalStack_PopFloat(tex2);\n"
"					EvalStack_PopFloat(tex1);\n"
"\n"
"					const float eval = LessThanTexture_ConstEvaluateFloat(tex1, tex2);\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					float tex1, tex2;\n"
"					EvalStack_PopFloat(tex2);\n"
"					EvalStack_PopFloat(tex1);\n"
"\n"
"					const float3 eval = LessThanTexture_ConstEvaluateSpectrum(tex1, tex2);\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"					Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"					Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP:\n"
"					Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// GREATER_THAN_TEX\n"
"		//----------------------------------------------------------------------\n"
"		case GREATER_THAN_TEX: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					float tex1, tex2;\n"
"					EvalStack_PopFloat(tex2);\n"
"					EvalStack_PopFloat(tex1);\n"
"\n"
"					const float eval = GreaterThanTexture_ConstEvaluateFloat(tex1, tex2);\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					float tex1, tex2;\n"
"					EvalStack_PopFloat(tex2);\n"
"					EvalStack_PopFloat(tex1);\n"
"\n"
"					const float3 eval = GreaterThanTexture_ConstEvaluateSpectrum(tex1, tex2);\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"					Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"					Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP:\n"
"					Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// ROUNDING_TEX\n"
"		//----------------------------------------------------------------------\n"
"		case ROUNDING_TEX: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					float tex1, tex2;\n"
"					EvalStack_PopFloat(tex2);\n"
"					EvalStack_PopFloat(tex1);\n"
"\n"
"					const float eval = RoundingTexture_ConstEvaluateFloat(tex1, tex2);\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					float tex1, tex2;\n"
"					EvalStack_PopFloat(tex2);\n"
"					EvalStack_PopFloat(tex1);\n"
"\n"
"					const float3 eval = RoundingTexture_ConstEvaluateSpectrum(tex1, tex2);\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"					Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"					Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP:\n"
"					Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// MODULO_TEX\n"
"		//----------------------------------------------------------------------\n"
"		case MODULO_TEX: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					float tex1, tex2;\n"
"					EvalStack_PopFloat(tex2);\n"
"					EvalStack_PopFloat(tex1);\n"
"\n"
"					const float eval = ModuloTexture_ConstEvaluateFloat(tex1, tex2);\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					float tex1, tex2;\n"
"					EvalStack_PopFloat(tex2);\n"
"					EvalStack_PopFloat(tex1);\n"
"\n"
"					const float3 eval = ModuloTexture_ConstEvaluateSpectrum(tex1, tex2);\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"					Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"					Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP:\n"
"					Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// SHADING_NORMAL_TEX\n"
"		//----------------------------------------------------------------------\n"
"		case SHADING_NORMAL_TEX:\n"
"			ShadingNormalTexture_EvalOp(texture, evalType, evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance TEXTURES_PARAM);\n"
"			break;\n"
"		//----------------------------------------------------------------------\n"
"		// POSITION_TEX\n"
"		//----------------------------------------------------------------------\n"
"		case POSITION_TEX: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					const float eval = PositionTexture_ConstEvaluateFloat(hitPoint);\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					const float3 eval = PositionTexture_ConstEvaluateSpectrum(hitPoint);\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"					Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"					Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP:\n"
"					Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// SPLIT_FLOAT3\n"
"		//----------------------------------------------------------------------\n"
"		case SPLIT_FLOAT3: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					float3 tex1;\n"
"					EvalStack_PopFloat3(tex1);\n"
"\n"
"					const float eval = SplitFloat3Texture_ConstEvaluateFloat(tex1,\n"
"							texture->splitFloat3Tex.channelIndex);\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					float3 tex1;\n"
"					EvalStack_PopFloat3(tex1);\n"
"\n"
"					const float3 eval = SplitFloat3Texture_ConstEvaluateSpectrum(tex1,\n"
"							texture->splitFloat3Tex.channelIndex);\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"					Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"					Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP:\n"
"					Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// MAKE_FLOAT3\n"
"		//----------------------------------------------------------------------\n"
"		case MAKE_FLOAT3: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					float tex1, tex2, tex3;\n"
"					EvalStack_PopFloat(tex3);\n"
"					EvalStack_PopFloat(tex2);\n"
"					EvalStack_PopFloat(tex1);\n"
"\n"
"					const float eval = MakeFloat3Texture_ConstEvaluateFloat(tex1,\n"
"							tex2, tex3);\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					float tex1, tex2, tex3;\n"
"					EvalStack_PopFloat(tex3);\n"
"					EvalStack_PopFloat(tex2);\n"
"					EvalStack_PopFloat(tex1);\n"
"\n"
"					const float3 eval = MakeFloat3Texture_ConstEvaluateSpectrum(tex1,\n"
"							tex2, tex3);\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"					Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"					Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP:\n"
"					Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// BRIGHT_CONTRAST_TEX\n"
"		//----------------------------------------------------------------------\n"
"		case BRIGHT_CONTRAST_TEX: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					float tex1, brightnessTex, contrastTex;\n"
"					EvalStack_PopFloat(contrastTex);\n"
"					EvalStack_PopFloat(brightnessTex);\n"
"					EvalStack_PopFloat(tex1);\n"
"\n"
"					const float eval = BrightContrastTexture_ConstEvaluateFloat(tex1,\n"
"							brightnessTex, contrastTex);\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					float3 tex1;\n"
"					float brightnessTex, contrastTex;\n"
"					EvalStack_PopFloat(contrastTex);\n"
"					EvalStack_PopFloat(brightnessTex);\n"
"					EvalStack_PopFloat3(tex1);\n"
"\n"
"					const float3 eval = BrightContrastTexture_ConstEvaluateSpectrum(tex1,\n"
"							brightnessTex, contrastTex);\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"					Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"					Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP:\n"
"					Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// BLENDER_BLEND\n"
"		//----------------------------------------------------------------------\n"
"		case BLENDER_BLEND:\n"
"			BlenderBlendTexture_EvalOp(texture, evalType, evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance TEXTURES_PARAM);\n"
"			break;\n"
"		//----------------------------------------------------------------------\n"
"		// BLENDER_CLOUDS\n"
"		//----------------------------------------------------------------------\n"
"		case BLENDER_CLOUDS:\n"
"			BlenderCloudsTexture_EvalOp(texture, evalType, evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance TEXTURES_PARAM);\n"
"			break;\n"
"		//----------------------------------------------------------------------\n"
"		// BLENDER_DISTORTED_NOISE\n"
"		//----------------------------------------------------------------------\n"
"		case BLENDER_DISTORTED_NOISE:\n"
"			BlenderDistortedNoiseTexture_EvalOp(texture, evalType, evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance TEXTURES_PARAM);\n"
"			break;\n"
"		//----------------------------------------------------------------------\n"
"		// BLENDER_MAGIC\n"
"		//----------------------------------------------------------------------\n"
"		case BLENDER_MAGIC:\n"
"			BlenderMagicTexture_EvalOp(texture, evalType, evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance TEXTURES_PARAM);\n"
"			break;\n"
"		//----------------------------------------------------------------------\n"
"		// BLENDER_MARBLE\n"
"		//----------------------------------------------------------------------\n"
"		case BLENDER_MARBLE:\n"
"			BlenderMarbleTexture_EvalOp(texture, evalType, evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance TEXTURES_PARAM);\n"
"			break;\n"
"		//----------------------------------------------------------------------\n"
"		// BLENDER_MUSGRAVE\n"
"		//----------------------------------------------------------------------\n"
"		case BLENDER_MUSGRAVE:\n"
"			BlenderMusgraveTexture_EvalOp(texture, evalType, evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance TEXTURES_PARAM);\n"
"			break;\n"
"		//----------------------------------------------------------------------\n"
"		// BLENDER_NOISE\n"
"		//----------------------------------------------------------------------\n"
"		case BLENDER_NOISE:\n"
"			BlenderNoiseTexture_EvalOp(texture, evalType, evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance TEXTURES_PARAM);\n"
"			break;\n"
"		//----------------------------------------------------------------------\n"
"		// BLENDER_STUCCI\n"
"		//----------------------------------------------------------------------\n"
"		case BLENDER_STUCCI:\n"
"			BlenderStucciTexture_EvalOp(texture, evalType, evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance TEXTURES_PARAM);\n"
"			break;\n"
"		//----------------------------------------------------------------------\n"
"		// BLENDER_WOOD\n"
"		//----------------------------------------------------------------------\n"
"		case BLENDER_WOOD:\n"
"			BlenderWoodTexture_EvalOp(texture, evalType, evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance TEXTURES_PARAM);\n"
"			break;\n"
"		//----------------------------------------------------------------------\n"
"		// BLENDER_VORONOI\n"
"		//----------------------------------------------------------------------\n"
"		case BLENDER_VORONOI:\n"
"			BlenderVoronoiTexture_EvalOp(texture, evalType, evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance TEXTURES_PARAM);\n"
"			break;\n"
"		//----------------------------------------------------------------------\n"
"		// CHECKERBOARD2D\n"
"		//----------------------------------------------------------------------\n"
"		case CHECKERBOARD2D: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					float tex1, tex2;\n"
"					EvalStack_PopFloat(tex2);\n"
"					EvalStack_PopFloat(tex1);\n"
"\n"
"					const float eval = CheckerBoard2DTexture_ConstEvaluateFloat(hitPoint,\n"
"							tex1, tex2, &texture->checkerBoard2D.mapping\n"
"							TEXTURES_PARAM);\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					float3 tex1, tex2;\n"
"					EvalStack_PopFloat3(tex2);\n"
"					EvalStack_PopFloat3(tex1);\n"
"\n"
"					const float3 eval = CheckerBoard2DTexture_ConstEvaluateSpectrum(hitPoint,\n"
"							tex1, tex2, &texture->checkerBoard2D.mapping\n"
"							TEXTURES_PARAM);\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"					Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"					Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP:\n"
"					Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// CHECKERBOARD3D\n"
"		//----------------------------------------------------------------------\n"
"		case CHECKERBOARD3D: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					float tex1, tex2;\n"
"					EvalStack_PopFloat(tex2);\n"
"					EvalStack_PopFloat(tex1);\n"
"\n"
"					const float eval = CheckerBoard3DTexture_ConstEvaluateFloat(hitPoint,\n"
"							tex1, tex2, &texture->checkerBoard3D.mapping\n"
"							TEXTURES_PARAM);\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					float3 tex1, tex2;\n"
"					EvalStack_PopFloat3(tex2);\n"
"					EvalStack_PopFloat3(tex1);\n"
"\n"
"					const float3 eval = CheckerBoard3DTexture_ConstEvaluateSpectrum(hitPoint,\n"
"							tex1, tex2, &texture->checkerBoard3D.mapping\n"
"							TEXTURES_PARAM);\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"					Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"					Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP:\n"
"					Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// CLOUD_TEX\n"
"		//----------------------------------------------------------------------\n"
"		case CLOUD_TEX: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					const float eval = CloudTexture_ConstEvaluateFloat(hitPoint,\n"
"							texture->cloud.radius, texture->cloud.numspheres,\n"
"							texture->cloud.spheresize, texture->cloud.sharpness,\n"
"							texture->cloud.basefadedistance, texture->cloud.baseflatness,\n"
"							texture->cloud.variability, texture->cloud.omega,\n"
"							texture->cloud.noisescale, texture->cloud.noiseoffset,\n"
"							texture->cloud.turbulence, texture->cloud.octaves,\n"
"							&texture->cloud.mapping\n"
"							TEXTURES_PARAM);\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					const float3 eval = CloudTexture_ConstEvaluateSpectrum(hitPoint,\n"
"							texture->cloud.radius, texture->cloud.numspheres,\n"
"							texture->cloud.spheresize, texture->cloud.sharpness,\n"
"							texture->cloud.basefadedistance, texture->cloud.baseflatness,\n"
"							texture->cloud.variability, texture->cloud.omega,\n"
"							texture->cloud.noisescale, texture->cloud.noiseoffset,\n"
"							texture->cloud.turbulence, texture->cloud.octaves,\n"
"							&texture->cloud.mapping\n"
"							TEXTURES_PARAM);\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"					Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"					Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP:\n"
"					Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// FBM_TEX\n"
"		//----------------------------------------------------------------------\n"
"		case FBM_TEX: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					const float eval = FBMTexture_ConstEvaluateFloat(hitPoint,\n"
"							texture->fbm.omega, texture->fbm.octaves,\n"
"							&texture->fbm.mapping\n"
"							TEXTURES_PARAM);\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					const float3 eval = FBMTexture_ConstEvaluateSpectrum(hitPoint,\n"
"							texture->fbm.omega, texture->fbm.octaves,\n"
"							&texture->fbm.mapping\n"
"							TEXTURES_PARAM);\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"					Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"					Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP:\n"
"					Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// MARBLE\n"
"		//----------------------------------------------------------------------\n"
"		case MARBLE: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					const float eval = MarbleTexture_ConstEvaluateFloat(hitPoint,\n"
"							texture->marble.scale, texture->marble.omega,\n"
"							texture->marble.octaves, texture->marble.variation,\n"
"							&texture->marble.mapping\n"
"							TEXTURES_PARAM);\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					const float3 eval = MarbleTexture_ConstEvaluateSpectrum(hitPoint,\n"
"							texture->marble.scale, texture->marble.omega,\n"
"							texture->marble.octaves, texture->marble.variation,\n"
"							&texture->marble.mapping\n"
"							TEXTURES_PARAM);\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"					Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"					Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP:\n"
"					Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// DOTS\n"
"		//----------------------------------------------------------------------\n"
"		case DOTS: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					float tex1, tex2;\n"
"					EvalStack_PopFloat(tex2);\n"
"					EvalStack_PopFloat(tex1);\n"
"\n"
"					const float eval = DotsTexture_ConstEvaluateFloat(hitPoint,\n"
"							tex1, tex2, &texture->checkerBoard2D.mapping\n"
"							TEXTURES_PARAM);\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					float3 tex1, tex2;\n"
"					EvalStack_PopFloat3(tex2);\n"
"					EvalStack_PopFloat3(tex1);\n"
"\n"
"					const float3 eval = DotsTexture_ConstEvaluateSpectrum(hitPoint,\n"
"							tex1, tex2, &texture->checkerBoard2D.mapping\n"
"							TEXTURES_PARAM);\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"					Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"					Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP:\n"
"					Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// BRICK\n"
"		//----------------------------------------------------------------------\n"
"		case BRICK:\n"
"			BrickTexture_EvalOp(texture, evalType, evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance TEXTURES_PARAM);\n"
"			break;\n"
"		//----------------------------------------------------------------------\n"
"		// WINDY\n"
"		//----------------------------------------------------------------------\n"
"		case WINDY: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					const float eval = WindyTexture_ConstEvaluateFloat(hitPoint,\n"
"							&texture->windy.mapping\n"
"							TEXTURES_PARAM);\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					const float3 eval = WindyTexture_ConstEvaluateSpectrum(hitPoint,\n"
"							&texture->windy.mapping\n"
"							TEXTURES_PARAM);\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"					Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"					Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP:\n"
"					Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// WRINKLED\n"
"		//----------------------------------------------------------------------\n"
"		case WRINKLED: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					const float eval = WrinkledTexture_ConstEvaluateFloat(hitPoint,\n"
"							texture->wrinkled.omega, texture->wrinkled.octaves,\n"
"							&texture->wrinkled.mapping\n"
"							TEXTURES_PARAM);\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					const float3 eval = WrinkledTexture_ConstEvaluateSpectrum(hitPoint,\n"
"							texture->wrinkled.omega, texture->wrinkled.octaves,\n"
"							&texture->wrinkled.mapping\n"
"							TEXTURES_PARAM);\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"					Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"					Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP:\n"
"					Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// UV_TEX\n"
"		//----------------------------------------------------------------------\n"
"		case UV_TEX: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					const float eval = UVTexture_ConstEvaluateFloat(hitPoint,\n"
"							&texture->uvTex.mapping\n"
"							TEXTURES_PARAM);\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					const float3 eval = UVTexture_ConstEvaluateSpectrum(hitPoint,\n"
"							&texture->uvTex.mapping\n"
"							TEXTURES_PARAM);\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"					Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"					Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP:\n"
"					Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// BAND_TEX\n"
"		//----------------------------------------------------------------------\n"
"		case BAND_TEX: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					float tex1;\n"
"					EvalStack_PopFloat(tex1);\n"
"\n"
"					const float eval = BandTexture_ConstEvaluateFloat(hitPoint,\n"
"							texture->band.interpType, texture->band.size,\n"
"							texture->band.offsets, texture->band.values,\n"
"							tex1);\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					float tex1;\n"
"					EvalStack_PopFloat(tex1);\n"
"\n"
"					const float3 eval = BandTexture_ConstEvaluateSpectrum(hitPoint,\n"
"							texture->band.interpType, texture->band.size,\n"
"							texture->band.offsets, texture->band.values,\n"
"							tex1);\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"					Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"					Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP:\n"
"					Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// WIREFRAME_TEX\n"
"		//----------------------------------------------------------------------\n"
"		case WIREFRAME_TEX: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					float tex1, tex2;\n"
"					EvalStack_PopFloat(tex2);\n"
"					EvalStack_PopFloat(tex1);\n"
"\n"
"					const float eval = WireFrameTexture_ConstEvaluateFloat(hitPoint,\n"
"							texture->wireFrameTex.width,\n"
"							tex1, tex2\n"
"							TEXTURES_PARAM);\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					float3 tex1, tex2;\n"
"					EvalStack_PopFloat3(tex2);\n"
"					EvalStack_PopFloat3(tex1);\n"
"\n"
"					const float3 eval = WireFrameTexture_ConstEvaluateSpectrum(hitPoint,\n"
"							texture->wireFrameTex.width,\n"
"							tex1, tex2\n"
"							TEXTURES_PARAM);\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"					Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"					Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				case EVAL_BUMP:\n"
"					Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"							hitPoint, sampleDistance);\n"
"					break;\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// TRIPLANAR_TEX\n"
"		//----------------------------------------------------------------------\n"
"		case TRIPLANAR_TEX:\n"
"			TriplanarTexture_EvalOp(texture, evalType, evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance TEXTURES_PARAM);\n"
"			break;\n"
"		//----------------------------------------------------------------------\n"
"		// DISTORT_TEX\n"
"		//----------------------------------------------------------------------\n"
"		case DISTORT_TEX:\n"
"			DistortTexture_EvalOp(texture, evalType, evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance TEXTURES_PARAM);\n"
"			break;\n"
"		//----------------------------------------------------------------------\n"
"		// BOMBING_TEX\n"
"		//----------------------------------------------------------------------\n"
"		case BOMBING_TEX:\n"
"			BombingTexture_EvalOp(texture, evalType, evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance TEXTURES_PARAM);\n"
"			break;\n"
"		//----------------------------------------------------------------------\n"
"		// FRESNELCOLOR_TEX\n"
"		//----------------------------------------------------------------------\n"
"		case FRESNELCOLOR_TEX: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					const float eval = FresnelColorTexture_ConstEvaluateFloat();\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					const float3 eval = FresnelColorTexture_ConstEvaluateSpectrum();\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_BUMP: {\n"
"					const float3 shadeN = ConstTexture_Bump(hitPoint);\n"
"					EvalStack_PushFloat3(shadeN);\n"
"					break;\n"
"				}\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// FRESNELCONST_TEX\n"
"		//----------------------------------------------------------------------\n"
"		case FRESNELCONST_TEX: {\n"
"			switch (evalType) {\n"
"				case EVAL_FLOAT: {\n"
"					const float eval = FresnelColorTexture_ConstEvaluateFloat();\n"
"					EvalStack_PushFloat(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_SPECTRUM: {\n"
"					const float3 eval = FresnelColorTexture_ConstEvaluateSpectrum();\n"
"					EvalStack_PushFloat3(eval);\n"
"					break;\n"
"				}\n"
"				case EVAL_BUMP: {\n"
"					const float3 shadeN = ConstTexture_Bump(hitPoint);\n"
"					EvalStack_PushFloat3(shadeN);\n"
"					break;\n"
"				}\n"
"				default:\n"
"					// Something wrong here\n"
"					break;\n"
"			}\n"
"			break;\n"
"		}\n"
"		//----------------------------------------------------------------------\n"
"		// RANDOM_TEX\n"
"		//----------------------------------------------------------------------\n"
"		case RANDOM_TEX:\n"
"			RandomTexture_EvalOp(texture, evalType, evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance TEXTURES_PARAM);\n"
"			break;\n"
"		//----------------------------------------------------------------------\n"
"		default:\n"
"			// Something wrong here\n"
"			break;\n"
"	}\n"
"}\n";
} }
