
#include <string>

namespace slg { namespace ocl {
std::string KernelSource_texture_funcs =
"#line 2 \"texture_funcs.cl\"\n"
"\n"
"/***************************************************************************\n"
" * Copyright 1998-2020 by authors (see AUTHORS.txt)                        *\n"
" *                                                                         *\n"
" *   This file is part of LuxCoreRender.                                   *\n"
" *                                                                         *\n"
" * Licensed under the Apache License, Version 2.0 (the \"License\");         *\n"
" * you may not use this file except in compliance with the License.        *\n"
" * You may obtain a copy of the License at                                 *\n"
" *                                                                         *\n"
" *     http://www.apache.org/licenses/LICENSE-2.0                          *\n"
" *                                                                         *\n"
" * Unless required by applicable law or agreed to in writing, software     *\n"
" * distributed under the License is distributed on an \"AS IS\" BASIS,       *\n"
" * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.*\n"
" * See the License for the specific language governing permissions and     *\n"
" * limitations under the License.                                          *\n"
" ***************************************************************************/\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Texture_GetFloatValue()\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float Texture_GetFloatValueSlowPath(const uint texIndex,\n"
"		__global const HitPoint *hitPoint\n"
"		TEXTURES_PARAM_DECL) {\n"
"#if defined(DEBUG_PRINTF_TEXTURE_EVAL)\n"
"	printf(\"===============================================================\\n\");\n"
"	printf(\"Texture_GetFloatValue()\\n\");\n"
"	printf(\"===============================================================\\n\");\n"
"#endif\n"
"\n"
"	__global const Texture* restrict startTex = &texs[texIndex];\n"
"	const size_t gid = get_global_id(0);\n"
"	__global float *evalStack = &texEvalStacks[gid * maxTextureEvalStackSize];\n"
"\n"
"	const uint evalFloatOpStartIndex = startTex->evalFloatOpStartIndex;\n"
"	const uint evalFloatOpLength = startTex->evalFloatOpLength;\n"
"\n"
"#if defined(DEBUG_PRINTF_TEXTURE_EVAL)\n"
"	printf(\"texIndex=%d evalFloatOpStartIndex=%d evalFloatOpLength=%d\\n\", texIndex, evalFloatOpStartIndex, evalFloatOpLength);\n"
"#endif\n"
"\n"
"	uint evalStackOffsetVal = 0;\n"
"	uint *evalStackOffset = &evalStackOffsetVal; // Used by macros\n"
"	for (uint i = 0; i < evalFloatOpLength; ++i) {\n"
"#if defined(DEBUG_PRINTF_TEXTURE_EVAL)\n"
"		printf(\"EvalOp: #%d\\n\", i);\n"
"#endif\n"
"\n"
"		__global const TextureEvalOp* restrict evalOp = &texEvalOps[evalFloatOpStartIndex + i];\n"
"\n"
"		Texture_EvalOp(evalOp, evalStack, &evalStackOffsetVal, hitPoint, 0.f TEXTURES_PARAM);\n"
"	}\n"
"#if defined(DEBUG_PRINTF_TEXTURE_EVAL)\n"
"	printf(\"evalStackOffset=#%d\\n\", evalStackOffsetVal);\n"
"#endif\n"
"\n"
"	float result;\n"
"	EvalStack_PopFloat(result);\n"
"\n"
"#if defined(DEBUG_PRINTF_TEXTURE_EVAL)\n"
"	printf(\"Result=%f\\n\", result);\n"
"#endif\n"
"	\n"
"	return result;\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float Texture_GetFloatValue(const uint texIndex,\n"
"		__global const HitPoint *hitPoint\n"
"		TEXTURES_PARAM_DECL) {\n"
"	__global const Texture* restrict tex = &texs[texIndex];\n"
"	switch (tex->type) {\n"
"		//----------------------------------------------------------------------\n"
"		// Fast paths\n"
"		//----------------------------------------------------------------------\n"
"		case CONST_FLOAT:\n"
"			return ConstFloatTexture_ConstEvaluateFloat(tex);\n"
"		case CONST_FLOAT3:\n"
"			return ConstFloat3Texture_ConstEvaluateFloat(tex);\n"
"		case IMAGEMAP:\n"
"			return ImageMapTexture_ConstEvaluateFloat(tex, hitPoint TEXTURES_PARAM);\n"
"		//----------------------------------------------------------------------\n"
"		// Fall back to the slow path\n"
"		//----------------------------------------------------------------------\n"
"		default:\n"
"			return Texture_GetFloatValueSlowPath(texIndex, hitPoint TEXTURES_PARAM);\n"
"	}\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Texture_GetSpectrumValue()\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float3 Texture_GetSpectrumValueSlowPath(const uint texIndex,\n"
"		__global const HitPoint *hitPoint\n"
"		TEXTURES_PARAM_DECL) {\n"
"#if defined(DEBUG_PRINTF_TEXTURE_EVAL)\n"
"	printf(\"===============================================================\\n\");\n"
"	printf(\"Texture_GetSpectrumValue()\\n\");\n"
"	printf(\"===============================================================\\n\");\n"
"#endif\n"
"\n"
"	__global const Texture* restrict startTex = &texs[texIndex];\n"
"	const size_t gid = get_global_id(0);\n"
"	__global float *evalStack = &texEvalStacks[gid * maxTextureEvalStackSize];\n"
"\n"
"	const uint evalSpectrumOpStartIndex = startTex->evalSpectrumOpStartIndex;\n"
"	const uint evalSpectrumOpLength = startTex->evalSpectrumOpLength;\n"
"\n"
"#if defined(DEBUG_PRINTF_TEXTURE_EVAL)\n"
"	printf(\"texIndex=%d evalSpectrumOpStartIndex=%d evalSpectrumOpLength=%d\\n\", texIndex, evalSpectrumOpStartIndex, evalSpectrumOpLength);\n"
"#endif\n"
"\n"
"	uint evalStackOffsetVal = 0;\n"
"	uint *evalStackOffset = &evalStackOffsetVal; // Used by macros\n"
"	for (uint i = 0; i < evalSpectrumOpLength; ++i) {\n"
"#if defined(DEBUG_PRINTF_TEXTURE_EVAL)\n"
"		printf(\"EvalOp: #%d\\n\", i);\n"
"#endif\n"
"\n"
"		__global const TextureEvalOp* restrict evalOp = &texEvalOps[evalSpectrumOpStartIndex + i];\n"
"\n"
"		Texture_EvalOp(evalOp, evalStack, &evalStackOffsetVal, hitPoint, 0.f TEXTURES_PARAM);\n"
"	}\n"
"#if defined(DEBUG_PRINTF_TEXTURE_EVAL)\n"
"	printf(\"evalStackOffset=#%d\\n\", evalStackOffsetVal);\n"
"#endif\n"
"	\n"
"	float3 result;\n"
"	EvalStack_PopFloat3(result);\n"
"\n"
"\n"
"#if defined(DEBUG_PRINTF_TEXTURE_EVAL)\n"
"	printf(\"Result=(%f, %f, %f)\\n\", result.s0, result.s1, result.s2);\n"
"#endif\n"
"\n"
"	return result;\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 Texture_GetSpectrumValue(const uint texIndex,\n"
"		__global const HitPoint *hitPoint\n"
"		TEXTURES_PARAM_DECL) {\n"
"	__global const Texture* restrict tex = &texs[texIndex];\n"
"	switch (tex->type) {\n"
"		//----------------------------------------------------------------------\n"
"		// Fast paths\n"
"		//----------------------------------------------------------------------\n"
"		case CONST_FLOAT:\n"
"			return ConstFloatTexture_ConstEvaluateSpectrum(tex);\n"
"		case CONST_FLOAT3:\n"
"			return ConstFloat3Texture_ConstEvaluateSpectrum(tex);\n"
"		case IMAGEMAP:\n"
"			return ImageMapTexture_ConstEvaluateSpectrum(tex, hitPoint TEXTURES_PARAM);\n"
"		//----------------------------------------------------------------------\n"
"		// Fall back to the slow path\n"
"		//----------------------------------------------------------------------\n"
"		default:\n"
"			return Texture_GetSpectrumValueSlowPath(texIndex, hitPoint TEXTURES_PARAM);\n"
"	}\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Texture_Bump()\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float3 Texture_Bump(const uint texIndex,\n"
"		__global HitPoint *hitPoint,\n"
"		const float sampleDistance\n"
"		TEXTURES_PARAM_DECL) {\n"
"#if defined(DEBUG_PRINTF_TEXTURE_EVAL)\n"
"	printf(\"===============================================================\\n\");\n"
"	printf(\"Texture_Bump()\\n\");\n"
"	printf(\"===============================================================\\n\");\n"
"#endif\n"
"\n"
"	__global const Texture* restrict startTex = &texs[texIndex];\n"
"	const size_t gid = get_global_id(0);\n"
"	__global float *evalStack = &texEvalStacks[gid * maxTextureEvalStackSize];\n"
"\n"
"	const uint evalBumpOpStartIndex = startTex->evalBumpOpStartIndex;\n"
"	const uint evalBumpOpLength = startTex->evalBumpOpLength;\n"
"\n"
"#if defined(DEBUG_PRINTF_TEXTURE_EVAL)\n"
"	printf(\"texIndex=%d evalSpectrumOpStartIndex=%d evalSpectrumOpLength=%d\\n\", texIndex, evalBumpOpStartIndex, evalBumpOpLength);\n"
"#endif\n"
"\n"
"	uint evalStackOffsetVal = 0;\n"
"	uint *evalStackOffset = &evalStackOffsetVal; // Used by macros\n"
"	for (uint i = 0; i < evalBumpOpLength; ++i) {\n"
"#if defined(DEBUG_PRINTF_TEXTURE_EVAL)\n"
"		printf(\"EvalOp: #%d\\n\", i);\n"
"#endif\n"
"\n"
"		__global const TextureEvalOp* restrict evalOp = &texEvalOps[evalBumpOpStartIndex + i];\n"
"\n"
"		Texture_EvalOp(evalOp, evalStack, &evalStackOffsetVal, hitPoint, sampleDistance TEXTURES_PARAM);\n"
"	}\n"
"#if defined(DEBUG_PRINTF_TEXTURE_EVAL)\n"
"	printf(\"evalStackOffset=#%d\\n\", evalStackOffsetVal);\n"
"#endif\n"
"\n"
"	float3 result;\n"
"	EvalStack_PopFloat3(result);\n"
"\n"
"#if defined(DEBUG_PRINTF_TEXTURE_EVAL)\n"
"	printf(\"Result=(%f, %f, %f)\\n\", result.x, result.y, result.z);\n"
"#endif\n"
"\n"
"	return result;\n"
"}\n";
} }
