
#include <string>

namespace slg { namespace ocl {
std::string KernelSource_texture_hitpoint_funcs =
"#line 2 \"texture_hitpoint_funcs.cl\"\n"
"\n"
"/***************************************************************************\n"
" * Copyright 1998-2020 by authors (see AUTHORS.txt)                        *\n"
" *                                                                         *\n"
" *   This file is part of LuxCoreRender.                                   *\n"
" *                                                                         *\n"
" * Licensed under the Apache License, Version 2.0 (the \"License\");         *\n"
" * you may not use this file except in compliance with the License.        *\n"
" * You may obtain a copy of the License at                                 *\n"
" *                                                                         *\n"
" *     http://www.apache.org/licenses/LICENSE-2.0                          *\n"
" *                                                                         *\n"
" * Unless required by applicable law or agreed to in writing, software     *\n"
" * distributed under the License is distributed on an \"AS IS\" BASIS,       *\n"
" * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.*\n"
" * See the License for the specific language governing permissions and     *\n"
" * limitations under the License.                                          *\n"
" ***************************************************************************/\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// HitPointColor texture\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE float HitPointColorTexture_ConstEvaluateFloat(__global const HitPoint *hitPoint,\n"
"		const uint dataIndex\n"
"		TEXTURES_PARAM_DECL) {\n"
"	return Spectrum_Y(HitPoint_GetColor(hitPoint, dataIndex EXTMESH_PARAM));\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 HitPointColorTexture_ConstEvaluateSpectrum(__global const HitPoint *hitPoint,\n"
"		const uint dataIndex\n"
"		TEXTURES_PARAM_DECL) {\n"
"	return HitPoint_GetColor(hitPoint, dataIndex EXTMESH_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE void HitPointColorTexture_EvalOp(\n"
"		__global const Texture* restrict texture,\n"
"		const TextureEvalOpType evalType,\n"
"		__global float *evalStack,\n"
"		uint *evalStackOffset,\n"
"		__global const HitPoint *hitPoint,\n"
"		const float sampleDistance\n"
"		TEXTURES_PARAM_DECL) {\n"
"	switch (evalType) {\n"
"		case EVAL_FLOAT: {\n"
"			const float eval = HitPointColorTexture_ConstEvaluateFloat(hitPoint,\n"
"					texture->hitPointColor.dataIndex\n"
"					TEXTURES_PARAM);\n"
"			EvalStack_PushFloat(eval);\n"
"			break;\n"
"		}\n"
"		case EVAL_SPECTRUM: {\n"
"			const float3 eval = HitPointColorTexture_ConstEvaluateSpectrum(hitPoint,\n"
"					texture->hitPointColor.dataIndex\n"
"					TEXTURES_PARAM);\n"
"			EvalStack_PushFloat3(eval);\n"
"			break;\n"
"		}\n"
"		case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"			Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"			Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		case EVAL_BUMP:\n"
"			Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		default:\n"
"			// Something wrong here\n"
"			break;\n"
"	}\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// HitPointAlpha texture\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE float HitPointAlphaTexture_ConstEvaluateFloat(__global const HitPoint *hitPoint,\n"
"		const uint dataIndex\n"
"		TEXTURES_PARAM_DECL) {\n"
"	return HitPoint_GetAlpha(hitPoint, dataIndex EXTMESH_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 HitPointAlphaTexture_ConstEvaluateSpectrum(__global const HitPoint *hitPoint,\n"
"		const uint dataIndex\n"
"		TEXTURES_PARAM_DECL) {\n"
"	const float alpha = HitPoint_GetAlpha(hitPoint, dataIndex EXTMESH_PARAM);\n"
"	return MAKE_FLOAT3(alpha, alpha, alpha);\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE void HitPointAlphaTexture_EvalOp(\n"
"		__global const Texture* restrict texture,\n"
"		const TextureEvalOpType evalType,\n"
"		__global float *evalStack,\n"
"		uint *evalStackOffset,\n"
"		__global const HitPoint *hitPoint,\n"
"		const float sampleDistance\n"
"		TEXTURES_PARAM_DECL) {\n"
"	switch (evalType) {\n"
"		case EVAL_FLOAT: {\n"
"			const float eval = HitPointAlphaTexture_ConstEvaluateFloat(hitPoint,\n"
"					texture->hitPointAlpha.dataIndex\n"
"					TEXTURES_PARAM);\n"
"			EvalStack_PushFloat(eval);\n"
"			break;\n"
"		}\n"
"		case EVAL_SPECTRUM: {\n"
"			const float3 eval = HitPointAlphaTexture_ConstEvaluateSpectrum(hitPoint,\n"
"					texture->hitPointAlpha.dataIndex\n"
"					TEXTURES_PARAM);\n"
"			EvalStack_PushFloat3(eval);\n"
"			break;\n"
"		}\n"
"		case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"			Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"			Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		case EVAL_BUMP:\n"
"			Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		default:\n"
"			// Something wrong here\n"
"			break;\n"
"	}\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// HitPointGrey texture\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE float HitPointGreyTexture_ConstEvaluateFloat(__global const HitPoint *hitPoint,\n"
"		const uint dataIndex, const uint channel\n"
"		TEXTURES_PARAM_DECL) {\n"
"	const float3 col = HitPoint_GetColor(hitPoint, dataIndex EXTMESH_PARAM);\n"
"\n"
"	switch (channel) {\n"
"		case 0:\n"
"			return col.x;\n"
"		case 1:\n"
"			return col.y;\n"
"		case 2:\n"
"			return col.z;\n"
"		default:\n"
"			return Spectrum_Y(col);\n"
"	}\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 HitPointGreyTexture_ConstEvaluateSpectrum(__global const HitPoint *hitPoint,\n"
"		const uint dataIndex, const uint channel\n"
"		TEXTURES_PARAM_DECL) {\n"
"	const float3 col = HitPoint_GetColor(hitPoint, dataIndex EXTMESH_PARAM);\n"
"\n"
"	float v;\n"
"	switch (channel) {\n"
"		case 0:\n"
"			v = col.x;\n"
"			break;\n"
"		case 1:\n"
"			v = col.y;\n"
"			break;\n"
"		case 2:\n"
"			v = col.z;\n"
"			break;\n"
"		default:\n"
"			v = Spectrum_Y(col);\n"
"			break;\n"
"	}\n"
"\n"
"	return TO_FLOAT3(v);\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE void HitPointGreyTexture_EvalOp(\n"
"		__global const Texture* restrict texture,\n"
"		const TextureEvalOpType evalType,\n"
"		__global float *evalStack,\n"
"		uint *evalStackOffset,\n"
"		__global const HitPoint *hitPoint,\n"
"		const float sampleDistance\n"
"		TEXTURES_PARAM_DECL) {\n"
"	switch (evalType) {\n"
"		case EVAL_FLOAT: {\n"
"			const float eval = HitPointGreyTexture_ConstEvaluateFloat(hitPoint,\n"
"					texture->hitPointGrey.dataIndex, texture->hitPointGrey.channelIndex\n"
"					TEXTURES_PARAM);\n"
"			EvalStack_PushFloat(eval);\n"
"			break;\n"
"		}\n"
"		case EVAL_SPECTRUM: {\n"
"			const float3 eval = HitPointGreyTexture_ConstEvaluateSpectrum(hitPoint,\n"
"					texture->hitPointGrey.dataIndex, texture->hitPointGrey.channelIndex\n"
"					TEXTURES_PARAM);\n"
"			EvalStack_PushFloat3(eval);\n"
"			break;\n"
"		}\n"
"		case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"			Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"			Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		case EVAL_BUMP:\n"
"			Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		default:\n"
"			// Something wrong here\n"
"			break;\n"
"	}\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// HitPointVertexAOV texture\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE float HitPointVertexAOVTexture_ConstEvaluateFloat(__global const HitPoint *hitPoint,\n"
"		const uint dataIndex\n"
"		TEXTURES_PARAM_DECL) {\n"
"	return HitPoint_GetVertexAOV(hitPoint, dataIndex EXTMESH_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 HitPointVertexAOVTexture_ConstEvaluateSpectrum(__global const HitPoint *hitPoint,\n"
"		const uint dataIndex\n"
"		TEXTURES_PARAM_DECL) {\n"
"	const float v = HitPoint_GetVertexAOV(hitPoint, dataIndex EXTMESH_PARAM);\n"
"	return TO_FLOAT3(v);\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE void HitPointVertexAOVTexture_EvalOp(\n"
"		__global const Texture* restrict texture,\n"
"		const TextureEvalOpType evalType,\n"
"		__global float *evalStack,\n"
"		uint *evalStackOffset,\n"
"		__global const HitPoint *hitPoint,\n"
"		const float sampleDistance\n"
"		TEXTURES_PARAM_DECL) {\n"
"	switch (evalType) {\n"
"		case EVAL_FLOAT: {\n"
"			const float eval = HitPointVertexAOVTexture_ConstEvaluateFloat(hitPoint,\n"
"					texture->hitPointVertexAOV.dataIndex\n"
"					TEXTURES_PARAM);\n"
"			EvalStack_PushFloat(eval);\n"
"			break;\n"
"		}\n"
"		case EVAL_SPECTRUM: {\n"
"			const float3 eval = HitPointVertexAOVTexture_ConstEvaluateSpectrum(hitPoint,\n"
"					texture->hitPointVertexAOV.dataIndex\n"
"					TEXTURES_PARAM);\n"
"			EvalStack_PushFloat3(eval);\n"
"			break;\n"
"		}\n"
"		case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"			Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"			Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		case EVAL_BUMP:\n"
"			Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		default:\n"
"			// Something wrong here\n"
"			break;\n"
"	}\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// HitPointTriangleAOV texture\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE float HitPointTriangleAOVTexture_ConstEvaluateFloat(__global const HitPoint *hitPoint,\n"
"		const uint dataIndex\n"
"		TEXTURES_PARAM_DECL) {\n"
"	return HitPoint_GetTriAOV(hitPoint, dataIndex EXTMESH_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 HitPointTriangleAOVTexture_ConstEvaluateSpectrum(__global const HitPoint *hitPoint,\n"
"		const uint dataIndex\n"
"		TEXTURES_PARAM_DECL) {\n"
"	const float t = HitPoint_GetTriAOV(hitPoint, dataIndex EXTMESH_PARAM);\n"
"	return MAKE_FLOAT3(t, t, t);\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE void HitPointTriangleAOVTexture_EvalOp(\n"
"		__global const Texture* restrict texture,\n"
"		const TextureEvalOpType evalType,\n"
"		__global float *evalStack,\n"
"		uint *evalStackOffset,\n"
"		__global const HitPoint *hitPoint,\n"
"		const float sampleDistance\n"
"		TEXTURES_PARAM_DECL) {\n"
"	switch (evalType) {\n"
"		case EVAL_FLOAT: {\n"
"			const float eval = HitPointTriangleAOVTexture_ConstEvaluateFloat(hitPoint,\n"
"					texture->hitPointTriangleAOV.dataIndex\n"
"					TEXTURES_PARAM);\n"
"			EvalStack_PushFloat(eval);\n"
"			break;\n"
"		}\n"
"		case EVAL_SPECTRUM: {\n"
"			const float3 eval = HitPointTriangleAOVTexture_ConstEvaluateSpectrum(hitPoint,\n"
"					texture->hitPointTriangleAOV.dataIndex\n"
"					TEXTURES_PARAM);\n"
"			EvalStack_PushFloat3(eval);\n"
"			break;\n"
"		}\n"
"		case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"			Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"			Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		case EVAL_BUMP:\n"
"			Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		default:\n"
"			// Something wrong here\n"
"			break;\n"
"	}\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Shading Normal texture\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE float ShadingNormalTexture_ConstEvaluateFloat(__global const HitPoint *hitPoint) {\n"
"	// This method doesn't really make sense for a vector - just return the first element\n"
"	return hitPoint->shadeN.x;\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 ShadingNormalTexture_ConstEvaluateSpectrum(__global const HitPoint *hitPoint) {\n"
"	return MAKE_FLOAT3(hitPoint->shadeN.x, hitPoint->shadeN.y, hitPoint->shadeN.z);\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE void ShadingNormalTexture_EvalOp(\n"
"		__global const Texture* restrict texture,\n"
"		const TextureEvalOpType evalType,\n"
"		__global float *evalStack,\n"
"		uint *evalStackOffset,\n"
"		__global const HitPoint *hitPoint,\n"
"		const float sampleDistance\n"
"		TEXTURES_PARAM_DECL) {\n"
"	switch (evalType) {\n"
"		case EVAL_FLOAT: {\n"
"			const float eval = ShadingNormalTexture_ConstEvaluateFloat(hitPoint);\n"
"			EvalStack_PushFloat(eval);\n"
"			break;\n"
"		}\n"
"		case EVAL_SPECTRUM: {\n"
"			const float3 eval = ShadingNormalTexture_ConstEvaluateSpectrum(hitPoint);\n"
"			EvalStack_PushFloat3(eval);\n"
"			break;\n"
"		}\n"
"		case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"			Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"			Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		case EVAL_BUMP:\n"
"			Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		default:\n"
"			// Something wrong here\n"
"			break;\n"
"	}\n"
"}\n";
} }
