
#include <string>

namespace slg { namespace ocl {
std::string KernelSource_texture_imagemap_funcs =
"#line 2 \"texture_brick_funcs.cl\"\n"
"\n"
"/***************************************************************************\n"
" * Copyright 1998-2020 by authors (see AUTHORS.txt)                        *\n"
" *                                                                         *\n"
" *   This file is part of LuxCoreRender.                                   *\n"
" *                                                                         *\n"
" * Licensed under the Apache License, Version 2.0 (the \"License\");         *\n"
" * you may not use this file except in compliance with the License.        *\n"
" * You may obtain a copy of the License at                                 *\n"
" *                                                                         *\n"
" *     http://www.apache.org/licenses/LICENSE-2.0                          *\n"
" *                                                                         *\n"
" * Unless required by applicable law or agreed to in writing, software     *\n"
" * distributed under the License is distributed on an \"AS IS\" BASIS,       *\n"
" * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.*\n"
" * See the License for the specific language governing permissions and     *\n"
" * limitations under the License.                                          *\n"
" ***************************************************************************/\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// ImageMap texture\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE float3 RGBToYCbCr(const float3 rgb) {\n"
"	// ITU-R BT.601 from https://en.wikipedia.org/wiki/YCbCr\n"
"	const float RGBToYCbCrOff[3] = {\n"
"		 16.f / 255.f,\n"
"		128.f / 255.f,\n"
"		128.f / 255.f\n"
"	};\n"
"    const float RGBToYCbCrMat[3][3] = {\n"
"        {  65.481f / 255.f, 128.553f / 255.f,   24.966f / 255.f },\n"
"        { -37.797f / 255.f, -74.203f / 255.f,  112.f    / 255.f },\n"
"        { 112.f    / 255.f, -93.786f / 255.f, -18.214f  / 255.f }\n"
"	};\n"
"\n"
"	// RGB to YCbCr color space transformation\n"
"	const float Y  = RGBToYCbCrOff[0] + RGBToYCbCrMat[0][0] * rgb.x + RGBToYCbCrMat[0][1] * rgb.y + RGBToYCbCrMat[0][2] * rgb.z;\n"
"	const float Cb = RGBToYCbCrOff[1] + RGBToYCbCrMat[1][0] * rgb.x + RGBToYCbCrMat[1][1] * rgb.y + RGBToYCbCrMat[1][2] * rgb.z;\n"
"	const float Cr = RGBToYCbCrOff[2] + RGBToYCbCrMat[2][0] * rgb.x + RGBToYCbCrMat[2][1] * rgb.y + RGBToYCbCrMat[2][2] * rgb.z;\n"
"\n"
"	return clamp(MAKE_FLOAT3(Y, Cb, Cr), 0.f, 1.f);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 YCbCrToRGB(const float3 YCbCr) {\n"
"	// ITU-R BT.601 from https://en.wikipedia.org/wiki/YCbCr\n"
"	const float YCbCrToRGBOff[3] = {\n"
"		-222.921f / 256.f,\n"
"		 135.576f / 256.f,\n"
"		-276.836f / 256.f\n"
"	};\n"
"    const float YCbCrToRGBMat[3][3] = {\n"
"		{ 298.082f / 256.f,    0.f    / 256.f,  408.583f / 256.f },\n"
"		{ 298.082f / 256.f, -100.291f / 256.f, -208.120f / 256.f },\n"
"		{ 298.082f / 256.f,  516.412f / 256.f,    0.f    / 256.f }\n"
"	};\n"
"\n"
"	// YCbCr to RGB color space transformation\n"
"	const float r = YCbCrToRGBOff[0] + YCbCrToRGBMat[0][0] * YCbCr.x + YCbCrToRGBMat[0][1] * YCbCr.y + YCbCrToRGBMat[0][2] * YCbCr.z;\n"
"	const float g = YCbCrToRGBOff[1] + YCbCrToRGBMat[1][0] * YCbCr.x + YCbCrToRGBMat[1][1] * YCbCr.y + YCbCrToRGBMat[1][2] * YCbCr.z;\n"
"	const float b = YCbCrToRGBOff[2] + YCbCrToRGBMat[2][0] * YCbCr.x + YCbCrToRGBMat[2][1] * YCbCr.y + YCbCrToRGBMat[2][2] * YCbCr.z;\n"
"\n"
"	return clamp(MAKE_FLOAT3(r, g, b), 0.f, 1.f);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 ImageMapTexture_SampleTile(__global const Texture* restrict tex,\n"
"		const float2 vertex, const float2 offset\n"
"		TEXTURES_PARAM_DECL) {\n"
"	__global const ImageMap *randomImageMap = &imageMapDescs[tex->imageMapTex.randomImageMapIndex];\n"
"	__global const ImageMap *imageMap = &imageMapDescs[tex->imageMapTex.imageMapIndex];\n"
"	__global const ImageMap *randomizedTilingLUT = &imageMapDescs[tex->imageMapTex.randomizedTilingLUTIndex];\n"
"	\n"
"	const float2 noiseP = MAKE_FLOAT2(vertex.x / randomImageMap->width, vertex.y / randomImageMap->height);\n"
"	const float3 noise = ImageMap_GetSpectrum(randomImageMap, noiseP.x, noiseP.y IMAGEMAPS_PARAM);\n"
"	const float2 pos = MAKE_FLOAT2(.25f, .25f) + MAKE_FLOAT2(noise.x, noise.y) * .5f + offset;\n"
"\n"
"	float3 YCbCr = RGBToYCbCr(ImageMap_GetSpectrum(imageMap, pos.x, pos.y IMAGEMAPS_PARAM));\n"
"\n"
"	YCbCr.x = ImageMap_GetFloat(randomizedTilingLUT, YCbCr.x, .5f IMAGEMAPS_PARAM);\n"
"\n"
"	return YCbCr;\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float ImageMapTexture_SoftClipContrast(const float x, const float w) {\n"
"	const float u = (x > .5f) ? (1.f - x) : x;\n"
"\n"
"	float result;\n"
"	if (u >= .5f - .25f * w)\n"
"		result = (u - .5f) / w + .5f;\n"
"	else if (w >= 2.f / 3.f)\n"
"		result = 8.f * (1.f / w - 1.f) * Sqr(u / (2.f - w)) + (3.f - 2.f / w) * u / (2.f - w);\n"
"	else if (u >= .5f - .75f * w)\n"
"		result = Sqr((u - (.5f - .75f * w)) / w);\n"
"	else\n"
"		result = 0.f;\n"
"\n"
"	if (x > .5f)\n"
"		result = 1.f - result;\n"
"\n"
"	return result;\n"
"}\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float3 ImageMapTexture_RandomizedTilingGetSpectrumValue(\n"
"		__global const Texture* restrict tex,\n"
"		const float2 pos\n"
"		TEXTURES_PARAM_DECL) {\n"
"	const float latticeToSurface_0_0 = .125f;\n"
"	const float latticeToSurface_0_1 = .25f;\n"
"	const float latticeToSurface_1_0 = .216506f;\n"
"	const float latticeToSurface_1_1 = 0.f;\n"
"\n"
"	const float surfaceToLattice_0_0 = 0.f;\n"
"	const float surfaceToLattice_0_1 = 4.6188f;\n"
"	const float surfaceToLattice_1_0 = 4.f;\n"
"	const float surfaceToLattice_1_1 = -2.3094f;\n"
"\n"
"	const float2 lattice = MAKE_FLOAT2(\n"
"		surfaceToLattice_0_0 * pos.x + surfaceToLattice_0_1 * pos.y,\n"
"		surfaceToLattice_1_0 * pos.x + surfaceToLattice_1_1 * pos.y\n"
"	);\n"
"	const float2 cell = MAKE_FLOAT2(floor(lattice.x), floor(lattice.y));\n"
"	float2 uv = lattice - cell;\n"
"\n"
"	float2 v0 = cell;\n"
"	if (uv.x + uv.y >= 1.f) {\n"
"		v0 += MAKE_FLOAT2(1.f, 1.f);\n"
"		uv = MAKE_FLOAT2(1.f, 1.f) - MAKE_FLOAT2(uv.y, uv.x);\n"
"	}\n"
"	float2 v1 = cell + MAKE_FLOAT2(1.f, 0.f);\n"
"	float2 v2 = cell + MAKE_FLOAT2(0.f, 1.f);\n"
"\n"
"	const float2 v0offset = MAKE_FLOAT2(\n"
"		pos.x - (latticeToSurface_0_0 * v0.x + latticeToSurface_0_1 * v0.y),\n"
"		pos.y - (latticeToSurface_1_0 * v0.x + latticeToSurface_1_1 * v0.y)\n"
"	);\n"
"	const float2 v1offset = MAKE_FLOAT2(\n"
"		pos.x - (latticeToSurface_0_0 * v1.x + latticeToSurface_0_1 * v1.y),\n"
"		pos.y - (latticeToSurface_1_0 * v1.x + latticeToSurface_1_1 * v1.y)\n"
"	);\n"
"	const float2 v2offset = MAKE_FLOAT2(\n"
"		pos.x - (latticeToSurface_0_0 * v2.x + latticeToSurface_0_1 * v2.y),\n"
"		pos.y - (latticeToSurface_1_0 * v2.x + latticeToSurface_1_1 * v2.y)\n"
"	);\n"
"\n"
"	const float3 color0 = ImageMapTexture_SampleTile(tex, v0, v0offset TEXTURES_PARAM);\n"
"	const float3 color1 = ImageMapTexture_SampleTile(tex, v1, v1offset TEXTURES_PARAM);\n"
"	const float3 color2 = ImageMapTexture_SampleTile(tex, v2, v2offset TEXTURES_PARAM);\n"
"\n"
"	float3 uvWeights = MAKE_FLOAT3(1.f - uv.x - uv.y, uv.x, uv.y);\n"
"\n"
"	uvWeights.x = uvWeights.x * uvWeights.x * uvWeights.x;\n"
"	uvWeights.y = uvWeights.y * uvWeights.y * uvWeights.y;\n"
"	uvWeights.z = uvWeights.z * uvWeights.z * uvWeights.z;\n"
"\n"
"	uvWeights /= uvWeights.x + uvWeights.y + uvWeights.z;\n"
"\n"
"	float3 YCbCr = uvWeights.x * color0 + uvWeights.y * color1 + uvWeights.z * color2;\n"
"\n"
"	YCbCr.x = ImageMapTexture_SoftClipContrast(YCbCr.x, uvWeights.x + uvWeights.y + uvWeights.z);\n"
"\n"
"	YCbCr.x = ImageMap_GetFloat(&imageMapDescs[tex->imageMapTex.randomizedTilingInvLUTIndex], YCbCr.x, .5f IMAGEMAPS_PARAM);\n"
"\n"
"	return YCbCrToRGB(YCbCr);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float ImageMapTexture_ConstEvaluateFloat(__global const Texture* restrict tex,\n"
"		__global const HitPoint *hitPoint\n"
"		TEXTURES_PARAM_DECL) {\n"
"	const float2 pos = TextureMapping2D_Map(&tex->imageMapTex.mapping, hitPoint TEXTURES_PARAM);\n"
"\n"
"	const float value = (tex->imageMapTex.randomizedTiling) ?\n"
"		Spectrum_Y(ImageMapTexture_RandomizedTilingGetSpectrumValue(tex, pos TEXTURES_PARAM)) :\n"
"		ImageMap_GetFloat(&imageMapDescs[tex->imageMapTex.imageMapIndex], pos.x, pos.y IMAGEMAPS_PARAM);\n"
"	\n"
"	return tex->imageMapTex.gain * value;\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 ImageMapTexture_ConstEvaluateSpectrum(__global const Texture* restrict tex,\n"
"		__global const HitPoint *hitPoint\n"
"		TEXTURES_PARAM_DECL) {\n"
"	const float2 pos = TextureMapping2D_Map(&tex->imageMapTex.mapping, hitPoint TEXTURES_PARAM);\n"
"\n"
"	const float3 value = (tex->imageMapTex.randomizedTiling) ?\n"
"		ImageMapTexture_RandomizedTilingGetSpectrumValue(tex, pos TEXTURES_PARAM) :\n"
"		ImageMap_GetSpectrum(&imageMapDescs[tex->imageMapTex.imageMapIndex], pos.x, pos.y IMAGEMAPS_PARAM);\n"
"\n"
"	return tex->imageMapTex.gain * value;\n"
"}\n"
"\n"
"// Note: ImageMapTexture_Bump() is defined in texture_bump_funcs.cl\n"
"\n"
"OPENCL_FORCE_NOT_INLINE void ImageMapTexture_EvalOp(\n"
"		__global const Texture* restrict texture,\n"
"		const TextureEvalOpType evalType,\n"
"		__global float *evalStack,\n"
"		uint *evalStackOffset,\n"
"		__global const HitPoint *hitPoint,\n"
"		const float sampleDistance\n"
"		TEXTURES_PARAM_DECL) {\n"
"	switch (evalType) {\n"
"		case EVAL_FLOAT: {\n"
"			const float eval = ImageMapTexture_ConstEvaluateFloat(texture, hitPoint TEXTURES_PARAM);\n"
"			EvalStack_PushFloat(eval);\n"
"			break;\n"
"		}\n"
"		case EVAL_SPECTRUM: {\n"
"			const float3 eval = ImageMapTexture_ConstEvaluateSpectrum(texture, hitPoint TEXTURES_PARAM);\n"
"			EvalStack_PushFloat3(eval);\n"
"			break;\n"
"		}\n"
"		case EVAL_BUMP: {\n"
"			const float3 shadeN = ImageMapTexture_Bump(texture, hitPoint TEXTURES_PARAM);\n"
"			EvalStack_PushFloat3(shadeN);\n"
"			break;\n"
"		}\n"
"		default:\n"
"			// Something wrong here\n"
"			break;\n"
"	}\n"
"}\n";
} }
