
#include <string>

namespace slg { namespace ocl {
std::string KernelSource_texture_triplanar_funcs =
"#line 2 \"texture_triplanar_funcs.cl\"\n"
"\n"
"/***************************************************************************\n"
" * Copyright 1998-2020 by authors (see AUTHORS.txt)                        *\n"
" *                                                                         *\n"
" *   This file is part of LuxCoreRender.                                   *\n"
" *                                                                         *\n"
" * Licensed under the Apache License, Version 2.0 (the \"License\");         *\n"
" * you may not use this file except in compliance with the License.        *\n"
" * You may obtain a copy of the License at                                 *\n"
" *                                                                         *\n"
" *     http://www.apache.org/licenses/LICENSE-2.0                          *\n"
" *                                                                         *\n"
" * Unless required by applicable law or agreed to in writing, software     *\n"
" * distributed under the License is distributed on an \"AS IS\" BASIS,       *\n"
" * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.*\n"
" * See the License for the specific language governing permissions and     *\n"
" * limitations under the License.                                          *\n"
" ***************************************************************************/\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Triplanar texture\n"
"//------------------------------------------------------------------------------\n"
"\n"
"//----------------------------------------------------------------------\n"
"// For the very special case of Triplanar texture evaluation\n"
"//\n"
"// EVAL_TRIPLANAR_STEP_1, EVAL_TRIPLANAR_STEP_2, EVAL_TRIPLANAR_STEP_3\n"
"// and EVAL_TRIPLANAR\n"
"//----------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_NOT_INLINE void TriplanarTexture_EvalOp(\n"
"		__global const Texture* restrict texture,\n"
"		const TextureEvalOpType evalType,\n"
"		__global float *evalStack,\n"
"		uint *evalStackOffset,\n"
"		__global const HitPoint *hitPoint,\n"
"		const float sampleDistance\n"
"		TEXTURES_PARAM_DECL) {\n"
"	switch (evalType) {\n"
"		case EVAL_TRIPLANAR_STEP_1: {\n"
"			// Save original UV\n"
"			EvalStack_PushFloat(hitPoint->defaultUV.u);\n"
"			EvalStack_PushFloat(hitPoint->defaultUV.v);\n"
"\n"
"			// Compute localPoint\n"
"			float3 localShadeN;\n"
"			const float3 localPoint = TextureMapping3D_Map(&texture->triplanarTex.mapping,\n"
"					hitPoint, &localShadeN TEXTURES_PARAM);\n"
"\n"
"			// Compute the 3 weights\n"
"			float weightsX = Sqr(Sqr(localShadeN.x));\n"
"			float weightsY = Sqr(Sqr(localShadeN.y));\n"
"			float weightsZ = Sqr(Sqr(localShadeN.z));\n"
"\n"
"			const float sum = weightsX + weightsY + weightsZ;\n"
"			weightsX = weightsX / sum;\n"
"			weightsY = weightsY / sum;\n"
"			weightsZ = weightsZ / sum;\n"
"\n"
"			// Save 3 weights	\n"
"			EvalStack_PushFloat(weightsX);\n"
"			EvalStack_PushFloat(weightsY);\n"
"			EvalStack_PushFloat(weightsZ);\n"
"			// Save localPoint\n"
"			EvalStack_PushFloat3(localPoint);\n"
"\n"
"			// Update HitPoint\n"
"			__global HitPoint *hitPointTmp = (__global HitPoint *)hitPoint;\n"
"			hitPointTmp->defaultUV.u = localPoint.y;\n"
"			hitPointTmp->defaultUV.v = localPoint.z;\n"
"			break;\n"
"		}\n"
"		case EVAL_TRIPLANAR_STEP_2: {\n"
"			// Read localPoint\n"
"			//\n"
"			// Note: -3 is there to skip the result of EVAL_TRIPLANAR_STEP_1\n"
"			const float3 localPoint = EvalStack_ReadFloat3(-3 - 3);\n"
"\n"
"			// Update HitPoint\n"
"			__global HitPoint *hitPointTmp = (__global HitPoint *)hitPoint;\n"
"			hitPointTmp->defaultUV.u = localPoint.x;\n"
"			hitPointTmp->defaultUV.v = localPoint.z;\n"
"			break;\n"
"		}\n"
"		case EVAL_TRIPLANAR_STEP_3: {\n"
"			// Read localPoint\n"
"			//\n"
"			// Note: -6 is there to skip the result of EVAL_TRIPLANAR_STEP_1 and EVAL_TRIPLANAR_STEP_2\n"
"			const float3 localPoint = EvalStack_ReadFloat3(-3 - 6);\n"
"\n"
"			// Update HitPoint\n"
"			__global HitPoint *hitPointTmp = (__global HitPoint *)hitPoint;\n"
"			hitPointTmp->defaultUV.u = localPoint.x;\n"
"			hitPointTmp->defaultUV.v = localPoint.y;\n"
"			break;\n"
"		}\n"
"		case EVAL_FLOAT:\n"
"		case EVAL_SPECTRUM: {\n"
"			// Read textures evaluation\n"
"			float3 tex1, tex2, tex3;\n"
"			EvalStack_PopFloat3(tex3);\n"
"			EvalStack_PopFloat3(tex2);\n"
"			EvalStack_PopFloat3(tex1);\n"
"\n"
"			// Read localPoint\n"
"			float3 localPoint;\n"
"			EvalStack_PopFloat3(localPoint);\n"
"\n"
"			// Read 3 weights\n"
"			float weightX, weightY, weightZ;\n"
"			EvalStack_PopFloat(weightZ);\n"
"			EvalStack_PopFloat(weightY);\n"
"			EvalStack_PopFloat(weightX);\n"
"\n"
"			// Restore original UV\n"
"			__global HitPoint *hitPointTmp = (__global HitPoint *)hitPoint;\n"
"			EvalStack_PopFloat(hitPointTmp->defaultUV.v);\n"
"			EvalStack_PopFloat(hitPointTmp->defaultUV.u);\n"
"\n"
"			const float3 result = tex1 * weightX + tex2 * weightY + tex3 * weightZ;\n"
"			if (evalType == EVAL_FLOAT) {\n"
"				EvalStack_PushFloat(Spectrum_Y(result));\n"
"			} else {\n"
"				EvalStack_PushFloat3(result);\n"
"			}\n"
"			break;\n"
"		}\n"
"		case EVAL_BUMP_TRIPLANAR_STEP_1: {\n"
"			// Save original hit point\n"
"			const float3 p = VLOAD3F(&hitPoint->p.x);\n"
"			EvalStack_PushFloat3(p);\n"
"\n"
"			// Update HitPoint\n"
"			__global HitPoint *hitPointTmp = (__global HitPoint *)hitPoint;\n"
"			hitPointTmp->p.x = p.x + sampleDistance;\n"
"			hitPointTmp->p.y = p.y;\n"
"			hitPointTmp->p.z = p.z;\n"
"			break;\n"
"		}\n"
"		case EVAL_BUMP_TRIPLANAR_STEP_2: {\n"
"			// Read original hit point\n"
"			//\n"
"			// Note: -1 is there to skip the result of EVAL_BUMP_TRIPLANAR_STEP_1\n"
"			const float3 p = EvalStack_ReadFloat3(-1 - 3);\n"
"\n"
"			// Update HitPoint\n"
"			__global HitPoint *hitPointTmp = (__global HitPoint *)hitPoint;\n"
"			hitPointTmp->p.x = p.x;\n"
"			hitPointTmp->p.y = p.y + sampleDistance;\n"
"			hitPointTmp->p.z = p.z;\n"
"			break;\n"
"		}\n"
"		case EVAL_BUMP_TRIPLANAR_STEP_3: {\n"
"			// Read original hit point\n"
"			//\n"
"			// Note: -2 is there to skip the result of EVAL_BUMP_TRIPLANAR_STEP_2\n"
"			const float3 p = EvalStack_ReadFloat3(-2 - 3);\n"
"\n"
"			// Update HitPoint\n"
"			__global HitPoint *hitPointTmp = (__global HitPoint *)hitPoint;\n"
"			hitPointTmp->p.x = p.x;\n"
"			hitPointTmp->p.y = p.y;\n"
"			hitPointTmp->p.z = p.z + sampleDistance;\n"
"			break;\n"
"		}\n"
"		case EVAL_BUMP_GENERIC_OFFSET_U:\n"
"			Texture_EvalOpGenericBumpOffsetU(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		case EVAL_BUMP_GENERIC_OFFSET_V:\n"
"			Texture_EvalOpGenericBumpOffsetV(evalStack, evalStackOffset,\n"
"					hitPoint, sampleDistance);\n"
"			break;\n"
"		case EVAL_BUMP:\n"
"			if (texture->triplanarTex.enableUVlessBumpMap) {\n"
"				float evalFloatTexBase, evalFloatTexOffsetX,\n"
"						evalFloatTexOffsetY, evalFloatTexOffsetZ;\n"
"\n"
"				// Read X/Y/Z textures evaluation\n"
"				EvalStack_PopFloat(evalFloatTexOffsetZ);\n"
"				EvalStack_PopFloat(evalFloatTexOffsetY);\n"
"				EvalStack_PopFloat(evalFloatTexOffsetX);\n"
"\n"
"				// Read and restore original hit point\n"
"				float3 p;\n"
"				EvalStack_PopFloat3(p);\n"
"				__global HitPoint *hitPointTmp = (__global HitPoint *)hitPoint;\n"
"				VSTORE3F(p, &hitPointTmp->p.x);\n"
"\n"
"				// Read base textures evaluation\n"
"				EvalStack_PopFloat(evalFloatTexBase);\n"
"\n"
"				const float3 shadeN = TriplanarTexture_BumpUVLess(hitPoint,\n"
"						sampleDistance, evalFloatTexBase, evalFloatTexOffsetX,\n"
"						evalFloatTexOffsetY, evalFloatTexOffsetZ);\n"
"				EvalStack_PushFloat3(shadeN);\n"
"			} else\n"
"				Texture_EvalOpGenericBump(evalStack, evalStackOffset,\n"
"						hitPoint, sampleDistance);\n"
"			break;\n"
"		default:\n"
"			// Something wrong here\n"
"			break;\n"
"	}\n"
"}\n";
} }
