
#include <string>

namespace slg { namespace ocl {
std::string KernelSource_varianceclamping_funcs =
"#line 2 \"varianceclamping_funcs.cl\"\n"
"\n"
"/***************************************************************************\n"
" * Copyright 1998-2020 by authors (see AUTHORS.txt)                        *\n"
" *                                                                         *\n"
" *   This file is part of LuxCoreRender.                                   *\n"
" *                                                                         *\n"
" * Licensed under the Apache License, Version 2.0 (the \"License\");         *\n"
" * you may not use this file except in compliance with the License.        *\n"
" * You may obtain a copy of the License at                                 *\n"
" *                                                                         *\n"
" *     http://www.apache.org/licenses/LICENSE-2.0                          *\n"
" *                                                                         *\n"
" * Unless required by applicable law or agreed to in writing, software     *\n"
" * distributed under the License is distributed on an \"AS IS\" BASIS,       *\n"
" * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.*\n"
" * See the License for the specific language governing permissions and     *\n"
" * limitations under the License.                                          *\n"
" ***************************************************************************/\n"
"\n"
"OPENCL_FORCE_INLINE void VarianceClamping_ScaledClamp3(__global float *value, const float low, const float high) {\n"
"	const float maxValue = fmax(value[0], fmax(value[1], value[2]));\n"
"\n"
"	if (maxValue > 0.f) {\n"
"		if (maxValue > high) {\n"
"			const float scale = high / maxValue;\n"
"\n"
"			value[0] *= scale;\n"
"			value[1] *= scale;\n"
"			value[2] *= scale;\n"
"			return;\n"
"		}\n"
"\n"
"		if (maxValue < low) {\n"
"			const float scale = low / maxValue;\n"
"\n"
"			value[0] *= scale;\n"
"			value[1] *= scale;\n"
"			value[2] *= scale;\n"
"			return;\n"
"		}\n"
"	}\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void VarianceClamping_Clamp3(const float sqrtVarianceClampMaxValue,\n"
"		__global const float *expectedValue, __global float *value) {\n"
"	if (expectedValue[3] > 0.f) {\n"
"		// Use the current pixel value as expected value\n"
"		const float invWeight = 1.f / expectedValue[3];\n"
"\n"
"		const float minExpectedValue = fmin(expectedValue[0] * invWeight,\n"
"				fmin(expectedValue[1] * invWeight, expectedValue[2] * invWeight));\n"
"		const float maxExpectedValue = fmax(expectedValue[0] * invWeight,\n"
"				fmax(expectedValue[1] * invWeight, expectedValue[2] * invWeight));\n"
"\n"
"		VarianceClamping_ScaledClamp3(value,\n"
"				fmax(minExpectedValue - sqrtVarianceClampMaxValue, 0.f),\n"
"				maxExpectedValue + sqrtVarianceClampMaxValue);\n"
"	} else\n"
"		VarianceClamping_ScaledClamp3(value, 0.f, sqrtVarianceClampMaxValue);	\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void VarianceClamping_Clamp(\n"
"		__global SampleResult *sampleResult, const float sqrtVarianceClampMaxValue\n"
"		FILM_PARAM_DECL) {\n"
"	// Recover the current pixel value\n"
"	const int x = sampleResult->pixelX;\n"
"	const int y = sampleResult->pixelY;\n"
"\n"
"	const uint index1 = x + y * filmWidth;\n"
"	const uint index4 = index1 * 4;\n"
"\n"
"	// Apply variance clamping to each radiance group. This help to avoid problems\n"
"	// with extreme clamping settings and multiple light groups\n"
"\n"
"	for (uint radianceGroupIndex = 0; radianceGroupIndex < film->radianceGroupCount; ++radianceGroupIndex) {\n"
"		VarianceClamping_Clamp3(sqrtVarianceClampMaxValue,\n"
"				&((filmRadianceGroup[radianceGroupIndex])[index4]),\n"
"				sampleResult->radiancePerPixelNormalized[radianceGroupIndex].c);\n"
"	}\n"
"\n"
"	// Clamp the AOVs too\n"
"\n"
"	// DIRECT_DIFFUSE\n"
"\n"
"	if (film->hasChannelDirectDiffuseReflect)\n"
"		VarianceClamping_Clamp3(sqrtVarianceClampMaxValue, &filmDirectDiffuseReflect[index4], sampleResult->directDiffuseReflect.c);\n"
"	else if (film->hasChannelDirectDiffuse)\n"
"		VarianceClamping_Clamp3(sqrtVarianceClampMaxValue, &filmDirectDiffuse[index4], sampleResult->directDiffuseReflect.c);\n"
"\n"
"	if (film->hasChannelDirectDiffuseTransmit)\n"
"		VarianceClamping_Clamp3(sqrtVarianceClampMaxValue, &filmDirectDiffuseTransmit[index4], sampleResult->directDiffuseTransmit.c);\n"
"	else if (film->hasChannelDirectDiffuse)\n"
"		VarianceClamping_Clamp3(sqrtVarianceClampMaxValue, &filmDirectDiffuse[index4], sampleResult->directDiffuseTransmit.c);\n"
"\n"
"	// DIRECT_GLOSSY\n"
"\n"
"	if (film->hasChannelDirectGlossyReflect)\n"
"		VarianceClamping_Clamp3(sqrtVarianceClampMaxValue, &filmDirectGlossyReflect[index4], sampleResult->directGlossyReflect.c);\n"
"	else if (film->hasChannelDirectGlossy)\n"
"		VarianceClamping_Clamp3(sqrtVarianceClampMaxValue, &filmDirectGlossy[index4], sampleResult->directGlossyReflect.c);\n"
"\n"
"	if (film->hasChannelDirectGlossyTransmit)\n"
"		VarianceClamping_Clamp3(sqrtVarianceClampMaxValue, &filmDirectGlossyTransmit[index4], sampleResult->directGlossyTransmit.c);\n"
"	else if (film->hasChannelDirectGlossy)\n"
"		VarianceClamping_Clamp3(sqrtVarianceClampMaxValue, &filmDirectGlossy[index4], sampleResult->directGlossyTransmit.c);\n"
"\n"
"	// EMISSION\n"
"\n"
"	if (film->hasChannelEmission)\n"
"		VarianceClamping_Clamp3(sqrtVarianceClampMaxValue, &filmEmission[index4], sampleResult->emission.c);\n"
"\n"
"	// INDIRECT_DIFFUSE\n"
"\n"
"	if (film->hasChannelIndirectDiffuseReflect)\n"
"		VarianceClamping_Clamp3(sqrtVarianceClampMaxValue, &filmIndirectDiffuseReflect[index4], sampleResult->indirectDiffuseReflect.c);\n"
"	else if (film->hasChannelIndirectDiffuse)\n"
"		VarianceClamping_Clamp3(sqrtVarianceClampMaxValue, &filmIndirectDiffuse[index4], sampleResult->indirectDiffuseReflect.c);\n"
"\n"
"	if (film->hasChannelIndirectDiffuseTransmit)\n"
"		VarianceClamping_Clamp3(sqrtVarianceClampMaxValue, &filmIndirectDiffuseTransmit[index4], sampleResult->indirectDiffuseTransmit.c);\n"
"	else if (film->hasChannelIndirectDiffuse)\n"
"		VarianceClamping_Clamp3(sqrtVarianceClampMaxValue, &filmIndirectDiffuse[index4], sampleResult->indirectDiffuseTransmit.c);\n"
"\n"
"	// INDIRECT_GLOSSY\n"
"\n"
"	if (film->hasChannelIndirectGlossyReflect)\n"
"		VarianceClamping_Clamp3(sqrtVarianceClampMaxValue, &filmIndirectGlossyReflect[index4], sampleResult->indirectGlossyReflect.c);\n"
"	else if (film->hasChannelIndirectGlossy)\n"
"		VarianceClamping_Clamp3(sqrtVarianceClampMaxValue, &filmIndirectGlossy[index4], sampleResult->indirectGlossyReflect.c);\n"
"\n"
"	if (film->hasChannelIndirectGlossyTransmit)\n"
"		VarianceClamping_Clamp3(sqrtVarianceClampMaxValue, &filmIndirectGlossyTransmit[index4], sampleResult->indirectGlossyTransmit.c);\n"
"	else if (film->hasChannelIndirectGlossy)\n"
"		VarianceClamping_Clamp3(sqrtVarianceClampMaxValue, &filmIndirectGlossy[index4], sampleResult->indirectGlossyTransmit.c);\n"
"\n"
"	// INDIRECT_SPECULAR\n"
"\n"
"	if (film->hasChannelIndirectSpecularReflect)\n"
"		VarianceClamping_Clamp3(sqrtVarianceClampMaxValue, &filmIndirectSpecularReflect[index4], sampleResult->indirectSpecularReflect.c);\n"
"	else if (film->hasChannelIndirectSpecular)\n"
"		VarianceClamping_Clamp3(sqrtVarianceClampMaxValue, &filmIndirectSpecular[index4], sampleResult->indirectSpecularReflect.c);\n"
"\n"
"	if (film->hasChannelIndirectSpecularTransmit)\n"
"		VarianceClamping_Clamp3(sqrtVarianceClampMaxValue, &filmIndirectSpecularTransmit[index4], sampleResult->indirectSpecularTransmit.c);\n"
"	else if (film->hasChannelIndirectSpecular)\n"
"		VarianceClamping_Clamp3(sqrtVarianceClampMaxValue, &filmIndirectSpecular[index4], sampleResult->indirectSpecularTransmit.c);\n"
"}\n";
} }
