
#include <string>

namespace slg { namespace ocl {
std::string KernelSource_volume_funcs =
"#line 2 \"volume_funcs.cl\"\n"
"\n"
"/***************************************************************************\n"
" * Copyright 1998-2020 by authors (see AUTHORS.txt)                        *\n"
" *                                                                         *\n"
" *   This file is part of LuxCoreRender.                                   *\n"
" *                                                                         *\n"
" * Licensed under the Apache License, Version 2.0 (the \"License\");         *\n"
" * you may not use this file except in compliance with the License.        *\n"
" * You may obtain a copy of the License at                                 *\n"
" *                                                                         *\n"
" *     http://www.apache.org/licenses/LICENSE-2.0                          *\n"
" *                                                                         *\n"
" * Unless required by applicable law or agreed to in writing, software     *\n"
" * distributed under the License is distributed on an \"AS IS\" BASIS,       *\n"
" * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.*\n"
" * See the License for the specific language governing permissions and     *\n"
" * limitations under the License.                                          *\n"
" ***************************************************************************/\n"
"\n"
"OPENCL_FORCE_INLINE float3 Volume_Emission(__global const Volume *vol, __global const HitPoint *hitPoint\n"
"	TEXTURES_PARAM_DECL) {\n"
"	const uint emiTexIndex = vol->volume.volumeEmissionTexIndex;\n"
"	if (emiTexIndex != NULL_INDEX) {\n"
"		const float3 emiTex = Texture_GetSpectrumValue(emiTexIndex, hitPoint\n"
"			TEXTURES_PARAM);\n"
"		return clamp(emiTex, 0.f, INFINITY);\n"
"	} else\n"
"		return BLACK;\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE void Volume_InitializeTmpHitPoint(__global HitPoint *tmpHitPoint,\n"
"		const float3 rayOrig, const float3 rayDir, const float passThroughEvent) {\n"
"	// Initialize tmpHitPoint\n"
"	HitPoint_InitDefault(tmpHitPoint);\n"
"\n"
"	VSTORE3F(rayDir, &tmpHitPoint->fixedDir.x);\n"
"	VSTORE3F(rayOrig, &tmpHitPoint->p.x);\n"
"	VSTORE3F(-rayDir, &tmpHitPoint->geometryN.x);\n"
"	VSTORE3F(-rayDir, &tmpHitPoint->interpolatedN.x);\n"
"	VSTORE3F(-rayDir, &tmpHitPoint->shadeN.x);\n"
"	tmpHitPoint->passThroughEvent = passThroughEvent;\n"
"	Transform_Init(&tmpHitPoint->localToWorld);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float HomogeneousVolume_SegmentScatter(const float u, \n"
"		const bool scatterAllowed, const float segmentLength,\n"
"		const float3 *sigmaA, const float3 *sigmaS, const float3 *emission,\n"
"		float3 *segmentTransmittance, float3 *segmentEmission) {\n"
"	// This code must work also with segmentLength = INFINITY\n"
"\n"
"	bool scatter = false;\n"
"	*segmentTransmittance = WHITE;\n"
"	*segmentEmission = BLACK;\n"
"\n"
"	//--------------------------------------------------------------------------\n"
"	// Check if there is a scattering event\n"
"	//--------------------------------------------------------------------------\n"
"\n"
"	float scatterDistance = segmentLength;\n"
"	const float sigmaSValue = Spectrum_Filter(*sigmaS);\n"
"	if (scatterAllowed && (sigmaSValue > 0.f)) {\n"
"		// Determine scattering distance\n"
"		const float proposedScatterDistance = -log(1.f - u) / sigmaSValue;\n"
"\n"
"		scatter = (proposedScatterDistance < segmentLength);\n"
"		scatterDistance = scatter ? proposedScatterDistance : segmentLength;\n"
"\n"
"		// Note: scatterDistance can not be infinity because otherwise there would\n"
"		// have been a scatter event before.\n"
"		const float tau = scatterDistance * sigmaSValue;\n"
"		const float pdf = exp(-tau) * (scatter ? sigmaSValue : 1.f);\n"
"		*segmentTransmittance *= 1.f / pdf;\n"
"	}\n"
"\n"
"	//--------------------------------------------------------------------------\n"
"	// Volume transmittance\n"
"	//--------------------------------------------------------------------------\n"
"	\n"
"	const float3 sigmaT = *sigmaA + *sigmaS;\n"
"	if (!Spectrum_IsBlack(sigmaT)) {\n"
"		if (isinf(scatterDistance)) {\n"
"			// This avoid NaN in case scatterDistance is inf\n"
"			*segmentTransmittance = BLACK;\n"
"		} else {\n"
"			const float3 tau = scatterDistance * sigmaT;\n"
"			*segmentTransmittance *= Spectrum_Exp(-tau) * (scatter ? sigmaT : WHITE);\n"
"		}\n"
"	}\n"
"\n"
"	//--------------------------------------------------------------------------\n"
"	// Volume emission\n"
"	//--------------------------------------------------------------------------\n"
"\n"
"	*segmentEmission += (*segmentTransmittance) * scatterDistance * (*emission);\n"
"\n"
"	return scatter ? scatterDistance : -1.f;\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// ClearVolume scatter\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE float3 ClearVolume_SigmaA(__global const Volume *vol, __global const HitPoint *hitPoint\n"
"	TEXTURES_PARAM_DECL) {\n"
"	const float3 sigmaA = Texture_GetSpectrumValue(vol->volume.clear.sigmaATexIndex, hitPoint\n"
"		TEXTURES_PARAM);\n"
"			\n"
"	return clamp(sigmaA, 0.f, INFINITY);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 ClearVolume_SigmaS(__global const Volume *vol, __global const HitPoint *hitPoint\n"
"	TEXTURES_PARAM_DECL) {\n"
"	return BLACK;\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 ClearVolume_SigmaT(__global const Volume *vol, __global const HitPoint *hitPoint\n"
"	TEXTURES_PARAM_DECL) {\n"
"	return\n"
"			ClearVolume_SigmaA(vol, hitPoint\n"
"				TEXTURES_PARAM) +\n"
"			ClearVolume_SigmaS(vol, hitPoint\n"
"				TEXTURES_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float ClearVolume_Scatter(__global const Volume *vol,\n"
"		__global Ray *ray, const float hitT,\n"
"		const float passThroughEvent,\n"
"		const bool scatteredStart, float3 *connectionThroughput,\n"
"		float3 *connectionEmission, __global HitPoint *tmpHitPoint\n"
"		TEXTURES_PARAM_DECL) {\n"
"	const float3 rayOrig = VLOAD3F(&ray->o.x);\n"
"	const float3 rayDir = VLOAD3F(&ray->d.x);\n"
"\n"
"	// Initialize tmpHitPoint\n"
"	Volume_InitializeTmpHitPoint(tmpHitPoint, rayOrig, rayDir, passThroughEvent);\n"
"\n"
"	const float distance = hitT - ray->mint;	\n"
"	float3 transmittance = WHITE;\n"
"\n"
"	const float3 sigmaT = ClearVolume_SigmaT(vol, tmpHitPoint\n"
"			TEXTURES_PARAM);\n"
"	if (!Spectrum_IsBlack(sigmaT)) {\n"
"		const float3 tau = clamp(distance * sigmaT, 0.f, INFINITY);\n"
"		transmittance = Spectrum_Exp(-tau);\n"
"	}\n"
"\n"
"	// Apply volume transmittance\n"
"	*connectionThroughput *= transmittance;\n"
"\n"
"	// Apply volume emission\n"
"	const uint emiTexIndex = vol->volume.volumeEmissionTexIndex;\n"
"	if (emiTexIndex != NULL_INDEX) {\n"
"		const float3 emiTex = Texture_GetSpectrumValue(emiTexIndex, tmpHitPoint\n"
"			TEXTURES_PARAM);\n"
"		*connectionEmission += *connectionThroughput * distance * clamp(emiTex, 0.f, INFINITY);\n"
"	}\n"
"\n"
"	return -1.f;\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// HomogeneousVolume scatter\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE float3 HomogeneousVolume_SigmaA(__global const Volume *vol, __global const HitPoint *hitPoint\n"
"	TEXTURES_PARAM_DECL) {\n"
"	const float3 sigmaA = Texture_GetSpectrumValue(vol->volume.homogenous.sigmaATexIndex, hitPoint\n"
"		TEXTURES_PARAM);\n"
"			\n"
"	return clamp(sigmaA, 0.f, INFINITY);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 HomogeneousVolume_SigmaS(__global const Volume *vol, __global const HitPoint *hitPoint\n"
"	TEXTURES_PARAM_DECL) {\n"
"	const float3 sigmaS = Texture_GetSpectrumValue(vol->volume.homogenous.sigmaSTexIndex, hitPoint\n"
"		TEXTURES_PARAM);\n"
"			\n"
"	return clamp(sigmaS, 0.f, INFINITY);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float HomogeneousVolume_Scatter(__global const Volume *vol,\n"
"		__global Ray *ray, const float hitT,\n"
"		const float passThroughEvent,\n"
"		const bool scatteredStart, float3 *connectionThroughput,\n"
"		float3 *connectionEmission, __global HitPoint *tmpHitPoint\n"
"		TEXTURES_PARAM_DECL) {\n"
"	const float3 rayOrig = VLOAD3F(&ray->o.x);\n"
"	const float3 rayDir = VLOAD3F(&ray->d.x);\n"
"\n"
"	// Initialize tmpHitPoint\n"
"	Volume_InitializeTmpHitPoint(tmpHitPoint, rayOrig, rayDir, passThroughEvent);\n"
"\n"
"	const float segmentLength = hitT - ray->mint;\n"
"\n"
"	// Check if I have to support multi-scattering\n"
"	const bool scatterAllowed = (!scatteredStart || vol->volume.homogenous.multiScattering);\n"
"	\n"
"	const float3 sigmaA = HomogeneousVolume_SigmaA(vol, tmpHitPoint\n"
"			TEXTURES_PARAM);\n"
"	const float3 sigmaS = HomogeneousVolume_SigmaS(vol, tmpHitPoint\n"
"			TEXTURES_PARAM);\n"
"	const float3 emission = Volume_Emission(vol, tmpHitPoint\n"
"			TEXTURES_PARAM);\n"
"\n"
"	float3 segmentTransmittance, segmentEmission;\n"
"	const float scatterDistance = HomogeneousVolume_SegmentScatter(passThroughEvent, scatterAllowed,\n"
"			segmentLength, &sigmaA, &sigmaS, &emission,\n"
"			&segmentTransmittance, &segmentEmission);\n"
"\n"
"	// I need to update first connectionEmission and than connectionThroughput\n"
"	*connectionEmission += *connectionThroughput * emission;\n"
"	*connectionThroughput *= segmentTransmittance;\n"
"\n"
"	return (scatterDistance == -1.f) ? -1.f : (ray->mint + scatterDistance);\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// HeterogeneousVolume scatter\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_INLINE float3 HeterogeneousVolume_SigmaA(__global const Volume *vol, __global const HitPoint *hitPoint\n"
"	TEXTURES_PARAM_DECL) {\n"
"	const float3 sigmaA = Texture_GetSpectrumValue(vol->volume.heterogenous.sigmaATexIndex, hitPoint\n"
"		TEXTURES_PARAM);\n"
"			\n"
"	return clamp(sigmaA, 0.f, INFINITY);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 HeterogeneousVolume_SigmaS(__global const Volume *vol, __global const HitPoint *hitPoint\n"
"	TEXTURES_PARAM_DECL) {\n"
"	const float3 sigmaS = Texture_GetSpectrumValue(vol->volume.heterogenous.sigmaSTexIndex, hitPoint\n"
"		TEXTURES_PARAM);\n"
"			\n"
"	return clamp(sigmaS, 0.f, INFINITY);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float3 HeterogeneousVolume_SigmaT(__global const Volume *vol, __global const HitPoint *hitPoint\n"
"	TEXTURES_PARAM_DECL) {\n"
"	return HeterogeneousVolume_SigmaA(vol, hitPoint TEXTURES_PARAM) +\n"
"			HeterogeneousVolume_SigmaS(vol, hitPoint TEXTURES_PARAM);\n"
"}\n"
"\n"
"OPENCL_FORCE_INLINE float HeterogeneousVolume_Scatter(__global const Volume *vol,\n"
"		__global Ray *ray, const float hitT,\n"
"		const float passThroughEvent,\n"
"		const bool scatteredStart, float3 *connectionThroughput,\n"
"		float3 *connectionEmission, __global HitPoint *tmpHitPoint\n"
"		TEXTURES_PARAM_DECL) {\n"
"	// I need a sequence of pseudo-random numbers starting form a floating point\n"
"	// pseudo-random number\n"
"	Seed seed;\n"
"	Rnd_InitFloat(passThroughEvent, &seed);\n"
"\n"
"	const float stepSize = vol->volume.heterogenous.stepSize;\n"
"	const uint maxStepsCount = vol->volume.heterogenous.maxStepsCount;\n"
"\n"
"	const float3 rayOrig = VLOAD3F(&ray->o.x);\n"
"	const float3 rayDir = VLOAD3F(&ray->d.x);\n"
"\n"
"	// Compute the number of steps to evaluate the volume\n"
"	const float segmentLength = hitT - ray->mint;\n"
"\n"
"	// Handle the case when segmentLength is infinity or a very large number\n"
"	//\n"
"	// Note: the old code\"Min(maxStepsCount, Ceil2UInt(segmentLength / stepSize))\"\n"
"	// can overflow for large values of segmentLength so I have to use\n"
"	// \"Ceil2UInt(Min((float)maxStepsCount, segmentLength / stepSize))\"\n"
"	const uint steps = Ceil2UInt(fmin((float)maxStepsCount, segmentLength / stepSize));\n"
"\n"
"	const float currentStepSize = fmin(segmentLength / steps, maxStepsCount * stepSize);\n"
"\n"
"	// Check if I have to support multi-scattering\n"
"	const bool scatterAllowed = (!scatteredStart || vol->volume.heterogenous.multiScattering);\n"
"\n"
"	// Initialize tmpHitPoint\n"
"	Volume_InitializeTmpHitPoint(tmpHitPoint, rayOrig, rayDir, passThroughEvent);\n"
"\n"
"	for (uint s = 0; s < steps; ++s) {\n"
"		// Compute the scattering over the current step\n"
"		const float evaluationPoint = (s + Rnd_FloatValue(&seed)) * currentStepSize;\n"
"\n"
"		VSTORE3F(rayOrig + (ray->mint + evaluationPoint) * rayDir, &tmpHitPoint->p.x);\n"
"\n"
"		// Volume segment values\n"
"		const float3 sigmaA = HeterogeneousVolume_SigmaA(vol, tmpHitPoint\n"
"				TEXTURES_PARAM);\n"
"		const float3 sigmaS = HeterogeneousVolume_SigmaS(vol, tmpHitPoint\n"
"				TEXTURES_PARAM);\n"
"		const float3 emission = Volume_Emission(vol, tmpHitPoint\n"
"				TEXTURES_PARAM);\n"
"\n"
"		// Evaluate the current segment like if it was an homogenous volume\n"
"		//\n"
"		// This could be optimized by inlining the code and exploiting\n"
"		// exp(a) * exp(b) = exp(a + b) in order to evaluate a single exp() at\n"
"		// the end instead of one each step.\n"
"		// However the code would be far less simple and readable.\n"
"		float3 segmentTransmittance, segmentEmission;\n"
"		const float scatterDistance = HomogeneousVolume_SegmentScatter(Rnd_FloatValue(&seed), scatterAllowed,\n"
"				currentStepSize, &sigmaA, &sigmaS, &emission,\n"
"				&segmentTransmittance, &segmentEmission);\n"
"\n"
"		// I need to update first connectionEmission and than connectionThroughput\n"
"		*connectionEmission += *connectionThroughput * emission;\n"
"		*connectionThroughput *= segmentTransmittance;\n"
"\n"
"		if (scatterDistance >= 0.f)\n"
"			return ray->mint + s * currentStepSize + scatterDistance;\n"
"	}\n"
"\n"
"	return -1.f;\n"
"}\n"
"\n"
"//------------------------------------------------------------------------------\n"
"// Volume scatter\n"
"//------------------------------------------------------------------------------\n"
"\n"
"OPENCL_FORCE_NOT_INLINE float Volume_Scatter(__global const Volume *vol,\n"
"		__global Ray *ray, const float hitT,\n"
"		const float passThrough,\n"
"		const bool scatteredStart, float3 *connectionThroughput,\n"
"		float3 *connectionEmission, __global HitPoint *tmpHitPoint\n"
"		TEXTURES_PARAM_DECL) {\n"
"	switch (vol->type) {\n"
"		case CLEAR_VOL:\n"
"			return ClearVolume_Scatter(vol, ray, hitT,\n"
"					passThrough, scatteredStart,\n"
"					connectionThroughput, connectionEmission, tmpHitPoint\n"
"					TEXTURES_PARAM);\n"
"		case HOMOGENEOUS_VOL:\n"
"			return HomogeneousVolume_Scatter(vol, ray, hitT,\n"
"					passThrough, scatteredStart,\n"
"					connectionThroughput, connectionEmission, tmpHitPoint\n"
"					TEXTURES_PARAM);\n"
"		case HETEROGENEOUS_VOL:\n"
"			return HeterogeneousVolume_Scatter(vol, ray, hitT,\n"
"					passThrough, scatteredStart,\n"
"					connectionThroughput, connectionEmission, tmpHitPoint\n"
"					TEXTURES_PARAM);\n"
"		default:\n"
"			return -1.f;\n"
"	}\n"
"}\n";
} }
